@@ -11,12 +11,11 @@ import { fileURLToPath } from 'url';
 import which from 'which';
 import { EventEmitter } from 'events';
 import * as filetype from 'file-type';
-import { assign, groupBy, IDisposable, toDisposable, dispose, mkdirp, readBytes, detectUnicodeEncoding, Encoding, onceEvent, splitInChunks, Limiter, Versions, isWindows, pathEquals, isMacintosh, isDescendant, relativePathWithNoFallback } from './util';
+import { assign, groupBy, IDisposable, toDisposable, dispose, mkdirp, readBytes, detectUnicodeEncoding, Encoding, onceEvent, splitInChunks, Limiter, Versions, isWindows, pathEquals, isMacintosh, isDescendant, relativePathWithNoFallback, Mutable } from './util';
 import { CancellationError, CancellationToken, ConfigurationChangeEvent, LogOutputChannel, Progress, Uri, workspace } from 'vscode';
-import { Commit as ApiCommit, Ref, RefType, Branch, Remote, ForcePushMode, GitErrorCodes, LogOptions, Change, Status, CommitOptions, RefQuery as ApiRefQuery, InitOptions } from './api/git';
+import { Commit as ApiCommit, Ref, RefType, Branch, Remote, ForcePushMode, GitErrorCodes, LogOptions, Change, Status, CommitOptions, RefQuery as ApiRefQuery, InitOptions, DiffChange, Worktree as ApiWorktree } from './api/git';
 import * as byline from 'byline';
 import { StringDecoder } from 'string_decoder';
-import { sequentialize } from './decorators';
 
 // https://github.com/microsoft/vscode/issues/65693
 const MAX_CLI_LENGTH = 30000;
@@ -26,6 +25,13 @@ export interface IGit {
 	version: string;
 }
 
+export interface IDotGit {
+	readonly path: string;
+	readonly commonPath?: string;
+	readonly superProjectPath?: string;
+	readonly isBare: boolean;
+}
+
 export interface IFileStatus {
 	x: string;
 	y: string;
@@ -39,6 +45,8 @@ export interface Stash {
 	readonly index: number;
 	readonly description: string;
 	readonly branchName?: string;
+	readonly authorDate?: Date;
+	readonly commitDate?: Date;
 }
 
 interface MutableRemote extends Remote {
@@ -112,7 +120,7 @@ function findGitDarwin(onValidate: (path: string) => boolean): Promise<IGit> {
 			}
 
 			// must check if XCode is installed
-			cp.exec('xcode-select -p', (err: any) => {
+			cp.exec('xcode-select -p', (err) => {
 				if (err && err.code === 2) {
 					// git is not installed, and launching /usr/bin/git
 					// will prompt the user to install it
@@ -302,8 +310,8 @@ export class GitError extends Error {
 			stderr: this.stderr
 		}, null, 2);
 
-		if (this.error) {
-			result += (<any>this.error).stack;
+		if (this.error?.stack) {
+			result += this.error.stack;
 		}
 
 		return result;
@@ -344,8 +352,13 @@ function getGitErrorCode(stderr: string): string | undefined {
 		return GitErrorCodes.DirtyWorkTree;
 	} else if (/detected dubious ownership in repository at/.test(stderr)) {
 		return GitErrorCodes.NotASafeGitRepository;
+	} else if (/contains modified or untracked files|use --force to delete it/.test(stderr)) {
+		return GitErrorCodes.WorktreeContainsChanges;
+	} else if (/fatal: '[^']+' already exists/.test(stderr)) {
+		return GitErrorCodes.WorktreeAlreadyExists;
+	} else if (/is already used by worktree at/.test(stderr)) {
+		return GitErrorCodes.WorktreeBranchAlreadyUsed;
 	}
-
 	return undefined;
 }
 
@@ -360,7 +373,7 @@ function sanitizeRelativePath(path: string): string {
 }
 
 const COMMIT_FORMAT = '%H%n%aN%n%aE%n%at%n%ct%n%P%n%D%n%B';
-const STASH_FORMAT = '%H%n%P%n%gd%n%gs';
+const STASH_FORMAT = '%H%n%P%n%gd%n%gs%n%at%n%ct';
 
 export interface ICloneOptions {
 	readonly parentPath: string;
@@ -404,7 +417,7 @@ export class Git {
 		return Versions.compare(Versions.fromString(this.version), Versions.fromString(version));
 	}
 
-	open(repositoryRoot: string, repositoryRootRealPath: string | undefined, dotGit: { path: string; commonPath?: string }, logger: LogOutputChannel): Repository {
+	open(repositoryRoot: string, repositoryRootRealPath: string | undefined, dotGit: IDotGit, logger: LogOutputChannel): Repository {
 		return new Repository(this, repositoryRoot, repositoryRootRealPath, dotGit, logger);
 	}
 
@@ -540,9 +553,16 @@ export class Git {
 		return repositoryRootPath;
 	}
 
-	async getRepositoryDotGit(repositoryPath: string): Promise<{ path: string; commonPath?: string }> {
-		const result = await this.exec(repositoryPath, ['rev-parse', '--git-dir', '--git-common-dir']);
-		let [dotGitPath, commonDotGitPath] = result.stdout.split('\n').map(r => r.trim());
+	async getRepositoryDotGit(repositoryPath: string): Promise<IDotGit> {
+		let dotGitPath: string | undefined, commonDotGitPath: string | undefined, superProjectPath: string | undefined;
+
+		const args = ['rev-parse', '--git-dir', '--git-common-dir'];
+		if (this.compareGitVersionTo('2.13.0') >= 0) {
+			args.push('--show-superproject-working-tree');
+		}
+
+		const result = await this.exec(repositoryPath, args);
+		[dotGitPath, commonDotGitPath, superProjectPath] = result.stdout.split('\n').map(r => r.trim());
 
 		if (!path.isAbsolute(dotGitPath)) {
 			dotGitPath = path.join(repositoryPath, dotGitPath);
@@ -554,11 +574,18 @@ export class Git {
 				commonDotGitPath = path.join(repositoryPath, commonDotGitPath);
 			}
 			commonDotGitPath = path.normalize(commonDotGitPath);
-
-			return { path: dotGitPath, commonPath: commonDotGitPath !== dotGitPath ? commonDotGitPath : undefined };
 		}
 
-		return { path: dotGitPath };
+		const raw = await fs.readFile(path.join(commonDotGitPath ?? dotGitPath, 'config'), 'utf8');
+		const coreSections = GitConfigParser.parse(raw).find(s => s.name === 'core');
+		const isBare = coreSections?.properties['bare'] === 'true';
+
+		return {
+			isBare,
+			path: dotGitPath,
+			commonPath: commonDotGitPath !== dotGitPath ? commonDotGitPath : undefined,
+			superProjectPath: superProjectPath ? path.normalize(superProjectPath) : undefined
+		};
 	}
 
 	async exec(cwd: string, args: string[], options: SpawnOptions = {}): Promise<IExecutionResult<string>> {
@@ -657,6 +684,7 @@ export class Git {
 
 		options.env = assign({}, process.env, this.env, options.env || {}, {
 			VSCODE_GIT_COMMAND: args[0],
+			LANGUAGE: 'en',
 			LC_ALL: 'en_US.UTF-8',
 			LANG: 'en_US.UTF-8',
 			GIT_PAGER: 'cat'
@@ -974,12 +1002,12 @@ export function parseLsFiles(raw: string): LsFilesElement[] {
 		.map(([, mode, object, stage, file]) => ({ mode, object, stage, file }));
 }
 
-const stashRegex = /([0-9a-f]{40})\n(.*)\nstash@{(\d+)}\n(WIP\s)*on([^:]+):(.*)(?:\x00)/gmi;
+const stashRegex = /([0-9a-f]{40})\n(.*)\nstash@{(\d+)}\n(WIP\s)?on\s([^:]+):\s(.*)\n(\d+)\n(\d+)(?:\x00)/gmi;
 
 function parseGitStashes(raw: string): Stash[] {
 	const result: Stash[] = [];
 
-	let match, hash, parents, index, wip, branchName, description;
+	let match, hash, parents, index, wip, branchName, description, authorDate, commitDate;
 
 	do {
 		match = stashRegex.exec(raw);
@@ -987,13 +1015,15 @@ function parseGitStashes(raw: string): Stash[] {
 			break;
 		}
 
-		[, hash, parents, index, wip, branchName, description] = match;
+		[, hash, parents, index, wip, branchName, description, authorDate, commitDate] = match;
 		result.push({
 			hash,
 			parents: parents.split(' '),
 			index: parseInt(index),
 			branchName: branchName.trim(),
-			description: wip ? `WIP (${description.trim()})` : description.trim()
+			description: wip ? `WIP (${description.trim()})` : description.trim(),
+			authorDate: authorDate ? new Date(Number(authorDate) * 1000) : undefined,
+			commitDate: commitDate ? new Date(Number(commitDate) * 1000) : undefined,
 		});
 	} while (true);
 
@@ -1061,6 +1091,90 @@ function parseGitChanges(repositoryRoot: string, raw: string): Change[] {
 	return result;
 }
 
+function parseGitChangesRaw(repositoryRoot: string, raw: string): DiffChange[] {
+	const changes: Change[] = [];
+	const numStats = new Map<string, { insertions: number; deletions: number }>();
+
+	let index = 0;
+	const segments = raw.trim().split('\x00').filter(s => s);
+
+	segmentsLoop:
+	while (index < segments.length) {
+		const segment = segments[index++];
+		if (!segment) {
+			break;
+		}
+
+		if (segment.startsWith(':')) {
+			// Parse --raw output
+			const [, , , , change] = segment.split(' ');
+			const filePath = segments[index++];
+			const originalUri = Uri.file(path.isAbsolute(filePath) ? filePath : path.join(repositoryRoot, filePath));
+
+			let uri = originalUri;
+			let renameUri = originalUri;
+			let status = Status.UNTRACKED;
+
+			switch (change[0]) {
+				case 'A':
+					status = Status.INDEX_ADDED;
+					break;
+				case 'M':
+					status = Status.MODIFIED;
+					break;
+				case 'D':
+					status = Status.DELETED;
+					break;
+				case 'R': {
+					if (index >= segments.length) {
+						break;
+					}
+					const newPath = segments[index++];
+					if (!newPath) {
+						break;
+					}
+
+					status = Status.INDEX_RENAMED;
+					uri = renameUri = Uri.file(path.isAbsolute(newPath) ? newPath : path.join(repositoryRoot, newPath));
+					break;
+				}
+				default:
+					// Unknown status
+					break segmentsLoop;
+			}
+
+			changes.push({ status, uri, originalUri, renameUri });
+		} else {
+			// Parse --numstat output
+			const [insertions, deletions, filePath] = segment.split('\t');
+
+			let numstatPath: string;
+			if (filePath === '') {
+				// For renamed files, filePath is empty and the old/new paths
+				// are in the next two null-terminated segments. We skip the
+				// old path and use the new path for the stats key.
+				index++;
+
+				const renamePath = segments[index++];
+				numstatPath = path.isAbsolute(renamePath) ? renamePath : path.join(repositoryRoot, renamePath);
+			} else {
+				numstatPath = path.isAbsolute(filePath) ? filePath : path.join(repositoryRoot, filePath);
+			}
+
+			numStats.set(numstatPath, {
+				insertions: insertions === '-' ? 0 : parseInt(insertions),
+				deletions: deletions === '-' ? 0 : parseInt(deletions),
+			});
+		}
+	}
+
+	return changes.map(change => ({
+		...change,
+		insertions: numStats.get(change.uri.fsPath)?.insertions ?? 0,
+		deletions: numStats.get(change.uri.fsPath)?.deletions ?? 0,
+	}));
+}
+
 export interface BlameInformation {
 	readonly hash: string;
 	readonly subject?: string;
@@ -1207,6 +1321,10 @@ export interface PullOptions {
 	readonly cancellationToken?: CancellationToken;
 }
 
+export interface Worktree extends ApiWorktree {
+	readonly commitDetails?: ApiCommit;
+}
+
 export class Repository {
 	private _isUsingRefTable = false;
 
@@ -1214,9 +1332,20 @@ export class Repository {
 		private _git: Git,
 		private repositoryRoot: string,
 		private repositoryRootRealPath: string | undefined,
-		readonly dotGit: { path: string; commonPath?: string },
+		readonly dotGit: IDotGit,
 		private logger: LogOutputChannel
-	) { }
+	) {
+		this._kind = this.dotGit.commonPath
+			? 'worktree'
+			: this.dotGit.superProjectPath
+				? 'submodule'
+				: 'repository';
+	}
+
+	private readonly _kind: 'repository' | 'submodule' | 'worktree';
+	get kind(): 'repository' | 'submodule' | 'worktree' {
+		return this._kind;
+	}
 
 	get git(): Git {
 		return this._git;
@@ -1242,7 +1371,6 @@ export class Repository {
 		return this.git.spawn(args, options);
 	}
 
-	@sequentialize
 	async config(command: string, scope: string, key: string, value: any = null, options: SpawnOptions = {}): Promise<string> {
 		const args = ['config', `--${command}`];
 
@@ -1551,11 +1679,19 @@ export class Repository {
 		}
 	}
 
-	async apply(patch: string, reverse?: boolean): Promise<void> {
+	async apply(patch: string, options?: { reverse?: boolean; threeWay?: boolean; allowEmpty?: boolean }): Promise<void> {
 		const args = ['apply', patch];
 
-		if (reverse) {
-			args.push('-R');
+		if (options?.allowEmpty) {
+			args.push('--allow-empty');
+		}
+
+		if (options?.reverse) {
+			args.push('--reverse');
+		}
+
+		if (options?.threeWay) {
+			args.push('--3way');
 		}
 
 		try {
@@ -1585,7 +1721,7 @@ export class Repository {
 	diffWithHEAD(path?: string | undefined): Promise<string | Change[]>;
 	async diffWithHEAD(path?: string | undefined): Promise<string | Change[]> {
 		if (!path) {
-			return await this.diffFiles(false);
+			return await this.diffFiles(undefined, { cached: false });
 		}
 
 		const args = ['diff', '--', this.sanitizeRelativePath(path)];
@@ -1593,12 +1729,16 @@ export class Repository {
 		return result.stdout;
 	}
 
+	async diffWithHEADShortStats(path?: string): Promise<CommitShortStat> {
+		return this.diffFilesShortStat(undefined, { cached: false, path });
+	}
+
 	diffWith(ref: string): Promise<Change[]>;
 	diffWith(ref: string, path: string): Promise<string>;
 	diffWith(ref: string, path?: string | undefined): Promise<string | Change[]>;
 	async diffWith(ref: string, path?: string): Promise<string | Change[]> {
 		if (!path) {
-			return await this.diffFiles(false, ref);
+			return await this.diffFiles(ref, { cached: false });
 		}
 
 		const args = ['diff', ref, '--', this.sanitizeRelativePath(path)];
@@ -1611,7 +1751,7 @@ export class Repository {
 	diffIndexWithHEAD(path?: string | undefined): Promise<Change[]>;
 	async diffIndexWithHEAD(path?: string): Promise<string | Change[]> {
 		if (!path) {
-			return await this.diffFiles(true);
+			return await this.diffFiles(undefined, { cached: true });
 		}
 
 		const args = ['diff', '--cached', '--', this.sanitizeRelativePath(path)];
@@ -1619,12 +1759,16 @@ export class Repository {
 		return result.stdout;
 	}
 
+	async diffIndexWithHEADShortStats(path?: string): Promise<CommitShortStat> {
+		return this.diffFilesShortStat(undefined, { cached: true, path });
+	}
+
 	diffIndexWith(ref: string): Promise<Change[]>;
 	diffIndexWith(ref: string, path: string): Promise<string>;
 	diffIndexWith(ref: string, path?: string | undefined): Promise<string | Change[]>;
 	async diffIndexWith(ref: string, path?: string): Promise<string | Change[]> {
 		if (!path) {
-			return await this.diffFiles(true, ref);
+			return await this.diffFiles(ref, { cached: true });
 		}
 
 		const args = ['diff', '--cached', ref, '--', this.sanitizeRelativePath(path)];
@@ -1644,7 +1788,7 @@ export class Repository {
 	async diffBetween(ref1: string, ref2: string, path?: string): Promise<string | Change[]> {
 		const range = `${ref1}...${ref2}`;
 		if (!path) {
-			return await this.diffFiles(false, range);
+			return await this.diffFiles(range, { cached: false });
 		}
 
 		const args = ['diff', range, '--', this.sanitizeRelativePath(path)];
@@ -1653,27 +1797,54 @@ export class Repository {
 		return result.stdout.trim();
 	}
 
-	async diffBetweenShortStat(ref1: string, ref2: string): Promise<{ files: number; insertions: number; deletions: number }> {
-		const args = ['diff', '--shortstat', `${ref1}...${ref2}`];
+	async diffBetweenPatch(ref: string, options: { path?: string }): Promise<string> {
+		const args = ['diff', ref, '--'];
 
-		const result = await this.exec(args);
-		if (result.exitCode) {
-			return { files: 0, insertions: 0, deletions: 0 };
+		if (options.path) {
+			args.push(this.sanitizeRelativePath(options.path));
 		}
 
-		return parseGitDiffShortStat(result.stdout.trim());
+		const result = await this.exec(args);
+		return result.stdout;
 	}
 
-	private async diffFiles(cached: boolean, ref?: string): Promise<Change[]> {
+	async diffBetweenWithStats(ref: string, options: { path?: string; similarityThreshold?: number }): Promise<DiffChange[]> {
+		const args = ['diff', '--raw', '--numstat', '--diff-filter=ADMR', '-z',];
+
+		if (options.similarityThreshold) {
+			args.push(`--find-renames=${options.similarityThreshold}%`);
+		}
+
+		args.push(...[ref, '--']);
+		if (options.path) {
+			args.push(this.sanitizeRelativePath(options.path));
+		}
+
+		const gitResult = await this.exec(args);
+		if (gitResult.exitCode) {
+			return [];
+		}
+
+		return parseGitChangesRaw(this.repositoryRoot, gitResult.stdout);
+	}
+
+	private async diffFiles(ref: string | undefined, options: { cached: boolean; similarityThreshold?: number }): Promise<Change[]> {
 		const args = ['diff', '--name-status', '-z', '--diff-filter=ADMR'];
-		if (cached) {
+
+		if (options.cached) {
 			args.push('--cached');
 		}
 
+		if (options.similarityThreshold) {
+			args.push(`--find-renames=${options.similarityThreshold}%`);
+		}
+
 		if (ref) {
 			args.push(ref);
 		}
 
+		args.push('--');
+
 		const gitResult = await this.exec(args);
 		if (gitResult.exitCode) {
 			return [];
@@ -1682,8 +1853,34 @@ export class Repository {
 		return parseGitChanges(this.repositoryRoot, gitResult.stdout);
 	}
 
-	async diffTrees(treeish1: string, treeish2?: string, options?: { similarityThreshold?: number }): Promise<Change[]> {
-		const args = ['diff-tree', '-r', '--name-status', '-z', '--diff-filter=ADMR'];
+	private async diffFilesShortStat(ref: string | undefined, options: { cached: boolean; path?: string }): Promise<CommitShortStat> {
+		const args = ['diff', '--shortstat'];
+
+		if (options.cached) {
+			args.push('--cached');
+		}
+
+		if (ref !== undefined) {
+			args.push(ref);
+		}
+
+		args.push('--');
+
+		if (options.path) {
+			args.push(this.sanitizeRelativePath(options.path));
+		}
+
+		const result = await this.exec(args);
+		if (result.exitCode) {
+			return { files: 0, insertions: 0, deletions: 0 };
+		}
+
+		return parseGitDiffShortStat(result.stdout.trim());
+	}
+
+
+	async diffTrees(treeish1: string, treeish2?: string, options?: { similarityThreshold?: number }): Promise<DiffChange[]> {
+		const args = ['diff-tree', '-r', '--raw', '--numstat', '--diff-filter=ADMR', '-z'];
 
 		if (options?.similarityThreshold) {
 			args.push(`--find-renames=${options.similarityThreshold}%`);
@@ -1695,12 +1892,14 @@ export class Repository {
 			args.push(treeish2);
 		}
 
+		args.push('--');
+
 		const gitResult = await this.exec(args);
 		if (gitResult.exitCode) {
 			return [];
 		}
 
-		return parseGitChanges(this.repositoryRoot, gitResult.stdout);
+		return parseGitChangesRaw(this.repositoryRoot, gitResult.stdout);
 	}
 
 	async getMergeBase(ref1: string, ref2: string, ...refs: string[]): Promise<string | undefined> {
@@ -1762,7 +1961,15 @@ export class Repository {
 	async stage(path: string, data: Uint8Array): Promise<void> {
 		const relativePath = this.sanitizeRelativePath(path);
 		const child = this.stream(['hash-object', '--stdin', '-w', '--path', relativePath], { stdio: [null, null, null] });
-		child.stdin!.end(data);
+
+		if (!child.stdin) {
+			throw new GitError({
+				message: 'Failed to spawn git process',
+				exitCode: -1
+			});
+		}
+
+		child.stdin.end(data);
 
 		const { exitCode, stdout } = await exec(child);
 		const hash = stdout.toString('utf8');
@@ -1866,8 +2073,12 @@ export class Repository {
 			args.push('--signoff');
 		}
 
-		if (opts.signCommit) {
-			args.push('-S');
+		if (opts.signCommit !== undefined) {
+			if (opts.signCommit) {
+				args.push('-S');
+			} else {
+				args.push('--no-gpg-sign');
+			}
 		}
 
 		if (opts.empty) {
@@ -1904,11 +2115,12 @@ export class Repository {
 		}
 	}
 
-	private async handleCommitError(commitErr: any): Promise<void> {
-		if (/not possible because you have unmerged files/.test(commitErr.stderr || '')) {
+
+	private async handleCommitError(commitErr: unknown): Promise<void> {
+		if (commitErr instanceof GitError && /not possible because you have unmerged files/.test(commitErr.stderr || '')) {
 			commitErr.gitErrorCode = GitErrorCodes.UnmergedChanges;
 			throw commitErr;
-		} else if (/Aborting commit due to empty commit message/.test(commitErr.stderr || '')) {
+		} else if (commitErr instanceof GitError && /Aborting commit due to empty commit message/.test(commitErr.stderr || '')) {
 			commitErr.gitErrorCode = GitErrorCodes.EmptyCommitMessage;
 			throw commitErr;
 		}
@@ -2004,6 +2216,29 @@ export class Repository {
 		await this.exec(args);
 	}
 
+	async addWorktree(options: { path: string; commitish: string; branch?: string }): Promise<void> {
+		const args = ['worktree', 'add'];
+
+		if (options.branch) {
+			args.push('-b', options.branch);
+		}
+
+		args.push(options.path, options.commitish);
+
+		await this.exec(args);
+	}
+
+	async deleteWorktree(path: string, options?: { force?: boolean }): Promise<void> {
+		const args = ['worktree', 'remove'];
+
+		if (options?.force) {
+			args.push('--force');
+		}
+
+		args.push(path);
+		await this.exec(args);
+	}
+
 	async deleteRemoteRef(remoteName: string, refName: string, options?: { force?: boolean }): Promise<void> {
 		const args = ['push', remoteName, '--delete'];
 
@@ -2019,8 +2254,8 @@ export class Repository {
 		const pathsByGroup = groupBy(paths.map(sanitizePath), p => path.dirname(p));
 		const groups = Object.keys(pathsByGroup).map(k => pathsByGroup[k]);
 
-		const limiter = new Limiter(5);
-		const promises: Promise<any>[] = [];
+		const limiter = new Limiter<IExecutionResult<string>>(5);
+		const promises: Promise<IExecutionResult<string>>[] = [];
 		const args = ['clean', '-f', '-q'];
 
 		for (const paths of groups) {
@@ -2321,10 +2556,14 @@ export class Repository {
 		}
 	}
 
-	async blame2(path: string, ref?: string): Promise<BlameInformation[] | undefined> {
+	async blame2(path: string, ref?: string, ignoreWhitespace?: boolean): Promise<BlameInformation[] | undefined> {
 		try {
 			const args = ['blame', '--root', '--incremental'];
 
+			if (ignoreWhitespace) {
+				args.push('-w');
+			}
+
 			if (ref) {
 				args.push(ref);
 			}
@@ -2366,13 +2605,19 @@ export class Repository {
 		}
 	}
 
-	async popStash(index?: number): Promise<void> {
+	async popStash(index?: number, options?: { reinstateStagedChanges?: boolean }): Promise<void> {
 		const args = ['stash', 'pop'];
+		if (options?.reinstateStagedChanges) {
+			args.push('--index');
+		}
 		await this.popOrApplyStash(args, index);
 	}
 
-	async applyStash(index?: number): Promise<void> {
+	async applyStash(index?: number, options?: { reinstateStagedChanges?: boolean }): Promise<void> {
 		const args = ['stash', 'apply'];
+		if (options?.reinstateStagedChanges) {
+			args.push('--index');
+		}
 		await this.popOrApplyStash(args, index);
 	}
 
@@ -2488,19 +2733,23 @@ export class Repository {
 
 				if (limit !== 0 && parser.status.length > limit) {
 					child.removeListener('close', onClose);
-					child.stdout!.removeListener('data', onStdoutData);
+					child.stdout?.removeListener('data', onStdoutData);
 					child.kill();
 
 					c({ status: parser.status.slice(0, limit), statusLength: parser.status.length, didHitLimit: true });
 				}
 			};
 
-			child.stdout!.setEncoding('utf8');
-			child.stdout!.on('data', onStdoutData);
+			if (child.stdout) {
+				child.stdout.setEncoding('utf8');
+				child.stdout.on('data', onStdoutData);
+			}
 
 			const stderrData: string[] = [];
-			child.stderr!.setEncoding('utf8');
-			child.stderr!.on('data', raw => stderrData.push(raw as string));
+			if (child.stderr) {
+				child.stderr.setEncoding('utf8');
+				child.stderr.on('data', raw => stderrData.push(raw as string));
+			}
 
 			child.on('error', cpErrorHandler(e));
 			child.on('close', onClose);
@@ -2716,6 +2965,77 @@ export class Repository {
 		return parseGitStashes(result.stdout.trim());
 	}
 
+	async getWorktrees(): Promise<Worktree[]> {
+		return await this.getWorktreesFS();
+	}
+
+	private async getWorktreesFS(): Promise<Worktree[]> {
+		const result: Worktree[] = [];
+		const mainRepositoryPath = this.dotGit.commonPath ?? this.dotGit.path;
+
+		try {
+			if (!this.dotGit.isBare) {
+				// Add main worktree for a non-bare repository
+				const headPath = path.join(mainRepositoryPath, 'HEAD');
+				const headContent = (await fs.readFile(headPath, 'utf8')).trim();
+
+				const mainRepositoryWorktreeName = path.basename(path.dirname(mainRepositoryPath));
+
+				result.push({
+					name: mainRepositoryWorktreeName,
+					path: path.dirname(mainRepositoryPath),
+					ref: headContent.replace(/^ref: /, ''),
+					detached: !headContent.startsWith('ref: '),
+					main: true
+				} satisfies Worktree);
+			}
+
+			// List all worktree folder names
+			const worktreesPath = path.join(mainRepositoryPath, 'worktrees');
+			const dirents = await fs.readdir(worktreesPath, { withFileTypes: true });
+
+			for (const dirent of dirents) {
+				if (!dirent.isDirectory()) {
+					continue;
+				}
+
+				try {
+					const headPath = path.join(worktreesPath, dirent.name, 'HEAD');
+					const headContent = (await fs.readFile(headPath, 'utf8')).trim();
+
+					const gitdirPath = path.join(worktreesPath, dirent.name, 'gitdir');
+					const gitdirContent = (await fs.readFile(gitdirPath, 'utf8')).trim();
+
+					result.push({
+						name: dirent.name,
+						// Remove '/.git' suffix
+						path: gitdirContent.replace(/\/.git.*$/, ''),
+						// Remove 'ref: ' prefix
+						ref: headContent.replace(/^ref: /, ''),
+						// Detached if HEAD does not start with 'ref: '
+						detached: !headContent.startsWith('ref: '),
+						main: false
+					});
+				} catch (err) {
+					if (/ENOENT/.test(err.message)) {
+						continue;
+					}
+
+					throw err;
+				}
+			}
+
+			return result;
+		}
+		catch (err) {
+			if (/ENOENT/.test(err.message) || /ENOTDIR/.test(err.message)) {
+				return result;
+			}
+
+			throw err;
+		}
+	}
+
 	async getRemotes(): Promise<Remote[]> {
 		const remotes: MutableRemote[] = [];
 
@@ -2856,8 +3176,8 @@ export class Repository {
 					const result = await this.exec(['rev-list', '--left-right', '--count', `${branch.name}...${branch.upstream.remote}/${branch.upstream.name}`]);
 					const [ahead, behind] = result.stdout.trim().split('\t');
 
-					(branch as any).ahead = Number(ahead) || 0;
-					(branch as any).behind = Number(behind) || 0;
+					(branch as Mutable<Branch>).ahead = Number(ahead) || 0;
+					(branch as Mutable<Branch>).behind = Number(behind) || 0;
 				} catch { }
 			}
 
@@ -2937,9 +3257,27 @@ export class Repository {
 		return commits[0];
 	}
 
-	async showCommit(ref: string): Promise<string> {
+	async showChanges(ref: string): Promise<string> {
 		try {
-			const result = await this.exec(['show', ref]);
+			const result = await this.exec(['log', '-p', '-n1', ref, '--']);
+			return result.stdout.trim();
+		} catch (err) {
+			if (/^fatal: bad revision '.+'/.test(err.stderr || '')) {
+				err.gitErrorCode = GitErrorCodes.BadRevision;
+			}
+
+			throw err;
+		}
+	}
+
+	async showChangesBetween(ref1: string, ref2: string, path?: string): Promise<string> {
+		try {
+			const args = ['log', '-p', `${ref1}..${ref2}`, '--'];
+			if (path) {
+				args.push(this.sanitizeRelativePath(path));
+			}
+
+			const result = await this.exec(args);
 			return result.stdout.trim();
 		} catch (err) {
 			if (/^fatal: bad revision '.+'/.test(err.stderr || '')) {
