@@ -12,13 +12,13 @@ import type * as Proto from '../tsServer/protocol/protocol';
 import * as typeConverters from '../typeConverters';
 import { ClientCapability, ITypeScriptServiceClient } from '../typescriptService';
 import { nulToken } from '../utils/cancellation';
-import { memoize } from '../utils/memoize';
+import { Lazy } from '../utils/lazy';
 import { equals } from '../utils/objects';
 import { DiagnosticsManager } from './diagnostics';
 import FileConfigurationManager from './fileConfigurationManager';
 import { applyCodeActionCommands, getEditForCodeAction } from './util/codeAction';
+import { CompositeCommand, EditorChatFollowUp, EditorChatFollowUp_Args, Expand } from './util/copilot';
 import { conditionalRegistration, requireSomeCapability } from './util/dependentRegistration';
-import { Expand, EditorChatFollowUp_Args, CompositeCommand, EditorChatFollowUp } from './util/copilot';
 
 type ApplyCodeActionCommand_args = {
 	readonly document: vscode.TextDocument;
@@ -199,17 +199,16 @@ class SupportedCodeActionProvider {
 	) { }
 
 	public async getFixableDiagnosticsForContext(diagnostics: readonly vscode.Diagnostic[]): Promise<DiagnosticsSet> {
-		const fixableCodes = await this.fixableDiagnosticCodes;
+		const fixableCodes = await this.fixableDiagnosticCodes.value;
 		return DiagnosticsSet.from(
 			diagnostics.filter(diagnostic => typeof diagnostic.code !== 'undefined' && fixableCodes.has(diagnostic.code + '')));
 	}
 
-	@memoize
-	private get fixableDiagnosticCodes(): Thenable<Set<string>> {
+	private readonly fixableDiagnosticCodes = new Lazy<Thenable<Set<string>>>(() => {
 		return this.client.execute('getSupportedCodeFixes', null, nulToken)
 			.then(response => response.type === 'response' ? response.body || [] : [])
 			.then(codes => new Set(codes));
-	}
+	});
 }
 
 class TypeScriptQuickFixProvider implements vscode.CodeActionProvider<VsCodeCodeAction> {
@@ -367,22 +366,19 @@ class TypeScriptQuickFixProvider implements vscode.CodeActionProvider<VsCodeCode
 			let expand: Expand | undefined;
 			let title = action.description;
 			if (action.fixName === fixNames.classIncorrectlyImplementsInterface) {
-				title += ' with Copilot';
-				message = `Implement the stubbed-out class members for ${document.getText(diagnostic.range)} with a useful implementation.`;
+				title = vscode.l10n.t('{0} with AI', action.description);
+				message = vscode.l10n.t('Implement the stubbed-out class members for {0} with a useful implementation.', document.getText(diagnostic.range));
 				expand = { kind: 'code-action', action };
-			}
-			else if (action.fixName === fixNames.fixClassDoesntImplementInheritedAbstractMember) {
-				title += ' with Copilot';
-				message = `Implement the stubbed-out class members for ${document.getText(diagnostic.range)} with a useful implementation.`;
+			} else if (action.fixName === fixNames.fixClassDoesntImplementInheritedAbstractMember) {
+				title = vscode.l10n.t('{0} with AI', action.description);
+				message = vscode.l10n.t(`Implement the stubbed-out class members for {0} with a useful implementation.`, document.getText(diagnostic.range));
 				expand = { kind: 'code-action', action };
-			}
-			else if (action.fixName === fixNames.fixMissingFunctionDeclaration) {
-				title = `Implement missing function declaration '${document.getText(diagnostic.range)}' using Copilot`;
-				message = `Provide a reasonable implementation of the function ${document.getText(diagnostic.range)} given its type and the context it's called in.`;
+			} else if (action.fixName === fixNames.fixMissingFunctionDeclaration) {
+				title = vscode.l10n.t(`Implement missing function declaration '{0}' using AI`, document.getText(diagnostic.range));
+				message = vscode.l10n.t(`Provide a reasonable implementation of the function {0} given its type and the context it's called in.`, document.getText(diagnostic.range));
 				expand = { kind: 'code-action', action };
-			}
-			else if (action.fixName === fixNames.inferFromUsage) {
-				const inferFromBody = new VsCodeCodeAction(action, 'Infer types using Copilot', vscode.CodeActionKind.QuickFix);
+			} else if (action.fixName === fixNames.inferFromUsage) {
+				const inferFromBody = new VsCodeCodeAction(action, vscode.l10n.t('Infer types using AI'), vscode.CodeActionKind.QuickFix);
 				inferFromBody.edit = new vscode.WorkspaceEdit();
 				inferFromBody.diagnostics = [diagnostic];
 				inferFromBody.ranges = [diagnostic.range];
@@ -390,7 +386,7 @@ class TypeScriptQuickFixProvider implements vscode.CodeActionProvider<VsCodeCode
 				inferFromBody.command = {
 					command: EditorChatFollowUp.ID,
 					arguments: [{
-						message: 'Add types to this code. Add separate interfaces when possible. Do not change the code except for adding types.',
+						message: vscode.l10n.t('Add types to this code. Add separate interfaces when possible. Do not change the code except for adding types.'),
 						expand: { kind: 'navtree-function', pos: diagnostic.range.start },
 						document,
 						action: { type: 'quickfix', quickfix: action }
@@ -401,8 +397,8 @@ class TypeScriptQuickFixProvider implements vscode.CodeActionProvider<VsCodeCode
 			}
 			else if (action.fixName === fixNames.addNameToNamelessParameter) {
 				const newText = action.changes.map(change => change.textChanges.map(textChange => textChange.newText).join('')).join('');
-				title = 'Add meaningful parameter name with Copilot';
-				message = `Rename the parameter ${newText} with a more meaningful name.`;
+				title = vscode.l10n.t('Add meaningful parameter name with AI');
+				message = vscode.l10n.t(`Rename the parameter {0} with a more meaningful name.`, newText);
 				expand = {
 					kind: 'navtree-function',
 					pos: diagnostic.range.start
