@@ -10,7 +10,7 @@ export function isWeb(): boolean {
 }
 
 export function isWebAndHasSharedArrayBuffers(): boolean {
-	return isWeb() && (globalThis as any)['crossOriginIsolated'];
+	return isWeb() && !!(globalThis as Record<string, unknown>)['crossOriginIsolated'];
 }
 
 export function supportsReadableByteStreams(): boolean {
