@@ -27,9 +27,9 @@ import { ScrollbarVisibility, ScrollEvent } from '../../../common/scrollable.js'
 import { ISpliceable } from '../../../common/sequence.js';
 import { isNumber } from '../../../common/types.js';
 import './list.css';
-import { IIdentityProvider, IKeyboardNavigationDelegate, IKeyboardNavigationLabelProvider, IListContextMenuEvent, IListDragAndDrop, IListDragOverReaction, IListEvent, IListGestureEvent, IListMouseEvent, IListElementRenderDetails, IListRenderer, IListTouchEvent, IListVirtualDelegate, ListError } from './list.js';
+import { IIdentityProvider, IKeyboardNavigationDelegate, IKeyboardNavigationLabelProvider, IListContextMenuEvent, IListDragAndDrop, IListDragOverReaction, IListEvent, IListGestureEvent, IListMouseEvent, IListElementRenderDetails, IListRenderer, IListTouchEvent, IListVirtualDelegate, ListError, NotSelectableGroupId, NotSelectableGroupIdType } from './list.js';
 import { IListView, IListViewAccessibilityProvider, IListViewDragAndDrop, IListViewOptions, IListViewOptionsUpdate, ListViewTargetSector, ListView } from './listView.js';
-import { StandardMouseEvent } from '../../mouseEvent.js';
+import { IMouseWheelEvent, StandardMouseEvent } from '../../mouseEvent.js';
 import { autorun, constObservable, IObservable } from '../../../common/observable.js';
 
 interface ITraitChangeEvent {
@@ -120,7 +120,7 @@ class Trait<T> implements ISpliceable<boolean>, IDisposable {
 	protected sortedIndexes: number[] = [];
 
 	private readonly _onChange = new Emitter<ITraitChangeEvent>();
-	readonly onChange: Event<ITraitChangeEvent> = this._onChange.event;
+	get onChange(): Event<ITraitChangeEvent> { return this._onChange.event; }
 
 	get name(): string { return this._trait; }
 
@@ -403,7 +403,17 @@ class KeyboardController<T> implements IDisposable {
 	private onCtrlA(e: StandardKeyboardEvent): void {
 		e.preventDefault();
 		e.stopPropagation();
-		this.list.setSelection(range(this.list.length), e.browserEvent);
+
+		let selection = range(this.list.length);
+
+		// Filter by group if identity provider has getGroupId
+		const focusedElements = this.list.getFocus();
+		const referenceGroupId = focusedElements.length > 0 ? this.list.getElementGroupId(focusedElements[0]) : undefined;
+		if (referenceGroupId !== undefined) {
+			selection = this.list.filterIndicesByGroup(selection, referenceGroupId);
+		}
+
+		this.list.setSelection(selection, e.browserEvent);
 		this.list.setAnchor(undefined);
 		this.view.domNode.focus();
 	}
@@ -631,6 +641,7 @@ class DOMFocusController<T> implements IDisposable {
 			return;
 		}
 
+		// eslint-disable-next-line no-restricted-syntax
 		const tabIndexElement = focusedDomElement.querySelector('[tabIndex]');
 
 		if (!tabIndexElement || !(isHTMLElement(tabIndexElement)) || tabIndexElement.tabIndex === -1) {
@@ -672,11 +683,11 @@ const DefaultMultipleSelectionController = {
 export class MouseController<T> implements IDisposable {
 
 	private multipleSelectionController: IMultipleSelectionController<T> | undefined;
-	private mouseSupport: boolean;
+	private readonly mouseSupport: boolean;
 	private readonly disposables = new DisposableStore();
 
-	private _onPointer = new Emitter<IListMouseEvent<T>>();
-	readonly onPointer: Event<IListMouseEvent<T>> = this._onPointer.event;
+	private readonly _onPointer = this.disposables.add(new Emitter<IListMouseEvent<T>>());
+	get onPointer() { return this._onPointer.event; }
 
 	constructor(protected list: List<T>) {
 		if (list.options.multipleSelectionSupport !== false) {
@@ -776,7 +787,11 @@ export class MouseController<T> implements IDisposable {
 		this.list.setAnchor(focus);
 
 		if (!isMouseRightClick(e.browserEvent)) {
-			this.list.setSelection([focus], e.browserEvent);
+			// Check if the element is selectable (getGroupId must not return undefined)
+			const focusGroupId = this.list.getElementGroupId(focus);
+			if (focusGroupId !== NotSelectableGroupId) {
+				this.list.setSelection([focus], e.browserEvent);
+			}
 		}
 
 		this._onPointer.fire(e);
@@ -813,7 +828,16 @@ export class MouseController<T> implements IDisposable {
 
 			const min = Math.min(anchor, focus);
 			const max = Math.max(anchor, focus);
-			const rangeSelection = range(min, max + 1);
+			let rangeSelection = range(min, max + 1);
+
+			const selectedElement = this.list.getSelection()[0];
+			if (selectedElement !== undefined) {
+				const referenceGroupId = this.list.getElementGroupId(selectedElement);
+				if (referenceGroupId !== undefined) {
+					rangeSelection = this.list.filterIndicesByGroup(rangeSelection, referenceGroupId);
+				}
+			}
+
 			const selection = this.list.getSelection();
 			const contiguousRange = getContiguousRangeContaining(disjunction(selection, [anchor]), anchor);
 
@@ -832,8 +856,16 @@ export class MouseController<T> implements IDisposable {
 			this.list.setFocus([focus]);
 			this.list.setAnchor(focus);
 
+			const focusGroupId = this.list.getElementGroupId(focus);
+			if (focusGroupId === NotSelectableGroupId) {
+				return; // Cannot select this element, do nothing
+			}
+
 			if (selection.length === newSelection.length) {
-				this.list.setSelection([...newSelection, focus], e.browserEvent);
+				const itemsToBeSelected = focusGroupId !== undefined ?
+					this.list.filterIndicesByGroup([...newSelection, focus], focusGroupId)
+					: [...newSelection, focus];
+				this.list.setSelection(itemsToBeSelected, e.browserEvent);
 			} else {
 				this.list.setSelection(newSelection, e.browserEvent);
 			}
@@ -856,10 +888,10 @@ export interface IStyleController {
 
 export interface IListAccessibilityProvider<T> extends IListViewAccessibilityProvider<T> {
 	getAriaLabel(element: T): string | IObservable<string> | null;
-	getWidgetAriaLabel(): string;
+	getWidgetAriaLabel(): string | IObservable<string>;
 	getWidgetRole?(): AriaRole;
 	getAriaLevel?(element: T): number | undefined;
-	onDidChangeActiveDescendant?: Event<void>;
+	readonly onDidChangeActiveDescendant?: Event<void>;
 	getActiveDescendantId?(element: T): string | undefined;
 }
 
@@ -954,7 +986,7 @@ export class DefaultStyleController implements IStyleController {
 			content.push(`
 				.monaco-drag-image${suffix},
 				.monaco-list${suffix}:focus .monaco-list-row.focused,
-				.monaco-workbench.context-menu-visible .monaco-list${suffix}.last-focused .monaco-list-row.focused { outline: 1px solid ${styles.listFocusOutline}; outline-offset: -1px; }
+				.context-menu-visible .monaco-list${suffix}.last-focused .monaco-list-row.focused { outline: 1px solid ${styles.listFocusOutline}; outline-offset: -1px; }
 			`);
 		}
 
@@ -1002,13 +1034,13 @@ export class DefaultStyleController implements IStyleController {
 			content.push(`
 				.monaco-table > .monaco-split-view2,
 				.monaco-table > .monaco-split-view2 .monaco-sash.vertical::before,
-				.monaco-workbench:not(.reduce-motion) .monaco-table:hover > .monaco-split-view2,
-				.monaco-workbench:not(.reduce-motion) .monaco-table:hover > .monaco-split-view2 .monaco-sash.vertical::before {
+				.monaco-enable-motion .monaco-table:hover > .monaco-split-view2,
+				.monaco-enable-motion .monaco-table:hover > .monaco-split-view2 .monaco-sash.vertical::before {
 					border-color: ${styles.tableColumnsBorder};
 				}
 
-				.monaco-workbench:not(.reduce-motion) .monaco-table > .monaco-split-view2,
-				.monaco-workbench:not(.reduce-motion) .monaco-table > .monaco-split-view2 .monaco-sash.vertical::before {
+				.monaco-enable-motion .monaco-table > .monaco-split-view2,
+				.monaco-enable-motion .monaco-table > .monaco-split-view2 .monaco-sash.vertical::before {
 					border-color: transparent;
 				}
 			`);
@@ -1530,7 +1562,12 @@ export class List<T> implements ISpliceable<T>, IDisposable {
 		this.onDidChangeSelection(this._onSelectionChange, this, this.disposables);
 
 		if (this.accessibilityProvider) {
-			this.ariaLabel = this.accessibilityProvider.getWidgetAriaLabel();
+			const ariaLabel = this.accessibilityProvider.getWidgetAriaLabel();
+			const observable = (ariaLabel && typeof ariaLabel !== 'string') ? ariaLabel : constObservable(ariaLabel);
+
+			this.disposables.add(autorun(reader => {
+				this.ariaLabel = reader.readObservable(observable);
+			}));
 		}
 
 		if (this._options.multipleSelectionSupport !== false) {
@@ -1692,6 +1729,8 @@ export class List<T> implements ISpliceable<T>, IDisposable {
 			}
 		}
 
+		indexes = indexes.filter(i => this.getElementGroupId(i) !== NotSelectableGroupId);
+
 		this.selection.set(indexes, browserEvent);
 	}
 
@@ -1725,6 +1764,42 @@ export class List<T> implements ISpliceable<T>, IDisposable {
 		return typeof anchor === 'undefined' ? undefined : this.element(anchor);
 	}
 
+	/**
+	 * Gets the group ID for an element at the given index.
+	 * Returns undefined if no identity provider, no getGroupId method, or if the group ID is undefined.
+	 */
+	getElementGroupId(index: number): number | NotSelectableGroupIdType | undefined {
+		const identityProvider = this.options.identityProvider;
+		if (!identityProvider?.getGroupId) {
+			return undefined;
+		}
+
+		const element = this.element(index);
+		return identityProvider.getGroupId(element);
+	}
+
+	/**
+	 * Filters the given indices to only include those with a matching group ID.
+	 * If no identity provider or getGroupId method exists, returns the original indices.
+	 * If referenceGroupId is undefined, returns an empty array (elements without group IDs are not selectable).
+	 */
+	filterIndicesByGroup(indices: number[], referenceGroupId: number | NotSelectableGroupIdType): number[] {
+		const identityProvider = this.options.identityProvider;
+		if (!identityProvider?.getGroupId) {
+			return indices;
+		}
+
+		if (referenceGroupId === NotSelectableGroupId) {
+			return [];
+		}
+
+		return indices.filter(index => {
+			const element = this.element(index);
+			const groupId = identityProvider.getGroupId!(element);
+			return groupId === referenceGroupId;
+		});
+	}
+
 	setFocus(indexes: number[], browserEvent?: UIEvent): void {
 		for (const index of indexes) {
 			if (index < 0 || index >= this.length) {
@@ -1966,6 +2041,10 @@ export class List<T> implements ISpliceable<T>, IDisposable {
 		this.styleController.style(styles);
 	}
 
+	delegateScrollFromMouseWheelEvent(browserEvent: IMouseWheelEvent) {
+		this.view.delegateScrollFromMouseWheelEvent(browserEvent);
+	}
+
 	private toListEvent({ indexes, browserEvent }: ITraitChangeEvent) {
 		return { indexes, elements: indexes.map(i => this.view.element(i)), browserEvent };
 	}
