@@ -161,32 +161,6 @@ export function raceTimeout<T>(promise: Promise<T>, timeout: number, onTimeout?:
 	]);
 }
 
-export function raceFilter<T>(promises: Promise<T>[], filter: (result: T) => boolean): Promise<T | undefined> {
-	return new Promise((resolve, reject) => {
-		if (promises.length === 0) {
-			resolve(undefined);
-			return;
-		}
-
-		let resolved = false;
-		let unresolvedCount = promises.length;
-		for (const promise of promises) {
-			promise.then(result => {
-				unresolvedCount--;
-				if (!resolved) {
-					if (filter(result)) {
-						resolved = true;
-						resolve(result);
-					} else if (unresolvedCount === 0) {
-						// Last one has to resolve the promise
-						resolve(undefined);
-					}
-				}
-			}).catch(reject);
-		}
-	});
-}
-
 export function asPromise<T>(callback: () => T | Thenable<T>): Promise<T> {
 	return new Promise<T>((resolve, reject) => {
 		const item = callback();
@@ -217,6 +191,10 @@ export interface ITask<T> {
 	(): T;
 }
 
+export interface ICancellableTask<T> {
+	(token: CancellationToken): T;
+}
+
 /**
  * A helper to prevent accumulation of sequential async tasks.
  *
@@ -247,18 +225,19 @@ export class Throttler implements IDisposable {
 
 	private activePromise: Promise<any> | null;
 	private queuedPromise: Promise<any> | null;
-	private queuedPromiseFactory: ITask<Promise<any>> | null;
-
-	private isDisposed = false;
+	private queuedPromiseFactory: ICancellableTask<Promise<any>> | null;
+	private cancellationTokenSource: CancellationTokenSource;
 
 	constructor() {
 		this.activePromise = null;
 		this.queuedPromise = null;
 		this.queuedPromiseFactory = null;
+
+		this.cancellationTokenSource = new CancellationTokenSource();
 	}
 
-	queue<T>(promiseFactory: ITask<Promise<T>>): Promise<T> {
-		if (this.isDisposed) {
+	queue<T>(promiseFactory: ICancellableTask<Promise<T>>): Promise<T> {
+		if (this.cancellationTokenSource.token.isCancellationRequested) {
 			return Promise.reject(new Error('Throttler is disposed'));
 		}
 
@@ -269,7 +248,7 @@ export class Throttler implements IDisposable {
 				const onComplete = () => {
 					this.queuedPromise = null;
 
-					if (this.isDisposed) {
+					if (this.cancellationTokenSource.token.isCancellationRequested) {
 						return;
 					}
 
@@ -289,7 +268,7 @@ export class Throttler implements IDisposable {
 			});
 		}
 
-		this.activePromise = promiseFactory();
+		this.activePromise = promiseFactory(this.cancellationTokenSource.token);
 
 		return new Promise((resolve, reject) => {
 			this.activePromise!.then((result: T) => {
@@ -303,7 +282,7 @@ export class Throttler implements IDisposable {
 	}
 
 	dispose(): void {
-		this.isDisposed = true;
+		this.cancellationTokenSource.cancel();
 	}
 }
 
@@ -334,6 +313,10 @@ export class SequencerByKey<TKey> {
 		return newPromise;
 	}
 
+	peek(key: TKey): Promise<unknown> | undefined {
+		return this.promiseMap.get(key) || undefined;
+	}
+
 	keys(): IterableIterator<TKey> {
 		return this.promiseMap.keys();
 	}
@@ -484,7 +467,7 @@ export class ThrottledDelayer<T> {
 		this.throttler = new Throttler();
 	}
 
-	trigger(promiseFactory: ITask<Promise<T>>, delay?: number): Promise<T> {
+	trigger(promiseFactory: ICancellableTask<Promise<T>>, delay?: number): Promise<T> {
 		return this.delayer.trigger(() => this.throttler.queue(promiseFactory), delay) as unknown as Promise<T>;
 	}
 
@@ -945,6 +928,12 @@ export class ResourceQueue implements IDisposable {
 
 export type Task<T = void> = () => (Promise<T> | T);
 
+/**
+ * Wrap a type in an optional promise. This can be useful to avoid the runtime
+ * overhead of creating a promise.
+ */
+export type MaybePromise<T> = Promise<T> | T;
+
 /**
  * Processes tasks in the order they were scheduled.
 */
@@ -1459,7 +1448,8 @@ export let runWhenGlobalIdle: (callback: (idle: IdleDeadline) => void, timeout?:
 export let _runWhenIdle: (targetWindow: IdleApi, callback: (idle: IdleDeadline) => void, timeout?: number) => IDisposable;
 
 (function () {
-	if (typeof globalThis.requestIdleCallback !== 'function' || typeof globalThis.cancelIdleCallback !== 'function') {
+	const safeGlobal: any = globalThis;
+	if (typeof safeGlobal.requestIdleCallback !== 'function' || typeof safeGlobal.cancelIdleCallback !== 'function') {
 		_runWhenIdle = (_targetWindow, runner, timeout?) => {
 			setTimeout0(() => {
 				if (disposed) {
@@ -1485,7 +1475,7 @@ export let _runWhenIdle: (targetWindow: IdleApi, callback: (idle: IdleDeadline)
 			};
 		};
 	} else {
-		_runWhenIdle = (targetWindow: IdleApi, runner, timeout?) => {
+		_runWhenIdle = (targetWindow: typeof safeGlobal, runner, timeout?) => {
 			const handle: number = targetWindow.requestIdleCallback(runner, typeof timeout === 'number' ? { timeout } : undefined);
 			let disposed = false;
 			return {
@@ -1667,8 +1657,8 @@ export class TaskSequentializer {
 			this._queued = {
 				run,
 				promise,
-				promiseResolve: promiseResolve!,
-				promiseReject: promiseReject!
+				promiseResolve,
+				promiseReject
 			};
 		}
 
@@ -1740,9 +1730,15 @@ const enum DeferredOutcome {
  */
 export class DeferredPromise<T> {
 
+	public static fromPromise<T>(promise: Promise<T>): DeferredPromise<T> {
+		const deferred = new DeferredPromise<T>();
+		deferred.settleWith(promise);
+		return deferred;
+	}
+
 	private completeCallback!: ValueCallback<T>;
 	private errorCallback!: (err: unknown) => void;
-	private outcome?: { outcome: DeferredOutcome.Rejected; value: any } | { outcome: DeferredOutcome.Resolved; value: T };
+	private outcome?: { outcome: DeferredOutcome.Rejected; value: unknown } | { outcome: DeferredOutcome.Resolved; value: T };
 
 	public get isRejected() {
 		return this.outcome?.outcome === DeferredOutcome.Rejected;
@@ -1770,6 +1766,10 @@ export class DeferredPromise<T> {
 	}
 
 	public complete(value: T) {
+		if (this.isSettled) {
+			return Promise.resolve();
+		}
+
 		return new Promise<void>(resolve => {
 			this.completeCallback(value);
 			this.outcome = { outcome: DeferredOutcome.Resolved, value };
@@ -1778,6 +1778,10 @@ export class DeferredPromise<T> {
 	}
 
 	public error(err: unknown) {
+		if (this.isSettled) {
+			return Promise.resolve();
+		}
+
 		return new Promise<void>(resolve => {
 			this.errorCallback(err);
 			this.outcome = { outcome: DeferredOutcome.Rejected, value: err };
@@ -2081,6 +2085,8 @@ export class AsyncIterableObject<T> implements AsyncIterable<T> {
 		});
 	}
 
+	public filter<T2 extends T>(filterFn: (item: T) => item is T2): AsyncIterableObject<T2>;
+	public filter(filterFn: (item: T) => boolean): AsyncIterableObject<T>;
 	public filter(filterFn: (item: T) => boolean): AsyncIterableObject<T> {
 		return AsyncIterableObject.filter(this, filterFn);
 	}
@@ -2165,24 +2171,12 @@ export class AsyncIterableObject<T> implements AsyncIterable<T> {
 	}
 }
 
-export class CancelableAsyncIterableObject<T> extends AsyncIterableObject<T> {
-	constructor(
-		private readonly _source: CancellationTokenSource,
-		executor: AsyncIterableExecutor<T>
-	) {
-		super(executor);
-	}
 
-	cancel(): void {
-		this._source.cancel();
-	}
-}
-
-export function createCancelableAsyncIterable<T>(callback: (token: CancellationToken) => AsyncIterable<T>): CancelableAsyncIterableObject<T> {
+export function createCancelableAsyncIterableProducer<T>(callback: (token: CancellationToken) => AsyncIterable<T>): CancelableAsyncIterableProducer<T> {
 	const source = new CancellationTokenSource();
 	const innerIterable = callback(source.token);
 
-	return new CancelableAsyncIterableObject<T>(source, async (emitter) => {
+	return new CancelableAsyncIterableProducer<T>(source, async (emitter) => {
 		const subscription = source.token.onCancellationRequested(() => {
 			subscription.dispose();
 			source.dispose();
@@ -2302,4 +2296,350 @@ export function cancellableIterable<T>(iterableOrIterator: AsyncIterator<T> | As
 	};
 }
 
+type ProducerConsumerValue<T> = {
+	ok: true;
+	value: T;
+} | {
+	ok: false;
+	error: Error;
+};
+
+class ProducerConsumer<T> {
+	private readonly _unsatisfiedConsumers: DeferredPromise<T>[] = [];
+	private readonly _unconsumedValues: ProducerConsumerValue<T>[] = [];
+	private _finalValue: ProducerConsumerValue<T> | undefined;
+
+	public get hasFinalValue(): boolean {
+		return !!this._finalValue;
+	}
+
+	produce(value: ProducerConsumerValue<T>): void {
+		this._ensureNoFinalValue();
+		if (this._unsatisfiedConsumers.length > 0) {
+			const deferred = this._unsatisfiedConsumers.shift()!;
+			this._resolveOrRejectDeferred(deferred, value);
+		} else {
+			this._unconsumedValues.push(value);
+		}
+	}
+
+	produceFinal(value: ProducerConsumerValue<T>): void {
+		this._ensureNoFinalValue();
+		this._finalValue = value;
+		for (const deferred of this._unsatisfiedConsumers) {
+			this._resolveOrRejectDeferred(deferred, value);
+		}
+		this._unsatisfiedConsumers.length = 0;
+	}
+
+	private _ensureNoFinalValue(): void {
+		if (this._finalValue) {
+			throw new BugIndicatingError('ProducerConsumer: cannot produce after final value has been set');
+		}
+	}
+
+	private _resolveOrRejectDeferred(deferred: DeferredPromise<T>, value: ProducerConsumerValue<T>): void {
+		if (value.ok) {
+			deferred.complete(value.value);
+		} else {
+			deferred.error(value.error);
+		}
+	}
+
+	consume(): Promise<T> {
+		if (this._unconsumedValues.length > 0 || this._finalValue) {
+			const value = this._unconsumedValues.length > 0 ? this._unconsumedValues.shift()! : this._finalValue!;
+			if (value.ok) {
+				return Promise.resolve(value.value);
+			} else {
+				return Promise.reject(value.error);
+			}
+		} else {
+			const deferred = new DeferredPromise<T>();
+			this._unsatisfiedConsumers.push(deferred);
+			return deferred.p;
+		}
+	}
+}
+
+/**
+ * Important difference to AsyncIterableObject:
+ * If it is iterated two times, the second iterator will not see the values emitted by the first iterator.
+ */
+export class AsyncIterableProducer<T> implements AsyncIterable<T> {
+	private readonly _producerConsumer = new ProducerConsumer<IteratorResult<T>>();
+
+	constructor(executor: AsyncIterableExecutor<T>, private readonly _onReturn?: () => void) {
+		queueMicrotask(async () => {
+			const p = executor({
+				emitOne: value => this._producerConsumer.produce({ ok: true, value: { done: false, value: value } }),
+				emitMany: values => {
+					for (const value of values) {
+						this._producerConsumer.produce({ ok: true, value: { done: false, value: value } });
+					}
+				},
+				reject: error => this._finishError(error),
+			});
+
+			if (!this._producerConsumer.hasFinalValue) {
+				try {
+					await p;
+					this._finishOk();
+				} catch (error) {
+					this._finishError(error);
+				}
+			}
+		});
+	}
+
+	public static fromArray<T>(items: T[]): AsyncIterableProducer<T> {
+		return new AsyncIterableProducer<T>((writer) => {
+			writer.emitMany(items);
+		});
+	}
+
+	public static fromPromise<T>(promise: Promise<T[]>): AsyncIterableProducer<T> {
+		return new AsyncIterableProducer<T>(async (emitter) => {
+			emitter.emitMany(await promise);
+		});
+	}
+
+	public static fromPromisesResolveOrder<T>(promises: Promise<T>[]): AsyncIterableProducer<T> {
+		return new AsyncIterableProducer<T>(async (emitter) => {
+			await Promise.all(promises.map(async (p) => emitter.emitOne(await p)));
+		});
+	}
+
+	public static merge<T>(iterables: AsyncIterable<T>[]): AsyncIterableProducer<T> {
+		return new AsyncIterableProducer(async (emitter) => {
+			await Promise.all(iterables.map(async (iterable) => {
+				for await (const item of iterable) {
+					emitter.emitOne(item);
+				}
+			}));
+		});
+	}
+
+	public static EMPTY = AsyncIterableProducer.fromArray<any>([]);
+
+	public static map<T, R>(iterable: AsyncIterable<T>, mapFn: (item: T) => R): AsyncIterableProducer<R> {
+		return new AsyncIterableProducer<R>(async (emitter) => {
+			for await (const item of iterable) {
+				emitter.emitOne(mapFn(item));
+			}
+		});
+	}
+
+	public static tee<T>(iterable: AsyncIterable<T>): [AsyncIterableProducer<T>, AsyncIterableProducer<T>] {
+		let emitter1: AsyncIterableEmitter<T> | undefined;
+		let emitter2: AsyncIterableEmitter<T> | undefined;
+
+		const defer = new DeferredPromise<void>();
+
+		const start = async () => {
+			if (!emitter1 || !emitter2) {
+				return; // not yet ready
+			}
+			try {
+				for await (const item of iterable) {
+					emitter1.emitOne(item);
+					emitter2.emitOne(item);
+				}
+			} catch (err) {
+				emitter1.reject(err);
+				emitter2.reject(err);
+			} finally {
+				defer.complete();
+			}
+		};
+
+		const p1 = new AsyncIterableProducer<T>(async (emitter) => {
+			emitter1 = emitter;
+			start();
+			return defer.p;
+		});
+		const p2 = new AsyncIterableProducer<T>(async (emitter) => {
+			emitter2 = emitter;
+			start();
+			return defer.p;
+		});
+		return [p1, p2];
+	}
+
+	public map<R>(mapFn: (item: T) => R): AsyncIterableProducer<R> {
+		return AsyncIterableProducer.map(this, mapFn);
+	}
+
+	public static coalesce<T>(iterable: AsyncIterable<T | undefined | null>): AsyncIterableProducer<T> {
+		return <AsyncIterableProducer<T>>AsyncIterableProducer.filter(iterable, item => !!item);
+	}
+
+	public coalesce(): AsyncIterableProducer<NonNullable<T>> {
+		return AsyncIterableProducer.coalesce(this) as AsyncIterableProducer<NonNullable<T>>;
+	}
+
+	public static filter<T>(iterable: AsyncIterable<T>, filterFn: (item: T) => boolean): AsyncIterableProducer<T> {
+		return new AsyncIterableProducer<T>(async (emitter) => {
+			for await (const item of iterable) {
+				if (filterFn(item)) {
+					emitter.emitOne(item);
+				}
+			}
+		});
+	}
+
+	public filter<T2 extends T>(filterFn: (item: T) => item is T2): AsyncIterableProducer<T2>;
+	public filter(filterFn: (item: T) => boolean): AsyncIterableProducer<T>;
+	public filter(filterFn: (item: T) => boolean): AsyncIterableProducer<T> {
+		return AsyncIterableProducer.filter(this, filterFn);
+	}
+
+	private _finishOk(): void {
+		if (!this._producerConsumer.hasFinalValue) {
+			this._producerConsumer.produceFinal({ ok: true, value: { done: true, value: undefined } });
+		}
+	}
+
+	private _finishError(error: Error): void {
+		if (!this._producerConsumer.hasFinalValue) {
+			this._producerConsumer.produceFinal({ ok: false, error: error });
+		}
+		// Warning: this can cause to dropped errors.
+	}
+
+	private readonly _iterator: AsyncIterator<T, void, void> = {
+		next: () => this._producerConsumer.consume(),
+		return: () => {
+			this._onReturn?.();
+			return Promise.resolve({ done: true, value: undefined });
+		},
+		throw: async (e) => {
+			this._finishError(e);
+			return { done: true, value: undefined };
+		},
+	};
+
+	[Symbol.asyncIterator](): AsyncIterator<T, void, void> {
+		return this._iterator;
+	}
+}
+
+export class CancelableAsyncIterableProducer<T> extends AsyncIterableProducer<T> {
+	constructor(
+		private readonly _source: CancellationTokenSource,
+		executor: AsyncIterableExecutor<T>
+	) {
+		super(executor);
+	}
+
+	cancel(): void {
+		this._source.cancel();
+	}
+}
+
 //#endregion
+
+export const AsyncReaderEndOfStream = Symbol('AsyncReaderEndOfStream');
+
+export class AsyncReader<T> {
+	private _buffer: T[] = [];
+	private _atEnd = false;
+
+	public get endOfStream(): boolean { return this._buffer.length === 0 && this._atEnd; }
+	private _extendBufferPromise: Promise<void> | undefined;
+
+	constructor(
+		private readonly _source: AsyncIterator<T>
+	) {
+	}
+
+	public async read(): Promise<T | typeof AsyncReaderEndOfStream> {
+		if (this._buffer.length === 0 && !this._atEnd) {
+			await this._extendBuffer();
+		}
+		if (this._buffer.length === 0) {
+			return AsyncReaderEndOfStream;
+		}
+		return this._buffer.shift()!;
+	}
+
+	public async readWhile(predicate: (value: T) => boolean, callback: (element: T) => unknown): Promise<void> {
+		do {
+			const piece = await this.peek();
+			if (piece === AsyncReaderEndOfStream) {
+				break;
+			}
+			if (!predicate(piece)) {
+				break;
+			}
+			await this.read(); // consume
+			await callback(piece);
+		} while (true);
+	}
+
+	public readBufferedOrThrow(): T | typeof AsyncReaderEndOfStream {
+		const value = this.peekBufferedOrThrow();
+		this._buffer.shift();
+		return value;
+	}
+
+	public async consumeToEnd(): Promise<void> {
+		while (!this.endOfStream) {
+			await this.read();
+		}
+	}
+
+	public async peek(): Promise<T | typeof AsyncReaderEndOfStream> {
+		if (this._buffer.length === 0 && !this._atEnd) {
+			await this._extendBuffer();
+		}
+		if (this._buffer.length === 0) {
+			return AsyncReaderEndOfStream;
+		}
+		return this._buffer[0];
+	}
+
+	public peekBufferedOrThrow(): T | typeof AsyncReaderEndOfStream {
+		if (this._buffer.length === 0) {
+			if (this._atEnd) {
+				return AsyncReaderEndOfStream;
+			}
+			throw new BugIndicatingError('No buffered elements');
+		}
+
+		return this._buffer[0];
+	}
+
+	public async peekTimeout(timeoutMs: number): Promise<T | typeof AsyncReaderEndOfStream | undefined> {
+		if (this._buffer.length === 0 && !this._atEnd) {
+			await raceTimeout(this._extendBuffer(), timeoutMs);
+		}
+		if (this._atEnd) {
+			return AsyncReaderEndOfStream;
+		}
+		if (this._buffer.length === 0) {
+			return undefined;
+		}
+		return this._buffer[0];
+	}
+
+	private _extendBuffer(): Promise<void> {
+		if (this._atEnd) {
+			return Promise.resolve();
+		}
+
+		if (!this._extendBufferPromise) {
+			this._extendBufferPromise = (async () => {
+				const { value, done } = await this._source.next();
+				this._extendBufferPromise = undefined;
+				if (done) {
+					this._atEnd = true;
+				} else {
+					this._buffer.push(value);
+				}
+			})();
+		}
+
+		return this._extendBufferPromise;
+	}
+}
