@@ -25,6 +25,9 @@ export const enum MarshalledId {
 	ChatViewContext,
 	LanguageModelToolResult,
 	LanguageModelTextPart,
+	LanguageModelThinkingPart,
 	LanguageModelPromptTsxPart,
 	LanguageModelDataPart,
+	AgentSessionContext,
+	ChatResponsePullRequestPart,
 }
