@@ -8,24 +8,28 @@ import { DebugNameData, IDebugNameData } from '../debugName.js';
 import { EqualityComparer, strictEquals } from '../commonFacade/deps.js';
 import { ObservableValue } from './observableValue.js';
 import { LazyObservableValue } from './lazyObservableValue.js';
+import { DebugLocation } from '../debugLocation.js';
 
 export function observableValueOpts<T, TChange = void>(
 	options: IDebugNameData & {
 		equalsFn?: EqualityComparer<T>;
 		lazy?: boolean;
 	},
-	initialValue: T
+	initialValue: T,
+	debugLocation = DebugLocation.ofCaller(),
 ): ISettableObservable<T, TChange> {
 	if (options.lazy) {
 		return new LazyObservableValue(
 			new DebugNameData(options.owner, options.debugName, undefined),
 			initialValue,
 			options.equalsFn ?? strictEquals,
+			debugLocation
 		);
 	}
 	return new ObservableValue(
 		new DebugNameData(options.owner, options.debugName, undefined),
 		initialValue,
 		options.equalsFn ?? strictEquals,
+		debugLocation
 	);
 }
