@@ -28,12 +28,22 @@ suite('Processes', () => {
 			VSCODE_CODE_CACHE_PATH: 'x',
 			VSCODE_NEW_VAR: 'x',
 			GDK_PIXBUF_MODULE_FILE: 'x',
-			GDK_PIXBUF_MODULEDIR: 'x'
+			GDK_PIXBUF_MODULEDIR: 'x',
+			VSCODE_PYTHON_BASH_ACTIVATE: 'source /path/to/venv/bin/activate',
+			VSCODE_PYTHON_ZSH_ACTIVATE: 'source /path/to/venv/bin/activate',
+			VSCODE_PYTHON_PWSH_ACTIVATE: '. /path/to/venv/Scripts/Activate.ps1',
+			VSCODE_PYTHON_FISH_ACTIVATE: 'source /path/to/venv/bin/activate.fish',
+			VSCODE_PYTHON_AUTOACTIVATE_GUARD: '1'
 		};
 		processes.sanitizeProcessEnvironment(env);
 		assert.strictEqual(env['FOO'], 'bar');
 		assert.strictEqual(env['VSCODE_SHELL_LOGIN'], '1');
 		assert.strictEqual(env['VSCODE_PORTABLE'], '3');
+		assert.strictEqual(env['VSCODE_PYTHON_BASH_ACTIVATE'], undefined);
+		assert.strictEqual(env['VSCODE_PYTHON_ZSH_ACTIVATE'], undefined);
+		assert.strictEqual(env['VSCODE_PYTHON_PWSH_ACTIVATE'], undefined);
+		assert.strictEqual(env['VSCODE_PYTHON_FISH_ACTIVATE'], undefined);
+		assert.strictEqual(env['VSCODE_PYTHON_AUTOACTIVATE_GUARD'], undefined);
 		assert.strictEqual(Object.keys(env).length, 3);
 	});
 });
