@@ -18,8 +18,9 @@ import { Position } from '../../../../common/core/position.js';
 import { Selection } from '../../../../common/core/selection.js';
 import { IAccessibilityService } from '../../../../../platform/accessibility/common/accessibility.js';
 import { ILogService } from '../../../../../platform/log/common/log.js';
-import { ClipboardDataToCopy, ClipboardEventUtils, ClipboardStoredMetadata, InMemoryClipboardMetadataManager } from '../clipboardUtils.js';
+import { ClipboardStoredMetadata, CopyOptions, createClipboardCopyEvent, createClipboardPasteEvent, IClipboardCopyEvent, IClipboardPasteEvent, InMemoryClipboardMetadataManager } from '../clipboardUtils.js';
 import { _debugComposition, ITextAreaWrapper, ITypeData, TextAreaState } from './textAreaEditContextState.js';
+import { ViewContext } from '../../../../common/viewModel/viewContext.js';
 
 export namespace TextAreaSyntethicEvents {
 	export const Tap = '-monaco-textarea-synthetic-tap';
@@ -36,7 +37,7 @@ export interface IPasteData {
 }
 
 export interface ITextAreaInputHost {
-	getDataToCopy(): ClipboardDataToCopy;
+	readonly context: ViewContext;
 	getScreenReaderContent(): TextAreaState;
 	deduceModelPosition(viewAnchorPosition: Position, deltaOffset: number, lineFeedCnt: number): Position;
 }
@@ -126,6 +127,15 @@ export class TextAreaInput extends Disposable {
 	private _onPaste = this._register(new Emitter<IPasteData>());
 	public readonly onPaste: Event<IPasteData> = this._onPaste.event;
 
+	private _onWillCopy = this._register(new Emitter<IClipboardCopyEvent>());
+	public readonly onWillCopy: Event<IClipboardCopyEvent> = this._onWillCopy.event;
+
+	private _onWillCut = this._register(new Emitter<IClipboardCopyEvent>());
+	public readonly onWillCut: Event<IClipboardCopyEvent> = this._onWillCut.event;
+
+	private _onWillPaste = this._register(new Emitter<IClipboardPasteEvent>());
+	public readonly onWillPaste: Event<IClipboardPasteEvent> = this._onWillPaste.event;
+
 	private _onType = this._register(new Emitter<ITypeData>());
 	public readonly onType: Event<ITypeData> = this._onType.event;
 
@@ -342,47 +352,86 @@ export class TextAreaInput extends Disposable {
 				|| typeInput.replaceNextCharCnt !== 0
 				|| typeInput.positionDelta !== 0
 			) {
-				this._onType.fire(typeInput);
+				// https://w3c.github.io/input-events/#interface-InputEvent-Attributes
+				if (e.inputType === 'insertFromPaste') {
+					this._onPaste.fire({
+						text: typeInput.text,
+						metadata: InMemoryClipboardMetadataManager.INSTANCE.get(typeInput.text)
+					});
+				} else {
+					this._onType.fire(typeInput);
+				}
 			}
 		}));
 
 		// --- Clipboard operations
 
 		this._register(this._textArea.onCut((e) => {
+			this._logService.trace(`TextAreaInput#onCut`, e);
+
+			// Fire onWillCut event to allow interception
+			const cutEvent = createClipboardCopyEvent(e, /* isCut */ true, this._host.context, this._logService, this._browser.isFirefox);
+			this._onWillCut.fire(cutEvent);
+			if (cutEvent.isHandled) {
+				// Event was handled externally, skip default processing
+				return;
+			}
+
 			// Pretend here we touched the text area, as the `cut` event will most likely
 			// result in a `selectionchange` event which we want to ignore
 			this._textArea.setIgnoreSelectionChangeTime('received cut event');
 
-			this._ensureClipboardGetsEditorSelection(e);
+			cutEvent.ensureClipboardGetsEditorData();
 			this._asyncTriggerCut.schedule();
 		}));
 
 		this._register(this._textArea.onCopy((e) => {
-			this._ensureClipboardGetsEditorSelection(e);
+			this._logService.trace(`TextAreaInput#onCopy`, e);
+
+			// !!!!!
+			// This is a workaround for what we think is an Electron bug where
+			// execCommand('copy') does not always work (it does not fire a clipboard event)
+			// !!!!!
+			// We signal that we have executed a copy command
+			CopyOptions.electronBugWorkaroundCopyEventHasFired = true;
+
+			// Fire onWillCopy event to allow interception
+			const copyEvent = createClipboardCopyEvent(e, /* isCut */ false, this._host.context, this._logService, this._browser.isFirefox);
+			this._onWillCopy.fire(copyEvent);
+			if (copyEvent.isHandled) {
+				// Event was handled externally, skip default processing
+				return;
+			}
+
+			copyEvent.ensureClipboardGetsEditorData();
 		}));
 
 		this._register(this._textArea.onPaste((e) => {
+			this._logService.trace(`TextAreaInput#onPaste`, e);
+
+			// Fire onWillPaste event to allow interception
+			const pasteEvent = createClipboardPasteEvent(e);
+			this._onWillPaste.fire(pasteEvent);
+			if (pasteEvent.isHandled) {
+				// Event was handled externally, skip default processing
+				return;
+			}
+
 			// Pretend here we touched the text area, as the `paste` event will most likely
 			// result in a `selectionchange` event which we want to ignore
 			this._textArea.setIgnoreSelectionChangeTime('received paste event');
 
 			e.preventDefault();
 
-			if (!e.clipboardData) {
+			this._logService.trace(`TextAreaInput#onPaste with id : `, pasteEvent.metadata?.id, ' with text.length: ', pasteEvent.text.length);
+			if (!pasteEvent.text) {
 				return;
 			}
 
-			let [text, metadata] = ClipboardEventUtils.getTextData(e.clipboardData);
-			if (!text) {
-				return;
-			}
-
-			// try the in-memory store
-			metadata = metadata || InMemoryClipboardMetadataManager.INSTANCE.get(text);
-
+			this._logService.trace(`TextAreaInput#onPaste (before onPaste)`);
 			this._onPaste.fire({
-				text: text,
-				metadata: metadata
+				text: pasteEvent.text,
+				metadata: pasteEvent.metadata
 			});
 		}));
 
@@ -594,27 +643,6 @@ export class TextAreaInput extends Disposable {
 		}
 		this._setAndWriteTextAreaState(reason, this._host.getScreenReaderContent());
 	}
-
-	private _ensureClipboardGetsEditorSelection(e: ClipboardEvent): void {
-		const dataToCopy = this._host.getDataToCopy();
-		const storedMetadata: ClipboardStoredMetadata = {
-			version: 1,
-			isFromEmptySelection: dataToCopy.isFromEmptySelection,
-			multicursorText: dataToCopy.multicursorText,
-			mode: dataToCopy.mode
-		};
-		InMemoryClipboardMetadataManager.INSTANCE.set(
-			// When writing "LINE\r\n" to the clipboard and then pasting,
-			// Firefox pastes "LINE\n", so let's work around this quirk
-			(this._browser.isFirefox ? dataToCopy.text.replace(/\r\n/g, '\n') : dataToCopy.text),
-			storedMetadata
-		);
-
-		e.preventDefault();
-		if (e.clipboardData) {
-			ClipboardEventUtils.setTextData(e.clipboardData, dataToCopy.text, dataToCopy.html, storedMetadata);
-		}
-	}
 }
 
 export class TextAreaWrapper extends Disposable implements ICompleteTextAreaWrapper {
