@@ -3,9 +3,9 @@
  *  Licensed under the MIT License. See License.txt in the project root for license information.
  *--------------------------------------------------------------------------------------------*/
 
-import { equalsIfDefined, itemsEquals } from '../../base/common/equals.js';
+import { equalsIfDefinedC, arrayEqualsC } from '../../base/common/equals.js';
 import { Disposable, DisposableStore, IDisposable, toDisposable } from '../../base/common/lifecycle.js';
-import { IObservable, IObservableWithChange, ITransaction, TransactionImpl, autorun, autorunOpts, derived, derivedOpts, derivedWithSetter, observableFromEvent, observableSignal, observableValue, observableValueOpts } from '../../base/common/observable.js';
+import { DebugLocation, IObservable, IObservableWithChange, IReader, ITransaction, TransactionImpl, autorun, autorunOpts, derived, derivedOpts, derivedWithSetter, observableFromEvent, observableFromEventOpts, observableSignal, observableSignalFromEvent, observableValue, observableValueOpts } from '../../base/common/observable.js';
 import { EditorOption, FindComputedEditorOptionValueById } from '../common/config/editorOptions.js';
 import { LineRange } from '../common/core/ranges/lineRange.js';
 import { OffsetRange } from '../common/core/ranges/offsetRange.js';
@@ -74,19 +74,19 @@ export class ObservableCodeEditor extends Disposable {
 		this._currentTransaction = undefined;
 		this._model = observableValue(this, this.editor.getModel());
 		this.model = this._model;
-		this.isReadonly = observableFromEvent(this, this.editor.onDidChangeConfiguration, () => this.editor.getOption(EditorOption.readOnly));
+		this.isReadonly = observableFromEventOpts({ owner: this, getTransaction: () => this._currentTransaction }, this.editor.onDidChangeConfiguration, () => this.editor.getOption(EditorOption.readOnly));
 		this._versionId = observableValueOpts<number | null, IModelContentChangedEvent | undefined>({ owner: this, lazy: true }, this.editor.getModel()?.getVersionId() ?? null);
 		this.versionId = this._versionId;
 		this._selections = observableValueOpts<Selection[] | null, ICursorSelectionChangedEvent | undefined>(
-			{ owner: this, equalsFn: equalsIfDefined(itemsEquals(Selection.selectionsEqual)), lazy: true },
+			{ owner: this, equalsFn: equalsIfDefinedC(arrayEqualsC(Selection.selectionsEqual)), lazy: true },
 			this.editor.getSelections() ?? null
 		);
 		this.selections = this._selections;
 		this.positions = derivedOpts<readonly Position[] | null>(
-			{ owner: this, equalsFn: equalsIfDefined(itemsEquals(Position.equals)) },
+			{ owner: this, equalsFn: equalsIfDefinedC(arrayEqualsC(Position.equals)) },
 			reader => this.selections.read(reader)?.map(s => s.getStartPosition()) ?? null
 		);
-		this.isFocused = observableFromEvent(this, e => {
+		this.isFocused = observableFromEventOpts({ owner: this, getTransaction: () => this._currentTransaction }, e => {
 			const d1 = this.editor.onDidFocusEditorWidget(e);
 			const d2 = this.editor.onDidBlurEditorWidget(e);
 			return {
@@ -96,7 +96,7 @@ export class ObservableCodeEditor extends Disposable {
 				}
 			};
 		}, () => this.editor.hasWidgetFocus());
-		this.isTextFocused = observableFromEvent(this, e => {
+		this.isTextFocused = observableFromEventOpts({ owner: this, getTransaction: () => this._currentTransaction }, e => {
 			const d1 = this.editor.onDidFocusEditorText(e);
 			const d2 = this.editor.onDidBlurEditorText(e);
 			return {
@@ -106,7 +106,7 @@ export class ObservableCodeEditor extends Disposable {
 				}
 			};
 		}, () => this.editor.hasTextFocus());
-		this.inComposition = observableFromEvent(this, e => {
+		this.inComposition = observableFromEventOpts({ owner: this, getTransaction: () => this._currentTransaction }, e => {
 			const d1 = this.editor.onDidCompositionStart(() => {
 				e(undefined);
 			});
@@ -132,22 +132,26 @@ export class ObservableCodeEditor extends Disposable {
 			}
 		);
 		this.valueIsEmpty = derived(this, reader => { this.versionId.read(reader); return this.editor.getModel()?.getValueLength() === 0; });
-		this.cursorSelection = derivedOpts({ owner: this, equalsFn: equalsIfDefined(Selection.selectionsEqual) }, reader => this.selections.read(reader)?.[0] ?? null);
+		this.cursorSelection = derivedOpts({ owner: this, equalsFn: equalsIfDefinedC(Selection.selectionsEqual) }, reader => this.selections.read(reader)?.[0] ?? null);
 		this.cursorPosition = derivedOpts({ owner: this, equalsFn: Position.equals }, reader => this.selections.read(reader)?.[0]?.getPosition() ?? null);
 		this.cursorLineNumber = derived<number | null>(this, reader => this.cursorPosition.read(reader)?.lineNumber ?? null);
 		this.onDidType = observableSignal<string>(this);
 		this.onDidPaste = observableSignal<IPasteEvent>(this);
-		this.scrollTop = observableFromEvent(this.editor.onDidScrollChange, () => this.editor.getScrollTop());
-		this.scrollLeft = observableFromEvent(this.editor.onDidScrollChange, () => this.editor.getScrollLeft());
-		this.layoutInfo = observableFromEvent(this.editor.onDidLayoutChange, () => this.editor.getLayoutInfo());
+		this.scrollTop = observableFromEventOpts({ owner: this, getTransaction: () => this._currentTransaction }, this.editor.onDidScrollChange, () => this.editor.getScrollTop());
+		this.scrollLeft = observableFromEventOpts({ owner: this, getTransaction: () => this._currentTransaction }, this.editor.onDidScrollChange, () => this.editor.getScrollLeft());
+		this.layoutInfo = observableFromEventOpts({ owner: this, getTransaction: () => this._currentTransaction }, this.editor.onDidLayoutChange, () => this.editor.getLayoutInfo());
 		this.layoutInfoContentLeft = this.layoutInfo.map(l => l.contentLeft);
 		this.layoutInfoDecorationsLeft = this.layoutInfo.map(l => l.decorationsLeft);
 		this.layoutInfoWidth = this.layoutInfo.map(l => l.width);
 		this.layoutInfoHeight = this.layoutInfo.map(l => l.height);
 		this.layoutInfoMinimap = this.layoutInfo.map(l => l.minimap);
 		this.layoutInfoVerticalScrollbarWidth = this.layoutInfo.map(l => l.verticalScrollbarWidth);
-		this.contentWidth = observableFromEvent(this.editor.onDidContentSizeChange, () => this.editor.getContentWidth());
-		this.contentHeight = observableFromEvent(this.editor.onDidContentSizeChange, () => this.editor.getContentHeight());
+		this.contentWidth = observableFromEventOpts({ owner: this, getTransaction: () => this._currentTransaction }, this.editor.onDidContentSizeChange, () => this.editor.getContentWidth());
+		this.contentHeight = observableFromEventOpts({ owner: this, getTransaction: () => this._currentTransaction }, this.editor.onDidContentSizeChange, () => this.editor.getContentHeight());
+		this._onDidChangeViewZones = observableSignalFromEvent(this, this.editor.onDidChangeViewZones);
+		this._onDidHiddenAreasChanged = observableSignalFromEvent(this, this.editor.onDidChangeHiddenAreas);
+		this._onDidLineHeightChanged = observableSignalFromEvent(this, this.editor.onDidChangeLineHeight);
+
 		this._widgetCounter = 0;
 		this.openedPeekWidgets = observableValue(this, 0);
 
@@ -203,6 +207,27 @@ export class ObservableCodeEditor extends Disposable {
 				this._endUpdate();
 			}
 		}));
+
+		this.domNode = derived(reader => {
+			this.model.read(reader);
+			return this.editor.getDomNode();
+		});
+	}
+
+	/**
+	 * Batches the transactions started by observableFromEvent.
+	 *
+	 * If the callback causes the editor to fire an event that updates
+	 * an observable value backed by observableFromEvent (such as scrollTop etc.),
+	 * then all such updates will be part of the same transaction.
+	*/
+	public transaction<T>(cb: (tx: ITransaction) => T): T {
+		this._beginUpdate();
+		try {
+			return cb(this._currentTransaction!);
+		} finally {
+			this._endUpdate();
+		}
 	}
 
 	public forceUpdate(): void;
@@ -272,10 +297,12 @@ export class ObservableCodeEditor extends Disposable {
 	public readonly contentWidth;
 	public readonly contentHeight;
 
-	public getOption<T extends EditorOption>(id: T): IObservable<FindComputedEditorOptionValueById<T>> {
+	public readonly domNode;
+
+	public getOption<T extends EditorOption>(id: T, debugLocation = DebugLocation.ofCaller()): IObservable<FindComputedEditorOptionValueById<T>> {
 		return observableFromEvent(this, cb => this.editor.onDidChangeConfiguration(e => {
 			if (e.hasChanged(id)) { cb(undefined); }
-		}), () => this.editor.getOption(id));
+		}), () => this.editor.getOption(id), debugLocation);
 	}
 
 	public setDecorations(decorations: IObservable<IModelDeltaDecoration[]>): IDisposable {
@@ -356,9 +383,26 @@ export class ObservableCodeEditor extends Disposable {
 		});
 	}
 
+	/**
+	 * Uses an approximation if the exact position cannot be determined.
+	 */
+	getLeftOfPosition(position: Position, reader: IReader | undefined): number {
+		this.layoutInfo.read(reader);
+		this.value.read(reader);
+
+		let offset = this.editor.getOffsetForColumn(position.lineNumber, position.column);
+		if (offset === -1) {
+			// approximation
+			const typicalHalfwidthCharacterWidth = this.editor.getOption(EditorOption.fontInfo).typicalHalfwidthCharacterWidth;
+			const approximation = position.column * typicalHalfwidthCharacterWidth;
+			offset = approximation;
+		}
+		return offset;
+	}
+
 	public observePosition(position: IObservable<Position | null>, store: DisposableStore): IObservable<Point | null> {
 		let pos = position.get();
-		const result = observableValueOpts<Point | null>({ owner: this, debugName: () => `topLeftOfPosition${pos?.toString()}`, equalsFn: equalsIfDefined(Point.equals) }, new Point(0, 0));
+		const result = observableValueOpts<Point | null>({ owner: this, debugName: () => `topLeftOfPosition${pos?.toString()}`, equalsFn: equalsIfDefinedC(Point.equals) }, new Point(0, 0));
 		const contentWidgetId = `observablePositionWidget` + (this._widgetCounter++);
 		const domNode = document.createElement('div');
 		const w: IContentWidget = {
@@ -368,6 +412,7 @@ export class ObservableCodeEditor extends Disposable {
 			},
 			getId: () => contentWidgetId,
 			allowEditorOverflow: false,
+			useDisplayNone: true,
 			afterRender: (position, coordinate) => {
 				const model = this._model.get();
 				if (model && pos && pos.lineNumber > model.getLineCount()) {
@@ -403,6 +448,129 @@ export class ObservableCodeEditor extends Disposable {
 		}));
 		return isHovered;
 	}
+
+	observeLineHeightForPosition(position: IObservable<Position> | Position): IObservable<number>;
+	observeLineHeightForPosition(position: IObservable<null>): IObservable<null>;
+	observeLineHeightForPosition(position: IObservable<Position | null> | Position): IObservable<number | null> {
+		return derived(reader => {
+			const pos = position instanceof Position ? position : position.read(reader);
+			if (pos === null) {
+				return null;
+			}
+
+			this.getOption(EditorOption.lineHeight).read(reader);
+
+			return this.editor.getLineHeightForPosition(pos);
+		});
+	}
+
+	observeLineHeightForLine(lineNumber: IObservable<number> | number): IObservable<number>;
+	observeLineHeightForLine(lineNumber: IObservable<null>): IObservable<null>;
+	observeLineHeightForLine(lineNumber: IObservable<number | null> | number): IObservable<number | null> {
+		if (typeof lineNumber === 'number') {
+			return this.observeLineHeightForPosition(new Position(lineNumber, 1));
+		}
+
+		return derived(reader => {
+			const line = lineNumber.read(reader);
+			if (line === null) {
+				return null;
+			}
+
+			return this.observeLineHeightForPosition(new Position(line, 1)).read(reader);
+		});
+	}
+
+	observeLineHeightsForLineRange(lineNumber: IObservable<LineRange> | LineRange): IObservable<number[]> {
+		return derived(reader => {
+			const range = lineNumber instanceof LineRange ? lineNumber : lineNumber.read(reader);
+
+			const heights: number[] = [];
+			for (let i = range.startLineNumber; i < range.endLineNumberExclusive; i++) {
+				heights.push(this.observeLineHeightForLine(i).read(reader));
+			}
+			return heights;
+		});
+	}
+
+	private readonly _onDidChangeViewZones;
+	private readonly _onDidHiddenAreasChanged;
+	private readonly _onDidLineHeightChanged;
+
+	/**
+	 * Tracks whether getWidthOfLine returned 0, indicating the editor may be hidden.
+	 * When resize happens and this flag is set, we reset cached line widths.
+	 */
+	private _sawZeroLineWidth = false;
+
+	/**
+	 * Fires when the editor container resizes.
+	 * This is lazily created only when someone subscribes to it.
+	 * Useful for detecting when a parent element's display changes from 'none' to 'block'.
+	 */
+	private readonly _onDidContainerResize = observableFromEventOpts(
+		{ owner: this, getTransaction: () => this._currentTransaction },
+		e => {
+			const container = this.editor.getContainerDomNode();
+			const resizeObserver = new ResizeObserver(() => {
+				// If we previously saw a 0 width, the editor was likely hidden.
+				// Now that it resized (became visible), flush the cached widths.
+				if (this._sawZeroLineWidth) {
+					this._sawZeroLineWidth = false;
+					this.editor.resetLineWidthCaches();
+				}
+				e(undefined);
+			});
+			resizeObserver.observe(container);
+			return { dispose: () => resizeObserver.disconnect() };
+		},
+		() => ({}) // Return new object each time to ensure change detection
+	);
+
+	/**
+	 * Get the width of a line in pixels.
+	 * Reading the returned value depends on layoutInfo, value, scrollTop, and container resize events.
+	 * The container resize dependency ensures correct values when the editor becomes visible after being hidden.
+	 */
+	getWidthOfLine(lineNumber: number, reader: IReader | undefined): number {
+		this.layoutInfo.read(reader);
+		this.value.read(reader);
+		this.scrollTop.read(reader);
+		const width = this.editor.getWidthOfLine(lineNumber);
+		this._onDidContainerResize.read(reader);
+		if (width === 0) {
+			this._sawZeroLineWidth = true;
+		}
+		return width;
+	}
+
+	/**
+	 * Get the vertical position (top offset) for the line's bottom w.r.t. to the first line.
+	 */
+	observeTopForLineNumber(lineNumber: number): IObservable<number> {
+		return derived(reader => {
+			this.layoutInfo.read(reader);
+			this._onDidChangeViewZones.read(reader);
+			this._onDidHiddenAreasChanged.read(reader);
+			this._onDidLineHeightChanged.read(reader);
+			this._versionId.read(reader);
+			return this.editor.getTopForLineNumber(lineNumber);
+		});
+	}
+
+	/**
+	 * Get the vertical position (top offset) for the line's bottom w.r.t. to the first line.
+	 */
+	observeBottomForLineNumber(lineNumber: number): IObservable<number> {
+		return derived(reader => {
+			this.layoutInfo.read(reader);
+			this._onDidChangeViewZones.read(reader);
+			this._onDidHiddenAreasChanged.read(reader);
+			this._onDidLineHeightChanged.read(reader);
+			this._versionId.read(reader);
+			return this.editor.getBottomForLineNumber(lineNumber);
+		});
+	}
 }
 
 interface IObservableOverlayWidget {
