@@ -211,22 +211,22 @@ export abstract class AbstractCodeEditorService extends Disposable implements IC
 	}
 
 	private readonly _transientWatchers = this._register(new DisposableMap<string, ModelTransientSettingWatcher>());
-	private readonly _modelProperties = new Map<string, Map<string, any>>();
+	private readonly _modelProperties = new Map<string, Map<string, unknown>>();
 
-	public setModelProperty(resource: URI, key: string, value: any): void {
+	public setModelProperty(resource: URI, key: string, value: unknown): void {
 		const key1 = resource.toString();
-		let dest: Map<string, any>;
+		let dest: Map<string, unknown>;
 		if (this._modelProperties.has(key1)) {
 			dest = this._modelProperties.get(key1)!;
 		} else {
-			dest = new Map<string, any>();
+			dest = new Map<string, unknown>();
 			this._modelProperties.set(key1, dest);
 		}
 
 		dest.set(key, value);
 	}
 
-	public getModelProperty(resource: URI, key: string): any {
+	public getModelProperty(resource: URI, key: string): unknown {
 		const key1 = resource.toString();
 		if (this._modelProperties.has(key1)) {
 			const innerMap = this._modelProperties.get(key1)!;
@@ -235,7 +235,7 @@ export abstract class AbstractCodeEditorService extends Disposable implements IC
 		return undefined;
 	}
 
-	public setTransientModelProperty(model: ITextModel, key: string, value: any): void {
+	public setTransientModelProperty(model: ITextModel, key: string, value: unknown): void {
 		const uri = model.uri.toString();
 
 		let w = this._transientWatchers.get(uri);
@@ -251,7 +251,7 @@ export abstract class AbstractCodeEditorService extends Disposable implements IC
 		}
 	}
 
-	public getTransientModelProperty(model: ITextModel, key: string): any {
+	public getTransientModelProperty(model: ITextModel, key: string): unknown {
 		const uri = model.uri.toString();
 
 		const watcher = this._transientWatchers.get(uri);
@@ -262,7 +262,7 @@ export abstract class AbstractCodeEditorService extends Disposable implements IC
 		return watcher.get(key);
 	}
 
-	public getTransientModelProperties(model: ITextModel): [string, any][] | undefined {
+	public getTransientModelProperties(model: ITextModel): [string, unknown][] | undefined {
 		const uri = model.uri.toString();
 
 		const watcher = this._transientWatchers.get(uri);
@@ -297,7 +297,7 @@ export abstract class AbstractCodeEditorService extends Disposable implements IC
 
 export class ModelTransientSettingWatcher extends Disposable {
 	public readonly uri: string;
-	private readonly _values: { [key: string]: any };
+	private readonly _values: { [key: string]: unknown };
 
 	constructor(uri: string, model: ITextModel, owner: AbstractCodeEditorService) {
 		super();
@@ -307,11 +307,11 @@ export class ModelTransientSettingWatcher extends Disposable {
 		this._register(model.onWillDispose(() => owner._removeWatcher(this)));
 	}
 
-	public set(key: string, value: any): void {
+	public set(key: string, value: unknown): void {
 		this._values[key] = value;
 	}
 
-	public get(key: string): any {
+	public get(key: string): unknown {
 		return this._values[key];
 	}
 
@@ -460,7 +460,11 @@ class DecorationTypeOptionsProvider implements IModelDecorationOptionsProvider {
 	public afterContentClassName: string | undefined;
 	public glyphMarginClassName: string | undefined;
 	public isWholeLine: boolean;
-	public lineHeight?: number;
+	public lineHeight: number | undefined;
+	public fontSize: string | undefined;
+	public fontFamily: string | undefined;
+	public fontWeight: string | undefined;
+	public fontStyle: string | undefined;
 	public overviewRuler: IModelDecorationOverviewRulerOptions | undefined;
 	public stickiness: TrackedRangeStickiness | undefined;
 	public beforeInjectedText: InjectedTextOptions | undefined;
@@ -522,6 +526,10 @@ class DecorationTypeOptionsProvider implements IModelDecorationOptionsProvider {
 		const options = providerArgs.options;
 		this.isWholeLine = Boolean(options.isWholeLine);
 		this.lineHeight = options.lineHeight;
+		this.fontFamily = options.fontFamily;
+		this.fontSize = options.fontSize;
+		this.fontWeight = options.fontWeight;
+		this.fontStyle = options.fontStyle;
 		this.stickiness = options.rangeBehavior;
 
 		const lightOverviewRulerColor = options.light && options.light.overviewRulerColor || options.overviewRulerColor;
@@ -552,6 +560,10 @@ class DecorationTypeOptionsProvider implements IModelDecorationOptionsProvider {
 			glyphMarginClassName: this.glyphMarginClassName,
 			isWholeLine: this.isWholeLine,
 			lineHeight: this.lineHeight,
+			fontFamily: this.fontFamily,
+			fontSize: this.fontSize,
+			fontWeight: this.fontWeight,
+			fontStyle: this.fontStyle,
 			overviewRuler: this.overviewRuler,
 			stickiness: this.stickiness,
 			before: this.beforeInjectedText,
@@ -759,7 +771,7 @@ class DecorationCSSRules {
 			return '';
 		}
 		const cssTextArr: string[] = [];
-		this.collectCSSText(opts, ['fontStyle', 'fontWeight', 'textDecoration', 'cursor', 'color', 'opacity', 'letterSpacing'], cssTextArr);
+		this.collectCSSText(opts, ['fontStyle', 'fontWeight', 'fontFamily', 'fontSize', 'textDecoration', 'cursor', 'color', 'opacity', 'letterSpacing'], cssTextArr);
 		if (opts.letterSpacing) {
 			this._hasLetterSpacing = true;
 		}
@@ -814,7 +826,7 @@ class DecorationCSSRules {
 		return cssTextArr.join('');
 	}
 
-	private collectBorderSettingsCSSText(opts: any, cssTextArr: string[]): boolean {
+	private collectBorderSettingsCSSText(opts: unknown, cssTextArr: string[]): boolean {
 		if (this.collectCSSText(opts, ['border', 'borderColor', 'borderRadius', 'borderSpacing', 'borderStyle', 'borderWidth'], cssTextArr)) {
 			cssTextArr.push(strings.format('box-sizing: border-box;'));
 			return true;
@@ -822,10 +834,10 @@ class DecorationCSSRules {
 		return false;
 	}
 
-	private collectCSSText(opts: any, properties: string[], cssTextArr: string[]): boolean {
+	private collectCSSText(opts: unknown, properties: string[], cssTextArr: string[]): boolean {
 		const lenBefore = cssTextArr.length;
 		for (const property of properties) {
-			const value = this.resolveValue(opts[property]);
+			const value = this.resolveValue((opts as Record<string, unknown>)[property] as string | ThemeColor);
 			if (typeof value === 'string') {
 				cssTextArr.push(strings.format(_CSS_MAP[property], value));
 			}
