@@ -26,7 +26,7 @@ import { RenderingContext, RestrictedRenderingContext } from '../../view/renderi
 import { ViewContext } from '../../../common/viewModel/viewContext.js';
 import { EditorTheme } from '../../../common/editorTheme.js';
 import * as viewEvents from '../../../common/viewEvents.js';
-import { ViewLineData, ViewModelDecoration } from '../../../common/viewModel.js';
+import { ViewLineData } from '../../../common/viewModel.js';
 import { minimapSelection, minimapBackground, minimapForegroundOpacity, editorForeground } from '../../../../platform/theme/common/colorRegistry.js';
 import { ModelDecorationMinimapOptions } from '../../../common/model/textModel.js';
 import { Selection } from '../../../common/core/selection.js';
@@ -37,6 +37,8 @@ import { MinimapPosition, MinimapSectionHeaderStyle, TextModelResolvedOptions }
 import { createSingleCallFunction } from '../../../../base/common/functional.js';
 import { LRUCache } from '../../../../base/common/map.js';
 import { DEFAULT_FONT_FAMILY } from '../../../../base/browser/fonts.js';
+import { ViewModelDecoration } from '../../../common/viewModel/viewModelDecoration.js';
+import { RunOnceScheduler } from '../../../../base/common/async.js';
 
 /**
  * The orthogonal distance to the slider at which dragging "resets". This implements "snapping"
@@ -54,7 +56,7 @@ class MinimapOptions {
 	public readonly paddingTop: number;
 	public readonly paddingBottom: number;
 	public readonly showSlider: 'always' | 'mouseover';
-	public readonly autohide: boolean;
+	public readonly autohide: 'none' | 'mouseover' | 'scroll';
 	public readonly pixelRatio: number;
 	public readonly typicalHalfwidthCharacterWidth: number;
 	public readonly lineHeight: number;
@@ -915,7 +917,7 @@ export class Minimap extends ViewPart implements IMinimapModel {
 		}
 	}
 	public override onScrollChanged(e: viewEvents.ViewScrollChangedEvent): boolean {
-		return this._actual.onScrollChanged();
+		return this._actual.onScrollChanged(e);
 	}
 	public override onThemeChanged(e: viewEvents.ViewThemeChangedEvent): boolean {
 		this._actual.onThemeChanged();
@@ -1171,6 +1173,8 @@ class InnerMinimap extends Disposable {
 	private _renderDecorations: boolean = false;
 	private _gestureInProgress: boolean = false;
 	private _buffers: MinimapBuffers | null;
+	private _isMouseOverMinimap: boolean = false;
+	private _hideDelayedScheduler: RunOnceScheduler;
 
 	constructor(
 		theme: EditorTheme,
@@ -1221,9 +1225,21 @@ class InnerMinimap extends Disposable {
 
 		this._applyLayout();
 
+		this._hideDelayedScheduler = this._register(new RunOnceScheduler(() => this._hideImmediatelyIfMouseIsOutside(), 500));
+
+		this._register(dom.addStandardDisposableListener(this._domNode.domNode, dom.EventType.MOUSE_OVER, () => {
+			this._isMouseOverMinimap = true;
+		}));
+		this._register(dom.addStandardDisposableListener(this._domNode.domNode, dom.EventType.MOUSE_LEAVE, () => {
+			this._isMouseOverMinimap = false;
+		}));
+
 		this._pointerDownListener = dom.addStandardDisposableListener(this._domNode.domNode, dom.EventType.POINTER_DOWN, (e) => {
 			e.preventDefault();
 
+			const isMouse = (e.pointerType === 'mouse');
+			const isLeftClick = (e.button === 0);
+
 			const renderMinimap = this._model.options.renderMinimap;
 			if (renderMinimap === RenderMinimap.None) {
 				return;
@@ -1232,7 +1248,7 @@ class InnerMinimap extends Disposable {
 				return;
 			}
 			if (this._model.options.size !== 'proportional') {
-				if (e.button === 0 && this._lastRenderData) {
+				if (isLeftClick && this._lastRenderData) {
 					// pretend the click occurred in the center of the slider
 					const position = dom.getDomNodePagePosition(this._slider.domNode);
 					const initialPosY = position.top + position.height / 2;
@@ -1240,14 +1256,17 @@ class InnerMinimap extends Disposable {
 				}
 				return;
 			}
-			const minimapLineHeight = this._model.options.minimapLineHeight;
-			const internalOffsetY = (this._model.options.canvasInnerHeight / this._model.options.canvasOuterHeight) * e.offsetY;
-			const lineIndex = Math.floor(internalOffsetY / minimapLineHeight);
 
-			let lineNumber = lineIndex + this._lastRenderData.renderedLayout.startLineNumber - this._lastRenderData.renderedLayout.topPaddingLineCount;
-			lineNumber = Math.min(lineNumber, this._model.getLineCount());
+			if (isLeftClick || !isMouse) {
+				const minimapLineHeight = this._model.options.minimapLineHeight;
+				const internalOffsetY = (this._model.options.canvasInnerHeight / this._model.options.canvasOuterHeight) * e.offsetY;
+				const lineIndex = Math.floor(internalOffsetY / minimapLineHeight);
 
-			this._model.revealLineNumber(lineNumber);
+				let lineNumber = lineIndex + this._lastRenderData.renderedLayout.startLineNumber - this._lastRenderData.renderedLayout.topPaddingLineCount;
+				lineNumber = Math.min(lineNumber, this._model.getLineCount());
+
+				this._model.revealLineNumber(lineNumber);
+			}
 		});
 
 		this._sliderPointerMoveMonitor = new GlobalPointerMoveMonitor();
@@ -1287,6 +1306,19 @@ class InnerMinimap extends Disposable {
 		});
 	}
 
+	private _hideSoon() {
+		this._hideDelayedScheduler.cancel();
+		this._hideDelayedScheduler.schedule();
+	}
+
+	private _hideImmediatelyIfMouseIsOutside() {
+		if (this._isMouseOverMinimap) {
+			this._hideSoon();
+			return;
+		}
+		this._domNode.toggleClassName('active', false);
+	}
+
 	private _startSliderDragging(e: PointerEvent, initialPosY: number, initialSliderState: MinimapLayout): void {
 		if (!e.target || !(e.target instanceof Element)) {
 			return;
@@ -1352,8 +1384,11 @@ class InnerMinimap extends Disposable {
 		} else {
 			class_.push('slider-mouseover');
 		}
-		if (this._model.options.autohide) {
-			class_.push('autohide');
+
+		if (this._model.options.autohide === 'mouseover') {
+			class_.push('minimap-autohide-mouseover');
+		} else if (this._model.options.autohide === 'scroll') {
+			class_.push('minimap-autohide-scroll');
 		}
 
 		return class_.join(' ');
@@ -1430,7 +1465,11 @@ class InnerMinimap extends Disposable {
 		this._lastRenderData?.onLinesInserted(insertFromLineNumber, insertToLineNumber);
 		return true;
 	}
-	public onScrollChanged(): boolean {
+	public onScrollChanged(e: viewEvents.ViewScrollChangedEvent): boolean {
+		if (this._model.options.autohide === 'scroll' && (e.scrollTopChanged || e.scrollHeightChanged)) {
+			this._domNode.toggleClassName('active', true);
+			this._hideSoon();
+		}
 		this._renderDecorations = true;
 		return true;
 	}
@@ -1624,7 +1663,7 @@ class InnerMinimap extends Disposable {
 					continue;
 				}
 				highlightedLines.set(line, true);
-				const y = layout.getYForLineNumber(startLineNumber, minimapLineHeight);
+				const y = layout.getYForLineNumber(line, minimapLineHeight);
 				canvasContext.fillRect(MINIMAP_GUTTER_WIDTH, y, canvasContext.canvas.width, minimapLineHeight);
 			}
 		}
@@ -1811,7 +1850,7 @@ class InnerMinimap extends Disposable {
 		canvasContext.letterSpacing = sectionHeaderLetterSpacing + 'px';
 		canvasContext.font = '500 ' + sectionHeaderFontSize + 'px ' + this._model.options.sectionHeaderFontFamily;
 		canvasContext.strokeStyle = separatorStroke;
-		canvasContext.lineWidth = 0.2;
+		canvasContext.lineWidth = 0.4;
 
 		const decorations = this._model.getSectionHeaderDecorationsInViewport(layout.startLineNumber, layout.endLineNumber);
 		decorations.sort((a, b) => a.range.startLineNumber - b.range.startLineNumber);
