@@ -11,15 +11,16 @@ import { RangeUtil } from './rangeUtil.js';
 import { StringBuilder } from '../../../common/core/stringBuilder.js';
 import { FloatHorizontalRange, VisibleRanges } from '../../view/renderingContext.js';
 import { LineDecoration } from '../../../common/viewLayout/lineDecorations.js';
-import { CharacterMapping, ForeignElementType, RenderLineInput, renderViewLine, DomPosition } from '../../../common/viewLayout/viewLineRenderer.js';
+import { CharacterMapping, ForeignElementType, RenderLineInput, renderViewLine, DomPosition, RenderWhitespace } from '../../../common/viewLayout/viewLineRenderer.js';
 import { ViewportData } from '../../../common/viewLayout/viewLinesViewportData.js';
-import { InlineDecorationType } from '../../../common/viewModel.js';
 import { isHighContrast } from '../../../../platform/theme/common/theme.js';
 import { EditorFontLigatures } from '../../../common/config/editorOptions.js';
 import { DomReadingContext } from './domReadingContext.js';
 import type { ViewLineOptions } from './viewLineOptions.js';
 import { ViewGpuContext } from '../../gpu/viewGpuContext.js';
 import { OffsetRange } from '../../../common/core/ranges/offsetRange.js';
+import { InlineDecorationType } from '../../../common/viewModel/inlineDecorations.js';
+import { TextDirection } from '../../../common/model.js';
 
 const canUseFastRenderedViewLine = (function () {
 	if (platform.isNative) {
@@ -91,7 +92,7 @@ export class ViewLine implements IVisibleLine {
 		this._options = newOptions;
 	}
 	public onSelectionChanged(): boolean {
-		if (isHighContrast(this._options.themeType) || this._options.renderWhitespace === 'selection') {
+		if (isHighContrast(this._options.themeType) || this._renderedViewLine?.input.renderWhitespace === RenderWhitespace.Selection) {
 			this._isMaybeInvalid = true;
 			return true;
 		}
@@ -115,10 +116,12 @@ export class ViewLine implements IVisibleLine {
 		const lineData = viewportData.getViewLineRenderingData(lineNumber);
 		const options = this._options;
 		const actualInlineDecorations = LineDecoration.filter(lineData.inlineDecorations, lineNumber, lineData.minColumn, lineData.maxColumn);
+		const renderWhitespace = (lineData.hasVariableFonts || options.experimentalWhitespaceRendering === 'off') ? options.renderWhitespace : 'none';
+		const allowFastRendering = !lineData.hasVariableFonts;
 
 		// Only send selection information when needed for rendering whitespace
 		let selectionsOnLine: OffsetRange[] | null = null;
-		if (isHighContrast(options.themeType) || this._options.renderWhitespace === 'selection') {
+		if (isHighContrast(options.themeType) || renderWhitespace === 'selection') {
 			const selections = viewportData.selections;
 			for (const selection of selections) {
 
@@ -134,7 +137,7 @@ export class ViewLine implements IVisibleLine {
 					if (isHighContrast(options.themeType)) {
 						actualInlineDecorations.push(new LineDecoration(startColumn, endColumn, 'inline-selected-text', InlineDecorationType.Regular));
 					}
-					if (this._options.renderWhitespace === 'selection') {
+					if (renderWhitespace === 'selection') {
 						if (!selectionsOnLine) {
 							selectionsOnLine = [];
 						}
@@ -161,10 +164,12 @@ export class ViewLine implements IVisibleLine {
 			options.middotWidth,
 			options.wsmiddotWidth,
 			options.stopRenderingLineAfter,
-			options.renderWhitespace,
+			renderWhitespace,
 			options.renderControlCharacters,
 			options.fontLigatures !== EditorFontLigatures.OFF,
-			selectionsOnLine
+			selectionsOnLine,
+			lineData.textDirection,
+			options.verticalScrollbarSize
 		);
 
 		if (this._renderedViewLine && this._renderedViewLine.input.equals(renderLineInput)) {
@@ -172,12 +177,22 @@ export class ViewLine implements IVisibleLine {
 			return false;
 		}
 
-		sb.appendString('<div style="top:');
+		sb.appendString('<div ');
+		if (lineData.textDirection === TextDirection.RTL) {
+			sb.appendString('dir="rtl" ');
+		} else if (lineData.containsRTL) {
+			sb.appendString('dir="ltr" ');
+		}
+		sb.appendString('style="top:');
 		sb.appendString(String(deltaTop));
 		sb.appendString('px;height:');
 		sb.appendString(String(lineHeight));
 		sb.appendString('px;line-height:');
 		sb.appendString(String(lineHeight));
+		if (lineData.textDirection === TextDirection.RTL) {
+			sb.appendString('px;padding-right:');
+			sb.appendString(String(options.verticalScrollbarSize));
+		}
 		sb.appendString('px;" class="');
 		sb.appendString(ViewLine.CLASS_NAME);
 		sb.appendString('">');
@@ -187,7 +202,15 @@ export class ViewLine implements IVisibleLine {
 		sb.appendString('</div>');
 
 		let renderedViewLine: IRenderedViewLine | null = null;
-		if (monospaceAssumptionsAreValid && canUseFastRenderedViewLine && lineData.isBasicASCII && options.useMonospaceOptimizations && output.containsForeignElements === ForeignElementType.None) {
+		if (
+			allowFastRendering
+			&& monospaceAssumptionsAreValid
+			&& canUseFastRenderedViewLine
+			&& lineData.isBasicASCII
+			&& renderLineInput.isLTR
+			&& options.useMonospaceOptimizations
+			&& output.containsForeignElements === ForeignElementType.None
+		) {
 			renderedViewLine = new FastRenderedViewLine(
 				this._renderedViewLine ? this._renderedViewLine.domNode : null,
 				renderLineInput,
@@ -200,7 +223,6 @@ export class ViewLine implements IVisibleLine {
 				this._renderedViewLine ? this._renderedViewLine.domNode : null,
 				renderLineInput,
 				output.characterMapping,
-				output.containsRTL,
 				output.containsForeignElements
 			);
 		}
@@ -220,6 +242,13 @@ export class ViewLine implements IVisibleLine {
 
 	// --- end IVisibleLineData
 
+	public isRenderedRTL(): boolean {
+		if (!this._renderedViewLine) {
+			return false;
+		}
+		return this._renderedViewLine.input.textDirection === TextDirection.RTL;
+	}
+
 	public getWidth(context: DomReadingContext | null): number {
 		if (!this._renderedViewLine) {
 			return 0;
@@ -294,6 +323,10 @@ export class ViewLine implements IVisibleLine {
 		}
 		return this._renderedViewLine.getColumnOfNodeOffset(spanNode, offset);
 	}
+
+	public resetCachedWidth(): void {
+		this._renderedViewLine?.resetCachedWidth();
+	}
 }
 
 interface IRenderedViewLine {
@@ -301,6 +334,7 @@ interface IRenderedViewLine {
 	readonly input: RenderLineInput;
 	getWidth(context: DomReadingContext | null): number;
 	getWidthIsFast(): boolean;
+	resetCachedWidth(): void;
 	getVisibleRangesForRange(lineNumber: number, startColumn: number, endColumn: number, context: DomReadingContext): FloatHorizontalRange[] | null;
 	getColumnOfNodeOffset(spanNode: HTMLElement, offset: number): number;
 }
@@ -362,6 +396,10 @@ class FastRenderedViewLine implements IRenderedViewLine {
 		return (this.input.lineContent.length < Constants.MaxMonospaceDistance) || this._cachedWidth !== -1;
 	}
 
+	public resetCachedWidth(): void {
+		this._cachedWidth = -1;
+	}
+
 	public monospaceAssumptionsAreValid(): boolean {
 		if (!this.domNode) {
 			return monospaceAssumptionsAreValid;
@@ -379,7 +417,7 @@ class FastRenderedViewLine implements IRenderedViewLine {
 	}
 
 	public toSlowRenderedLine(): RenderedViewLine {
-		return createRenderedLine(this.domNode, this.input, this._characterMapping, false, ForeignElementType.None);
+		return createRenderedLine(this.domNode, this.input, this._characterMapping, ForeignElementType.None);
 	}
 
 	public getVisibleRangesForRange(lineNumber: number, startColumn: number, endColumn: number, context: DomReadingContext): FloatHorizontalRange[] | null {
@@ -451,11 +489,11 @@ class RenderedViewLine implements IRenderedViewLine {
 	private _cachedWidth: number;
 
 	/**
-	 * This is a map that is used only when the line is guaranteed to have no RTL text.
+	 * This is a map that is used only when the line is guaranteed to be rendered LTR and has no RTL text.
 	 */
 	private readonly _pixelOffsetCache: Float32Array | null;
 
-	constructor(domNode: FastDomNode<HTMLElement> | null, renderLineInput: RenderLineInput, characterMapping: CharacterMapping, containsRTL: boolean, containsForeignElements: ForeignElementType) {
+	constructor(domNode: FastDomNode<HTMLElement> | null, renderLineInput: RenderLineInput, characterMapping: CharacterMapping, containsForeignElements: ForeignElementType) {
 		this.domNode = domNode;
 		this.input = renderLineInput;
 		this._characterMapping = characterMapping;
@@ -464,7 +502,7 @@ class RenderedViewLine implements IRenderedViewLine {
 		this._cachedWidth = -1;
 
 		this._pixelOffsetCache = null;
-		if (!containsRTL || this._characterMapping.length === 0 /* the line is empty */) {
+		if (renderLineInput.isLTR) {
 			this._pixelOffsetCache = new Float32Array(Math.max(2, this._characterMapping.length + 1));
 			for (let column = 0, len = this._characterMapping.length; column <= len; column++) {
 				this._pixelOffsetCache[column] = -1;
@@ -499,6 +537,15 @@ class RenderedViewLine implements IRenderedViewLine {
 		return true;
 	}
 
+	public resetCachedWidth(): void {
+		this._cachedWidth = -1;
+		if (this._pixelOffsetCache !== null) {
+			for (let column = 0, len = this._pixelOffsetCache.length; column < len; column++) {
+				this._pixelOffsetCache[column] = -1;
+			}
+		}
+	}
+
 	/**
 	 * Visible ranges for a model range
 	 */
@@ -507,7 +554,7 @@ class RenderedViewLine implements IRenderedViewLine {
 			return null;
 		}
 		if (this._pixelOffsetCache !== null) {
-			// the text is LTR
+			// the text is guaranteed to be entirely LTR
 			const startOffset = this._readPixelOffset(this.domNode, lineNumber, startColumn, context);
 			if (startOffset === -1) {
 				return null;
@@ -538,7 +585,7 @@ class RenderedViewLine implements IRenderedViewLine {
 	}
 
 	protected _readPixelOffset(domNode: FastDomNode<HTMLElement>, lineNumber: number, column: number, context: DomReadingContext): number {
-		if (this._characterMapping.length === 0) {
+		if (this.input.isLTR && this._characterMapping.length === 0) {
 			// This line has no content
 			if (this._containsForeignElements === ForeignElementType.None) {
 				// We can assume the line is really empty
@@ -563,7 +610,7 @@ class RenderedViewLine implements IRenderedViewLine {
 		}
 
 		if (this._pixelOffsetCache !== null) {
-			// the text is LTR
+			// the text is guaranteed to be LTR
 
 			const cachedPixelOffset = this._pixelOffsetCache[column];
 			if (cachedPixelOffset !== -1) {
@@ -588,7 +635,7 @@ class RenderedViewLine implements IRenderedViewLine {
 			return r[0].left;
 		}
 
-		if (column === this._characterMapping.length && this._isWhitespaceOnly && this._containsForeignElements === ForeignElementType.None) {
+		if (this.input.isLTR && column === this._characterMapping.length && this._isWhitespaceOnly && this._containsForeignElements === ForeignElementType.None) {
 			// This branch helps in the case of whitespace only lines which have a width set
 			return this.getWidth(context);
 		}
@@ -612,7 +659,7 @@ class RenderedViewLine implements IRenderedViewLine {
 
 	private _readRawVisibleRangesForRange(domNode: FastDomNode<HTMLElement>, startColumn: number, endColumn: number, context: DomReadingContext): FloatHorizontalRange[] | null {
 
-		if (startColumn === 1 && endColumn === this._characterMapping.length) {
+		if (this.input.isLTR && startColumn === 1 && endColumn === this._characterMapping.length) {
 			// This branch helps IE with bidi text & gives a performance boost to other browsers when reading visible ranges for an entire line
 
 			return [new FloatHorizontalRange(0, this.getWidth(context))];
@@ -642,7 +689,7 @@ class WebKitRenderedViewLine extends RenderedViewLine {
 
 		// WebKit is buggy and returns an expanded range (to contain words in some cases)
 		// The last client rect is enlarged (I think)
-		if (!this.input.containsRTL) {
+		if (this.input.isLTR) {
 			// This is an attempt to patch things up
 			// Find position of last column
 			const endPixelOffset = this._readPixelOffset(domNode, lineNumber, endColumn, context);
@@ -659,23 +706,23 @@ class WebKitRenderedViewLine extends RenderedViewLine {
 	}
 }
 
-const createRenderedLine: (domNode: FastDomNode<HTMLElement> | null, renderLineInput: RenderLineInput, characterMapping: CharacterMapping, containsRTL: boolean, containsForeignElements: ForeignElementType) => RenderedViewLine = (function () {
+const createRenderedLine: (domNode: FastDomNode<HTMLElement> | null, renderLineInput: RenderLineInput, characterMapping: CharacterMapping, containsForeignElements: ForeignElementType) => RenderedViewLine = (function () {
 	if (browser.isWebKit) {
 		return createWebKitRenderedLine;
 	}
 	return createNormalRenderedLine;
 })();
 
-function createWebKitRenderedLine(domNode: FastDomNode<HTMLElement> | null, renderLineInput: RenderLineInput, characterMapping: CharacterMapping, containsRTL: boolean, containsForeignElements: ForeignElementType): RenderedViewLine {
-	return new WebKitRenderedViewLine(domNode, renderLineInput, characterMapping, containsRTL, containsForeignElements);
+function createWebKitRenderedLine(domNode: FastDomNode<HTMLElement> | null, renderLineInput: RenderLineInput, characterMapping: CharacterMapping, containsForeignElements: ForeignElementType): RenderedViewLine {
+	return new WebKitRenderedViewLine(domNode, renderLineInput, characterMapping, containsForeignElements);
 }
 
-function createNormalRenderedLine(domNode: FastDomNode<HTMLElement> | null, renderLineInput: RenderLineInput, characterMapping: CharacterMapping, containsRTL: boolean, containsForeignElements: ForeignElementType): RenderedViewLine {
-	return new RenderedViewLine(domNode, renderLineInput, characterMapping, containsRTL, containsForeignElements);
+function createNormalRenderedLine(domNode: FastDomNode<HTMLElement> | null, renderLineInput: RenderLineInput, characterMapping: CharacterMapping, containsForeignElements: ForeignElementType): RenderedViewLine {
+	return new RenderedViewLine(domNode, renderLineInput, characterMapping, containsForeignElements);
 }
 
 export function getColumnOfNodeOffset(characterMapping: CharacterMapping, spanNode: HTMLElement, offset: number): number {
-	const spanNodeTextContentLength = spanNode.textContent!.length;
+	const spanNodeTextContentLength = spanNode.textContent.length;
 
 	let spanIndex = -1;
 	while (spanNode) {
