@@ -40,7 +40,7 @@ export class DiffEditorViewModel extends Disposable implements IDiffEditorViewMo
 		} else {
 			// Reset state
 			transaction(tx => {
-				for (const r of this._unchangedRegions.get()?.regions || []) {
+				for (const r of this._unchangedRegions.read(undefined)?.regions || []) {
 					r.collapseAll(tx);
 				}
 			});
@@ -107,9 +107,9 @@ export class DiffEditorViewModel extends Disposable implements IDiffEditorViewMo
 			const updatedLastUnchangedRegions = lastUnchangedRegions.regions.map((r, idx) =>
 				(!lastUnchangedRegionsOrigRanges[idx] || !lastUnchangedRegionsModRanges[idx]) ? undefined :
 					new UnchangedRegion(
-						lastUnchangedRegionsOrigRanges[idx]!.startLineNumber,
-						lastUnchangedRegionsModRanges[idx]!.startLineNumber,
-						lastUnchangedRegionsOrigRanges[idx]!.length,
+						lastUnchangedRegionsOrigRanges[idx].startLineNumber,
+						lastUnchangedRegionsModRanges[idx].startLineNumber,
+						lastUnchangedRegionsOrigRanges[idx].length,
 						r.visibleLineCountTop.read(reader),
 						r.visibleLineCountBottom.read(reader),
 					)).filter(isDefined);
@@ -121,7 +121,7 @@ export class DiffEditorViewModel extends Disposable implements IDiffEditorViewMo
 				if (touching.length > 1) {
 					didChange = true;
 					const sumLineCount = touching.reduce((sum, r) => sum + r.lineCount, 0);
-					const r = new UnchangedRegion(touching[0].originalLineNumber, touching[0].modifiedLineNumber, sumLineCount, touching[0].visibleLineCountTop.get(), touching[touching.length - 1].visibleLineCountBottom.get());
+					const r = new UnchangedRegion(touching[0].originalLineNumber, touching[0].modifiedLineNumber, sumLineCount, touching[0].visibleLineCountTop.read(undefined), touching[touching.length - 1].visibleLineCountBottom.read(undefined));
 					newRanges.push(r);
 				} else {
 					newRanges.push(touching[0]);
@@ -174,10 +174,10 @@ export class DiffEditorViewModel extends Disposable implements IDiffEditorViewMo
 					lastUnchangedRegions.regions
 						.map((r, idx) => {
 							if (!lastUnchangedRegionsOrigRanges[idx] || !lastUnchangedRegionsModRanges[idx]) { return undefined; }
-							const length = lastUnchangedRegionsOrigRanges[idx]!.length;
+							const length = lastUnchangedRegionsOrigRanges[idx].length;
 							return new UnchangedRegion(
-								lastUnchangedRegionsOrigRanges[idx]!.startLineNumber,
-								lastUnchangedRegionsModRanges[idx]!.startLineNumber,
+								lastUnchangedRegionsOrigRanges[idx].startLineNumber,
+								lastUnchangedRegionsModRanges[idx].startLineNumber,
 								length,
 								// The visible area can shrink by edits -> we have to account for this
 								Math.min(r.visibleLineCountTop.get(), length),
@@ -315,7 +315,7 @@ export class DiffEditorViewModel extends Disposable implements IDiffEditorViewMo
 				const state = DiffState.fromDiffResult(result);
 				this._diff.set(state, tx);
 				this._isDiffUpToDate.set(true, tx);
-				const currentSyncedMovedText = this.movedTextToCompare.get();
+				const currentSyncedMovedText = this.movedTextToCompare.read(undefined);
 				this.movedTextToCompare.set(currentSyncedMovedText ? this._lastDiff.moves.find(m => m.lineRangeMapping.modified.intersect(currentSyncedMovedText.lineRangeMapping.modified)) : undefined, tx);
 			});
 		}));
