@@ -153,7 +153,7 @@ export interface IContentSizeChangedEvent {
 export interface ITriggerEditorOperationEvent {
 	source: string | null | undefined;
 	handlerId: string;
-	payload: any;
+	payload: unknown;
 }
 
 export interface INewScrollPosition {
@@ -198,7 +198,7 @@ export interface IViewState {
 export interface ICodeEditorViewState {
 	cursorState: ICursorState[];
 	viewState: IViewState;
-	contributionsState: { [id: string]: any };
+	contributionsState: { [id: string]: unknown };
 }
 /**
  * (Serializable) View state for the diff editor.
@@ -464,7 +464,7 @@ export interface IEditor {
 	 * @param handlerId The id of the handler or the id of a contribution.
 	 * @param payload Extra data to be sent to the handler.
 	 */
-	trigger(source: string | null | undefined, handlerId: string, payload: any): void;
+	trigger(source: string | null | undefined, handlerId: string, payload: unknown): void;
 
 	/**
 	 * Gets the current model attached to this editor.
@@ -495,7 +495,7 @@ export interface IEditor {
 	 * @see {@link ITextModel.changeDecorations}
 	 * @internal
 	 */
-	changeDecorations(callback: (changeAccessor: IModelDecorationsChangeAccessor) => any): any;
+	changeDecorations<T>(callback: (changeAccessor: IModelDecorationsChangeAccessor) => T): T | null;
 }
 
 /**
@@ -546,7 +546,7 @@ export interface IEditorDecorationsCollection {
 	 * An event emitted when decorations change in the editor,
 	 * but the change is not caused by us setting or clearing the collection.
 	 */
-	onDidChange: Event<IModelDecorationsChangedEvent>;
+	readonly onDidChange: Event<IModelDecorationsChangedEvent>;
 	/**
 	 * Get the decorations count.
 	 */
@@ -588,11 +588,11 @@ export interface IEditorContribution {
 	/**
 	 * Store view state.
 	 */
-	saveViewState?(): any;
+	saveViewState?(): unknown;
 	/**
 	 * Restore view state.
 	 */
-	restoreViewState?(state: any): void;
+	restoreViewState?(state: unknown): void;
 }
 
 /**
@@ -609,8 +609,8 @@ export interface IDiffEditorContribution {
 /**
  * @internal
  */
-export function isThemeColor(o: any): o is ThemeColor {
-	return o && typeof o.id === 'string';
+export function isThemeColor(o: unknown): o is ThemeColor {
+	return !!o && typeof (o as ThemeColor).id === 'string';
 }
 
 /**
@@ -633,6 +633,7 @@ export interface IThemeDecorationRenderOptions {
 
 	fontStyle?: string;
 	fontWeight?: string;
+	fontFamily?: string;
 	fontSize?: string;
 	lineHeight?: number;
 	textDecoration?: string;
