@@ -7,9 +7,8 @@ import { createTrustedTypesPolicy } from '../../../../../../base/browser/trusted
 import { renderIcon } from '../../../../../../base/browser/ui/iconLabel/iconLabels.js';
 import { Codicon } from '../../../../../../base/common/codicons.js';
 import { Emitter, Event } from '../../../../../../base/common/event.js';
-import { createHotClass } from '../../../../../../base/common/hotReloadHelpers.js';
 import { Disposable, DisposableStore, MutableDisposable, toDisposable } from '../../../../../../base/common/lifecycle.js';
-import { IObservable, autorun, autorunWithStore, constObservable, derived, observableSignalFromEvent, observableValue } from '../../../../../../base/common/observable.js';
+import { IObservable, autorun, autorunWithStore, constObservable, derived, derivedOpts, observableSignalFromEvent, observableValue } from '../../../../../../base/common/observable.js';
 import * as strings from '../../../../../../base/common/strings.js';
 import { applyFontInfo } from '../../../../../browser/config/domFontInfo.js';
 import { ContentWidgetPositionPreference, ICodeEditor, IContentWidgetPosition, IViewZoneChangeAccessor, MouseTargetType } from '../../../../../browser/editorBrowser.js';
@@ -19,13 +18,12 @@ import { StringEdit, StringReplacement } from '../../../../../common/core/edits/
 import { Position } from '../../../../../common/core/position.js';
 import { Range } from '../../../../../common/core/range.js';
 import { StringBuilder } from '../../../../../common/core/stringBuilder.js';
-import { IconPath } from '../../../../../common/languages.js';
+import { IconPath, InlineCompletionWarning } from '../../../../../common/languages.js';
 import { ILanguageService } from '../../../../../common/languages/language.js';
 import { IModelDeltaDecoration, ITextModel, InjectedTextCursorStops, PositionAffinity } from '../../../../../common/model.js';
 import { LineTokens } from '../../../../../common/tokens/lineTokens.js';
 import { LineDecoration } from '../../../../../common/viewLayout/lineDecorations.js';
 import { RenderLineInput, renderViewLine } from '../../../../../common/viewLayout/viewLineRenderer.js';
-import { InlineDecorationType } from '../../../../../common/viewModel.js';
 import { GhostText, GhostTextReplacement, IGhostTextLine } from '../../model/ghostText.js';
 import { RangeSingleLine } from '../../../../../common/core/ranges/rangeSingleLine.js';
 import { ColumnRange } from '../../../../../common/core/ranges/columnRange.js';
@@ -34,161 +32,88 @@ import './ghostTextView.css';
 import { IMouseEvent, StandardMouseEvent } from '../../../../../../base/browser/mouseEvent.js';
 import { CodeEditorWidget } from '../../../../../browser/widget/codeEditor/codeEditorWidget.js';
 import { TokenWithTextArray } from '../../../../../common/tokens/tokenWithTextArray.js';
+import { InlineCompletionViewData } from '../inlineEdits/inlineEditsViewInterface.js';
+import { InlineDecorationType } from '../../../../../common/viewModel/inlineDecorations.js';
+import { equals, sum } from '../../../../../../base/common/arrays.js';
+import { equalsIfDefinedC, IEquatable, thisEqualsC } from '../../../../../../base/common/equals.js';
 
-export interface IGhostTextWidgetModel {
-	readonly targetTextModel: IObservable<ITextModel | undefined>;
-	readonly ghostText: IObservable<GhostText | GhostTextReplacement | undefined>;
-	readonly warning: IObservable<{ icon: IconPath | undefined } | undefined>;
-	readonly minReservedLineCount: IObservable<number>;
+export interface IGhostTextWidgetData {
+	readonly ghostText: GhostText | GhostTextReplacement;
+	readonly warning: GhostTextWidgetWarning | undefined;
+	handleInlineCompletionShown(viewData: InlineCompletionViewData): void;
+}
+
+export class GhostTextWidgetWarning {
+	public static from(warning: InlineCompletionWarning | undefined): GhostTextWidgetWarning | undefined {
+		if (!warning) {
+			return undefined;
+		}
+		return new GhostTextWidgetWarning(warning.icon);
+	}
+
+	constructor(
+		public readonly icon: IconPath = Codicon.warning,
+	) { }
 }
 
 const USE_SQUIGGLES_FOR_WARNING = true;
 const GHOST_TEXT_CLASS_NAME = 'ghost-text';
 
 export class GhostTextView extends Disposable {
-	private readonly _isDisposed;
+	private readonly _isDisposed = observableValue(this, false);
 	private readonly _editorObs;
-	public static hot = createHotClass(GhostTextView);
+	private readonly _warningState = derived(reader => {
+		const model = this._data.read(reader);
+		const warning = model?.warning;
+		if (!model || !warning) { return undefined; }
+		const gt = model.ghostText;
+		return { lineNumber: gt.lineNumber, position: new Position(gt.lineNumber, gt.parts[0].column), icon: warning.icon };
+	});
 
-	private _warningState;
+	private readonly _onDidClick = this._register(new Emitter<IMouseEvent>());
+	public readonly onDidClick = this._onDidClick.event;
 
-	private readonly _onDidClick;
-	public readonly onDidClick;
+	private readonly _extraClasses: readonly string[];
+	private readonly _isClickable: boolean;
+	private readonly _shouldKeepCursorStable: boolean;
+	private readonly _minReservedLineCount: IObservable<number>;
+	private readonly _useSyntaxHighlighting: IObservable<boolean>;
+	private readonly _highlightShortText: boolean;
 
 	constructor(
 		private readonly _editor: ICodeEditor,
-		private readonly _model: IGhostTextWidgetModel,
-		private readonly _options: IObservable<{
-			extraClasses?: string[];
-			syntaxHighlightingEnabled: boolean;
-		}>,
-		private readonly _shouldKeepCursorStable: boolean,
-		private readonly _isClickable: boolean,
-		@ILanguageService private readonly _languageService: ILanguageService,
+		private readonly _data: IObservable<IGhostTextWidgetData | undefined>,
+		options: {
+			extraClasses?: readonly string[]; // TODO@benibenj improve
+			isClickable?: boolean;
+			shouldKeepCursorStable?: boolean;
+			minReservedLineCount?: IObservable<number>;
+			useSyntaxHighlighting?: IObservable<boolean>;
+			highlightShortSuggestions?: boolean;
+		},
+		@ILanguageService private readonly _languageService: ILanguageService
 	) {
 		super();
-		this._isDisposed = observableValue(this, false);
+
+		this._extraClasses = options.extraClasses ?? [];
+		this._isClickable = options.isClickable ?? false;
+		this._shouldKeepCursorStable = options.shouldKeepCursorStable ?? false;
+		this._minReservedLineCount = options.minReservedLineCount ?? constObservable(0);
+		this._useSyntaxHighlighting = options.useSyntaxHighlighting ?? constObservable(true);
+		this._highlightShortText = options.highlightShortSuggestions ?? false;
+
 		this._editorObs = observableCodeEditor(this._editor);
-		this._warningState = derived(reader => {
-			const gt = this._model.ghostText.read(reader);
-			if (!gt) { return undefined; }
-			const warning = this._model.warning.read(reader);
-			if (!warning) { return undefined; }
-			return { lineNumber: gt.lineNumber, position: new Position(gt.lineNumber, gt.parts[0].column), icon: warning.icon };
-		});
-		this._onDidClick = this._register(new Emitter<IMouseEvent>());
-		this.onDidClick = this._onDidClick.event;
-		this._useSyntaxHighlighting = this._options.map(o => o.syntaxHighlightingEnabled);
-		this._extraClassNames = derived(this, reader => {
-			const extraClasses = [...this._options.read(reader).extraClasses ?? []];
-			if (this._useSyntaxHighlighting.read(reader)) {
-				extraClasses.push('syntax-highlighted');
-			}
-			if (USE_SQUIGGLES_FOR_WARNING && this._warningState.read(reader)) {
-				extraClasses.push('warning');
-			}
-			const extraClassNames = extraClasses.map(c => ` ${c}`).join('');
-			return extraClassNames;
-		});
-		this.uiState = derived(this, reader => {
-			if (this._isDisposed.read(reader)) { return undefined; }
-			const textModel = this._editorObs.model.read(reader);
-			if (textModel !== this._model.targetTextModel.read(reader)) { return undefined; }
-			const ghostText = this._model.ghostText.read(reader);
-			if (!ghostText) { return undefined; }
-
-			const replacedRange = ghostText instanceof GhostTextReplacement ? ghostText.columnRange : undefined;
-
-			const syntaxHighlightingEnabled = this._useSyntaxHighlighting.read(reader);
-			const extraClassNames = this._extraClassNames.read(reader);
-			const { inlineTexts, additionalLines, hiddenRange, additionalLinesOriginalSuffix } = computeGhostTextViewData(ghostText, textModel, GHOST_TEXT_CLASS_NAME + extraClassNames);
-
-			const currentLine = textModel.getLineContent(ghostText.lineNumber);
-			const edit = new StringEdit(inlineTexts.map(t => StringReplacement.insert(t.column - 1, t.text)));
-			const tokens = syntaxHighlightingEnabled ? textModel.tokenization.tokenizeLinesAt(ghostText.lineNumber, [edit.apply(currentLine), ...additionalLines.map(l => l.content)]) : undefined;
-			const newRanges = edit.getNewRanges();
-			const inlineTextsWithTokens = inlineTexts.map((t, idx) => ({ ...t, tokens: tokens?.[0]?.getTokensInRange(newRanges[idx]) }));
-
-			const tokenizedAdditionalLines: LineData[] = additionalLines.map((l, idx) => {
-				let content = tokens?.[idx + 1] ?? LineTokens.createEmpty(l.content, this._languageService.languageIdCodec);
-				if (idx === additionalLines.length - 1 && additionalLinesOriginalSuffix) {
-					const t = TokenWithTextArray.fromLineTokens(textModel.tokenization.getLineTokens(additionalLinesOriginalSuffix.lineNumber));
-					const existingContent = t.slice(additionalLinesOriginalSuffix.columnRange.toZeroBasedOffsetRange());
-					content = TokenWithTextArray.fromLineTokens(content).append(existingContent).toLineTokens(content.languageIdCodec);
-				}
-				return {
-					content,
-					decorations: l.decorations,
-				};
-			});
-
-			return {
-				replacedRange,
-				inlineTexts: inlineTextsWithTokens,
-				additionalLines: tokenizedAdditionalLines,
-				hiddenRange,
-				lineNumber: ghostText.lineNumber,
-				additionalReservedLineCount: this._model.minReservedLineCount.read(reader),
-				targetTextModel: textModel,
-				syntaxHighlightingEnabled,
-			};
-		});
-		this.decorations = derived(this, reader => {
-			const uiState = this.uiState.read(reader);
-			if (!uiState) { return []; }
-
-			const decorations: IModelDeltaDecoration[] = [];
-
-			const extraClassNames = this._extraClassNames.read(reader);
-
-			if (uiState.replacedRange) {
-				decorations.push({
-					range: uiState.replacedRange.toRange(uiState.lineNumber),
-					options: { inlineClassName: 'inline-completion-text-to-replace' + extraClassNames, description: 'GhostTextReplacement' }
-				});
-			}
-
-			if (uiState.hiddenRange) {
-				decorations.push({
-					range: uiState.hiddenRange.toRange(uiState.lineNumber),
-					options: { inlineClassName: 'ghost-text-hidden', description: 'ghost-text-hidden', }
-				});
-			}
-
-			for (const p of uiState.inlineTexts) {
-				decorations.push({
-					range: Range.fromPositions(new Position(uiState.lineNumber, p.column)),
-					options: {
-						description: 'ghost-text-decoration',
-						after: {
-							content: p.text,
-							tokens: p.tokens,
-							inlineClassName: (p.preview ? 'ghost-text-decoration-preview' : 'ghost-text-decoration')
-								+ (this._isClickable ? ' clickable' : '')
-								+ extraClassNames
-								+ p.lineDecorations.map(d => ' ' + d.className).join(' '), // TODO: take the ranges into account for line decorations
-							cursorStops: InjectedTextCursorStops.Left,
-							attachedData: new GhostTextAttachedData(this),
-						},
-						showIfCollapsed: true,
-					}
-				});
-			}
-
-			return decorations;
-		});
 		this._additionalLinesWidget = this._register(
 			new AdditionalLinesWidget(
 				this._editor,
-				derived(reader => {
+				derivedOpts({ owner: this, equalsFn: equalsIfDefinedC(thisEqualsC()) }, reader => {
 					/** @description lines */
-					const uiState = this.uiState.read(reader);
-					return uiState ? {
-						lineNumber: uiState.lineNumber,
-						additionalLines: uiState.additionalLines,
-						minReservedLineCount: uiState.additionalReservedLineCount,
-						targetTextModel: uiState.targetTextModel,
-					} : undefined;
+					const uiState = this._state.read(reader);
+					return uiState ? new AdditionalLinesData(
+						uiState.lineNumber,
+						uiState.additionalLines,
+						uiState.additionalReservedLineCount,
+					) : undefined;
 				}),
 				this._shouldKeepCursorStable,
 				this._isClickable
@@ -200,17 +125,9 @@ export class GhostTextView extends Disposable {
 				p.target.detail.injectedText.options.attachedData.owner === this,
 			this._store
 		);
-		this.isHovered = derived(this, reader => {
-			if (this._isDisposed.read(reader)) { return false; }
-			return this._isInlineTextHovered.read(reader) || this._additionalLinesWidget.isHovered.read(reader);
-		});
-		this.height = derived(this, reader => {
-			const lineHeight = this._editorObs.getOption(EditorOption.lineHeight).read(reader);
-			return lineHeight + (this._additionalLinesWidget.viewZoneHeight.read(reader) ?? 0);
-		});
 
 		this._register(toDisposable(() => { this._isDisposed.set(true, undefined); }));
-		this._register(this._editorObs.setDecorations(this.decorations));
+		this._register(this._editorObs.setDecorations(this._decorations));
 
 		if (this._isClickable) {
 			this._register(this._additionalLinesWidget.onDidClick((e) => this._onDidClick.fire(e)));
@@ -225,6 +142,11 @@ export class GhostTextView extends Disposable {
 			}));
 		}
 
+		this._register(autorun(reader => {
+			const state = this._state.read(reader);
+			state?.handleInlineCompletionShown(state.telemetryViewData);
+		}));
+
 		this._register(autorunWithStore((reader, store) => {
 			if (USE_SQUIGGLES_FOR_WARNING) {
 				return;
@@ -252,6 +174,7 @@ export class GhostTextView extends Disposable {
 						color: 'orange',
 					},
 					ref: (dom) => {
+						// eslint-disable-next-line local/code-no-any-casts
 						(dom as any as WidgetDomElement).ghostTextViewWarningWidgetData = { range: Range.fromPositions(state.position) };
 					}
 				}, [
@@ -264,15 +187,16 @@ export class GhostTextView extends Disposable {
 							alignContent: 'center',
 							alignItems: 'center',
 						}
-					}, [
-						renderIcon((state.icon && 'id' in state.icon) ? state.icon : Codicon.warning),
-					])
+					},
+						[renderIcon(state.icon)]
+					)
 				]).keepUpdated(store).element,
 			}));
 		}));
 	}
 
 	public static getWarningWidgetContext(domNode: HTMLElement): { range: Range } | undefined {
+		// eslint-disable-next-line local/code-no-any-casts
 		const data = (domNode as any as WidgetDomElement).ghostTextViewWarningWidgetData;
 		if (data) {
 			return data;
@@ -282,21 +206,155 @@ export class GhostTextView extends Disposable {
 		return undefined;
 	}
 
-	private readonly _useSyntaxHighlighting;
+	private readonly _nonWhitespaceCount = derived(this, reader => {
+		const data = this._data.read(reader);
+		if (!data) { return undefined; }
+		const ghostText = data.ghostText;
+		const allText = ghostText.parts.map(p => p.lines.map(l => l.line).join('')).join('');
+		return allText.replace(/\s/g, '').length;
+	});
 
-	private readonly _extraClassNames;
+	private readonly _extraClassNames = derived(this, reader => {
+		const extraClasses = this._extraClasses.slice();
+		if (USE_SQUIGGLES_FOR_WARNING && this._warningState.read(reader)) {
+			extraClasses.push('warning');
+		}
+		const nonWhitespaceCount = this._nonWhitespaceCount.read(reader);
+		if (this._highlightShortText && nonWhitespaceCount && nonWhitespaceCount < 3) {
+			extraClasses.push('short-text');
+		} else if (this._useSyntaxHighlighting.read(reader)) {
+			extraClasses.push('syntax-highlighted');
+		}
+		const extraClassNames = extraClasses.map(c => ` ${c}`).join('');
+		return extraClassNames;
+	});
 
-	private readonly uiState;
+	private readonly _state = derived(this, reader => {
+		if (this._isDisposed.read(reader)) { return undefined; }
 
-	private readonly decorations;
+		const props = this._data.read(reader);
+		if (!props) { return undefined; }
+
+		const textModel = this._editorObs.model.read(reader);
+		if (!textModel) { return undefined; }
+
+		const ghostText = props.ghostText;
+		const replacedRange = ghostText instanceof GhostTextReplacement ? ghostText.columnRange : undefined;
+
+		const syntaxHighlightingEnabled = this._useSyntaxHighlighting.read(reader);
+		const extraClassNames = this._extraClassNames.read(reader);
+		const { inlineTexts, additionalLines, hiddenRange, additionalLinesOriginalSuffix } = computeGhostTextViewData(ghostText, textModel, GHOST_TEXT_CLASS_NAME + extraClassNames);
+
+		const currentLine = textModel.getLineContent(ghostText.lineNumber);
+		const edit = new StringEdit(inlineTexts.map(t => StringReplacement.insert(t.column - 1, t.text)));
+		const tokens = syntaxHighlightingEnabled ? textModel.tokenization.tokenizeLinesAt(ghostText.lineNumber, [edit.apply(currentLine), ...additionalLines.map(l => l.content)]) : undefined;
+		const newRanges = edit.getNewRanges();
+		const inlineTextsWithTokens = inlineTexts.map((t, idx) => ({ ...t, tokens: tokens?.[0]?.getTokensInRange(newRanges[idx]) }));
+
+		const tokenizedAdditionalLines: LineData[] = additionalLines.map((l, idx) => {
+			let content = tokens?.[idx + 1] ?? LineTokens.createEmpty(l.content, this._languageService.languageIdCodec);
+			if (idx === additionalLines.length - 1 && additionalLinesOriginalSuffix) {
+				const t = TokenWithTextArray.fromLineTokens(textModel.tokenization.getLineTokens(additionalLinesOriginalSuffix.lineNumber));
+				const existingContent = t.slice(additionalLinesOriginalSuffix.columnRange.toZeroBasedOffsetRange());
+				content = TokenWithTextArray.fromLineTokens(content).append(existingContent).toLineTokens(content.languageIdCodec);
+			}
+			return new LineData(
+				content,
+				l.decorations,
+			);
+		});
+
+		const cursorColumn = this._editor.getSelection()?.getStartPosition().column!;
+		const disjointInlineTexts = inlineTextsWithTokens.filter(inline => inline.text !== '');
+		const hasInsertionOnCurrentLine = disjointInlineTexts.length !== 0;
+		const telemetryViewData = new InlineCompletionViewData(
+			(hasInsertionOnCurrentLine ? disjointInlineTexts[0].column : 1) - cursorColumn,
+			hasInsertionOnCurrentLine ? 0 : (additionalLines.findIndex(line => line.content !== '') + 1),
+			hasInsertionOnCurrentLine ? 1 : 0,
+			additionalLines.length + (hasInsertionOnCurrentLine ? 1 : 0),
+			0,
+			sum(disjointInlineTexts.map(inline => inline.text.length)) + sum(tokenizedAdditionalLines.map(line => line.content.getTextLength())),
+			disjointInlineTexts.length + (additionalLines.length > 0 ? 1 : 0),
+			disjointInlineTexts.length > 1 && tokenizedAdditionalLines.length === 0 ? disjointInlineTexts.every(inline => inline.text === disjointInlineTexts[0].text) : undefined
+		);
+
+		return {
+			replacedRange,
+			inlineTexts: inlineTextsWithTokens,
+			additionalLines: tokenizedAdditionalLines,
+			hiddenRange,
+			lineNumber: ghostText.lineNumber,
+			additionalReservedLineCount: this._minReservedLineCount.read(reader),
+			targetTextModel: textModel,
+			syntaxHighlightingEnabled,
+			telemetryViewData,
+			handleInlineCompletionShown: props.handleInlineCompletionShown,
+		};
+	});
+
+	private readonly _decorations = derived(this, reader => {
+		const uiState = this._state.read(reader);
+		if (!uiState) { return []; }
+
+		const decorations: IModelDeltaDecoration[] = [];
+
+		const extraClassNames = this._extraClassNames.read(reader);
+
+		if (uiState.replacedRange) {
+			decorations.push({
+				range: uiState.replacedRange.toRange(uiState.lineNumber),
+				options: { inlineClassName: 'inline-completion-text-to-replace' + extraClassNames, description: 'GhostTextReplacement' }
+			});
+		}
+
+		if (uiState.hiddenRange) {
+			decorations.push({
+				range: uiState.hiddenRange.toRange(uiState.lineNumber),
+				options: { inlineClassName: 'ghost-text-hidden', description: 'ghost-text-hidden', }
+			});
+		}
+
+		for (const p of uiState.inlineTexts) {
+			let inlineExtraClassNames = '';
+			if (this._highlightShortText && p.text.length < 5) {
+				inlineExtraClassNames += ' short-text';
+			}
+			decorations.push({
+				range: Range.fromPositions(new Position(uiState.lineNumber, p.column)),
+				options: {
+					description: 'ghost-text-decoration',
+					after: {
+						content: p.text,
+						tokens: p.tokens,
+						inlineClassName: (p.preview ? 'ghost-text-decoration-preview' : 'ghost-text-decoration')
+							+ (this._isClickable ? ' clickable' : '')
+							+ extraClassNames
+							+ inlineExtraClassNames
+							+ p.lineDecorations.map(d => ' ' + d.className).join(' '), // TODO: take the ranges into account for line decorations
+						cursorStops: InjectedTextCursorStops.Left,
+						attachedData: new GhostTextAttachedData(this),
+					},
+					showIfCollapsed: true,
+				}
+			});
+		}
+
+		return decorations;
+	});
 
 	private readonly _additionalLinesWidget;
 
 	private readonly _isInlineTextHovered;
 
-	public readonly isHovered;
+	public readonly isHovered = derived(this, reader => {
+		if (this._isDisposed.read(reader)) { return false; }
+		return this._isInlineTextHovered.read(reader) || this._additionalLinesWidget.isHovered.read(reader);
+	});
 
-	public readonly height;
+	public readonly height = derived(this, reader => {
+		const lineHeight = this._editorObs.getOption(EditorOption.lineHeight).read(reader);
+		return lineHeight + (this._additionalLinesWidget.viewZoneHeight.read(reader) ?? 0);
+	});
 
 	public ownsViewZone(viewZoneId: string): boolean {
 		return this._additionalLinesWidget.viewZoneId === viewZoneId;
@@ -382,6 +440,24 @@ function computeGhostTextViewData(ghostText: GhostText | GhostTextReplacement, t
 	};
 }
 
+class AdditionalLinesData implements IEquatable<AdditionalLinesData> {
+	constructor(
+		public readonly lineNumber: number,
+		public readonly additionalLines: readonly LineData[],
+		public readonly minReservedLineCount: number,
+	) { }
+
+	equals(other: AdditionalLinesData): boolean {
+		if (this.lineNumber !== other.lineNumber) {
+			return false;
+		}
+		if (this.minReservedLineCount !== other.minReservedLineCount) {
+			return false;
+		}
+		return equals(this.additionalLines, other.additionalLines, thisEqualsC());
+	}
+}
+
 export class AdditionalLinesWidget extends Disposable {
 	private _viewZoneInfo: { viewZoneId: string; heightInLines: number; lineNumber: number } | undefined;
 	public get viewZoneId(): string | undefined { return this._viewZoneInfo?.viewZoneId; }
@@ -402,12 +478,7 @@ export class AdditionalLinesWidget extends Disposable {
 
 	constructor(
 		private readonly _editor: ICodeEditor,
-		private readonly _lines: IObservable<{
-			targetTextModel: ITextModel;
-			lineNumber: number;
-			additionalLines: LineData[];
-			minReservedLineCount: number;
-		} | undefined>,
+		private readonly _lines: IObservable<AdditionalLinesData | undefined>,
 		private readonly _shouldKeepCursorStable: boolean,
 		private readonly _isClickable: boolean,
 	) {
@@ -463,7 +534,7 @@ export class AdditionalLinesWidget extends Disposable {
 		});
 	}
 
-	private updateLines(lineNumber: number, additionalLines: LineData[], minReservedLineCount: number): void {
+	private updateLines(lineNumber: number, additionalLines: readonly LineData[], minReservedLineCount: number): void {
 		const textModel = this._editor.getModel();
 		if (!textModel) {
 			return;
@@ -471,31 +542,33 @@ export class AdditionalLinesWidget extends Disposable {
 
 		const { tabSize } = textModel.getOptions();
 
-		this._editor.changeViewZones((changeAccessor) => {
-			const store = new DisposableStore();
+		observableCodeEditor(this._editor).transaction(_ => {
+			this._editor.changeViewZones((changeAccessor) => {
+				const store = new DisposableStore();
 
-			this.removeActiveViewZone(changeAccessor);
+				this.removeActiveViewZone(changeAccessor);
 
-			const heightInLines = Math.max(additionalLines.length, minReservedLineCount);
-			if (heightInLines > 0) {
-				const domNode = document.createElement('div');
-				renderLines(domNode, tabSize, additionalLines, this._editor.getOptions(), this._isClickable);
+				const heightInLines = Math.max(additionalLines.length, minReservedLineCount);
+				if (heightInLines > 0) {
+					const domNode = document.createElement('div');
+					renderLines(domNode, tabSize, additionalLines, this._editor.getOptions(), this._isClickable);
 
-				if (this._isClickable) {
-					store.add(addDisposableListener(domNode, 'mousedown', (e) => {
-						e.preventDefault(); // This prevents that the editor loses focus
-					}));
-					store.add(addDisposableListener(domNode, 'click', (e) => {
-						if (isTargetGhostText(e.target)) {
-							this._onDidClick.fire(new StandardMouseEvent(getWindow(e), e));
-						}
-					}));
+					if (this._isClickable) {
+						store.add(addDisposableListener(domNode, 'mousedown', (e) => {
+							e.preventDefault(); // This prevents that the editor loses focus
+						}));
+						store.add(addDisposableListener(domNode, 'click', (e) => {
+							if (isTargetGhostText(e.target)) {
+								this._onDidClick.fire(new StandardMouseEvent(getWindow(e), e));
+							}
+						}));
+					}
+
+					this.addViewZone(changeAccessor, lineNumber, heightInLines, domNode);
 				}
 
-				this.addViewZone(changeAccessor, lineNumber, heightInLines, domNode);
-			}
-
-			this._viewZoneListener.value = store;
+				this._viewZoneListener.value = store;
+			});
 		});
 	}
 
@@ -544,12 +617,21 @@ function isTargetGhostText(target: EventTarget | null): boolean {
 	return isHTMLElement(target) && target.classList.contains(GHOST_TEXT_CLASS_NAME);
 }
 
-export interface LineData {
-	content: LineTokens; // Must not contain a linebreak!
-	decorations: LineDecoration[];
+export class LineData implements IEquatable<LineData> {
+	constructor(
+		public readonly content: LineTokens, // Must not contain a linebreak!
+		public readonly decorations: readonly LineDecoration[]
+	) { }
+
+	equals(other: LineData): boolean {
+		if (!this.content.equals(other.content)) {
+			return false;
+		}
+		return LineDecoration.equalsArr(this.decorations, other.decorations);
+	}
 }
 
-function renderLines(domNode: HTMLElement, tabSize: number, lines: LineData[], opts: IComputedEditorOptions, isClickable: boolean): void {
+function renderLines(domNode: HTMLElement, tabSize: number, lines: readonly LineData[], opts: IComputedEditorOptions, isClickable: boolean): void {
 	const disableMonospaceOptimizations = opts.get(EditorOption.disableMonospaceOptimizations);
 	const stopRenderingLineAfter = opts.get(EditorOption.stopRenderingLineAfter);
 	// To avoid visual confusion, we don't want to render visible whitespace
@@ -588,7 +670,7 @@ function renderLines(domNode: HTMLElement, tabSize: number, lines: LineData[], o
 			containsRTL,
 			0,
 			lineTokens,
-			lineData.decorations,
+			lineData.decorations.slice(),
 			tabSize,
 			0,
 			fontInfo.spaceWidth,
@@ -598,7 +680,9 @@ function renderLines(domNode: HTMLElement, tabSize: number, lines: LineData[], o
 			renderWhitespace,
 			renderControlCharacters,
 			fontLigatures !== EditorFontLigatures.OFF,
-			null
+			null,
+			null,
+			0
 		), sb);
 
 		sb.appendString('</div>');
