@@ -3,30 +3,28 @@
  *  Licensed under the MIT License. See License.txt in the project root for license information.
  *--------------------------------------------------------------------------------------------*/
 import { $, getWindow, n } from '../../../../../../../base/browser/dom.js';
-import { IMouseEvent, StandardMouseEvent } from '../../../../../../../base/browser/mouseEvent.js';
 import { Color } from '../../../../../../../base/common/color.js';
 import { Emitter } from '../../../../../../../base/common/event.js';
 import { Disposable } from '../../../../../../../base/common/lifecycle.js';
 import { IObservable, IReader, autorun, constObservable, derived, derivedObservableWithCache, observableFromEvent } from '../../../../../../../base/common/observable.js';
 import { IInstantiationService } from '../../../../../../../platform/instantiation/common/instantiation.js';
-import { editorBackground } from '../../../../../../../platform/theme/common/colorRegistry.js';
-import { asCssVariable, asCssVariableWithDefault } from '../../../../../../../platform/theme/common/colorUtils.js';
+import { asCssVariable } from '../../../../../../../platform/theme/common/colorUtils.js';
 import { IThemeService } from '../../../../../../../platform/theme/common/themeService.js';
 import { ICodeEditor } from '../../../../../../browser/editorBrowser.js';
 import { observableCodeEditor } from '../../../../../../browser/observableCodeEditor.js';
 import { Rect } from '../../../../../../common/core/2d/rect.js';
 import { EmbeddedCodeEditorWidget } from '../../../../../../browser/widget/codeEditor/embeddedCodeEditorWidget.js';
-import { EditorOption } from '../../../../../../common/config/editorOptions.js';
 import { OffsetRange } from '../../../../../../common/core/ranges/offsetRange.js';
 import { Position } from '../../../../../../common/core/position.js';
 import { Range } from '../../../../../../common/core/range.js';
 import { ITextModel } from '../../../../../../common/model.js';
 import { StickyScrollController } from '../../../../../stickyScroll/browser/stickyScrollController.js';
 import { InlineCompletionContextKeys } from '../../../controller/inlineCompletionContextKeys.js';
-import { IInlineEditsView, InlineEditTabAction } from '../inlineEditsViewInterface.js';
+import { IInlineEditsView, InlineEditClickEvent, InlineEditTabAction } from '../inlineEditsViewInterface.js';
 import { InlineEditWithChanges } from '../inlineEditWithChanges.js';
-import { getEditorBlendedColor, getModifiedBorderColor, getOriginalBorderColor, modifiedBackgroundColor, originalBackgroundColor } from '../theme.js';
-import { PathBuilder, getContentRenderWidth, getOffsetForPos, mapOutFalsy, maxContentWidthInRange } from '../utils/utils.js';
+import { getEditorBackgroundColor, getEditorBlendedColor, getModifiedBorderColor, getOriginalBorderColor, INLINE_EDITS_BORDER_RADIUS, modifiedBackgroundColor, originalBackgroundColor } from '../theme.js';
+import { PathBuilder, getContentRenderWidth, getOffsetForPos, mapOutFalsy, maxContentWidthInRange, observeEditorBoundingClientRect } from '../utils/utils.js';
+import { InlineCompletionEditorType } from '../../../model/provideInlineCompletions.js';
 
 const HORIZONTAL_PADDING = 0;
 const VERTICAL_PADDING = 0;
@@ -34,7 +32,8 @@ const ENABLE_OVERFLOW = false;
 
 const BORDER_WIDTH = 1;
 const WIDGET_SEPARATOR_WIDTH = 1;
-const BORDER_RADIUS = 4;
+const WIDGET_SEPARATOR_DIFF_EDITOR_WIDTH = 3;
+const BORDER_RADIUS = INLINE_EDITS_BORDER_RADIUS;
 const ORIGINAL_END_PADDING = 20;
 const MODIFIED_END_PADDING = 12;
 
@@ -58,8 +57,8 @@ export class InlineEditsSideBySideView extends Disposable implements IInlineEdit
 
 	private readonly _editorObs;
 
-	private readonly _onDidClick;
-	readonly onDidClick;
+	private readonly _onDidClick = this._register(new Emitter<InlineEditClickEvent>());
+	readonly onDidClick = this._onDidClick.event;
 
 	constructor(
 		private readonly _editor: ICodeEditor,
@@ -67,6 +66,7 @@ export class InlineEditsSideBySideView extends Disposable implements IInlineEdit
 		private readonly _previewTextModel: ITextModel,
 		private readonly _uiState: IObservable<{
 			newTextLineCount: number;
+			editorType: InlineCompletionEditorType;
 		} | undefined>,
 		private readonly _tabAction: IObservable<InlineEditTabAction>,
 		@IInstantiationService private readonly _instantiationService: IInstantiationService,
@@ -74,10 +74,9 @@ export class InlineEditsSideBySideView extends Disposable implements IInlineEdit
 	) {
 		super();
 		this._editorObs = observableCodeEditor(this._editor);
-		this._onDidClick = this._register(new Emitter<IMouseEvent>());
-		this.onDidClick = this._onDidClick.event;
 		this._display = derived(this, reader => !!this._uiState.read(reader) ? 'block' : 'none');
 		this.previewRef = n.ref<HTMLDivElement>();
+		const separatorWidthObs = this._uiState.map(s => s?.editorType === InlineCompletionEditorType.DiffEditor ? WIDGET_SEPARATOR_DIFF_EDITOR_WIDTH : WIDGET_SEPARATOR_WIDTH);
 		this._editorContainer = n.div({
 			class: ['editorContainer'],
 			style: { position: 'absolute', overflow: 'hidden', cursor: 'pointer' },
@@ -85,7 +84,7 @@ export class InlineEditsSideBySideView extends Disposable implements IInlineEdit
 				e.preventDefault(); // This prevents that the editor loses focus
 			},
 			onclick: (e) => {
-				this._onDidClick.fire(new StandardMouseEvent(getWindow(e), e));
+				this._onDidClick.fire(InlineEditClickEvent.create(e));
 			}
 		}, [
 			n.div({ class: 'preview', style: { pointerEvents: 'none' }, ref: this.previewRef }),
@@ -104,6 +103,7 @@ export class InlineEditsSideBySideView extends Disposable implements IInlineEdit
 					bracketPairsHorizontal: false,
 					highlightActiveIndentation: false,
 				},
+				editContext: false, // is a bit faster
 				rulers: [],
 				padding: { top: 0, bottom: 0 },
 				folding: false,
@@ -137,7 +137,7 @@ export class InlineEditsSideBySideView extends Disposable implements IInlineEdit
 		));
 		this._previewEditorObs = observableCodeEditor(this.previewEditor);
 		this._activeViewZones = [];
-		this._updatePreviewEditor = derived(reader => {
+		this._updatePreviewEditor = derived(this, reader => {
 			this._editorContainer.readEffect(reader);
 			this._previewEditorObs.model.read(reader); // update when the model is set
 
@@ -224,6 +224,9 @@ export class InlineEditsSideBySideView extends Disposable implements IInlineEdit
 				return Math.max(maxWidth, lastValue ?? 0);
 			});
 		}).map((v, r) => v.read(r));
+
+		const editorDomContentRect = observeEditorBoundingClientRect(this._editor, this._store);
+
 		this._previewEditorLayoutInfo = derived(this, (reader) => {
 			const inlineEdit = this._edit.read(reader);
 			if (!inlineEdit) {
@@ -242,7 +245,7 @@ export class InlineEditsSideBySideView extends Disposable implements IInlineEdit
 			const editorLayout = this._editorObs.layoutInfo.read(reader);
 			const previewContentWidth = this._previewEditorWidth.read(reader);
 			const editorContentAreaWidth = editorLayout.contentWidth - editorLayout.verticalScrollbarWidth;
-			const editorBoundingClientRect = this._editor.getContainerDomNode().getBoundingClientRect();
+			const editorBoundingClientRect = editorDomContentRect.read(reader);
 			const clientContentAreaRight = editorLayout.contentLeft + editorLayout.contentWidth + editorBoundingClientRect.left;
 			const remainingWidthRightOfContent = getWindow(this._editor.getContainerDomNode()).innerWidth - clientContentAreaRight;
 			const remainingWidthRightOfEditor = getWindow(this._editor.getContainerDomNode()).innerWidth - editorBoundingClientRect.right;
@@ -289,7 +292,8 @@ export class InlineEditsSideBySideView extends Disposable implements IInlineEdit
 				codeRect = codeRect.withMargin(VERTICAL_PADDING, HORIZONTAL_PADDING);
 			}
 
-			const editHeight = this._editor.getOption(EditorOption.lineHeight) * inlineEdit.modifiedLineRange.length;
+			const previewLineHeights = this._previewEditorObs.observeLineHeightsForLineRange(inlineEdit.modifiedLineRange).read(reader);
+			const editHeight = previewLineHeights.reduce((acc, h) => acc + h, 0);
 			const codeHeight = selectionBottom - selectionTop;
 			const previewEditorHeight = Math.max(codeHeight, editHeight);
 
@@ -322,7 +326,7 @@ export class InlineEditsSideBySideView extends Disposable implements IInlineEdit
 		});
 		this._stickyScrollController = StickyScrollController.get(this._editorObs.editor);
 		this._stickyScrollHeight = this._stickyScrollController ? observableFromEvent(this._stickyScrollController.onDidChangeStickyScrollHeight, () => this._stickyScrollController!.stickyScrollWidgetHeight) : constObservable(0);
-		this._shouldOverflow = derived(reader => {
+		this._shouldOverflow = derived(this, reader => {
 			if (!ENABLE_OVERFLOW) {
 				return false;
 			}
@@ -344,13 +348,16 @@ export class InlineEditsSideBySideView extends Disposable implements IInlineEdit
 		this._originalBackgroundColor = observableFromEvent(this, this._themeService.onDidColorThemeChange, () => {
 			return this._themeService.getColorTheme().getColor(originalBackgroundColor) ?? Color.transparent;
 		});
+		this._editorBackgroundColor = this._uiState.map(s => {
+			return getEditorBackgroundColor(s?.editorType ?? InlineCompletionEditorType.TextEditor);
+		});
 		this._backgroundSvg = n.svg({
 			transform: 'translate(-0.5 -0.5)',
 			style: { overflow: 'visible', pointerEvents: 'none', position: 'absolute' },
 		}, [
 			n.svgElem('path', {
 				class: 'rightOfModifiedBackgroundCoverUp',
-				d: derived(reader => {
+				d: derived(this, reader => {
 					const layoutInfo = this._previewEditorLayoutInfo.read(reader);
 					if (!layoutInfo) {
 						return undefined;
@@ -368,18 +375,21 @@ export class InlineEditsSideBySideView extends Disposable implements IInlineEdit
 						.build();
 				}),
 				style: {
-					fill: asCssVariableWithDefault(editorBackground, 'transparent'),
+					fill: this._editorBackgroundColor,
 				}
 			}),
 		]).keepUpdated(this._store);
 		this._originalOverlay = n.div({
 			style: { pointerEvents: 'none', display: this._previewEditorLayoutInfo.map(layoutInfo => layoutInfo?.isInsertion ? 'none' : 'block') },
-		}, derived(reader => {
+		}, derived(this, reader => {
 			const layoutInfoObs = mapOutFalsy(this._previewEditorLayoutInfo).read(reader);
 			if (!layoutInfoObs) { return undefined; }
 
+			const editorBackground = this._editorBackgroundColor.read(reader);
+
+			const separatorWidth = separatorWidthObs.read(reader);
 			const borderStyling = getOriginalBorderColor(this._tabAction).map(bc => `${BORDER_WIDTH}px solid ${asCssVariable(bc)}`);
-			const borderStylingSeparator = `${BORDER_WIDTH + WIDGET_SEPARATOR_WIDTH}px solid ${asCssVariable(editorBackground)}`;
+			const borderStylingSeparator = `${BORDER_WIDTH + separatorWidth}px solid ${editorBackground}`;
 
 			const hasBorderLeft = layoutInfoObs.read(reader).codeScrollLeft !== 0;
 			const isModifiedLower = layoutInfoObs.map(layoutInfo => layoutInfo.codeRect.bottom < layoutInfo.editRect.bottom);
@@ -396,7 +406,7 @@ export class InlineEditsSideBySideView extends Disposable implements IInlineEdit
 
 			const intersectionLine = new OffsetRange(overlayHider.left, Number.MAX_SAFE_INTEGER);
 			const overlayRect = layoutInfoObs.map(layoutInfo => layoutInfo.codeRect.intersectHorizontal(intersectionLine));
-			const separatorRect = overlayRect.map(overlayRect => overlayRect.withMargin(WIDGET_SEPARATOR_WIDTH, 0, WIDGET_SEPARATOR_WIDTH, WIDGET_SEPARATOR_WIDTH).intersectHorizontal(intersectionLine));
+			const separatorRect = overlayRect.map(overlayRect => overlayRect.withMargin(separatorWidth, 0, separatorWidth, separatorWidth).intersectHorizontal(intersectionLine));
 
 			const transitionRect = overlayRect.map(overlayRect => Rect.fromLeftTopWidthHeight(overlayRect.right - transitionRectSize + BORDER_WIDTH, overlayRect.bottom - BORDER_WIDTH, transitionRectSize, transitionRectSize).intersectHorizontal(intersectionLine));
 
@@ -449,7 +459,7 @@ export class InlineEditsSideBySideView extends Disposable implements IInlineEdit
 							borderTop: borderStyling,
 							borderRight: borderStyling,
 							borderRadius: `0 100% 0 0`,
-							backgroundColor: asCssVariable(editorBackground)
+							backgroundColor: editorBackground
 						}
 					})
 				]),
@@ -457,27 +467,29 @@ export class InlineEditsSideBySideView extends Disposable implements IInlineEdit
 					class: 'originalOverlaySideBySideHider',
 					style: {
 						...overlayHider.toStyles(),
-						backgroundColor: asCssVariable(editorBackground),
+						backgroundColor: editorBackground,
 					}
 				}),
 			];
 		})).keepUpdated(this._store);
 		this._modifiedOverlay = n.div({
 			style: { pointerEvents: 'none', }
-		}, derived(reader => {
+		}, derived(this, reader => {
 			const layoutInfoObs = mapOutFalsy(this._previewEditorLayoutInfo).read(reader);
 			if (!layoutInfoObs) { return undefined; }
 
 			const isModifiedLower = layoutInfoObs.map(layoutInfo => layoutInfo.codeRect.bottom < layoutInfo.editRect.bottom);
+			const editorBackground = this._editorBackgroundColor.read(reader);
 
+			const separatorWidth = separatorWidthObs.read(reader);
 			const borderRadius = isModifiedLower.map(isLower => `0 ${BORDER_RADIUS}px ${BORDER_RADIUS}px ${isLower ? BORDER_RADIUS : 0}px`);
 			const borderStyling = getEditorBlendedColor(getModifiedBorderColor(this._tabAction), this._themeService).map(c => `1px solid ${c.toString()}`);
-			const borderStylingSeparator = `${BORDER_WIDTH + WIDGET_SEPARATOR_WIDTH}px solid ${asCssVariable(editorBackground)}`;
+			const borderStylingSeparator = `${BORDER_WIDTH + separatorWidth}px solid ${editorBackground}`;
 
 			const overlayRect = layoutInfoObs.map(layoutInfo => layoutInfo.editRect.withMargin(0, BORDER_WIDTH));
-			const separatorRect = overlayRect.map(overlayRect => overlayRect.withMargin(WIDGET_SEPARATOR_WIDTH, WIDGET_SEPARATOR_WIDTH, WIDGET_SEPARATOR_WIDTH, 0));
+			const separatorRect = overlayRect.map(overlayRect => overlayRect.withMargin(separatorWidth, separatorWidth, separatorWidth, 0));
 
-			const insertionRect = derived(reader => {
+			const insertionRect = derived(this, reader => {
 				const overlay = overlayRect.read(reader);
 				const layoutinfo = layoutInfoObs.read(reader);
 				if (!layoutinfo.isInsertion || layoutinfo.contentLeft >= overlay.left) {
@@ -535,7 +547,7 @@ export class InlineEditsSideBySideView extends Disposable implements IInlineEdit
 			domNode: this._nonOverflowView.element,
 			position: constObservable(null),
 			allowEditorOverflow: false,
-			minContentWidthInPx: derived(reader => {
+			minContentWidthInPx: derived(this, reader => {
 				const x = this._previewEditorLayoutInfo.read(reader)?.maxContentWidth;
 				if (x === undefined) { return 0; }
 				return x;
@@ -608,6 +620,8 @@ export class InlineEditsSideBySideView extends Disposable implements IInlineEdit
 
 	private readonly _originalBackgroundColor;
 
+	private readonly _editorBackgroundColor;
+
 	private readonly _backgroundSvg;
 
 	private readonly _originalOverlay;
