@@ -3,8 +3,8 @@
  *  Licensed under the MIT License. See License.txt in the project root for license information.
  *--------------------------------------------------------------------------------------------*/
 
-import { Disposable, DisposableStore, toDisposable } from '../../../../base/common/lifecycle.js';
-import { IActiveCodeEditor, ICodeEditor, MouseTargetType } from '../../../browser/editorBrowser.js';
+import { IDisposable, Disposable, DisposableStore, toDisposable } from '../../../../base/common/lifecycle.js';
+import { ICodeEditor, MouseTargetType } from '../../../browser/editorBrowser.js';
 import { IEditorContribution, ScrollType } from '../../../common/editorCommon.js';
 import { ILanguageFeaturesService } from '../../../common/services/languageFeatures.js';
 import { EditorOption, RenderLineNumbersType, ConfigurationChangedEvent } from '../../../common/config/editorOptions.js';
@@ -44,7 +44,7 @@ export interface IStickyScrollController {
 	findScrollWidgetState(): StickyScrollWidgetState;
 	dispose(): void;
 	selectEditor(): void;
-	onDidChangeStickyScrollHeight: Event<{ height: number }>;
+	readonly onDidChangeStickyScrollHeight: Event<{ height: number }>;
 }
 
 export class StickyScrollController extends Disposable implements IEditorContribution, IStickyScrollController {
@@ -75,6 +75,7 @@ export class StickyScrollController extends Disposable implements IEditorContrib
 	private _showEndForLine: number | undefined;
 	private _minRebuildFromLine: number | undefined;
 	private _mouseTarget: EventTarget | null = null;
+	private _cursorPositionListener: IDisposable | undefined;
 
 	private readonly _onDidChangeStickyScrollHeight = this._register(new Emitter<{ height: number }>());
 	public readonly onDidChangeStickyScrollHeight = this._onDidChangeStickyScrollHeight.event;
@@ -104,6 +105,14 @@ export class StickyScrollController extends Disposable implements IEditorContrib
 				}
 			});
 		}));
+		this._register(this._editor.onDidChangeFont((e) => {
+			e.changes.forEach((change) => {
+				const lineNumber = change.lineNumber;
+				if (this._widgetState.startLineNumbers.includes(lineNumber)) {
+					this._renderStickyScroll(lineNumber);
+				}
+			});
+		}));
 		this._register(this._editor.onDidChangeConfiguration(e => {
 			this._readConfigurationChange(e);
 		}));
@@ -310,26 +319,18 @@ export class StickyScrollController extends Disposable implements IEditorContrib
 			}
 			this._revealPosition(position);
 		}));
-		const mouseMoveListener = (mouseEvent: MouseEvent) => {
+		this._register(dom.addDisposableListener(mainWindow, dom.EventType.MOUSE_MOVE, mouseEvent => {
 			this._mouseTarget = mouseEvent.target;
 			this._onMouseMoveOrKeyDown(mouseEvent);
-		};
-		const keyDownListener = (mouseEvent: KeyboardEvent) => {
+		}));
+		this._register(dom.addDisposableListener(mainWindow, dom.EventType.KEY_DOWN, mouseEvent => {
 			this._onMouseMoveOrKeyDown(mouseEvent);
-		};
-		const keyUpListener = (e: KeyboardEvent) => {
+		}));
+		this._register(dom.addDisposableListener(mainWindow, dom.EventType.KEY_UP, () => {
 			if (this._showEndForLine !== undefined) {
 				this._showEndForLine = undefined;
 				this._renderStickyScroll();
 			}
-		};
-		mainWindow.addEventListener(dom.EventType.MOUSE_MOVE, mouseMoveListener);
-		mainWindow.addEventListener(dom.EventType.KEY_DOWN, keyDownListener);
-		mainWindow.addEventListener(dom.EventType.KEY_UP, keyUpListener);
-		this._register(toDisposable(() => {
-			mainWindow.removeEventListener(dom.EventType.MOUSE_MOVE, mouseMoveListener);
-			mainWindow.removeEventListener(dom.EventType.KEY_DOWN, keyDownListener);
-			mainWindow.removeEventListener(dom.EventType.KEY_UP, keyUpListener);
 		}));
 
 		this._register(gesture.onMouseMoveOrRelevantKeyDown(([mouseEvent, _keyboardEvent]) => {
@@ -400,7 +401,7 @@ export class StickyScrollController extends Disposable implements IEditorContrib
 				}
 				this._revealPosition({ lineNumber: position.lineNumber, column: 1 });
 			}
-			this._instaService.invokeFunction(goToDefinitionWithLocation, e, this._editor as IActiveCodeEditor, { uri: this._editor.getModel().uri, range: this._stickyRangeProjectedOnEditor });
+			this._instaService.invokeFunction(goToDefinitionWithLocation, e, this._editor, { uri: this._editor.getModel().uri, range: this._stickyRangeProjectedOnEditor });
 		}));
 	}
 
@@ -410,6 +411,7 @@ export class StickyScrollController extends Disposable implements IEditorContrib
 		this._contextMenuService.showContextMenu({
 			menuId: MenuId.StickyScrollContext,
 			getAnchor: () => event,
+			menuActionOptions: { renderShortTitle: true },
 		});
 	}
 
@@ -475,10 +477,17 @@ export class StickyScrollController extends Disposable implements IEditorContrib
 
 		const lineNumberOption = this._editor.getOption(EditorOption.lineNumbers);
 		if (lineNumberOption.renderType === RenderLineNumbersType.Relative) {
-			this._sessionStore.add(this._editor.onDidChangeCursorPosition(() => {
-				this._showEndForLine = undefined;
-				this._renderStickyScroll(0);
-			}));
+			if (!this._cursorPositionListener) {
+				this._cursorPositionListener = this._editor.onDidChangeCursorPosition(() => {
+					this._showEndForLine = undefined;
+					this._renderStickyScroll(0);
+				});
+				this._sessionStore.add(this._cursorPositionListener);
+			}
+		} else if (this._cursorPositionListener) {
+			this._sessionStore.delete(this._cursorPositionListener);
+			this._cursorPositionListener.dispose();
+			this._cursorPositionListener = undefined;
 		}
 	}
 
@@ -592,10 +601,6 @@ export class StickyScrollController extends Disposable implements IEditorContrib
 	}
 
 	findScrollWidgetState(): StickyScrollWidgetState {
-		if (!this._editor.hasModel()) {
-			return StickyScrollWidgetState.Empty;
-		}
-		const textModel = this._editor.getModel();
 		const maxNumberStickyLines = Math.min(this._maxStickyLines, this._editor.getOption(EditorOption.stickyScroll).maxLineCount);
 		const scrollTop: number = this._editor.getScrollTop();
 		let lastLineRelativePosition: number = 0;
@@ -608,23 +613,20 @@ export class StickyScrollController extends Disposable implements IEditorContrib
 			for (const range of candidateRanges) {
 				const start = range.startLineNumber;
 				const end = range.endLineNumber;
-				const isValidRange = textModel.isValidRange({ startLineNumber: start, endLineNumber: end, startColumn: 1, endColumn: 1 });
-				if (isValidRange && end - start > 0) {
-					const topOfElement = range.top;
-					const bottomOfElement = topOfElement + range.height;
-					const topOfBeginningLine = this._editor.getTopForLineNumber(start) - scrollTop;
-					const bottomOfEndLine = this._editor.getBottomForLineNumber(end) - scrollTop;
-					if (topOfElement > topOfBeginningLine && topOfElement <= bottomOfEndLine) {
-						startLineNumbers.push(start);
-						endLineNumbers.push(end + 1);
-						if (bottomOfElement > bottomOfEndLine) {
-							lastLineRelativePosition = bottomOfEndLine - bottomOfElement;
-						}
-					}
-					if (startLineNumbers.length === maxNumberStickyLines) {
-						break;
+				const topOfElement = range.top;
+				const bottomOfElement = topOfElement + range.height;
+				const topOfBeginningLine = this._editor.getTopForLineNumber(start) - scrollTop;
+				const bottomOfEndLine = this._editor.getBottomForLineNumber(end) - scrollTop;
+				if (topOfElement > topOfBeginningLine && topOfElement <= bottomOfEndLine) {
+					startLineNumbers.push(start);
+					endLineNumbers.push(end + 1);
+					if (bottomOfElement > bottomOfEndLine) {
+						lastLineRelativePosition = bottomOfEndLine - bottomOfElement;
 					}
 				}
+				if (startLineNumbers.length === maxNumberStickyLines) {
+					break;
+				}
 			}
 		}
 		this._endLineNumbers = endLineNumbers;
