@@ -20,6 +20,7 @@ import { CharacterMapping, RenderLineInput, renderViewLine } from '../../../comm
 import { foldingCollapsedIcon, foldingExpandedIcon } from '../../folding/browser/foldingDecorations.js';
 import { FoldingModel } from '../../folding/browser/foldingModel.js';
 import { Emitter } from '../../../../base/common/event.js';
+import { IViewModel } from '../../../common/viewModel.js';
 
 export class StickyScrollWidgetState {
 	constructor(
@@ -50,7 +51,7 @@ const STICKY_IS_FOLDING_ICON_ATTR = 'data-sticky-is-folding-icon';
 
 export class StickyScrollWidget extends Disposable implements IOverlayWidget {
 
-	private readonly _foldingIconStore = new DisposableStore();
+	private readonly _foldingIconStore = this._register(new DisposableStore());
 	private readonly _rootDomNode: HTMLElement = document.createElement('div');
 	private readonly _lineNumbersDomNode: HTMLElement = document.createElement('div');
 	private readonly _linesDomNodeScrollable: HTMLElement = document.createElement('div');
@@ -58,8 +59,7 @@ export class StickyScrollWidget extends Disposable implements IOverlayWidget {
 
 	private readonly _editor: ICodeEditor;
 
-	private _previousState: StickyScrollWidgetState | undefined;
-	private _lineHeight: number;
+	private _state: StickyScrollWidgetState | undefined;
 	private _renderedStickyLines: RenderedStickyLine[] = [];
 	private _lineNumbers: number[] = [];
 	private _lastLineRelativePosition: number = 0;
@@ -78,7 +78,6 @@ export class StickyScrollWidget extends Disposable implements IOverlayWidget {
 		super();
 
 		this._editor = editor;
-		this._lineHeight = editor.getOption(EditorOption.lineHeight);
 		this._lineNumbersDomNode.className = 'sticky-widget-line-numbers';
 		this._lineNumbersDomNode.setAttribute('role', 'none');
 
@@ -101,9 +100,6 @@ export class StickyScrollWidget extends Disposable implements IOverlayWidget {
 			if (e.hasChanged(EditorOption.stickyScroll)) {
 				updateScrollLeftPosition();
 			}
-			if (e.hasChanged(EditorOption.lineHeight)) {
-				this._lineHeight = this._editor.getOption(EditorOption.lineHeight);
-			}
 		}));
 		this._register(this._editor.onDidScrollChange((e) => {
 			if (e.scrollLeftChanged) {
@@ -117,7 +113,6 @@ export class StickyScrollWidget extends Disposable implements IOverlayWidget {
 			updateScrollLeftPosition();
 			this._updateWidgetWidth();
 		}));
-		this._register(this._foldingIconStore);
 		updateScrollLeftPosition();
 
 		this._register(this._editor.onDidLayoutChange((e) => {
@@ -142,49 +137,48 @@ export class StickyScrollWidget extends Disposable implements IOverlayWidget {
 		return this._lineNumbers;
 	}
 
-	setState(_state: StickyScrollWidgetState | undefined, foldingModel: FoldingModel | undefined, _rebuildFromLine?: number): void {
-		if (_rebuildFromLine === undefined &&
-			((!this._previousState && !_state) || (this._previousState && this._previousState.equals(_state)))
-		) {
+	setState(state: StickyScrollWidgetState | undefined, foldingModel: FoldingModel | undefined, rebuildFromIndexCandidate?: number): void {
+		const currentStateAndPreviousStateUndefined = !this._state && !state;
+		const currentStateDefinedAndEqualsPreviousState = this._state && this._state.equals(state);
+		if (rebuildFromIndexCandidate === undefined && (currentStateAndPreviousStateUndefined || currentStateDefinedAndEqualsPreviousState)) {
 			return;
 		}
-		const isWidgetHeightZero = this._isWidgetHeightZero(_state);
-		const state = isWidgetHeightZero ? undefined : _state;
-		const rebuildFromLine = isWidgetHeightZero ? 0 : this._findLineToRebuildWidgetFrom(_state, _rebuildFromLine);
-
-		this._renderRootNode(state, foldingModel, rebuildFromLine);
-		this._previousState = _state;
+		const data = this._findRenderingData(state);
+		const previousLineNumbers = this._lineNumbers;
+		this._lineNumbers = data.lineNumbers;
+		this._lastLineRelativePosition = data.lastLineRelativePosition;
+		const rebuildFromIndex = this._findIndexToRebuildFrom(previousLineNumbers, this._lineNumbers, rebuildFromIndexCandidate);
+		this._renderRootNode(this._lineNumbers, this._lastLineRelativePosition, foldingModel, rebuildFromIndex);
+		this._state = state;
 	}
 
-	private _isWidgetHeightZero(state: StickyScrollWidgetState | undefined): boolean {
+	private _findRenderingData(state: StickyScrollWidgetState | undefined): { lineNumbers: number[]; lastLineRelativePosition: number } {
 		if (!state) {
-			return true;
+			return { lineNumbers: [], lastLineRelativePosition: 0 };
 		}
-		const futureWidgetHeight = this._getHeightOfLines(state.startLineNumbers, state.lastLineRelativePosition);
-		if (futureWidgetHeight > 0) {
-			this._lastLineRelativePosition = state.lastLineRelativePosition;
-			const lineNumbers = [...state.startLineNumbers];
-			if (state.showEndForLine !== null) {
-				lineNumbers[state.showEndForLine] = state.endLineNumbers[state.showEndForLine];
-			}
-			this._lineNumbers = lineNumbers;
-		} else {
-			this._lastLineRelativePosition = 0;
-			this._lineNumbers = [];
+		const candidateLineNumbers = [...state.startLineNumbers];
+		if (state.showEndForLine !== null) {
+			candidateLineNumbers[state.showEndForLine] = state.endLineNumbers[state.showEndForLine];
 		}
-		return futureWidgetHeight === 0;
+		let totalHeight = 0;
+		for (let i = 0; i < candidateLineNumbers.length; i++) {
+			totalHeight += this._editor.getLineHeightForPosition(new Position(candidateLineNumbers[i], 1));
+		}
+		if (totalHeight === 0) {
+			return { lineNumbers: [], lastLineRelativePosition: 0 };
+		}
+		return { lineNumbers: candidateLineNumbers, lastLineRelativePosition: state.lastLineRelativePosition };
 	}
 
-	private _findLineToRebuildWidgetFrom(state: StickyScrollWidgetState | undefined, _rebuildFromLine?: number): number {
-		if (!state || !this._previousState) {
+	private _findIndexToRebuildFrom(previousLineNumbers: number[], newLineNumbers: number[], rebuildFromIndexCandidate?: number): number {
+		if (newLineNumbers.length === 0) {
 			return 0;
 		}
-		if (_rebuildFromLine !== undefined) {
-			return _rebuildFromLine;
+		if (rebuildFromIndexCandidate !== undefined) {
+			return rebuildFromIndexCandidate;
 		}
-		const previousState = this._previousState;
-		const indexOfLinesAlreadyRendered = state.startLineNumbers.findIndex(startLineNumber => !previousState.startLineNumbers.includes(startLineNumber));
-		return (indexOfLinesAlreadyRendered === -1) ? 0 : indexOfLinesAlreadyRendered;
+		const validIndex = newLineNumbers.findIndex(startLineNumber => !previousLineNumbers.includes(startLineNumber));
+		return validIndex === -1 ? 0 : validIndex;
 	}
 
 	private _updateWidgetWidth(): void {
@@ -195,18 +189,6 @@ export class StickyScrollWidget extends Disposable implements IOverlayWidget {
 		this._rootDomNode.style.width = `${layoutInfo.width - layoutInfo.verticalScrollbarWidth}px`;
 	}
 
-	private _clearStickyLinesFromLine(clearFromLine: number) {
-		this._foldingIconStore.clear();
-		// Removing only the lines that need to be rerendered
-		for (let i = clearFromLine; i < this._renderedStickyLines.length; i++) {
-			const stickyLine = this._renderedStickyLines[i];
-			stickyLine.lineNumberDomNode.remove();
-			stickyLine.lineDomNode.remove();
-		}
-		// Keep the lines that need to be updated
-		this._renderedStickyLines = this._renderedStickyLines.slice(0, clearFromLine);
-	}
-
 	private _useFoldingOpacityTransition(requireTransitions: boolean) {
 		this._lineNumbersDomNode.style.setProperty('--vscode-editorStickyScroll-foldingOpacityTransition', `opacity ${requireTransitions ? 0.5 : 0}s`);
 	}
@@ -221,51 +203,55 @@ export class StickyScrollWidget extends Disposable implements IOverlayWidget {
 		}
 	}
 
-	private async _renderRootNode(state: StickyScrollWidgetState | undefined, foldingModel: FoldingModel | undefined, rebuildFromLine: number): Promise<void> {
-		this._clearStickyLinesFromLine(rebuildFromLine);
-		if (!state) {
-			// make sure the dom is 0 height and display:none
-			this._setHeight(0);
+	private async _renderRootNode(lineNumbers: number[], lastLineRelativePosition: number, foldingModel: FoldingModel | undefined, rebuildFromIndex: number): Promise<void> {
+		const viewModel = this._editor._getViewModel();
+		if (!viewModel) {
+			this._clearWidget();
 			return;
 		}
-		let top: number = 0;
-		// For existing sticky lines update the top and z-index
-		for (const stickyLine of this._renderedStickyLines) {
-			this._updatePosition(stickyLine, top);
-			top += stickyLine.height;
+		if (lineNumbers.length === 0) {
+			this._clearWidget();
+			return;
 		}
-		// For new sticky lines
-		const layoutInfo = this._editor.getLayoutInfo();
-		const linesToRender = this._lineNumbers.slice(rebuildFromLine);
-		for (const [index, line] of linesToRender.entries()) {
-			const stickyLine = this._renderChildNode(index + rebuildFromLine, line, top, foldingModel, layoutInfo);
-			if (!stickyLine) {
-				continue;
+		const renderedStickyLines: RenderedStickyLine[] = [];
+		const lastLineNumber = lineNumbers[lineNumbers.length - 1];
+		let top: number = 0;
+		for (let i = 0; i < this._renderedStickyLines.length; i++) {
+			if (i < rebuildFromIndex) {
+				const renderedLine = this._renderedStickyLines[i];
+				renderedStickyLines.push(this._updatePosition(renderedLine, top, renderedLine.lineNumber === lastLineNumber));
+				top += renderedLine.height;
+			} else {
+				const renderedLine = this._renderedStickyLines[i];
+				renderedLine.lineNumberDomNode.remove();
+				renderedLine.lineDomNode.remove();
 			}
+		}
+		const layoutInfo = this._editor.getLayoutInfo();
+		for (let i = rebuildFromIndex; i < lineNumbers.length; i++) {
+			const stickyLine = this._renderChildNode(viewModel, i, lineNumbers[i], top, lastLineNumber === lineNumbers[i], foldingModel, layoutInfo);
 			top += stickyLine.height;
 			this._linesDomNode.appendChild(stickyLine.lineDomNode);
 			this._lineNumbersDomNode.appendChild(stickyLine.lineNumberDomNode);
-			this._renderedStickyLines.push(stickyLine);
+			renderedStickyLines.push(stickyLine);
 		}
 		if (foldingModel) {
 			this._setFoldingHoverListeners();
 			this._useFoldingOpacityTransition(!this._isOnGlyphMargin);
 		}
-
-		const widgetHeight = top + this._lastLineRelativePosition;
-		this._setHeight(widgetHeight);
-
-		this._rootDomNode.style.marginLeft = '0px';
 		this._minContentWidthInPx = Math.max(...this._renderedStickyLines.map(l => l.scrollWidth)) + layoutInfo.verticalScrollbarWidth;
+		this._renderedStickyLines = renderedStickyLines;
+		this._setHeight(top + lastLineRelativePosition);
 		this._editor.layoutOverlayWidget(this);
 	}
 
-	private _getHeightOfLines(lineNumbers: number[], lastLineRelativePosition: number): number {
-		let totalHeight = 0;
-		for (let i = 0; i < lineNumbers.length; i++) {
-			totalHeight += this._editor.getLineHeightForPosition(new Position(lineNumbers[i], 1));
+	private _clearWidget(): void {
+		for (let i = 0; i < this._renderedStickyLines.length; i++) {
+			const stickyLine = this._renderedStickyLines[i];
+			stickyLine.lineNumberDomNode.remove();
+			stickyLine.lineDomNode.remove();
 		}
-		return totalHeight + lastLineRelativePosition;
+		this._setHeight(0);
 	}
 
 	private _setHeight(height: number): void {
@@ -287,10 +273,12 @@ export class StickyScrollWidget extends Disposable implements IOverlayWidget {
 	}
 
 	private _setFoldingHoverListeners(): void {
+		this._foldingIconStore.clear();
 		const showFoldingControls: 'mouseover' | 'always' | 'never' = this._editor.getOption(EditorOption.showFoldingControls);
 		if (showFoldingControls !== 'mouseover') {
 			return;
 		}
+		this._foldingIconStore.clear();
 		this._foldingIconStore.add(dom.addDisposableListener(this._lineNumbersDomNode, dom.EventType.MOUSE_ENTER, () => {
 			this._isOnGlyphMargin = true;
 			this._setFoldingIconsVisibility(true);
@@ -302,95 +290,23 @@ export class StickyScrollWidget extends Disposable implements IOverlayWidget {
 		}));
 	}
 
-	private _renderChildNode(index: number, line: number, top: number, foldingModel: FoldingModel | undefined, layoutInfo: EditorLayoutInfo): RenderedStickyLine | undefined {
-		const viewModel = this._editor._getViewModel();
-		if (!viewModel) {
-			return;
-		}
-		const viewLineNumber = viewModel.coordinatesConverter.convertModelPositionToViewPosition(new Position(line, 1)).lineNumber;
-		const lineRenderingData = viewModel.getViewLineRenderingData(viewLineNumber);
-		const lineNumberOption = this._editor.getOption(EditorOption.lineNumbers);
+	private _renderChildNode(viewModel: IViewModel, index: number, line: number, top: number, isLastLine: boolean, foldingModel: FoldingModel | undefined, layoutInfo: EditorLayoutInfo): RenderedStickyLine {
 
-		let actualInlineDecorations: LineDecoration[];
-		try {
-			actualInlineDecorations = LineDecoration.filter(lineRenderingData.inlineDecorations, viewLineNumber, lineRenderingData.minColumn, lineRenderingData.maxColumn);
-		} catch (err) {
-			actualInlineDecorations = [];
-		}
-
-		const lineHeight = this._editor.getLineHeightForPosition(new Position(line, 1));
-		const renderLineInput: RenderLineInput = new RenderLineInput(true, true, lineRenderingData.content,
-			lineRenderingData.continuesWithWrappedLine,
-			lineRenderingData.isBasicASCII, lineRenderingData.containsRTL, 0,
-			lineRenderingData.tokens, actualInlineDecorations,
-			lineRenderingData.tabSize, lineRenderingData.startVisibleColumn,
-			1, 1, 1, 500, 'none', true, true, null
+		const renderedLine = new RenderedStickyLine(
+			this._editor,
+			viewModel,
+			layoutInfo,
+			foldingModel,
+			this._isOnGlyphMargin,
+			index,
+			line
 		);
-
-		const sb = new StringBuilder(2000);
-		const renderOutput = renderViewLine(renderLineInput, sb);
-
-		let newLine;
-		if (_ttPolicy) {
-			newLine = _ttPolicy.createHTML(sb.build());
-		} else {
-			newLine = sb.build();
-		}
-
-		const lineHTMLNode = document.createElement('span');
-		lineHTMLNode.setAttribute(STICKY_INDEX_ATTR, String(index));
-		lineHTMLNode.setAttribute(STICKY_IS_LINE_ATTR, '');
-		lineHTMLNode.setAttribute('role', 'listitem');
-		lineHTMLNode.tabIndex = 0;
-		lineHTMLNode.className = 'sticky-line-content';
-		lineHTMLNode.classList.add(`stickyLine${line}`);
-		lineHTMLNode.style.lineHeight = `${lineHeight}px`;
-		lineHTMLNode.innerHTML = newLine as string;
-
-		const lineNumberHTMLNode = document.createElement('span');
-		lineNumberHTMLNode.setAttribute(STICKY_INDEX_ATTR, String(index));
-		lineNumberHTMLNode.setAttribute(STICKY_IS_LINE_NUMBER_ATTR, '');
-		lineNumberHTMLNode.className = 'sticky-line-number';
-		lineNumberHTMLNode.style.lineHeight = `${lineHeight}px`;
-		const lineNumbersWidth = layoutInfo.contentLeft;
-		lineNumberHTMLNode.style.width = `${lineNumbersWidth}px`;
-
-		const innerLineNumberHTML = document.createElement('span');
-		if (lineNumberOption.renderType === RenderLineNumbersType.On || lineNumberOption.renderType === RenderLineNumbersType.Interval && line % 10 === 0) {
-			innerLineNumberHTML.innerText = line.toString();
-		} else if (lineNumberOption.renderType === RenderLineNumbersType.Relative) {
-			innerLineNumberHTML.innerText = Math.abs(line - this._editor.getPosition()!.lineNumber).toString();
-		}
-		innerLineNumberHTML.className = 'sticky-line-number-inner';
-		innerLineNumberHTML.style.width = `${layoutInfo.lineNumbersWidth}px`;
-		innerLineNumberHTML.style.paddingLeft = `${layoutInfo.lineNumbersLeft}px`;
-
-		lineNumberHTMLNode.appendChild(innerLineNumberHTML);
-		const foldingIcon = this._renderFoldingIconForLine(foldingModel, line);
-		if (foldingIcon) {
-			lineNumberHTMLNode.appendChild(foldingIcon.domNode);
-			foldingIcon.domNode.style.left = `${layoutInfo.lineNumbersWidth + layoutInfo.lineNumbersLeft}px`;
-			foldingIcon.domNode.style.lineHeight = `${lineHeight}px`;
-		}
-
-		this._editor.applyFontInfo(lineHTMLNode);
-		this._editor.applyFontInfo(lineNumberHTMLNode);
-
-
-		lineNumberHTMLNode.style.lineHeight = `${lineHeight}px`;
-		lineHTMLNode.style.lineHeight = `${lineHeight}px`;
-		lineNumberHTMLNode.style.height = `${lineHeight}px`;
-		lineHTMLNode.style.height = `${lineHeight}px`;
-
-		const renderedLine = new RenderedStickyLine(index, line, lineHTMLNode, lineNumberHTMLNode, foldingIcon, renderOutput.characterMapping, lineHTMLNode.scrollWidth, lineHeight);
-		return this._updatePosition(renderedLine, top);
+		return this._updatePosition(renderedLine, top, isLastLine);
 	}
 
-	private _updatePosition(stickyLine: RenderedStickyLine, top: number): RenderedStickyLine {
-		const index = stickyLine.index;
+	private _updatePosition(stickyLine: RenderedStickyLine, top: number, isLastLine: boolean): RenderedStickyLine {
 		const lineHTMLNode = stickyLine.lineDomNode;
 		const lineNumberHTMLNode = stickyLine.lineNumberDomNode;
-		const isLastLine = index === this._lineNumbers.length - 1;
 		if (isLastLine) {
 			const zIndex = '0';
 			lineHTMLNode.style.zIndex = zIndex;
@@ -408,25 +324,6 @@ export class StickyScrollWidget extends Disposable implements IOverlayWidget {
 		return stickyLine;
 	}
 
-	private _renderFoldingIconForLine(foldingModel: FoldingModel | undefined, line: number): StickyFoldingIcon | undefined {
-		const showFoldingControls: 'mouseover' | 'always' | 'never' = this._editor.getOption(EditorOption.showFoldingControls);
-		if (!foldingModel || showFoldingControls === 'never') {
-			return;
-		}
-		const foldingRegions = foldingModel.regions;
-		const indexOfFoldingRegion = foldingRegions.findRange(line);
-		const startLineNumber = foldingRegions.getStartLineNumber(indexOfFoldingRegion);
-		const isFoldingScope = line === startLineNumber;
-		if (!isFoldingScope) {
-			return;
-		}
-		const isCollapsed = foldingRegions.isCollapsed(indexOfFoldingRegion);
-		const foldingIcon = new StickyFoldingIcon(isCollapsed, startLineNumber, foldingRegions.getEndLineNumber(indexOfFoldingRegion), this._lineHeight);
-		foldingIcon.setVisible(this._isOnGlyphMargin ? true : (isCollapsed || showFoldingControls === 'always'));
-		foldingIcon.domNode.setAttribute(STICKY_IS_FOLDING_ICON_ATTR, '');
-		return foldingIcon;
-	}
-
 	getId(): string {
 		return 'editor.contrib.stickyScrollWidget';
 	}
@@ -438,7 +335,7 @@ export class StickyScrollWidget extends Disposable implements IOverlayWidget {
 	getPosition(): IOverlayWidgetPosition | null {
 		return {
 			preference: OverlayWidgetPositionPreference.TOP_CENTER,
-			stackOridinal: 10,
+			stackOrdinal: 10,
 		};
 	}
 
@@ -524,16 +421,127 @@ export class StickyScrollWidget extends Disposable implements IOverlayWidget {
 }
 
 class RenderedStickyLine {
+
+	public readonly lineDomNode: HTMLElement;
+	public readonly lineNumberDomNode: HTMLElement;
+
+	public readonly foldingIcon: StickyFoldingIcon | undefined;
+	public readonly characterMapping: CharacterMapping;
+
+	public readonly scrollWidth: number;
+	public readonly height: number;
+
 	constructor(
+		editor: ICodeEditor,
+		viewModel: IViewModel,
+		layoutInfo: EditorLayoutInfo,
+		foldingModel: FoldingModel | undefined,
+		isOnGlyphMargin: boolean,
 		public readonly index: number,
 		public readonly lineNumber: number,
-		public readonly lineDomNode: HTMLElement,
-		public readonly lineNumberDomNode: HTMLElement,
-		public readonly foldingIcon: StickyFoldingIcon | undefined,
-		public readonly characterMapping: CharacterMapping,
-		public readonly scrollWidth: number,
-		public readonly height: number
-	) { }
+	) {
+		const viewLineNumber = viewModel.coordinatesConverter.convertModelPositionToViewPosition(new Position(lineNumber, 1)).lineNumber;
+		const lineRenderingData = viewModel.getViewLineRenderingData(viewLineNumber);
+		const lineNumberOption = editor.getOption(EditorOption.lineNumbers);
+		const verticalScrollbarSize = editor.getOption(EditorOption.scrollbar).verticalScrollbarSize;
+
+		let actualInlineDecorations: LineDecoration[];
+		try {
+			actualInlineDecorations = LineDecoration.filter(lineRenderingData.inlineDecorations, viewLineNumber, lineRenderingData.minColumn, lineRenderingData.maxColumn);
+		} catch (err) {
+			actualInlineDecorations = [];
+		}
+
+		const lineHeight = editor.getLineHeightForPosition(new Position(lineNumber, 1));
+		const textDirection = viewModel.getTextDirection(lineNumber);
+		const renderLineInput: RenderLineInput = new RenderLineInput(true, true, lineRenderingData.content,
+			lineRenderingData.continuesWithWrappedLine,
+			lineRenderingData.isBasicASCII, lineRenderingData.containsRTL, 0,
+			lineRenderingData.tokens, actualInlineDecorations,
+			lineRenderingData.tabSize, lineRenderingData.startVisibleColumn,
+			1, 1, 1, 500, 'none', true, true, null,
+			textDirection, verticalScrollbarSize
+		);
+
+		const sb = new StringBuilder(2000);
+		const renderOutput = renderViewLine(renderLineInput, sb);
+		this.characterMapping = renderOutput.characterMapping;
+
+		let newLine;
+		if (_ttPolicy) {
+			newLine = _ttPolicy.createHTML(sb.build());
+		} else {
+			newLine = sb.build();
+		}
+
+		const lineHTMLNode = document.createElement('span');
+		lineHTMLNode.setAttribute(STICKY_INDEX_ATTR, String(index));
+		lineHTMLNode.setAttribute(STICKY_IS_LINE_ATTR, '');
+		lineHTMLNode.setAttribute('role', 'listitem');
+		lineHTMLNode.tabIndex = 0;
+		lineHTMLNode.className = 'sticky-line-content';
+		lineHTMLNode.classList.add(`stickyLine${lineNumber}`);
+		lineHTMLNode.style.lineHeight = `${lineHeight}px`;
+		lineHTMLNode.innerHTML = newLine as string;
+
+		const lineNumberHTMLNode = document.createElement('span');
+		lineNumberHTMLNode.setAttribute(STICKY_INDEX_ATTR, String(index));
+		lineNumberHTMLNode.setAttribute(STICKY_IS_LINE_NUMBER_ATTR, '');
+		lineNumberHTMLNode.className = 'sticky-line-number';
+		lineNumberHTMLNode.style.lineHeight = `${lineHeight}px`;
+		const lineNumbersWidth = layoutInfo.contentLeft;
+		lineNumberHTMLNode.style.width = `${lineNumbersWidth}px`;
+
+		const innerLineNumberHTML = document.createElement('span');
+		if (lineNumberOption.renderType === RenderLineNumbersType.On || lineNumberOption.renderType === RenderLineNumbersType.Interval && lineNumber % 10 === 0) {
+			innerLineNumberHTML.innerText = lineNumber.toString();
+		} else if (lineNumberOption.renderType === RenderLineNumbersType.Relative) {
+			innerLineNumberHTML.innerText = Math.abs(lineNumber - editor.getPosition()!.lineNumber).toString();
+		}
+		innerLineNumberHTML.className = 'sticky-line-number-inner';
+		innerLineNumberHTML.style.width = `${layoutInfo.lineNumbersWidth}px`;
+		innerLineNumberHTML.style.paddingLeft = `${layoutInfo.lineNumbersLeft}px`;
+
+		lineNumberHTMLNode.appendChild(innerLineNumberHTML);
+		const foldingIcon = this._renderFoldingIconForLine(editor, foldingModel, lineNumber, lineHeight, isOnGlyphMargin);
+		if (foldingIcon) {
+			lineNumberHTMLNode.appendChild(foldingIcon.domNode);
+			foldingIcon.domNode.style.left = `${layoutInfo.lineNumbersWidth + layoutInfo.lineNumbersLeft}px`;
+			foldingIcon.domNode.style.lineHeight = `${lineHeight}px`;
+		}
+
+		editor.applyFontInfo(lineHTMLNode);
+		editor.applyFontInfo(lineNumberHTMLNode);
+
+		lineNumberHTMLNode.style.lineHeight = `${lineHeight}px`;
+		lineHTMLNode.style.lineHeight = `${lineHeight}px`;
+		lineNumberHTMLNode.style.height = `${lineHeight}px`;
+		lineHTMLNode.style.height = `${lineHeight}px`;
+
+		this.scrollWidth = lineHTMLNode.scrollWidth;
+		this.lineDomNode = lineHTMLNode;
+		this.lineNumberDomNode = lineNumberHTMLNode;
+		this.height = lineHeight;
+	}
+
+	private _renderFoldingIconForLine(editor: ICodeEditor, foldingModel: FoldingModel | undefined, line: number, lineHeight: number, isOnGlyphMargin: boolean): StickyFoldingIcon | undefined {
+		const showFoldingControls: 'mouseover' | 'always' | 'never' = editor.getOption(EditorOption.showFoldingControls);
+		if (!foldingModel || showFoldingControls === 'never') {
+			return;
+		}
+		const foldingRegions = foldingModel.regions;
+		const indexOfFoldingRegion = foldingRegions.findRange(line);
+		const startLineNumber = foldingRegions.getStartLineNumber(indexOfFoldingRegion);
+		const isFoldingScope = line === startLineNumber;
+		if (!isFoldingScope) {
+			return;
+		}
+		const isCollapsed = foldingRegions.isCollapsed(indexOfFoldingRegion);
+		const foldingIcon = new StickyFoldingIcon(isCollapsed, startLineNumber, foldingRegions.getEndLineNumber(indexOfFoldingRegion), lineHeight);
+		foldingIcon.setVisible(isOnGlyphMargin ? true : (isCollapsed || showFoldingControls === 'always'));
+		foldingIcon.domNode.setAttribute(STICKY_IS_FOLDING_ICON_ATTR, '');
+		return foldingIcon;
+	}
 }
 
 class StickyFoldingIcon {
