@@ -5,7 +5,7 @@
 
 import * as dom from '../../../base/browser/dom.js';
 import { StandardKeyboardEvent } from '../../../base/browser/keyboardEvent.js';
-import { ActionBar } from '../../../base/browser/ui/actionbar/actionbar.js';
+import { ToolBar } from '../../../base/browser/ui/toolbar/toolbar.js';
 import { Button, IButtonStyles } from '../../../base/browser/ui/button/button.js';
 import { CountBadge, ICountBadgeStyles } from '../../../base/browser/ui/countBadge/countBadge.js';
 import { IHoverDelegate, IHoverDelegateOptions } from '../../../base/browser/ui/hover/hoverDelegate.js';
@@ -13,7 +13,7 @@ import { IInputBoxStyles } from '../../../base/browser/ui/inputbox/inputBox.js';
 import { IKeybindingLabelStyles } from '../../../base/browser/ui/keybindingLabel/keybindingLabel.js';
 import { IListStyles } from '../../../base/browser/ui/list/listWidget.js';
 import { IProgressBarStyles, ProgressBar } from '../../../base/browser/ui/progressbar/progressbar.js';
-import { Checkbox, IToggleStyles, Toggle } from '../../../base/browser/ui/toggle/toggle.js';
+import { IToggleStyles, TriStateCheckbox } from '../../../base/browser/ui/toggle/toggle.js';
 import { equals } from '../../../base/common/arrays.js';
 import { TimeoutTimer } from '../../../base/common/async.js';
 import { Codicon } from '../../../base/common/codicons.js';
@@ -25,14 +25,16 @@ import Severity from '../../../base/common/severity.js';
 import { ThemeIcon } from '../../../base/common/themables.js';
 import './media/quickInput.css';
 import { localize } from '../../../nls.js';
-import { IInputBox, IKeyMods, IQuickInput, IQuickInputButton, IQuickInputHideEvent, IQuickInputToggle, IQuickNavigateConfiguration, IQuickPick, IQuickPickDidAcceptEvent, IQuickPickItem, IQuickPickItemButtonEvent, IQuickPickSeparator, IQuickPickSeparatorButtonEvent, IQuickPickWillAcceptEvent, IQuickWidget, ItemActivation, NO_KEY_MODS, QuickInputButtonLocation, QuickInputHideReason, QuickInputType, QuickPickFocus } from '../common/quickInput.js';
+import { IInputBox, IKeyMods, IQuickInput, IQuickInputButton, IQuickInputHideEvent, IQuickNavigateConfiguration, IQuickPick, IQuickPickDidAcceptEvent, IQuickPickItem, IQuickPickItemButtonEvent, IQuickPickSeparator, IQuickPickSeparatorButtonEvent, IQuickPickWillAcceptEvent, IQuickWidget, ItemActivation, NO_KEY_MODS, QuickInputButtonLocation, QuickInputHideReason, QuickInputType, QuickPickFocus } from '../common/quickInput.js';
 import { QuickInputBox } from './quickInputBox.js';
-import { quickInputButtonToAction, renderQuickInputDescription } from './quickInputUtils.js';
+import { quickInputButtonToAction, quickInputButtonsToActionArrays, renderQuickInputDescription } from './quickInputUtils.js';
 import { IConfigurationService } from '../../configuration/common/configuration.js';
 import { IHoverService, WorkbenchHoverDelegate } from '../../hover/browser/hover.js';
-import { QuickInputTree } from './quickInputTree.js';
+import { QuickInputList } from './quickInputList.js';
 import type { IHoverOptions } from '../../../base/browser/ui/hover/hover.js';
 import { ContextKeyExpr, RawContextKey } from '../../contextkey/common/contextkey.js';
+import { QuickInputTreeController } from './tree/quickInputTreeController.js';
+import { observableValue } from '../../../base/common/observable.js';
 
 export const inQuickInputContextKeyValue = 'inQuickInput';
 export const InQuickInputContextKey = new RawContextKey<boolean>(inQuickInputContextKeyValue, false, localize('inQuickInput', "Whether keyboard focus is inside the quick input control"));
@@ -95,15 +97,15 @@ export const backButton = {
 export interface QuickInputUI {
 	container: HTMLElement;
 	styleSheet: HTMLStyleElement;
-	leftActionBar: ActionBar;
+	leftActionBar: ToolBar;
 	titleBar: HTMLElement;
 	title: HTMLElement;
 	description1: HTMLElement;
 	description2: HTMLElement;
 	widget: HTMLElement;
-	rightActionBar: ActionBar;
-	inlineActionBar: ActionBar;
-	checkAll: Checkbox;
+	rightActionBar: ToolBar;
+	inlineActionBar: ToolBar;
+	checkAll: TriStateCheckbox;
 	inputContainer: HTMLElement;
 	filterContainer: HTMLElement;
 	inputBox: QuickInputBox;
@@ -117,10 +119,11 @@ export interface QuickInputUI {
 	customButtonContainer: HTMLElement;
 	customButton: Button;
 	progressBar: ProgressBar;
-	list: QuickInputTree;
-	onDidAccept: Event<void>;
-	onDidCustom: Event<void>;
-	onDidTriggerButton: Event<IQuickInputButton>;
+	list: QuickInputList;
+	tree: QuickInputTreeController;
+	readonly onDidAccept: Event<void>;
+	readonly onDidCustom: Event<void>;
+	readonly onDidTriggerButton: Event<IQuickInputButton>;
 	ignoreFocusOut: boolean;
 	keyMods: Writeable<IKeyMods>;
 	show(controller: QuickInput): void;
@@ -141,21 +144,20 @@ export type Visibilities = {
 	count?: boolean;
 	message?: boolean;
 	list?: boolean;
+	tree?: boolean;
 	ok?: boolean;
 	customButton?: boolean;
 	progressBar?: boolean;
 };
 
-abstract class QuickInput extends Disposable implements IQuickInput {
+export abstract class QuickInput extends Disposable implements IQuickInput {
 	protected static readonly noPromptMessage = localize('inputModeEntry', "Press 'Enter' to confirm your input or 'Escape' to cancel");
 
+	protected _visible = observableValue('visible', false);
 	private _title: string | undefined;
 	private _description: string | undefined;
-	private _widget: HTMLElement | undefined;
-	private _widgetUpdated = false;
 	private _steps: number | undefined;
 	private _totalSteps: number | undefined;
-	protected visible = false;
 	private _enabled = true;
 	private _contextKey: string | undefined;
 	private _busy = false;
@@ -163,10 +165,9 @@ abstract class QuickInput extends Disposable implements IQuickInput {
 	private _leftButtons: IQuickInputButton[] = [];
 	private _rightButtons: IQuickInputButton[] = [];
 	private _inlineButtons: IQuickInputButton[] = [];
+	private _inputButtons: IQuickInputButton[] = [];
 	private buttonsUpdated = false;
-	private _toggles: IQuickInputToggle[] = [];
-	private togglesUpdated = false;
-	protected noValidationMessage = QuickInput.noPromptMessage;
+	protected noValidationMessage: string | undefined = QuickInput.noPromptMessage;
 	private _validationMessage: string | undefined;
 	private _lastValidationMessage: string | undefined;
 	private _severity: Severity = Severity.Ignore;
@@ -188,6 +189,10 @@ abstract class QuickInput extends Disposable implements IQuickInput {
 		super();
 	}
 
+	protected get visible(): boolean {
+		return this._visible.get();
+	}
+
 	get title() {
 		return this._title;
 	}
@@ -206,21 +211,6 @@ abstract class QuickInput extends Disposable implements IQuickInput {
 		this.update();
 	}
 
-	get widget() {
-		return this._widget;
-	}
-
-	set widget(widget: unknown | undefined) {
-		if (!(dom.isHTMLElement(widget))) {
-			return;
-		}
-		if (this._widget !== widget) {
-			this._widget = widget;
-			this._widgetUpdated = true;
-			this.update();
-		}
-	}
-
 	get step() {
 		return this._steps;
 	}
@@ -288,28 +278,43 @@ abstract class QuickInput extends Disposable implements IQuickInput {
 		return [
 			...this._leftButtons,
 			...this._rightButtons,
-			...this._inlineButtons
+			...this._inlineButtons,
+			...this._inputButtons
 		];
 	}
 
 	set buttons(buttons: IQuickInputButton[]) {
-		this._leftButtons = buttons.filter(b => b === backButton);
-		this._rightButtons = buttons.filter(b => b !== backButton && b.location !== QuickInputButtonLocation.Inline);
-		this._inlineButtons = buttons.filter(b => b.location === QuickInputButtonLocation.Inline);
+		const leftButtons: IQuickInputButton[] = [];
+		const rightButtons: IQuickInputButton[] = [];
+		const inlineButtons: IQuickInputButton[] = [];
+		const inputButtons: IQuickInputButton[] = [];
+
+		for (const button of buttons) {
+			if (button === backButton) {
+				leftButtons.push(button);
+			} else {
+				switch (button.location) {
+					case QuickInputButtonLocation.Inline:
+						inlineButtons.push(button);
+						break;
+					case QuickInputButtonLocation.Input:
+						inputButtons.push(button);
+						break;
+					default:
+						rightButtons.push(button);
+						break;
+				}
+			}
+		}
+
+		this._leftButtons = leftButtons;
+		this._rightButtons = rightButtons;
+		this._inlineButtons = inlineButtons;
+		this._inputButtons = inputButtons;
 		this.buttonsUpdated = true;
 		this.update();
 	}
 
-	get toggles() {
-		return this._toggles;
-	}
-
-	set toggles(toggles: IQuickInputToggle[]) {
-		this._toggles = toggles ?? [];
-		this.togglesUpdated = true;
-		this.update();
-	}
-
 	get validationMessage() {
 		return this._validationMessage;
 	}
@@ -344,7 +349,7 @@ abstract class QuickInput extends Disposable implements IQuickInput {
 		this.ui.show(this);
 
 		// update properties in the controller that get reset in the ui.show() call
-		this.visible = true;
+		this._visible.set(true, undefined);
 		// This ensures the message/prompt gets rendered
 		this._lastValidationMessage = undefined;
 		// This ensures the input box has the right severity applied
@@ -354,11 +359,6 @@ abstract class QuickInput extends Disposable implements IQuickInput {
 			// rerender them.
 			this.buttonsUpdated = true;
 		}
-		if (this.toggles.length) {
-			// if there are toggles, the ui.show() clears them out of the UI so we should
-			// rerender them.
-			this.togglesUpdated = true;
-		}
 
 		this.update();
 	}
@@ -371,7 +371,7 @@ abstract class QuickInput extends Disposable implements IQuickInput {
 	}
 
 	didHide(reason = QuickInputHideReason.Other): void {
-		this.visible = false;
+		this._visible.set(false, undefined);
 		this.visibleDisposables.clear();
 		this.onDidHideEmitter.fire({ reason });
 	}
@@ -400,61 +400,50 @@ abstract class QuickInput extends Disposable implements IQuickInput {
 		if (this.ui.description2.textContent !== description) {
 			this.ui.description2.textContent = description;
 		}
-		if (this._widgetUpdated) {
-			this._widgetUpdated = false;
-			if (this._widget) {
-				dom.reset(this.ui.widget, this._widget);
-			} else {
-				dom.reset(this.ui.widget);
-			}
-		}
 		if (this.busy && !this.busyDelay) {
 			this.busyDelay = new TimeoutTimer();
 			this.busyDelay.setIfNotSet(() => {
 				if (this.visible) {
 					this.ui.progressBar.infinite();
+					this.ui.progressBar.getContainer().removeAttribute('aria-hidden');
 				}
 			}, 800);
 		}
 		if (!this.busy && this.busyDelay) {
 			this.ui.progressBar.stop();
+			this.ui.progressBar.getContainer().setAttribute('aria-hidden', 'true');
 			this.busyDelay.cancel();
 			this.busyDelay = undefined;
 		}
 		if (this.buttonsUpdated) {
 			this.buttonsUpdated = false;
-			this.ui.leftActionBar.clear();
-			const leftButtons = this._leftButtons
+			const leftActions = quickInputButtonsToActionArrays(
+				this._leftButtons,
+				'left-button',
+				(button) => this.onDidTriggerButtonEmitter.fire(button)
+			);
+			this.ui.leftActionBar.setActions(leftActions.primary, leftActions.secondary);
+			const rightActions = quickInputButtonsToActionArrays(
+				this._rightButtons,
+				'right-button',
+				(button) => this.onDidTriggerButtonEmitter.fire(button)
+			);
+			this.ui.rightActionBar.setActions(rightActions.primary, rightActions.secondary);
+			const inlineActions = quickInputButtonsToActionArrays(
+				this._inlineButtons,
+				'inline-button',
+				(button) => this.onDidTriggerButtonEmitter.fire(button)
+			);
+			this.ui.inlineActionBar.setActions(inlineActions.primary, inlineActions.secondary);
+			// Adjust count badge position based on input buttons (each button/toggle is ~22px wide)
+			const inputButtonOffset = this._inputButtons.length * 22;
+			this.ui.countContainer.style.right = inputButtonOffset > 0 ? `${4 + inputButtonOffset}px` : '4px';
+			this.ui.inputBox.actions = this._inputButtons
 				.map((button, index) => quickInputButtonToAction(
 					button,
 					`id-${index}`,
 					async () => this.onDidTriggerButtonEmitter.fire(button)
 				));
-			this.ui.leftActionBar.push(leftButtons, { icon: true, label: false });
-			this.ui.rightActionBar.clear();
-			const rightButtons = this._rightButtons
-				.map((button, index) => quickInputButtonToAction(
-					button,
-					`id-${index}`,
-					async () => this.onDidTriggerButtonEmitter.fire(button)
-				));
-			this.ui.rightActionBar.push(rightButtons, { icon: true, label: false });
-			this.ui.inlineActionBar.clear();
-			const inlineButtons = this._inlineButtons
-				.map((button, index) => quickInputButtonToAction(
-					button,
-					`id-${index}`,
-					async () => this.onDidTriggerButtonEmitter.fire(button)
-				));
-			this.ui.inlineActionBar.push(inlineButtons, { icon: true, label: false });
-		}
-		if (this.togglesUpdated) {
-			this.togglesUpdated = false;
-			// HACK: Filter out toggles here that are not concrete Toggle objects. This is to workaround
-			// a layering issue as quick input's interface is in common but Toggle is in browser and
-			// it requires a HTMLElement on its interface
-			const concreteToggles = this.toggles?.filter(opts => opts instanceof Toggle) as Toggle[] ?? [];
-			this.ui.inputBox.toggles = concreteToggles;
 		}
 		this.ui.ignoreFocusOut = this.ignoreFocusOut;
 		this.ui.setEnabled(this.enabled);
@@ -464,12 +453,14 @@ abstract class QuickInput extends Disposable implements IQuickInput {
 		if (this._lastValidationMessage !== validationMessage) {
 			this._lastValidationMessage = validationMessage;
 			dom.reset(this.ui.message);
-			renderQuickInputDescription(validationMessage, this.ui.message, {
-				callback: (content) => {
-					this.ui.linkOpenerDelegate(content);
-				},
-				disposables: this.visibleDisposables,
-			});
+			if (validationMessage) {
+				renderQuickInputDescription(validationMessage, this.ui.message, {
+					callback: (content) => {
+						this.ui.linkOpenerDelegate(content);
+					},
+					disposables: this.visibleDisposables,
+				});
+			}
 		}
 		if (this._lastSeverity !== this.severity) {
 			this._lastSeverity = this.severity;
@@ -569,6 +560,7 @@ export class QuickPick<T extends IQuickPickItem, O extends { useSeparators: bool
 	private _customButton = false;
 	private _customButtonLabel: string | undefined;
 	private _customButtonHover: string | undefined;
+	private _customButtonSecondary = false;
 	private _quickNavigate: IQuickNavigateConfiguration | undefined;
 	private _hideInput: boolean | undefined;
 	private _hideCountBadge: boolean | undefined;
@@ -577,6 +569,11 @@ export class QuickPick<T extends IQuickPickItem, O extends { useSeparators: bool
 
 	readonly type = QuickInputType.QuickPick;
 
+	constructor(ui: QuickInputUI) {
+		super(ui);
+		this.noValidationMessage = undefined;
+	}
+
 	get quickNavigate() {
 		return this._quickNavigate;
 	}
@@ -630,6 +627,15 @@ export class QuickPick<T extends IQuickPickItem, O extends { useSeparators: bool
 		this.update();
 	}
 
+	get prompt() {
+		return this.noValidationMessage;
+	}
+
+	set prompt(prompt: string | undefined) {
+		this.noValidationMessage = prompt;
+		this.update();
+	}
+
 	onDidChangeValue = this.onDidChangeValueEmitter.event;
 
 	onWillAccept = this.onWillAcceptEmitter.event;
@@ -807,6 +813,15 @@ export class QuickPick<T extends IQuickPickItem, O extends { useSeparators: bool
 		this.update();
 	}
 
+	get customButtonSecondary() {
+		return this._customButtonSecondary;
+	}
+
+	set customButtonSecondary(secondary: boolean | undefined) {
+		this._customButtonSecondary = secondary ?? false;
+		this.update();
+	}
+
 	get ok() {
 		return this._ok;
 	}
@@ -1029,7 +1044,7 @@ export class QuickPick<T extends IQuickPickItem, O extends { useSeparators: bool
 			count: this.canSelectMany && !this._hideCountBadge,
 			ok: this.ok === 'default' ? this.canSelectMany : this.ok,
 			list: true,
-			message: !!this.validationMessage,
+			message: !!this.validationMessage || !!this.prompt,
 			customButton: this.customButton
 		};
 		this.ui.setVisibilities(visibilities);
@@ -1048,10 +1063,15 @@ export class QuickPick<T extends IQuickPickItem, O extends { useSeparators: bool
 		let ariaLabel = this.ariaLabel;
 		// Only set aria label to the input box placeholder if we actually have an input box.
 		if (!ariaLabel && visibilities.inputBox) {
-			ariaLabel = this.placeholder || QuickPick.DEFAULT_ARIA_LABEL;
+			ariaLabel = this.placeholder;
 			// If we have a title, include it in the aria label.
 			if (this.title) {
-				ariaLabel += ` - ${this.title}`;
+				ariaLabel = ariaLabel
+					? `${ariaLabel} - ${this.title}`
+					: this.title;
+			}
+			if (!ariaLabel) {
+				ariaLabel = QuickPick.DEFAULT_ARIA_LABEL;
 			}
 		}
 		if (this.ui.list.ariaLabel !== ariaLabel) {
@@ -1120,6 +1140,7 @@ export class QuickPick<T extends IQuickPickItem, O extends { useSeparators: bool
 		this.ui.ok.label = this.okLabel || '';
 		this.ui.customButton.label = this.customLabel || '';
 		this.ui.customButton.element.title = this.customHover || '';
+		this.ui.customButton.secondary = this.customButtonSecondary || false;
 		if (!visibilities.inputBox) {
 			// we need to move focus into the tree to detect keybindings
 			// properly when the input box is not visible (quick nav)
@@ -1150,11 +1171,11 @@ export class QuickPick<T extends IQuickPickItem, O extends { useSeparators: bool
 			return; // needs to be enabled
 		}
 
-		if (this.activeItems[0]) {
+		if (this.activeItems[0] && !this._canSelectMany) {
 			this._selectedItems = [this.activeItems[0]];
 			this.onDidChangeSelectionEmitter.fire(this.selectedItems);
-			this.handleAccept(inBackground ?? false);
 		}
+		this.handleAccept(inBackground ?? false);
 	}
 }
 
@@ -1253,6 +1274,10 @@ export class InputBox extends QuickInput implements IInputBox {
 		super.show();
 	}
 
+	accept(): void {
+		this.onDidAcceptEmitter.fire();
+	}
+
 	protected override update() {
 		if (!this.visible) {
 			return;
@@ -1302,17 +1327,37 @@ export class InputBox extends QuickInput implements IInputBox {
 export class QuickWidget extends QuickInput implements IQuickWidget {
 	readonly type = QuickInputType.QuickWidget;
 
+	private _widget: HTMLElement | undefined;
+	private _widgetUpdated = false;
+
+	get widget() {
+		return this._widget;
+	}
+
+	set widget(widget: HTMLElement | undefined) {
+		if (this._widget !== widget) {
+			this._widget = widget;
+			this._widgetUpdated = true;
+			this.update();
+		}
+	}
+
 	protected override update() {
 		if (!this.visible) {
 			return;
 		}
-
-		const visibilities: Visibilities = {
+		this.ui.setVisibilities({
 			title: !!this.title || !!this.step || !!this.titleButtons.length,
 			description: !!this.description || !!this.step
-		};
-
-		this.ui.setVisibilities(visibilities);
+		});
+		if (this._widgetUpdated) {
+			this._widgetUpdated = false;
+			if (this._widget) {
+				dom.reset(this.ui.widget, this._widget);
+			} else {
+				dom.reset(this.ui.widget);
+			}
+		}
 		super.update();
 	}
 }
@@ -1323,7 +1368,7 @@ export class QuickInputHoverDelegate extends WorkbenchHoverDelegate {
 		@IConfigurationService configurationService: IConfigurationService,
 		@IHoverService hoverService: IHoverService
 	) {
-		super('element', undefined, (options) => this.getOverrideOptions(options), configurationService, hoverService);
+		super('mouse', undefined, (options) => this.getOverrideOptions(options), configurationService, hoverService);
 	}
 
 	private getOverrideOptions(options: IHoverDelegateOptions): Partial<IHoverOptions> {
