@@ -8,17 +8,18 @@ import { Disposable, dispose, IDisposable, toDisposable } from '../../../../base
 import { TerminalCapabilityStore } from '../capabilities/terminalCapabilityStore.js';
 import { CommandDetectionCapability } from '../capabilities/commandDetectionCapability.js';
 import { CwdDetectionCapability } from '../capabilities/cwdDetectionCapability.js';
-import { IBufferMarkCapability, ICommandDetectionCapability, ICwdDetectionCapability, ISerializedCommandDetectionCapability, IShellEnvDetectionCapability, TerminalCapability } from '../capabilities/capabilities.js';
+import { IBufferMarkCapability, ICommandDetectionCapability, ICwdDetectionCapability, IPromptTypeDetectionCapability, ISerializedCommandDetectionCapability, IShellEnvDetectionCapability, TerminalCapability } from '../capabilities/capabilities.js';
 import { PartialCommandDetectionCapability } from '../capabilities/partialCommandDetectionCapability.js';
 import { ILogService } from '../../../log/common/log.js';
 import { ITelemetryService } from '../../../telemetry/common/telemetry.js';
-import { Emitter } from '../../../../base/common/event.js';
+import { Emitter, Event } from '../../../../base/common/event.js';
 import { BufferMarkCapability } from '../capabilities/bufferMarkCapability.js';
 import type { ITerminalAddon, Terminal } from '@xterm/headless';
 import { URI } from '../../../../base/common/uri.js';
 import { sanitizeCwd } from '../terminalEnvironment.js';
 import { removeAnsiEscapeCodesFromPrompt } from '../../../../base/common/strings.js';
 import { ShellEnvDetectionCapability } from '../capabilities/shellEnvDetectionCapability.js';
+import { PromptTypeDetectionCapability } from '../capabilities/promptTypeDetectionCapability.js';
 
 
 /**
@@ -206,7 +207,7 @@ const enum VSCodeOscPt {
 	 * Known properties:
 	 *
 	 * - `Cwd` - Reports the current working directory to the terminal.
-	 * - `IsWindows` - Reports whether the shell is using a Windows backend like winpty or conpty.
+	 * - `IsWindows` - Reports whether the shell is using a Windows backend (conpty).
 	 *   This may be used to enable additional heuristics as the positioning of the shell
 	 *   integration sequences are not guaranteed to be correct. Valid values: `True`, `False`.
 	 * - `ContinuationPrompt` - Reports the continuation prompt that is printed at the start of
@@ -342,6 +343,7 @@ export class ShellIntegrationAddon extends Disposable implements IShellIntegrati
 	constructor(
 		private _nonce: string,
 		private readonly _disableTelemetry: boolean | undefined,
+		private _onDidExecuteText: Event<void> | undefined,
 		private readonly _telemetryService: ITelemetryService | undefined,
 		private readonly _logService: ILogService
 	) {
@@ -359,7 +361,7 @@ export class ShellIntegrationAddon extends Disposable implements IShellIntegrati
 
 	activate(xterm: Terminal) {
 		this._terminal = xterm;
-		this.capabilities.add(TerminalCapability.PartialCommandDetection, this._register(new PartialCommandDetectionCapability(this._terminal)));
+		this.capabilities.add(TerminalCapability.PartialCommandDetection, this._register(new PartialCommandDetectionCapability(this._terminal, this._onDidExecuteText)));
 		this._register(xterm.parser.registerOscHandler(ShellIntegrationOscPs.VSCode, data => this._handleVSCodeSequence(data)));
 		this._register(xterm.parser.registerOscHandler(ShellIntegrationOscPs.ITerm, data => this._doHandleITermSequence(data)));
 		this._commonProtocolDisposables.push(
@@ -374,6 +376,12 @@ export class ShellIntegrationAddon extends Disposable implements IShellIntegrati
 		this._createOrGetBufferMarkDetection(terminal).getMark(vscodeMarkerId);
 	}
 
+	setNextCommandId(command: string, commandId: string): void {
+		if (this._terminal) {
+			this._createOrGetCommandDetection(this._terminal).setNextCommandId(command, commandId);
+		}
+	}
+
 	private _markSequenceSeen(sequence: string) {
 		if (!this._seenSequences.has(sequence)) {
 			this._seenSequences.add(sequence);
@@ -488,7 +496,7 @@ export class ShellIntegrationAddon extends Disposable implements IShellIntegrati
 				const arg1 = args[1];
 				let commandLine: string;
 				if (arg0 !== undefined) {
-					commandLine = deserializeMessage(arg0);
+					commandLine = deserializeVSCodeOscMessage(arg0);
 				} else {
 					commandLine = '';
 				}
@@ -508,7 +516,7 @@ export class ShellIntegrationAddon extends Disposable implements IShellIntegrati
 				const arg1 = args[1];
 				if (arg0 !== undefined) {
 					try {
-						const env = JSON.parse(deserializeMessage(arg0));
+						const env = JSON.parse(deserializeVSCodeOscMessage(arg0));
 						this._createOrGetShellEnvDetection().setEnvironment(env, arg1 === this._nonce);
 					} catch (e) {
 						this._logService.warn('Failed to parse environment from shell integration sequence', arg0);
@@ -526,7 +534,7 @@ export class ShellIntegrationAddon extends Disposable implements IShellIntegrati
 				const arg1 = args[1];
 				const arg2 = args[2];
 				if (arg0 !== undefined && arg1 !== undefined) {
-					const env = deserializeMessage(arg1);
+					const env = deserializeVSCodeOscMessage(arg1);
 					this._createOrGetShellEnvDetection().deleteEnvironmentSingleVar(arg0, env, arg2 === this._nonce);
 				}
 				return true;
@@ -536,7 +544,7 @@ export class ShellIntegrationAddon extends Disposable implements IShellIntegrati
 				const arg1 = args[1];
 				const arg2 = args[2];
 				if (arg0 !== undefined && arg1 !== undefined) {
-					const env = deserializeMessage(arg1);
+					const env = deserializeVSCodeOscMessage(arg1);
 					this._createOrGetShellEnvDetection().setEnvironmentSingleVar(arg0, env, arg2 === this._nonce);
 				}
 				return true;
@@ -555,7 +563,7 @@ export class ShellIntegrationAddon extends Disposable implements IShellIntegrati
 			}
 			case VSCodeOscPt.Property: {
 				const arg0 = args[0];
-				const deserialized = arg0 !== undefined ? deserializeMessage(arg0) : '';
+				const deserialized = arg0 !== undefined ? deserializeVSCodeOscMessage(arg0) : '';
 				const { key, value } = parseKeyValueAssignment(deserialized);
 				if (value === undefined) {
 					return true;
@@ -584,7 +592,7 @@ export class ShellIntegrationAddon extends Disposable implements IShellIntegrati
 						return true;
 					}
 					case 'PromptType': {
-						this._createOrGetCommandDetection(this._terminal).setPromptType(value);
+						this._createOrGetPromptTypeDetection().setPromptType(value);
 						return true;
 					}
 					case 'Task': {
@@ -616,7 +624,14 @@ export class ShellIntegrationAddon extends Disposable implements IShellIntegrati
 			return;
 		}
 		const lastPromptLine = prompt.substring(prompt.lastIndexOf('\n') + 1);
-		const promptTerminator = lastPromptLine.substring(lastPromptLine.lastIndexOf(' '));
+		const lastPromptLineTrimmed = lastPromptLine.trim();
+		const promptTerminator = (
+			lastPromptLineTrimmed.length === 1
+				// The prompt line contains a single character, treat the full line as the
+				// terminator for example "\u2b9e "
+				? lastPromptLine
+				: lastPromptLine.substring(lastPromptLine.lastIndexOf(' '))
+		);
 		if (promptTerminator) {
 			this._createOrGetCommandDetection(this._terminal).setPromptTerminator(promptTerminator, lastPromptLine);
 		}
@@ -767,9 +782,18 @@ export class ShellIntegrationAddon extends Disposable implements IShellIntegrati
 		}
 		return shellEnvDetection;
 	}
+
+	protected _createOrGetPromptTypeDetection(): IPromptTypeDetectionCapability {
+		let promptTypeDetection = this.capabilities.get(TerminalCapability.PromptTypeDetection);
+		if (!promptTypeDetection) {
+			promptTypeDetection = this._register(new PromptTypeDetectionCapability());
+			this.capabilities.add(TerminalCapability.PromptTypeDetection, promptTypeDetection);
+		}
+		return promptTypeDetection;
+	}
 }
 
-export function deserializeMessage(message: string): string {
+export function deserializeVSCodeOscMessage(message: string): string {
 	return message.replaceAll(
 		// Backslash ('\') followed by an escape operator: either another '\', or 'x' and two hex chars.
 		/\\(\\|x([0-9a-f]{2}))/gi,
@@ -778,6 +802,22 @@ export function deserializeMessage(message: string): string {
 		(_match: string, op: string, hex?: string) => hex ? String.fromCharCode(parseInt(hex, 16)) : op);
 }
 
+export function serializeVSCodeOscMessage(message: string): string {
+	return message.replace(
+		// Match backslash ('\'), semicolon (';'), or characters 0x20 and below
+		/[\\;\x00-\x20]/g,
+		(char: string) => {
+			// Escape backslash as '\\'
+			if (char === '\\') {
+				return '\\\\';
+			}
+			// Escape other characters as '\xAB' where AB is the hex representation
+			const charCode = char.charCodeAt(0);
+			return `\\x${charCode.toString(16).padStart(2, '0')}`;
+		}
+	);
+}
+
 export function parseKeyValueAssignment(message: string): { key: string; value: string | undefined } {
 	const separatorIndex = message.indexOf('=');
 	if (separatorIndex === -1) {
