@@ -6,8 +6,8 @@
 import { Disposable, DisposableMap } from '../../../base/common/lifecycle.js';
 import * as nls from '../../../nls.js';
 import { extHostNamedCustomer, IExtHostContext } from '../../services/extensions/common/extHostCustomers.js';
-import { IAuthenticationCreateSessionOptions, AuthenticationSession, AuthenticationSessionsChangeEvent, IAuthenticationProvider, IAuthenticationService, IAuthenticationExtensionsService, INTERNAL_AUTH_PROVIDER_PREFIX as INTERNAL_MODEL_AUTH_PROVIDER_PREFIX, AuthenticationSessionAccount, IAuthenticationProviderSessionOptions } from '../../services/authentication/common/authentication.js';
-import { ExtHostAuthenticationShape, ExtHostContext, MainContext, MainThreadAuthenticationShape } from '../common/extHost.protocol.js';
+import { AuthenticationSession, AuthenticationSessionsChangeEvent, IAuthenticationProvider, IAuthenticationService, IAuthenticationExtensionsService, AuthenticationSessionAccount, IAuthenticationProviderSessionOptions, isAuthenticationWwwAuthenticateRequest, IAuthenticationConstraint, IAuthenticationWwwAuthenticateRequest } from '../../services/authentication/common/authentication.js';
+import { ExtHostAuthenticationShape, ExtHostContext, IRegisterAuthenticationProviderDetails, IRegisterDynamicAuthenticationProviderDetails, MainContext, MainThreadAuthenticationShape } from '../common/extHost.protocol.js';
 import { IDialogService, IPromptButton } from '../../../platform/dialogs/common/dialogs.js';
 import Severity from '../../../base/common/severity.js';
 import { INotificationService } from '../../../platform/notification/common/notification.js';
@@ -27,6 +27,8 @@ import { DeferredPromise, raceTimeout } from '../../../base/common/async.js';
 import { IAuthorizationTokenResponse } from '../../../base/common/oauth.js';
 import { IDynamicAuthenticationProviderStorageService } from '../../services/authentication/common/dynamicAuthenticationProviderStorage.js';
 import { IClipboardService } from '../../../platform/clipboard/common/clipboardService.js';
+import { IQuickInputService } from '../../../platform/quickinput/common/quickInput.js';
+import { IProductService } from '../../../platform/product/common/productService.js';
 
 export interface AuthenticationInteractiveOptions {
 	detail?: string;
@@ -43,17 +45,17 @@ export interface AuthenticationGetSessionOptions {
 	authorizationServer?: UriComponents;
 }
 
-export class MainThreadAuthenticationProvider extends Disposable implements IAuthenticationProvider {
+class MainThreadAuthenticationProvider extends Disposable implements IAuthenticationProvider {
 
 	readonly onDidChangeSessions: Event<AuthenticationSessionsChangeEvent>;
 
 	constructor(
-		private readonly _proxy: ExtHostAuthenticationShape,
+		protected readonly _proxy: ExtHostAuthenticationShape,
 		public readonly id: string,
 		public readonly label: string,
 		public readonly supportsMultipleAccounts: boolean,
 		public readonly authorizationServers: ReadonlyArray<URI>,
-		private readonly notificationService: INotificationService,
+		public readonly resourceServer: URI | undefined,
 		onDidChangeSessionsEmitter: Emitter<AuthenticationSessionsChangeEvent>,
 	) {
 		super();
@@ -64,13 +66,43 @@ export class MainThreadAuthenticationProvider extends Disposable implements IAut
 		return this._proxy.$getSessions(this.id, scopes, options);
 	}
 
-	createSession(scopes: string[], options: IAuthenticationCreateSessionOptions): Promise<AuthenticationSession> {
+	createSession(scopes: string[], options: IAuthenticationProviderSessionOptions): Promise<AuthenticationSession> {
 		return this._proxy.$createSession(this.id, scopes, options);
 	}
 
 	async removeSession(sessionId: string): Promise<void> {
 		await this._proxy.$removeSession(this.id, sessionId);
-		this.notificationService.info(nls.localize('signedOut', "Successfully signed out."));
+	}
+}
+
+class MainThreadAuthenticationProviderWithChallenges extends MainThreadAuthenticationProvider implements IAuthenticationProvider {
+
+	constructor(
+		proxy: ExtHostAuthenticationShape,
+		id: string,
+		label: string,
+		supportsMultipleAccounts: boolean,
+		authorizationServers: ReadonlyArray<URI>,
+		resourceServer: URI | undefined,
+		onDidChangeSessionsEmitter: Emitter<AuthenticationSessionsChangeEvent>,
+	) {
+		super(
+			proxy,
+			id,
+			label,
+			supportsMultipleAccounts,
+			authorizationServers,
+			resourceServer,
+			onDidChangeSessionsEmitter
+		);
+	}
+
+	getSessionsFromChallenges(constraint: IAuthenticationConstraint, options: IAuthenticationProviderSessionOptions): Promise<readonly AuthenticationSession[]> {
+		return this._proxy.$getSessionsFromChallenges(this.id, constraint, options);
+	}
+
+	createSessionFromChallenges(constraint: IAuthenticationConstraint, options: IAuthenticationProviderSessionOptions): Promise<AuthenticationSession> {
+		return this._proxy.$createSessionFromChallenges(this.id, constraint, options);
 	}
 }
 
@@ -80,9 +112,11 @@ export class MainThreadAuthentication extends Disposable implements MainThreadAu
 
 	private readonly _registrations = this._register(new DisposableMap<string>());
 	private _sentProviderUsageEvents = new Set<string>();
+	private _suppressUnregisterEvent = false;
 
 	constructor(
 		extHostContext: IExtHostContext,
+		@IProductService private readonly productService: IProductService,
 		@IAuthenticationService private readonly authenticationService: IAuthenticationService,
 		@IAuthenticationExtensionsService private readonly authenticationExtensionsService: IAuthenticationExtensionsService,
 		@IAuthenticationAccessService private readonly authenticationAccessService: IAuthenticationAccessService,
@@ -95,13 +129,18 @@ export class MainThreadAuthentication extends Disposable implements MainThreadAu
 		@ILogService private readonly logService: ILogService,
 		@IURLService private readonly urlService: IURLService,
 		@IDynamicAuthenticationProviderStorageService private readonly dynamicAuthProviderStorageService: IDynamicAuthenticationProviderStorageService,
-		@IClipboardService private readonly clipboardService: IClipboardService
+		@IClipboardService private readonly clipboardService: IClipboardService,
+		@IQuickInputService private readonly quickInputService: IQuickInputService
 	) {
 		super();
 		this._proxy = extHostContext.getProxy(ExtHostContext.ExtHostAuthentication);
 
 		this._register(this.authenticationService.onDidChangeSessions(e => this._proxy.$onDidChangeAuthenticationSessions(e.providerId, e.label)));
-		this._register(this.authenticationService.onDidUnregisterAuthenticationProvider(e => this._proxy.$onDidUnregisterAuthenticationProvider(e.id)));
+		this._register(this.authenticationService.onDidUnregisterAuthenticationProvider(e => {
+			if (!this._suppressUnregisterEvent) {
+				this._proxy.$onDidUnregisterAuthenticationProvider(e.id);
+			}
+		}));
 		this._register(this.authenticationExtensionsService.onDidChangeAccountPreference(e => {
 			const providerInfo = this.authenticationService.getProvider(e.providerId);
 			this._proxy.$onDidChangeAuthenticationSessions(providerInfo.id, providerInfo.label, e.extensionIds);
@@ -116,24 +155,32 @@ export class MainThreadAuthentication extends Disposable implements MainThreadAu
 			// Prefer Node.js extension hosts when they're available. No CORS issues etc.
 			priority: extHostContext.extensionHostKind === ExtensionHostKind.LocalWebWorker ? 0 : 1,
 			create: async (authorizationServer, serverMetadata, resource) => {
-				const authProviderId = authorizationServer.toString(true);
-				const clientId = this.dynamicAuthProviderStorageService.getClientId(authProviderId);
+				// Auth Provider Id is a combination of the authorization server and the resource, if provided.
+				const authProviderId = resource ? `${authorizationServer.toString(true)} ${resource.resource}` : authorizationServer.toString(true);
+				const clientDetails = await this.dynamicAuthProviderStorageService.getClientRegistration(authProviderId);
+				let clientId = clientDetails?.clientId;
+				const clientSecret = clientDetails?.clientSecret;
 				let initialTokens: (IAuthorizationTokenResponse & { created_at: number })[] | undefined = undefined;
 				if (clientId) {
 					initialTokens = await this.dynamicAuthProviderStorageService.getSessionsForDynamicAuthProvider(authProviderId, clientId);
+					// If we don't already have a client id, check if the server supports the Client Id Metadata flow (see docs on the property)
+					// and add the "client id" if so.
+				} else if (serverMetadata.client_id_metadata_document_supported) {
+					clientId = this.productService.authClientIdMetadataUrl;
 				}
 				return await this._proxy.$registerDynamicAuthProvider(
 					authorizationServer,
 					serverMetadata,
 					resource,
 					clientId,
+					clientSecret,
 					initialTokens
 				);
 			}
 		}));
 	}
 
-	async $registerAuthenticationProvider(id: string, label: string, supportsMultipleAccounts: boolean, supportedAuthorizationServer: UriComponents[] = []): Promise<void> {
+	async $registerAuthenticationProvider({ id, label, supportsMultipleAccounts, resourceServer, supportedAuthorizationServers, supportsChallenges }: IRegisterAuthenticationProviderDetails): Promise<void> {
 		if (!this.authenticationService.declaredProviders.find(p => p.id === id)) {
 			// If telemetry shows that this is not happening much, we can instead throw an error here.
 			this.logService.warn(`Authentication provider ${id} was not declared in the Extension Manifest.`);
@@ -146,14 +193,39 @@ export class MainThreadAuthentication extends Disposable implements MainThreadAu
 		}
 		const emitter = new Emitter<AuthenticationSessionsChangeEvent>();
 		this._registrations.set(id, emitter);
-		const supportedAuthorizationServerUris = supportedAuthorizationServer.map(i => URI.revive(i));
-		const provider = new MainThreadAuthenticationProvider(this._proxy, id, label, supportsMultipleAccounts, supportedAuthorizationServerUris, this.notificationService, emitter);
+		const supportedAuthorizationServerUris = (supportedAuthorizationServers ?? []).map(i => URI.revive(i));
+		const provider =
+			supportsChallenges
+				? new MainThreadAuthenticationProviderWithChallenges(
+					this._proxy,
+					id,
+					label,
+					supportsMultipleAccounts,
+					supportedAuthorizationServerUris,
+					resourceServer ? URI.revive(resourceServer) : undefined,
+					emitter
+				)
+				: new MainThreadAuthenticationProvider(
+					this._proxy,
+					id,
+					label,
+					supportsMultipleAccounts,
+					supportedAuthorizationServerUris,
+					resourceServer ? URI.revive(resourceServer) : undefined,
+					emitter
+				);
 		this.authenticationService.registerAuthenticationProvider(id, provider);
 	}
 
 	async $unregisterAuthenticationProvider(id: string): Promise<void> {
 		this._registrations.deleteAndDispose(id);
-		this.authenticationService.unregisterAuthenticationProvider(id);
+		// The ext host side already unregisters the provider, so we can suppress the event here.
+		this._suppressUnregisterEvent = true;
+		try {
+			this.authenticationService.unregisterAuthenticationProvider(id);
+		} finally {
+			this._suppressUnregisterEvent = false;
+		}
 	}
 
 	async $ensureProvider(id: string): Promise<void> {
@@ -214,21 +286,45 @@ export class MainThreadAuthentication extends Disposable implements MainThreadAu
 		return deferredPromise.p;
 	}
 
-	async $registerDynamicAuthenticationProvider(id: string, label: string, authorizationServer: UriComponents, clientId: string): Promise<void> {
-		await this.$registerAuthenticationProvider(id, label, false, [authorizationServer]);
-		this.dynamicAuthProviderStorageService.storeClientId(id, URI.revive(authorizationServer).toString(true), clientId, label);
+	async $registerDynamicAuthenticationProvider(details: IRegisterDynamicAuthenticationProviderDetails): Promise<void> {
+		await this.$registerAuthenticationProvider({
+			id: details.id,
+			label: details.label,
+			supportsMultipleAccounts: true,
+			supportedAuthorizationServers: [details.authorizationServer],
+			resourceServer: details.resourceServer,
+		});
+		await this.dynamicAuthProviderStorageService.storeClientRegistration(details.id, URI.revive(details.authorizationServer).toString(true), details.clientId, details.clientSecret, details.label);
 	}
 
 	async $setSessionsForDynamicAuthProvider(authProviderId: string, clientId: string, sessions: (IAuthorizationTokenResponse & { created_at: number })[]): Promise<void> {
 		await this.dynamicAuthProviderStorageService.setSessionsForDynamicAuthProvider(authProviderId, clientId, sessions);
 	}
 
+	async $sendDidChangeDynamicProviderInfo({ providerId, clientId, authorizationServer, label, clientSecret }: Partial<{ providerId: string; clientId: string; authorizationServer: UriComponents; label: string; clientSecret: string }>): Promise<void> {
+		this.logService.info(`Client ID for authentication provider ${providerId} changed to ${clientId}`);
+		const existing = this.dynamicAuthProviderStorageService.getInteractedProviders().find(p => p.providerId === providerId);
+		if (!existing) {
+			throw new Error(`Dynamic authentication provider ${providerId} not found. Has it been registered?`);
+		}
+
+		// Store client credentials together
+		await this.dynamicAuthProviderStorageService.storeClientRegistration(
+			providerId || existing.providerId,
+			authorizationServer ? URI.revive(authorizationServer).toString(true) : existing.authorizationServer,
+			clientId || existing.clientId,
+			clientSecret,
+			label || existing.label
+		);
+	}
+
 	private async loginPrompt(provider: IAuthenticationProvider, extensionName: string, recreatingSession: boolean, options?: AuthenticationInteractiveOptions): Promise<boolean> {
 		let message: string;
 
-		// An internal provider is a special case which is for model access only.
-		if (provider.id.startsWith(INTERNAL_MODEL_AUTH_PROVIDER_PREFIX)) {
-			message = nls.localize('confirmModelAccess', "The extension '{0}' wants to access the language models provided by {1}.", extensionName, provider.label);
+		// Check if the provider has a custom confirmation message
+		const customMessage = provider.confirmation?.(extensionName, recreatingSession);
+		if (customMessage) {
+			message = customMessage;
 		} else {
 			message = recreatingSession
 				? nls.localize('confirmRelogin', "The extension '{0}' wants you to sign in again using {1}.", extensionName, provider.label)
@@ -289,9 +385,9 @@ export class MainThreadAuthentication extends Disposable implements MainThreadAu
 		return result.result === chosenAccountLabel;
 	}
 
-	private async doGetSession(providerId: string, scopes: string[], extensionId: string, extensionName: string, options: AuthenticationGetSessionOptions): Promise<AuthenticationSession | undefined> {
+	private async doGetSession(providerId: string, scopeListOrRequest: ReadonlyArray<string> | IAuthenticationWwwAuthenticateRequest, extensionId: string, extensionName: string, options: AuthenticationGetSessionOptions): Promise<AuthenticationSession | undefined> {
 		const authorizationServer = URI.revive(options.authorizationServer);
-		const sessions = await this.authenticationService.getSessions(providerId, scopes, { account: options.account, authorizationServer }, true);
+		const sessions = await this.authenticationService.getSessions(providerId, scopeListOrRequest, { account: options.account, authorizationServer }, true);
 		const provider = this.authenticationService.getProvider(providerId);
 
 		// Error cases
@@ -308,7 +404,7 @@ export class MainThreadAuthentication extends Disposable implements MainThreadAu
 		if (options.clearSessionPreference) {
 			// Clearing the session preference is usually paired with createIfNone, so just remove the preference and
 			// defer to the rest of the logic in this function to choose the session.
-			this._removeAccountPreference(extensionId, providerId, scopes);
+			this.authenticationExtensionsService.removeAccountPreference(extensionId, providerId);
 		}
 
 		const matchingAccountPreferenceSession =
@@ -316,7 +412,7 @@ export class MainThreadAuthentication extends Disposable implements MainThreadAu
 			options.account
 				// We only support one session per account per set of scopes so grab the first one here
 				? sessions[0]
-				: this._getAccountPreference(extensionId, providerId, scopes, sessions);
+				: this._getAccountPreference(extensionId, providerId, sessions);
 
 		// Check if the sessions we have are valid
 		if (!options.forceNewSession && sessions.length) {
@@ -351,14 +447,14 @@ export class MainThreadAuthentication extends Disposable implements MainThreadAu
 			let session: AuthenticationSession;
 			if (sessions?.length && !options.forceNewSession) {
 				session = provider.supportsMultipleAccounts && !options.account
-					? await this.authenticationExtensionsService.selectSession(providerId, extensionId, extensionName, scopes, sessions)
+					? await this.authenticationExtensionsService.selectSession(providerId, extensionId, extensionName, scopeListOrRequest, sessions)
 					: sessions[0];
 			} else {
 				const accountToCreate: AuthenticationSessionAccount | undefined = options.account ?? matchingAccountPreferenceSession?.account;
 				do {
 					session = await this.authenticationService.createSession(
 						providerId,
-						scopes,
+						scopeListOrRequest,
 						{
 							activateImmediate: true,
 							account: accountToCreate,
@@ -372,15 +468,16 @@ export class MainThreadAuthentication extends Disposable implements MainThreadAu
 			}
 
 			this.authenticationAccessService.updateAllowedExtensions(providerId, session.account.label, [{ id: extensionId, name: extensionName, allowed: true }]);
-			this._updateAccountPreference(extensionId, providerId, session);
+			this.authenticationExtensionsService.updateNewSessionRequests(providerId, [session]);
+			this.authenticationExtensionsService.updateAccountPreference(extensionId, providerId, session.account);
 			return session;
 		}
 
-		// For the silent flows, if we have a session but we don't have a session preference, we'll return the first one that is valid.
-		if (!matchingAccountPreferenceSession && !this.authenticationExtensionsService.getAccountPreference(extensionId, providerId)) {
-			const validSession = sessions.find(session => this.authenticationAccessService.isAccessAllowed(providerId, session.account.label, extensionId));
-			if (validSession) {
-				return validSession;
+		// For the silent flows, if we don't have a session that matches the account preference, we can return any valid session if there is only one to choose from.
+		if (!matchingAccountPreferenceSession) {
+			const validSessions = sessions.filter(session => this.authenticationAccessService.isAccessAllowed(providerId, session.account.label, extensionId));
+			if (validSessions.length === 1) {
+				return validSessions[0];
 			}
 		}
 
@@ -389,19 +486,22 @@ export class MainThreadAuthentication extends Disposable implements MainThreadAu
 			// If there is a potential session, but the extension doesn't have access to it, use the "grant access" flow,
 			// otherwise request a new one.
 			sessions.length
-				? this.authenticationExtensionsService.requestSessionAccess(providerId, extensionId, extensionName, scopes, sessions)
-				: await this.authenticationExtensionsService.requestNewSession(providerId, scopes, extensionId, extensionName);
+				? this.authenticationExtensionsService.requestSessionAccess(providerId, extensionId, extensionName, scopeListOrRequest, sessions)
+				: await this.authenticationExtensionsService.requestNewSession(providerId, scopeListOrRequest, extensionId, extensionName);
 		}
 		return undefined;
 	}
 
-	async $getSession(providerId: string, scopes: string[], extensionId: string, extensionName: string, options: AuthenticationGetSessionOptions): Promise<AuthenticationSession | undefined> {
-		this.sendClientIdUsageTelemetry(extensionId, providerId, scopes);
-		const session = await this.doGetSession(providerId, scopes, extensionId, extensionName, options);
+	async $getSession(providerId: string, scopeListOrRequest: ReadonlyArray<string> | IAuthenticationWwwAuthenticateRequest, extensionId: string, extensionName: string, options: AuthenticationGetSessionOptions): Promise<AuthenticationSession | undefined> {
+		const scopes = isAuthenticationWwwAuthenticateRequest(scopeListOrRequest) ? scopeListOrRequest.fallbackScopes : scopeListOrRequest;
+		if (scopes) {
+			this.sendClientIdUsageTelemetry(extensionId, providerId, scopes);
+		}
+		const session = await this.doGetSession(providerId, scopeListOrRequest, extensionId, extensionName, options);
 
 		if (session) {
 			this.sendProviderUsageTelemetry(extensionId, providerId);
-			this.authenticationUsageService.addAccountUsage(providerId, session.account.label, scopes, extensionId, extensionName);
+			this.authenticationUsageService.addAccountUsage(providerId, session.account.label, session.scopes, extensionId, extensionName);
 		}
 
 		return session;
@@ -417,7 +517,7 @@ export class MainThreadAuthentication extends Disposable implements MainThreadAu
 	// due to the adoption of the Microsoft broker.
 	// Remove this in a few iterations.
 	private _sentClientIdUsageEvents = new Set<string>();
-	private sendClientIdUsageTelemetry(extensionId: string, providerId: string, scopes: string[]): void {
+	private sendClientIdUsageTelemetry(extensionId: string, providerId: string, scopes: readonly string[]): void {
 		const containsVSCodeClientIdScope = scopes.some(scope => scope.startsWith('VSCODE_CLIENT_ID:'));
 		const key = `${extensionId}|${providerId}|${containsVSCodeClientIdScope}`;
 		if (this._sentClientIdUsageEvents.has(key)) {
@@ -452,7 +552,7 @@ export class MainThreadAuthentication extends Disposable implements MainThreadAu
 	//#region Account Preferences
 	// TODO@TylerLeonhardt: Update this after a few iterations to no longer fallback to the session preference
 
-	private _getAccountPreference(extensionId: string, providerId: string, scopes: string[], sessions: ReadonlyArray<AuthenticationSession>): AuthenticationSession | undefined {
+	private _getAccountPreference(extensionId: string, providerId: string, sessions: ReadonlyArray<AuthenticationSession>): AuthenticationSession | undefined {
 		if (sessions.length === 0) {
 			return undefined;
 		}
@@ -461,29 +561,8 @@ export class MainThreadAuthentication extends Disposable implements MainThreadAu
 			const session = sessions.find(session => session.account.label === accountNamePreference);
 			return session;
 		}
-
-		const sessionIdPreference = this.authenticationExtensionsService.getSessionPreference(providerId, extensionId, scopes);
-		if (sessionIdPreference) {
-			const session = sessions.find(session => session.id === sessionIdPreference);
-			if (session) {
-				// Migrate the session preference to the account preference
-				this.authenticationExtensionsService.updateAccountPreference(extensionId, providerId, session.account);
-				return session;
-			}
-		}
 		return undefined;
 	}
-
-	private _updateAccountPreference(extensionId: string, providerId: string, session: AuthenticationSession): void {
-		this.authenticationExtensionsService.updateAccountPreference(extensionId, providerId, session.account);
-		this.authenticationExtensionsService.updateSessionPreference(providerId, extensionId, session);
-	}
-
-	private _removeAccountPreference(extensionId: string, providerId: string, scopes: string[]): void {
-		this.authenticationExtensionsService.removeAccountPreference(extensionId, providerId);
-		this.authenticationExtensionsService.removeSessionPreference(providerId, extensionId, scopes);
-	}
-
 	//#endregion
 
 	async $showDeviceCodeModal(userCode: string, verificationUri: string): Promise<boolean> {
@@ -511,4 +590,68 @@ export class MainThreadAuthentication extends Disposable implements MainThreadAu
 		}
 		return false;
 	}
+
+	async $promptForClientRegistration(authorizationServerUrl: string): Promise<{ clientId: string; clientSecret?: string } | undefined> {
+		const redirectUrls = 'http://127.0.0.1:33418\nhttps://vscode.dev/redirect';
+
+		// Show modal dialog first to explain the situation and get user consent
+		const result = await this.dialogService.prompt({
+			type: Severity.Info,
+			message: nls.localize('dcrNotSupported', "Dynamic Client Registration not supported"),
+			detail: nls.localize('dcrNotSupportedDetail', "The authorization server '{0}' does not support automatic client registration. Do you want to proceed by manually providing a client registration (client ID)?\n\nNote: When registering your OAuth application, make sure to include these redirect URIs:\n{1}", authorizationServerUrl, redirectUrls),
+			buttons: [
+				{
+					label: nls.localize('dcrCopyUrlsAndProceed', "Copy URIs & Proceed"),
+					run: async () => {
+						try {
+							await this.clipboardService.writeText(redirectUrls);
+						} catch (error) {
+							this.notificationService.error(nls.localize('dcrFailedToCopy', "Failed to copy redirect URIs to clipboard."));
+						}
+						return true;
+					}
+				},
+			],
+			cancelButton: {
+				label: nls.localize('cancel', "Cancel"),
+				run: () => false
+			}
+		});
+
+		if (!result) {
+			return undefined;
+		}
+
+		const sharedTitle = nls.localize('addClientRegistrationDetails', "Add Client Registration Details");
+
+		const clientId = await this.quickInputService.input({
+			title: sharedTitle,
+			prompt: nls.localize('clientIdPrompt', "Enter an existing client ID that has been registered with the following redirect URIs: http://127.0.0.1:33418, https://vscode.dev/redirect"),
+			placeHolder: nls.localize('clientIdPlaceholder', "OAuth client ID (azye39d...)"),
+			ignoreFocusLost: true,
+			validateInput: async (value: string) => {
+				if (!value || value.trim().length === 0) {
+					return nls.localize('clientIdRequired', "Client ID is required");
+				}
+				return undefined;
+			}
+		});
+
+		if (!clientId || clientId.trim().length === 0) {
+			return undefined;
+		}
+
+		const clientSecret = await this.quickInputService.input({
+			title: sharedTitle,
+			prompt: nls.localize('clientSecretPrompt', "(optional) Enter an existing client secret associated with the client id '{0}' or leave this field blank", clientId),
+			placeHolder: nls.localize('clientSecretPlaceholder', "OAuth client secret (wer32o50f...) or leave it blank"),
+			password: true,
+			ignoreFocusLost: true
+		});
+
+		return {
+			clientId: clientId.trim(),
+			clientSecret: clientSecret?.trim() || undefined
+		};
+	}
 }
