@@ -33,7 +33,14 @@ import * as callh from '../../contrib/callHierarchy/common/callHierarchy.js';
 import * as search from '../../contrib/search/common/search.js';
 import * as typeh from '../../contrib/typeHierarchy/common/typeHierarchy.js';
 import { extHostNamedCustomer, IExtHostContext } from '../../services/extensions/common/extHostCustomers.js';
-import { ExtHostContext, ExtHostLanguageFeaturesShape, HoverWithId, ICallHierarchyItemDto, ICodeActionDto, ICodeActionProviderMetadataDto, IdentifiableInlineCompletion, IdentifiableInlineCompletions, IdentifiableInlineEdit, IDocumentDropEditDto, IDocumentDropEditProviderMetadata, IDocumentFilterDto, IIndentationRuleDto, IInlayHintDto, ILanguageConfigurationDto, ILanguageWordDefinitionDto, ILinkDto, ILocationDto, ILocationLinkDto, IOnEnterRuleDto, IPasteEditDto, IPasteEditProviderMetadataDto, IRegExpDto, ISignatureHelpProviderMetadataDto, ISuggestDataDto, ISuggestDataDtoField, ISuggestResultDtoField, ITypeHierarchyItemDto, IWorkspaceSymbolDto, MainContext, MainThreadLanguageFeaturesShape } from '../common/extHost.protocol.js';
+import { ExtHostContext, ExtHostLanguageFeaturesShape, HoverWithId, ICallHierarchyItemDto, ICodeActionDto, ICodeActionProviderMetadataDto, IdentifiableInlineCompletion, IdentifiableInlineCompletions, IDocumentDropEditDto, IDocumentDropEditProviderMetadata, IDocumentFilterDto, IIndentationRuleDto, IInlayHintDto, IInlineCompletionChangeHintDto, IInlineCompletionModelInfoDto, ILanguageConfigurationDto, ILanguageWordDefinitionDto, ILinkDto, ILocationDto, ILocationLinkDto, IOnEnterRuleDto, IPasteEditDto, IPasteEditProviderMetadataDto, IRegExpDto, ISignatureHelpProviderMetadataDto, ISuggestDataDto, ISuggestDataDtoField, ISuggestResultDtoField, ITypeHierarchyItemDto, IWorkspaceSymbolDto, MainContext, MainThreadLanguageFeaturesShape } from '../common/extHost.protocol.js';
+import { InlineCompletionEndOfLifeReasonKind } from '../common/extHostTypes.js';
+import { IInstantiationService } from '../../../platform/instantiation/common/instantiation.js';
+import { DataChannelForwardingTelemetryService, forwardToChannelIf, isCopilotLikeExtension } from '../../../platform/dataChannel/browser/forwardingTelemetryService.js';
+import { IAiEditTelemetryService } from '../../contrib/editTelemetry/browser/telemetry/aiEditTelemetry/aiEditTelemetryService.js';
+import { EditDeltaInfo } from '../../../editor/common/textModelEditSource.js';
+import { IInlineCompletionsUnificationService } from '../../services/inlineCompletions/common/inlineCompletionsUnification.js';
+import { InlineCompletionEndOfLifeEvent, sendInlineCompletionsEndOfLifeTelemetry } from '../../../editor/contrib/inlineCompletions/browser/telemetry.js';
 
 @extHostNamedCustomer(MainContext.MainThreadLanguageFeatures)
 export class MainThreadLanguageFeatures extends Disposable implements MainThreadLanguageFeaturesShape {
@@ -46,7 +53,9 @@ export class MainThreadLanguageFeatures extends Disposable implements MainThread
 		@ILanguageService private readonly _languageService: ILanguageService,
 		@ILanguageConfigurationService private readonly _languageConfigurationService: ILanguageConfigurationService,
 		@ILanguageFeaturesService private readonly _languageFeaturesService: ILanguageFeaturesService,
-		@IUriIdentityService private readonly _uriIdentService: IUriIdentityService
+		@IUriIdentityService private readonly _uriIdentService: IUriIdentityService,
+		@IInstantiationService private readonly _instantiationService: IInstantiationService,
+		@IInlineCompletionsUnificationService private readonly _inlineCompletionsUnificationService: IInlineCompletionsUnificationService,
 	) {
 		super();
 
@@ -79,6 +88,13 @@ export class MainThreadLanguageFeatures extends Disposable implements MainThread
 			}));
 			updateAllWordDefinitions();
 		}
+
+		if (this._inlineCompletionsUnificationService) {
+			this._register(this._inlineCompletionsUnificationService.onDidStateChange(() => {
+				this._proxy.$acceptInlineCompletionsUnificationState(this._inlineCompletionsUnificationService.state);
+			}));
+			this._proxy.$acceptInlineCompletionsUnificationState(this._inlineCompletionsUnificationService.state);
+		}
 	}
 
 	$unregister(handle: number): void {
@@ -120,7 +136,7 @@ export class MainThreadLanguageFeatures extends Disposable implements MainThread
 	private static _reviveWorkspaceSymbolDto(data: undefined): undefined;
 	private static _reviveWorkspaceSymbolDto(data: IWorkspaceSymbolDto | IWorkspaceSymbolDto[] | undefined): search.IWorkspaceSymbol | search.IWorkspaceSymbol[] | undefined {
 		if (!data) {
-			return <undefined>data;
+			return data;
 		} else if (Array.isArray(data)) {
 			data.forEach(MainThreadLanguageFeatures._reviveWorkspaceSymbolDto);
 			return <search.IWorkspaceSymbol[]>data;
@@ -186,9 +202,10 @@ export class MainThreadLanguageFeatures extends Disposable implements MainThread
 			},
 			resolveCodeLens: async (model: ITextModel, codeLens: languages.CodeLens, token: CancellationToken): Promise<languages.CodeLens | undefined> => {
 				const result = await this._proxy.$resolveCodeLens(handle, codeLens, token);
-				if (!result) {
+				if (!result || token.isCancellationRequested) {
 					return undefined;
 				}
+
 				return {
 					...result,
 					range: model.validateRange(result.range),
@@ -205,7 +222,7 @@ export class MainThreadLanguageFeatures extends Disposable implements MainThread
 		this._registrations.set(handle, this._languageFeaturesService.codeLensProvider.register(selector, provider));
 	}
 
-	$emitCodeLensEvent(eventHandle: number, event?: any): void {
+	$emitCodeLensEvent(eventHandle: number, event?: unknown): void {
 		const obj = this._registrations.get(eventHandle);
 		if (obj instanceof Emitter) {
 			obj.fire(event);
@@ -297,7 +314,7 @@ export class MainThreadLanguageFeatures extends Disposable implements MainThread
 		this._registrations.set(handle, this._languageFeaturesService.inlineValuesProvider.register(selector, provider));
 	}
 
-	$emitInlineValuesEvent(eventHandle: number, event?: any): void {
+	$emitInlineValuesEvent(eventHandle: number, event?: unknown): void {
 		const obj = this._registrations.get(eventHandle);
 		if (obj instanceof Emitter) {
 			obj.fire(event);
@@ -532,8 +549,21 @@ export class MainThreadLanguageFeatures extends Disposable implements MainThread
 		}
 	}
 
-	$registerDocumentRangeSemanticTokensProvider(handle: number, selector: IDocumentFilterDto[], legend: languages.SemanticTokensLegend): void {
-		this._registrations.set(handle, this._languageFeaturesService.documentRangeSemanticTokensProvider.register(selector, new MainThreadDocumentRangeSemanticTokensProvider(this._proxy, handle, legend)));
+	$emitDocumentRangeSemanticTokensEvent(eventHandle: number): void {
+		const obj = this._registrations.get(eventHandle);
+		if (obj instanceof Emitter) {
+			obj.fire(undefined);
+		}
+	}
+
+	$registerDocumentRangeSemanticTokensProvider(handle: number, selector: IDocumentFilterDto[], legend: languages.SemanticTokensLegend, eventHandle: number | undefined): void {
+		let event: Event<void> | undefined = undefined;
+		if (typeof eventHandle === 'number') {
+			const emitter = new Emitter<void>();
+			this._registrations.set(eventHandle, emitter);
+			event = emitter.event;
+		}
+		this._registrations.set(handle, this._languageFeaturesService.documentRangeSemanticTokensProvider.register(selector, new MainThreadDocumentRangeSemanticTokensProvider(this._proxy, handle, legend, event)));
 	}
 
 	// --- suggest
@@ -614,84 +644,57 @@ export class MainThreadLanguageFeatures extends Disposable implements MainThread
 		this._registrations.set(handle, this._languageFeaturesService.completionProvider.register(selector, provider));
 	}
 
-	$registerInlineCompletionsSupport(handle: number, selector: IDocumentFilterDto[], supportsHandleEvents: boolean, extensionId: string, yieldsToExtensionIds: string[], displayName: string | undefined, debounceDelayMs: number | undefined, eventHandle: number | undefined): void {
-		const provider: languages.InlineCompletionsProvider<IdentifiableInlineCompletions> = {
-			provideInlineCompletions: async (model: ITextModel, position: EditorPosition, context: languages.InlineCompletionContext, token: CancellationToken): Promise<IdentifiableInlineCompletions | undefined> => {
-				return this._proxy.$provideInlineCompletions(handle, model.uri, position, context, token);
-			},
-			provideInlineEditsForRange: async (model: ITextModel, range: EditorRange, context: languages.InlineCompletionContext, token: CancellationToken): Promise<IdentifiableInlineCompletions | undefined> => {
-				return this._proxy.$provideInlineEditsForRange(handle, model.uri, range, context, token);
-			},
-			handleItemDidShow: async (completions: IdentifiableInlineCompletions, item: IdentifiableInlineCompletion, updatedInsertText: string): Promise<void> => {
-				if (supportsHandleEvents) {
-					await this._proxy.$handleInlineCompletionDidShow(handle, completions.pid, item.idx, updatedInsertText);
-				}
-			},
-			handlePartialAccept: async (completions, item, acceptedCharacters, info: languages.PartialAcceptInfo): Promise<void> => {
-				if (supportsHandleEvents) {
-					await this._proxy.$handleInlineCompletionPartialAccept(handle, completions.pid, item.idx, acceptedCharacters, info);
-				}
-			},
-			handleEndOfLifetime: async (completions, item, reason) => {
+	$registerInlineCompletionsSupport(
+		handle: number,
+		selector: IDocumentFilterDto[],
+		supportsHandleEvents: boolean,
+		extensionId: string,
+		extensionVersion: string,
+		groupId: string | undefined,
+		yieldsToExtensionIds: string[],
+		displayName: string | undefined,
+		debounceDelayMs: number | undefined,
+		excludesExtensionIds: string[],
+		supportsOnDidChange: boolean,
+		supportsSetModelId: boolean,
+		initialModelInfo: IInlineCompletionModelInfoDto | undefined,
+		supportsOnDidChangeModelInfo: boolean,
+	): void {
+		const providerId = new languages.ProviderId(extensionId, extensionVersion, groupId);
 
-				function mapReason<T1, T2>(reason: languages.InlineCompletionEndOfLifeReason<T1>, f: (reason: T1) => T2): languages.InlineCompletionEndOfLifeReason<T2> {
-					if (reason.kind === languages.InlineCompletionEndOfLifeReasonKind.Ignored) {
-						return {
-							...reason,
-							supersededBy: reason.supersededBy ? f(reason.supersededBy) : undefined,
-						};
-					}
-					return reason;
-				}
-
-				if (supportsHandleEvents) {
-					await this._proxy.$handleInlineCompletionEndOfLifetime(handle, completions.pid, item.idx, mapReason(reason, i => ({ pid: completions.pid, idx: i.idx })));
-				}
-			},
-			freeInlineCompletions: (completions: IdentifiableInlineCompletions): void => {
-				this._proxy.$freeInlineCompletionsList(handle, completions.pid);
-			},
-			handleRejection: async (completions, item): Promise<void> => {
-				if (supportsHandleEvents) {
-					await this._proxy.$handleInlineCompletionRejection(handle, completions.pid, item.idx);
-				}
-			},
-			groupId: extensionId,
-			yieldsToGroupIds: yieldsToExtensionIds,
+		const provider = this._instantiationService.createInstance(
+			ExtensionBackedInlineCompletionsProvider,
+			handle,
+			groupId ?? extensionId,
+			providerId,
+			yieldsToExtensionIds,
+			excludesExtensionIds,
 			debounceDelayMs,
 			displayName,
-			toString() {
-				return `InlineCompletionsProvider(${extensionId})`;
-			},
-		};
-		this._registrations.set(handle, this._languageFeaturesService.inlineCompletionsProvider.register(selector, provider));
+			initialModelInfo,
+			supportsHandleEvents,
+			supportsSetModelId,
+			supportsOnDidChange,
+			supportsOnDidChangeModelInfo,
+			selector,
+			this._proxy,
+		);
 
-		if (typeof eventHandle === 'number') {
-			const emitter = new Emitter<void>();
-			this._registrations.set(eventHandle, emitter);
-			provider.onDidChangeInlineCompletions = emitter.event;
-		}
+		this._registrations.set(handle, provider);
 	}
 
-	$emitInlineCompletionsChange(handle: number): void {
+	$emitInlineCompletionsChange(handle: number, changeHint: IInlineCompletionChangeHintDto | undefined): void {
 		const obj = this._registrations.get(handle);
-		if (obj instanceof Emitter) {
-			obj.fire(undefined);
+		if (obj instanceof ExtensionBackedInlineCompletionsProvider) {
+			obj._emitDidChange(changeHint);
 		}
 	}
 
-	$registerInlineEditProvider(handle: number, selector: IDocumentFilterDto[], extensionId: ExtensionIdentifier, displayName: string): void {
-		const provider: languages.InlineEditProvider<IdentifiableInlineEdit> = {
-			displayName,
-			provideInlineEdit: async (model: ITextModel, context: languages.IInlineEditContext, token: CancellationToken): Promise<IdentifiableInlineEdit | undefined> => {
-				return this._proxy.$provideInlineEdit(handle, model.uri, context, token);
-			},
-			freeInlineEdit: (edit: IdentifiableInlineEdit): void => {
-				this._proxy.$freeInlineEdit(handle, edit.pid);
-			}
-
-		};
-		this._registrations.set(handle, this._languageFeaturesService.inlineEditProvider.register(selector, provider));
+	$emitInlineCompletionModelInfoChange(handle: number, data: IInlineCompletionModelInfoDto | undefined): void {
+		const obj = this._registrations.get(handle);
+		if (obj instanceof ExtensionBackedInlineCompletionsProvider) {
+			obj._setModelInfo(data);
+		}
 	}
 
 	// --- parameter hints
@@ -861,7 +864,7 @@ export class MainThreadLanguageFeatures extends Disposable implements MainThread
 		this._registrations.set(handle, this._languageFeaturesService.foldingRangeProvider.register(selector, provider));
 	}
 
-	$emitFoldingRangeEvent(eventHandle: number, event?: any): void {
+	$emitFoldingRangeEvent(eventHandle: number, event?: unknown): void {
 		const obj = this._registrations.get(eventHandle);
 		if (obj instanceof Emitter) {
 			obj.fire(event);
@@ -906,6 +909,7 @@ export class MainThreadLanguageFeatures extends Disposable implements MainThread
 				outgoing.forEach(value => {
 					value.to = MainThreadLanguageFeatures._reviveCallHierarchyItemDto(value.to);
 				});
+				// eslint-disable-next-line local/code-no-any-casts, @typescript-eslint/no-explicit-any
 				return <any>outgoing;
 			},
 			provideIncomingCalls: async (item, token) => {
@@ -916,6 +920,7 @@ export class MainThreadLanguageFeatures extends Disposable implements MainThread
 				incoming.forEach(value => {
 					value.from = MainThreadLanguageFeatures._reviveCallHierarchyItemDto(value.from);
 				});
+				// eslint-disable-next-line local/code-no-any-casts, @typescript-eslint/no-explicit-any
 				return <any>incoming;
 			}
 		}));
@@ -1256,6 +1261,7 @@ export class MainThreadDocumentRangeSemanticTokensProvider implements languages.
 		private readonly _proxy: ExtHostLanguageFeaturesShape,
 		private readonly _handle: number,
 		private readonly _legend: languages.SemanticTokensLegend,
+		public readonly onDidChange: Event<void> | undefined,
 	) {
 	}
 
@@ -1282,3 +1288,209 @@ export class MainThreadDocumentRangeSemanticTokensProvider implements languages.
 	}
 }
 
+class ExtensionBackedInlineCompletionsProvider extends Disposable implements languages.InlineCompletionsProvider<IdentifiableInlineCompletions> {
+	public readonly setModelId: ((modelId: string) => Promise<void>) | undefined;
+	public readonly _onDidChangeEmitter = new Emitter<languages.IInlineCompletionChangeHint | void>();
+	public readonly onDidChangeInlineCompletions: Event<languages.IInlineCompletionChangeHint | void> | undefined;
+
+	public readonly _onDidChangeModelInfoEmitter = new Emitter<void>();
+	public readonly onDidChangeModelInfo: Event<void> | undefined;
+
+	constructor(
+		public readonly handle: number,
+		public readonly groupId: string,
+		public readonly providerId: languages.ProviderId,
+		public readonly yieldsToGroupIds: string[],
+		public readonly excludesGroupIds: string[],
+		public readonly debounceDelayMs: number | undefined,
+		public readonly displayName: string | undefined,
+		public modelInfo: languages.IInlineCompletionModelInfo | undefined,
+		private readonly _supportsHandleEvents: boolean,
+		private readonly _supportsSetModelId: boolean,
+		private readonly _supportsOnDidChange: boolean,
+		private readonly _supportsOnDidChangeModelInfo: boolean,
+		private readonly _selector: IDocumentFilterDto[],
+		private readonly _proxy: ExtHostLanguageFeaturesShape,
+		@ILanguageFeaturesService private readonly _languageFeaturesService: ILanguageFeaturesService,
+		@IAiEditTelemetryService private readonly _aiEditTelemetryService: IAiEditTelemetryService,
+		@IInstantiationService private readonly _instantiationService: IInstantiationService,
+	) {
+		super();
+
+		this.setModelId = this._supportsSetModelId ? async (modelId: string) => {
+			await this._proxy.$handleInlineCompletionSetCurrentModelId(this.handle, modelId);
+		} : undefined;
+
+		this.onDidChangeInlineCompletions = this._supportsOnDidChange ? this._onDidChangeEmitter.event : undefined;
+		this.onDidChangeModelInfo = this._supportsOnDidChangeModelInfo ? this._onDidChangeModelInfoEmitter.event : undefined;
+
+		this._register(this._languageFeaturesService.inlineCompletionsProvider.register(this._selector, this));
+	}
+
+	public _setModelInfo(newModelInfo: languages.IInlineCompletionModelInfo | undefined) {
+		this.modelInfo = newModelInfo;
+		if (this._supportsOnDidChangeModelInfo) {
+			this._onDidChangeModelInfoEmitter.fire();
+		}
+	}
+
+	public _emitDidChange(changeHint: IInlineCompletionChangeHintDto | undefined) {
+		if (this._supportsOnDidChange) {
+			this._onDidChangeEmitter.fire(changeHint);
+		}
+	}
+
+	public async provideInlineCompletions(model: ITextModel, position: EditorPosition, context: languages.InlineCompletionContext, token: CancellationToken): Promise<IdentifiableInlineCompletions | undefined> {
+		const result = await this._proxy.$provideInlineCompletions(this.handle, model.uri, position, context, token);
+		return result;
+	}
+
+	public async handleItemDidShow(completions: IdentifiableInlineCompletions, item: IdentifiableInlineCompletion, updatedInsertText: string, editDeltaInfo: EditDeltaInfo): Promise<void> {
+		if (item.suggestionId === undefined) {
+			item.suggestionId = this._aiEditTelemetryService.createSuggestionId({
+				applyCodeBlockSuggestionId: undefined,
+				feature: 'inlineSuggestion',
+				source: this.providerId,
+				languageId: completions.languageId,
+				editDeltaInfo: editDeltaInfo,
+				modeId: undefined,
+				modelId: undefined,
+				presentation: item.isInlineEdit ? 'nextEditSuggestion' : 'inlineCompletion',
+			});
+		}
+
+		if (this._supportsHandleEvents) {
+			await this._proxy.$handleInlineCompletionDidShow(this.handle, completions.pid, item.idx, updatedInsertText);
+		}
+	}
+
+	public async handlePartialAccept(completions: IdentifiableInlineCompletions, item: IdentifiableInlineCompletion, acceptedCharacters: number, info: languages.PartialAcceptInfo): Promise<void> {
+		if (this._supportsHandleEvents) {
+			await this._proxy.$handleInlineCompletionPartialAccept(this.handle, completions.pid, item.idx, acceptedCharacters, info);
+		}
+	}
+
+	public async handleEndOfLifetime(completions: IdentifiableInlineCompletions, item: IdentifiableInlineCompletion, reason: languages.InlineCompletionEndOfLifeReason<IdentifiableInlineCompletion>, lifetimeSummary: languages.LifetimeSummary): Promise<void> {
+		function mapReason<T1, T2>(reason: languages.InlineCompletionEndOfLifeReason<T1>, f: (reason: T1) => T2): languages.InlineCompletionEndOfLifeReason<T2> {
+			if (reason.kind === languages.InlineCompletionEndOfLifeReasonKind.Ignored) {
+				return {
+					...reason,
+					supersededBy: reason.supersededBy ? f(reason.supersededBy) : undefined,
+				};
+			}
+			return reason;
+		}
+
+		if (this._supportsHandleEvents) {
+			await this._proxy.$handleInlineCompletionEndOfLifetime(this.handle, completions.pid, item.idx, mapReason(reason, i => ({ pid: completions.pid, idx: i.idx })));
+		}
+
+		if (reason.kind === languages.InlineCompletionEndOfLifeReasonKind.Accepted) {
+			if (item.suggestionId !== undefined) {
+				this._aiEditTelemetryService.handleCodeAccepted({
+					suggestionId: item.suggestionId,
+					feature: 'inlineSuggestion',
+					source: this.providerId,
+					languageId: completions.languageId,
+					editDeltaInfo: EditDeltaInfo.tryCreate(
+						lifetimeSummary.lineCountModified,
+						lifetimeSummary.lineCountOriginal,
+						lifetimeSummary.characterCountModified,
+						lifetimeSummary.characterCountOriginal,
+					),
+					modeId: undefined,
+					modelId: undefined,
+					presentation: item.isInlineEdit ? 'nextEditSuggestion' : 'inlineCompletion',
+					acceptanceMethod: 'accept',
+					applyCodeBlockSuggestionId: undefined,
+				});
+			}
+		}
+
+		const endOfLifeSummary: InlineCompletionEndOfLifeEvent = {
+			opportunityId: lifetimeSummary.requestUuid,
+			correlationId: lifetimeSummary.correlationId,
+			shown: lifetimeSummary.shown,
+			shownDuration: lifetimeSummary.shownDuration,
+			shownDurationUncollapsed: lifetimeSummary.shownDurationUncollapsed,
+			timeUntilShown: lifetimeSummary.timeUntilShown,
+			timeUntilProviderRequest: lifetimeSummary.timeUntilProviderRequest,
+			timeUntilProviderResponse: lifetimeSummary.timeUntilProviderResponse,
+			editorType: lifetimeSummary.editorType,
+			viewKind: lifetimeSummary.viewKind,
+			preceeded: lifetimeSummary.preceeded,
+			requestReason: lifetimeSummary.requestReason,
+			typingInterval: lifetimeSummary.typingInterval,
+			typingIntervalCharacterCount: lifetimeSummary.typingIntervalCharacterCount,
+			languageId: lifetimeSummary.languageId,
+			cursorColumnDistance: lifetimeSummary.cursorColumnDistance,
+			cursorLineDistance: lifetimeSummary.cursorLineDistance,
+			lineCountOriginal: lifetimeSummary.lineCountOriginal,
+			lineCountModified: lifetimeSummary.lineCountModified,
+			characterCountOriginal: lifetimeSummary.characterCountOriginal,
+			characterCountModified: lifetimeSummary.characterCountModified,
+			disjointReplacements: lifetimeSummary.disjointReplacements,
+			sameShapeReplacements: lifetimeSummary.sameShapeReplacements,
+			selectedSuggestionInfo: lifetimeSummary.selectedSuggestionInfo,
+			extensionId: this.providerId.extensionId!,
+			extensionVersion: this.providerId.extensionVersion!,
+			groupId: extractEngineFromCorrelationId(lifetimeSummary.correlationId) ?? this.groupId,
+			skuPlan: lifetimeSummary.skuPlan,
+			skuType: lifetimeSummary.skuType,
+			performanceMarkers: lifetimeSummary.performanceMarkers,
+			availableProviders: lifetimeSummary.availableProviders,
+			partiallyAccepted: lifetimeSummary.partiallyAccepted,
+			partiallyAcceptedCountSinceOriginal: lifetimeSummary.partiallyAcceptedCountSinceOriginal,
+			partiallyAcceptedRatioSinceOriginal: lifetimeSummary.partiallyAcceptedRatioSinceOriginal,
+			partiallyAcceptedCharactersSinceOriginal: lifetimeSummary.partiallyAcceptedCharactersSinceOriginal,
+			superseded: reason.kind === InlineCompletionEndOfLifeReasonKind.Ignored && !!reason.supersededBy,
+			reason: reason.kind === InlineCompletionEndOfLifeReasonKind.Accepted ? 'accepted'
+				: reason.kind === InlineCompletionEndOfLifeReasonKind.Rejected ? 'rejected'
+					: reason.kind === InlineCompletionEndOfLifeReasonKind.Ignored ? 'ignored' : undefined,
+			acceptedAlternativeAction: reason.kind === InlineCompletionEndOfLifeReasonKind.Accepted && reason.alternativeAction,
+			noSuggestionReason: undefined,
+			notShownReason: lifetimeSummary.notShownReason,
+			renameCreated: lifetimeSummary.renameCreated,
+			renameDuration: lifetimeSummary.renameDuration,
+			renameTimedOut: lifetimeSummary.renameTimedOut,
+			renameDroppedOtherEdits: lifetimeSummary.renameDroppedOtherEdits,
+			renameDroppedRenameEdits: lifetimeSummary.renameDroppedRenameEdits,
+			editKind: lifetimeSummary.editKind,
+			longDistanceHintVisible: lifetimeSummary.longDistanceHintVisible,
+			longDistanceHintDistance: lifetimeSummary.longDistanceHintDistance,
+			...forwardToChannelIf(isCopilotLikeExtension(this.providerId.extensionId!)),
+		};
+
+		const dataChannelForwardingTelemetryService = this._instantiationService.createInstance(DataChannelForwardingTelemetryService);
+		sendInlineCompletionsEndOfLifeTelemetry(dataChannelForwardingTelemetryService, endOfLifeSummary);
+	}
+
+	public disposeInlineCompletions(completions: IdentifiableInlineCompletions, reason: languages.InlineCompletionsDisposeReason): void {
+		this._proxy.$freeInlineCompletionsList(this.handle, completions.pid, reason);
+	}
+
+	public async handleRejection(completions: IdentifiableInlineCompletions, item: IdentifiableInlineCompletion): Promise<void> {
+		if (this._supportsHandleEvents) {
+			await this._proxy.$handleInlineCompletionRejection(this.handle, completions.pid, item.idx);
+		}
+	}
+
+	override toString() {
+		return `InlineCompletionsProvider(${this.providerId.toString()})`;
+	}
+}
+
+function extractEngineFromCorrelationId(correlationId: string | undefined): string | undefined {
+	if (!correlationId) {
+		return undefined;
+	}
+	try {
+		const parsed = JSON.parse(correlationId);
+		if (typeof parsed === 'object' && parsed !== null && typeof parsed.engine === 'string') {
+			return parsed.engine;
+		}
+		return undefined;
+	} catch {
+		return undefined;
+	}
+}
