@@ -31,6 +31,7 @@ import { localize } from '../../../../../nls.js';
 import { MenuId } from '../../../../../platform/actions/common/actions.js';
 import { IConfigurationService } from '../../../../../platform/configuration/common/configuration.js';
 import { ContextKeyExpr, IContextKey, IContextKeyService } from '../../../../../platform/contextkey/common/contextkey.js';
+import { IDialogService } from '../../../../../platform/dialogs/common/dialogs.js';
 
 import { ITextResourceEditorInput } from '../../../../../platform/editor/common/editor.js';
 import { IInstantiationService } from '../../../../../platform/instantiation/common/instantiation.js';
@@ -54,7 +55,7 @@ import { IChatModel, IChatModelInputState, IChatResponseModel } from '../../comm
 import { ChatMode, IChatModeService } from '../../common/chatModes.js';
 import { chatAgentLeader, ChatRequestAgentPart, ChatRequestDynamicVariablePart, ChatRequestSlashPromptPart, ChatRequestToolPart, ChatRequestToolSetPart, chatSubcommandLeader, formatChatQuestion, IParsedChatRequest } from '../../common/requestParser/chatParserTypes.js';
 import { ChatRequestParser } from '../../common/requestParser/chatRequestParser.js';
-import { IChatLocationData, IChatSendRequestOptions, IChatService } from '../../common/chatService/chatService.js';
+import { ChatRequestQueueKind, ChatSendResult, IChatLocationData, IChatSendRequestOptions, IChatService } from '../../common/chatService/chatService.js';
 import { IChatSessionsService } from '../../common/chatSessionsService.js';
 import { IChatSlashCommandService } from '../../common/participants/chatSlashCommands.js';
 import { IChatTodoListService } from '../../common/tools/chatTodoListService.js';
@@ -65,7 +66,7 @@ import { ChatAgentLocation, ChatConfiguration, ChatModeKind } from '../../common
 import { ILanguageModelToolsService, isToolSet } from '../../common/tools/languageModelToolsService.js';
 import { ComputeAutomaticInstructions } from '../../common/promptSyntax/computeAutomaticInstructions.js';
 import { PromptsConfig } from '../../common/promptSyntax/config/config.js';
-import { IHandOff, PromptHeader, Target } from '../../common/promptSyntax/promptFileParser.js';
+import { IHandOff, PromptHeader } from '../../common/promptSyntax/promptFileParser.js';
 import { IPromptsService } from '../../common/promptSyntax/service/promptsService.js';
 import { handleModeSwitch } from '../actions/chatActions.js';
 import { ChatTreeItem, IChatAcceptInputOptions, IChatAccessibilityService, IChatCodeBlockInfo, IChatFileTreeInfo, IChatListItemRendererOptions, IChatWidget, IChatWidgetService, IChatWidgetViewContext, IChatWidgetViewModelChangeEvent, IChatWidgetViewOptions, isIChatResourceViewContext, isIChatViewViewContext } from '../chat.js';
@@ -237,7 +238,6 @@ export class ChatWidget extends Disposable implements IChatWidget {
 	private visibleChangeCount = 0;
 	private requestInProgress: IContextKey<boolean>;
 	private agentInInput: IContextKey<boolean>;
-	private currentRequest: Promise<void> | undefined;
 
 	private _visible = false;
 	get visible() { return this._visible; }
@@ -257,6 +257,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 	private readonly _lockedToCodingAgentContextKey: IContextKey<boolean>;
 	private readonly _agentSupportsAttachmentsContextKey: IContextKey<boolean>;
 	private readonly _sessionIsEmptyContextKey: IContextKey<boolean>;
+	private readonly _hasPendingRequestsContextKey: IContextKey<boolean>;
 	private _attachmentCapabilities: IChatAgentAttachmentCapabilities = supportsAllAttachments;
 
 	// Cache for prompt file descriptions to avoid async calls during rendering
@@ -288,7 +289,6 @@ export class ChatWidget extends Disposable implements IChatWidget {
 			this.logService.debug('ChatWidget#setViewModel: no viewModel');
 		}
 
-		this.currentRequest = undefined;
 		this._onDidChangeViewModel.fire({ previousSessionResource, currentSessionResource: this._viewModel?.sessionResource });
 	}
 
@@ -344,6 +344,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 		@ICodeEditorService private readonly codeEditorService: ICodeEditorService,
 		@IEditorService private readonly editorService: IEditorService,
 		@IConfigurationService private readonly configurationService: IConfigurationService,
+		@IDialogService private readonly dialogService: IDialogService,
 		@IContextKeyService private readonly contextKeyService: IContextKeyService,
 		@IInstantiationService private readonly instantiationService: IInstantiationService,
 		@IChatService private readonly chatService: IChatService,
@@ -371,6 +372,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 		this._lockedToCodingAgentContextKey = ChatContextKeys.lockedToCodingAgent.bindTo(this.contextKeyService);
 		this._agentSupportsAttachmentsContextKey = ChatContextKeys.agentSupportsAttachments.bindTo(this.contextKeyService);
 		this._sessionIsEmptyContextKey = ChatContextKeys.chatSessionIsEmpty.bindTo(this.contextKeyService);
+		this._hasPendingRequestsContextKey = ChatContextKeys.hasPendingRequests.bindTo(this.contextKeyService);
 
 		this.viewContext = viewContext ?? {};
 
@@ -887,17 +889,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 	 */
 	private async _checkForAgentInstructionFiles(): Promise<boolean> {
 		try {
-			const useCopilotInstructionsFiles = this.configurationService.getValue(PromptsConfig.USE_COPILOT_INSTRUCTION_FILES);
-			const useAgentMd = this.configurationService.getValue(PromptsConfig.USE_AGENT_MD);
-			if (!useCopilotInstructionsFiles && !useAgentMd) {
-				// If both settings are disabled, return true to hide the hint (since the features aren't enabled)
-				return true;
-			}
-			return (
-				(await this.promptsService.listCopilotInstructionsMDs(CancellationToken.None)).length > 0 ||
-				// Note: only checking for AGENTS.md files at the root folder, not ones in subfolders.
-				(await this.promptsService.listAgentMDs(CancellationToken.None, false)).length > 0
-			);
+			return (await this.promptsService.listAgentInstructions(CancellationToken.None)).length > 0;
 		} catch (error) {
 			// On error, assume no instruction files exist to be safe
 			this.logService.warn('[ChatWidget] Error checking for instruction files:', error);
@@ -1477,9 +1469,9 @@ export class ChatWidget extends Disposable implements IChatWidget {
 				if (request.id === currentElement.id) {
 					request.setShouldBeBlocked(false); // unblocking just this request.
 					request.attachedContext?.forEach(addToContext);
-					currentElement.variables.forEach(addToContext);
 				}
 			}
+			currentElement.variables.forEach(addToContext);
 
 			// set states
 			this.viewModel?.setEditing(currentElement);
@@ -1487,8 +1479,9 @@ export class ChatWidget extends Disposable implements IChatWidget {
 				ChatContextKeys.currentlyEditing.bindTo(item.contextKeyService).set(true);
 			}
 
+			const isEditingSentRequest = currentElement.pendingKind === undefined ? ChatContextKeys.EditingRequestType.Sent : ChatContextKeys.EditingRequestType.QueueOrSteer;
 			const isInput = this.configurationService.getValue<string>('chat.editRequests') === 'input';
-			this.inputPart?.setEditing(!!this.viewModel?.editing && isInput);
+			this.inputPart?.setEditing(!!this.viewModel?.editing && isInput, isEditingSentRequest);
 
 			if (!isInput) {
 				const rowContainer = item.rowContainer;
@@ -1496,6 +1489,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 				rowContainer.appendChild(this.inputContainer);
 				this.createInput(this.inputContainer);
 				this.input.setChatMode(this.inputPart.currentModeObs.get().id);
+				this.input.setEditing(true, isEditingSentRequest);
 			} else {
 				this.inputPart.element.classList.add('editing');
 			}
@@ -1505,7 +1499,6 @@ export class ChatWidget extends Disposable implements IChatWidget {
 				this.input.attachmentModel.addContext(...currentContext);
 			}
 
-
 			// rerenders
 			this.inputPart.dnd.setDisabledOverlay(!isInput);
 			this.input.renderAttachedContext();
@@ -1586,8 +1579,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 			this.inputPart.element.classList.remove('editing');
 		}
 		this.viewModel?.setEditing(undefined);
-
-		this.inputPart?.setEditing(!!this.viewModel?.editing && isInput);
+		this.inputPart?.setEditing(false, undefined);
 
 		this.onDidChangeItems();
 
@@ -1780,6 +1772,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 			}
 			this.viewModel = undefined;
 			this.onDidChangeItems();
+			this._hasPendingRequestsContextKey.set(false);
 			return;
 		}
 
@@ -1843,6 +1836,12 @@ export class ChatWidget extends Disposable implements IChatWidget {
 			this.onDidChangeItems();
 		}));
 		this._sessionIsEmptyContextKey.set(model.getRequests().length === 0);
+		const updatePendingRequestKeys = () => {
+			const pendingCount = model.getPendingRequests().length;
+			this._hasPendingRequestsContextKey.set(pendingCount > 0);
+		};
+		updatePendingRequestKeys();
+		this.viewModelDisposables.add(model.onDidChangePendingRequests(() => updatePendingRequestKeys()));
 
 		this.refreshParsedInput();
 		this.viewModelDisposables.add(model.onDidChange((e) => {
@@ -2031,11 +2030,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 		}
 	}
 
-	private async _acceptInput(query: { query: string } | undefined, options?: IChatAcceptInputOptions): Promise<IChatResponseModel | undefined> {
-		if (this.viewModel?.model.requestInProgress.get()) {
-			return;
-		}
-
+	private async _acceptInput(query: { query: string } | undefined, options: IChatAcceptInputOptions = {}): Promise<IChatResponseModel | undefined> {
 		if (!query && this.input.generating) {
 			// if the user submits the input and generation finishes quickly, just submit it for them
 			const generatingAutoSubmitWindow = 500;
@@ -2073,12 +2068,31 @@ export class ChatWidget extends Disposable implements IChatWidget {
 		};
 
 		const isUserQuery = !query;
+		const isEditing = this.viewModel?.editing;
+		if (isEditing) {
+			const editingPendingRequest = this.viewModel.editing!.pendingKind;
+			if (editingPendingRequest !== undefined) {
+				const editingRequestId = this.viewModel.editing!.id;
+				this.chatService.removePendingRequest(this.viewModel.sessionResource, editingRequestId);
+				options.queue ??= editingPendingRequest;
+			} else {
+				this.chatService.cancelCurrentRequestForSession(this.viewModel.sessionResource);
+				options.queue = undefined;
+			}
 
-		if (this.viewModel?.editing) {
 			this.finishedEditing(true);
 			this.viewModel.model?.setCheckpoint(undefined);
 		}
 
+		const model = this.viewModel.model;
+		const requestInProgress = model.requestInProgress.get();
+		if (requestInProgress) {
+			options.queue ??= ChatRequestQueueKind.Queued;
+		}
+		if (!requestInProgress && !isEditing && !(await this.confirmPendingRequestsBeforeSend(model, options))) {
+			return;
+		}
+
 		// process the prompt command
 		await this._applyPromptFileIfSet(requestInputs);
 		await this._autoAttachInstructions(requestInputs);
@@ -2114,12 +2128,6 @@ export class ChatWidget extends Disposable implements IChatWidget {
 			};
 			this.telemetryService.publicLog2<ChatEditingWorkingSetEvent, ChatEditingWorkingSetClassification>('chatEditing/workingSetSize', { originalSize: uniqueWorkingSetEntries.size, actualSize: uniqueWorkingSetEntries.size });
 		}
-		this.chatService.cancelCurrentRequestForSession(this.viewModel.sessionResource);
-		if (this.currentRequest) {
-			// We have to wait the current request to be properly cancelled so that it has a chance to update the model with its result metadata.
-			// This is awkward, it's basically a limitation of the chat provider-based agent.
-			await Promise.race([this.currentRequest, timeout(1000)]);
-		}
 
 		this.input.validateAgentMode();
 
@@ -2132,7 +2140,8 @@ export class ChatWidget extends Disposable implements IChatWidget {
 				}
 			}
 		}
-		if (this.viewModel.sessionResource) {
+		if (this.viewModel.sessionResource && !options.queue) {
+			// todo@connor4312: move chatAccessibilityService.acceptRequest to a refcount model to handle queue messages
 			this.chatAccessibilityService.acceptRequest(this._viewModel!.sessionResource);
 		}
 
@@ -2146,20 +2155,29 @@ export class ChatWidget extends Disposable implements IChatWidget {
 			...this.getModeRequestOptions(),
 			modeInfo: this.input.currentModeInfo,
 			agentIdSilent: this._lockedAgent?.id,
+			queue: options?.queue,
 		});
 
-		if (!result) {
+		if (this.viewModel.sessionResource && !options.queue) {
 			this.chatAccessibilityService.disposeRequest(this.viewModel.sessionResource);
+		}
+
+		if (ChatSendResult.isRejected(result)) {
 			return;
 		}
 
-		// visibility sync before we accept input to hide the welcome view
+		// visibility sync before firing events to hide the welcome view
 		this.updateChatViewVisibility();
-
 		this.input.acceptInput(options?.storeToHistory ?? isUserQuery);
-		this._onDidSubmitAgent.fire({ agent: result.agent, slashCommand: result.slashCommand });
-		this.handleDelegationExitIfNeeded(this._lockedAgent, result.agent);
-		this.currentRequest = result.responseCompletePromise.then(() => {
+
+		const sent = ChatSendResult.isQueued(result) ? await result.deferred : result;
+		if (!ChatSendResult.isSent(sent)) {
+			return;
+		}
+
+		this._onDidSubmitAgent.fire({ agent: sent.data.agent, slashCommand: sent.data.slashCommand });
+		this.handleDelegationExitIfNeeded(this._lockedAgent, sent.data.agent);
+		sent.data.responseCompletePromise.then(() => {
 			const responses = this.viewModel?.getItems().filter(isResponseVM);
 			const lastResponse = responses?.[responses.length - 1];
 			this.chatAccessibilityService.acceptResponse(this, this.container, lastResponse, this.viewModel?.sessionResource, options?.isVoiceInput);
@@ -2170,10 +2188,49 @@ export class ChatWidget extends Disposable implements IChatWidget {
 					this.input.setValue(question, false);
 				}
 			}
-			this.currentRequest = undefined;
 		});
 
-		return result.responseCreatedPromise;
+		return sent.data.responseCreatedPromise;
+	}
+
+	private async confirmPendingRequestsBeforeSend(model: IChatModel, options: IChatAcceptInputOptions): Promise<boolean> {
+		if (options.queue) {
+			return true;
+		}
+
+		const hasPendingRequests = model.getPendingRequests().length > 0;
+		if (!hasPendingRequests) {
+			return true;
+		}
+
+		const promptResult = await this.dialogService.prompt({
+			type: 'question',
+			message: localize('chat.pendingRequests.prompt.message', "You already have pending requests."),
+			detail: localize('chat.pendingRequests.prompt.detail', "Do you want to keep them in the queue or remove them before sending this message?"),
+			buttons: [
+				{
+					label: localize('chat.pendingRequests.prompt.keep', "Keep Pending Requests"),
+					run: () => 'keep'
+				},
+				{
+					label: localize('chat.pendingRequests.prompt.remove', "Remove Pending Requests"),
+					run: () => 'remove'
+				}
+			],
+			cancelButton: true
+		});
+
+		if (!promptResult.result) {
+			return false;
+		}
+
+		if (promptResult.result === 'remove') {
+			for (const pendingRequest of [...model.getPendingRequests()]) {
+				this.chatService.removePendingRequest(model.sessionResource, pendingRequest.request.id);
+			}
+		}
+
+		return true;
 	}
 
 	getModeRequestOptions(): Partial<IChatSendRequestOptions> {
@@ -2382,7 +2439,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 
 		// if not tools to enable are present, we are done
 		if (tools !== undefined && this.input.currentModeKind === ChatModeKind.Agent) {
-			const enablementMap = this.toolsService.toToolAndToolSetEnablementMap(tools, Target.VSCode, this.input.selectedLanguageModel.get()?.metadata);
+			const enablementMap = this.toolsService.toToolAndToolSetEnablementMap(tools, this.input.selectedLanguageModel.get()?.metadata);
 			this.input.selectedToolsModel.set(enablementMap, true);
 		}
 
