@@ -14,6 +14,7 @@ import { ThemeIcon } from '../../../../../base/common/themables.js';
 import { hasKey } from '../../../../../base/common/types.js';
 import { URI, UriComponents } from '../../../../../base/common/uri.js';
 import { IRange, Range } from '../../../../../editor/common/core/range.js';
+import { HookTypeValue } from '../promptSyntax/hookSchema.js';
 import { ISelection } from '../../../../../editor/common/core/selection.js';
 import { Command, Location, TextEdit } from '../../../../../editor/common/languages.js';
 import { FileType } from '../../../../../platform/files/common/files.js';
@@ -407,6 +408,26 @@ export interface IChatThinkingPart {
 	generatedTitle?: string;
 }
 
+/**
+ * A progress part representing the execution result of a hook.
+ * Aligned with the hook output JSON structure: { stopReason, systemMessage, hookSpecificOutput }.
+ * If {@link stopReason} is set, the hook blocked/denied the operation.
+ */
+export interface IChatHookPart {
+	kind: 'hook';
+	/** The type of hook that was executed */
+	hookType: HookTypeValue;
+	/** If set, the hook blocked processing. This message is shown to the user. */
+	stopReason?: string;
+	/** Warning/system message from the hook, shown to the user */
+	systemMessage?: string;
+	/** Display name of the tool that was affected by the hook */
+	toolDisplayName?: string;
+	metadata?: { readonly [key: string]: unknown };
+	/** If set, this hook was executed within a subagent invocation and should be grouped with it. */
+	subAgentInvocationId?: string;
+}
+
 export interface IChatTerminalToolInvocationData {
 	kind: 'terminal';
 	commandLine: {
@@ -600,6 +621,8 @@ export namespace IChatToolInvocation {
 	interface IChatToolInvocationCancelledState extends IChatToolInvocationStateBase, IChatToolInvocationPostStreamState {
 		type: StateKind.Cancelled;
 		reason: ToolConfirmKind.Denied | ToolConfirmKind.Skipped;
+		/** Optional message explaining why the tool was cancelled (e.g., from hook denial) */
+		reasonMessage?: string | IMarkdownString;
 	}
 
 	export type State =
@@ -816,6 +839,7 @@ export interface IChatSubagentToolInvocationData {
 	agentName?: string;
 	prompt?: string;
 	result?: string;
+	modelName?: string;
 }
 
 export interface IChatTodoListContent {
@@ -840,6 +864,10 @@ export interface IChatMcpServersStartingSerialized {
 	didStartServerIds?: string[];
 }
 
+export interface IChatDisabledClaudeHooksPart {
+	readonly kind: 'disabledClaudeHooks';
+}
+
 export class ChatMcpServersStarting implements IChatMcpServersStarting {
 	public readonly kind = 'mcpServersStarting';
 
@@ -902,7 +930,9 @@ export type IChatProgress =
 	| IChatElicitationRequest
 	| IChatElicitationRequestSerialized
 	| IChatMcpServersStarting
-	| IChatMcpServersStartingSerialized;
+	| IChatMcpServersStartingSerialized
+	| IChatHookPart
+	| IChatDisabledClaudeHooksPart;
 
 export interface IChatFollowup {
 	kind: 'reply';
@@ -1141,6 +1171,54 @@ export interface IChatSendRequestData extends IChatSendRequestResponseState {
 	slashCommand?: IChatAgentCommand;
 }
 
+/**
+ * Result of a sendRequest call - a discriminated union of possible outcomes.
+ */
+export type ChatSendResult =
+	| ChatSendResultRejected
+	| ChatSendResultSent
+	| ChatSendResultQueued;
+
+export interface ChatSendResultRejected {
+	readonly kind: 'rejected';
+	readonly reason: string;
+}
+
+export interface ChatSendResultSent {
+	readonly kind: 'sent';
+	readonly data: IChatSendRequestData;
+}
+
+export interface ChatSendResultQueued {
+	readonly kind: 'queued';
+	/**
+	 * Promise that resolves when the queued message is actually processed.
+	 * Will resolve to a 'sent' or 'rejected' result.
+	 */
+	readonly deferred: Promise<ChatSendResult>;
+}
+
+export namespace ChatSendResult {
+	export function isSent(result: ChatSendResult): result is ChatSendResultSent {
+		return result.kind === 'sent';
+	}
+
+	export function isRejected(result: ChatSendResult): result is ChatSendResultRejected {
+		return result.kind === 'rejected';
+	}
+
+	export function isQueued(result: ChatSendResult): result is ChatSendResultQueued {
+		return result.kind === 'queued';
+	}
+
+	/** Assertion function for tests - asserts that the result is a sent result */
+	export function assertSent(result: ChatSendResult): asserts result is ChatSendResultSent {
+		if (result.kind !== 'sent') {
+			throw new Error(`Expected ChatSendResult to be 'sent', but was '${result.kind}'`);
+		}
+	}
+}
+
 export interface IChatEditorLocationData {
 	type: ChatAgentLocation.EditorInline;
 	id: string;
@@ -1161,6 +1239,16 @@ export interface IChatTerminalLocationData {
 
 export type IChatLocationData = IChatEditorLocationData | IChatNotebookLocationData | IChatTerminalLocationData;
 
+/**
+ * The kind of queue request.
+ */
+export const enum ChatRequestQueueKind {
+	/** Request is queued to be sent after current request completes */
+	Queued = 'queued',
+	/** Request is queued and signals the active request to yield */
+	Steering = 'steering'
+}
+
 export interface IChatSendRequestOptions {
 	modeInfo?: IChatRequestModeInfo;
 	userSelectedModelId?: string;
@@ -1187,6 +1275,12 @@ export interface IChatSendRequestOptions {
 	 */
 	confirmation?: string;
 
+	/**
+	 * When set, queues this message to be sent after the current request completes.
+	 * If Steering, also sets yieldRequested on any active request to signal it should wrap up.
+	 */
+	queue?: ChatRequestQueueKind;
+
 }
 
 export type IChatModelReference = IReference<IChatModel>;
@@ -1228,9 +1322,10 @@ export interface IChatService {
 	getChatSessionFromInternalUri(sessionResource: URI): IChatSessionContext | undefined;
 
 	/**
-	 * Returns whether the request was accepted.`
+	 * Sends a chat request for the given session.
+	 * @returns A result indicating whether the request was sent, queued, or rejected.
 	 */
-	sendRequest(sessionResource: URI, message: string, options?: IChatSendRequestOptions): Promise<IChatSendRequestData | undefined>;
+	sendRequest(sessionResource: URI, message: string, options?: IChatSendRequestOptions): Promise<ChatSendResult>;
 
 	/**
 	 * Sets a custom title for a chat model.
@@ -1241,6 +1336,26 @@ export interface IChatService {
 	adoptRequest(sessionResource: URI, request: IChatRequestModel): Promise<void>;
 	removeRequest(sessionResource: URI, requestId: string): Promise<void>;
 	cancelCurrentRequestForSession(sessionResource: URI): void;
+	/**
+	 * Sets yieldRequested on the active request for the given session.
+	 */
+	setYieldRequested(sessionResource: URI): void;
+	/**
+	 * Removes a pending request from the session's queue.
+	 */
+	removePendingRequest(sessionResource: URI, requestId: string): void;
+	/**
+	 * Sets the pending requests for a session, allowing for deletions/reordering.
+	 * Adding new requests should go through sendRequest with the queue option.
+	 */
+	setPendingRequests(sessionResource: URI, requests: readonly { requestId: string; kind: ChatRequestQueueKind }[]): void;
+	/**
+	 * Ensures pending requests for the session are processing. If restoring from
+	 * storage or after an error, pending requests may be present without an
+	 * active chat message 'loop' happening. THis triggers the loop to happen
+	 * as needed. Idempotent, safe to call at any time.
+	 */
+	processPendingRequests(sessionResource: URI): void;
 	addCompleteRequest(sessionResource: URI, message: IParsedChatRequest | string, variableData: IChatRequestVariableData | undefined, attempt: number | undefined, response: IChatCompleteResponse): void;
 	setChatSessionTitle(sessionResource: URI, title: string): void;
 	getLocalSessionHistory(): Promise<IChatDetail[]>;
