@@ -18,7 +18,7 @@ import { ChatAgentLocation, ChatConfiguration } from '../../../../common/constan
 import { ILanguageModelToolsService, IToolData, ToolDataSource } from '../../../../common/tools/languageModelToolsService.js';
 import { ILanguageModelChatMetadata, ILanguageModelsService } from '../../../../common/languageModels.js';
 import { PromptHoverProvider } from '../../../../common/promptSyntax/languageProviders/promptHovers.js';
-import { IPromptsService, PromptsStorage } from '../../../../common/promptSyntax/service/promptsService.js';
+import { IPromptsService, PromptsStorage, Target } from '../../../../common/promptSyntax/service/promptsService.js';
 import { MockChatModeService } from '../../../common/mockChatModeService.js';
 import { createTextModel } from '../../../../../../../editor/test/common/testTextModel.js';
 import { URI } from '../../../../../../../base/common/uri.js';
@@ -55,6 +55,10 @@ suite('PromptHoverProvider', () => {
 		const testModels: ILanguageModelChatMetadata[] = [
 			{ id: 'mae-4', name: 'MAE 4', vendor: 'olama', version: '1.0', family: 'mae', modelPickerCategory: undefined, extension: new ExtensionIdentifier('a.b'), isUserSelectable: true, maxInputTokens: 8192, maxOutputTokens: 1024, capabilities: { agentMode: true, toolCalling: true }, isDefaultForLocation: { [ChatAgentLocation.Chat]: true } } satisfies ILanguageModelChatMetadata,
 			{ id: 'mae-4.1', name: 'MAE 4.1', vendor: 'copilot', version: '1.0', family: 'mae', modelPickerCategory: undefined, extension: new ExtensionIdentifier('a.b'), isUserSelectable: true, maxInputTokens: 8192, maxOutputTokens: 1024, capabilities: { agentMode: true, toolCalling: true }, isDefaultForLocation: { [ChatAgentLocation.Chat]: true } } satisfies ILanguageModelChatMetadata,
+			// Claude model equivalents
+			{ id: 'claude-sonnet-4.5', name: 'Claude Sonnet 4.5', vendor: 'copilot', version: '1.0', family: 'claude', modelPickerCategory: undefined, extension: new ExtensionIdentifier('a.b'), isUserSelectable: true, maxInputTokens: 200000, maxOutputTokens: 8192, capabilities: { agentMode: true, toolCalling: true }, isDefaultForLocation: {} } satisfies ILanguageModelChatMetadata,
+			{ id: 'claude-opus-4.6', name: 'Claude Opus 4.6', vendor: 'copilot', version: '1.0', family: 'claude', modelPickerCategory: undefined, extension: new ExtensionIdentifier('a.b'), isUserSelectable: true, maxInputTokens: 200000, maxOutputTokens: 8192, capabilities: { agentMode: true, toolCalling: true }, isDefaultForLocation: {} } satisfies ILanguageModelChatMetadata,
+			{ id: 'claude-haiku-4.5', name: 'Claude Haiku 4.5', vendor: 'copilot', version: '1.0', family: 'claude', modelPickerCategory: undefined, extension: new ExtensionIdentifier('a.b'), isUserSelectable: true, maxInputTokens: 200000, maxOutputTokens: 8192, capabilities: { agentMode: true, toolCalling: true }, isDefaultForLocation: {} } satisfies ILanguageModelChatMetadata,
 		];
 
 		instaService.stub(ILanguageModelsService, {
@@ -74,6 +78,7 @@ suite('PromptHoverProvider', () => {
 			name: 'BeastMode',
 			agentInstructions: { content: 'Beast mode instructions', toolReferences: [] },
 			source: { storage: PromptsStorage.local },
+			target: Target.Undefined,
 			visibility: { userInvokable: true, agentInvokable: true }
 		});
 		instaService.stub(IChatModeService, new MockChatModeService({ builtin: [ChatMode.Agent, ChatMode.Ask, ChatMode.Edit], custom: [customChatMode] }));
@@ -88,9 +93,11 @@ suite('PromptHoverProvider', () => {
 		hoverProvider = instaService.createInstance(PromptHoverProvider);
 	});
 
-	async function getHover(content: string, line: number, column: number, promptType: PromptsType): Promise<string | undefined> {
+	async function getHover(content: string, line: number, column: number, promptType: PromptsType, options?: { claudeAgent?: boolean }): Promise<string | undefined> {
 		const languageId = getLanguageIdForPromptsType(promptType);
-		const uri = URI.parse('test:///test' + getPromptFileExtension(promptType));
+		const ext = getPromptFileExtension(promptType);
+		const path = options?.claudeAgent ? `/.claude/agents/test${ext}` : `/test${ext}`;
+		const uri = URI.parse('test://' + path);
 		const model = disposables.add(createTextModel(content, languageId, undefined, uri));
 		const position = new Position(line, column);
 		const hover = await hoverProvider.provideHover(model, position, CancellationToken.None);
@@ -168,7 +175,7 @@ suite('PromptHoverProvider', () => {
 			const expected = [
 				'Possible handoff actions when the agent has completed its task.',
 				'',
-				'Note: This attribute is not used when target is github-copilot.'
+				'Note: This attribute is not used in GitHub Copilot or Claude targets.'
 			].join('\n');
 			assert.strictEqual(hover, expected);
 		});
@@ -495,4 +502,281 @@ suite('PromptHoverProvider', () => {
 			assert.strictEqual(hover, undefined);
 		});
 	});
+
+	suite('claude agent hovers', () => {
+		// Helper that creates a hover in a Claude agent file (URI under .claude/agents/)
+		async function getClaudeHover(content: string, line: number, column: number): Promise<string | undefined> {
+			return getHover(content, line, column, PromptsType.agent, { claudeAgent: true });
+		}
+
+		test('hover on name attribute shows Claude description', async () => {
+			const content = [
+				'---',
+				'name: security-reviewer',
+				'description: Reviews code for security vulnerabilities',
+				'---',
+			].join('\n');
+			const hover = await getClaudeHover(content, 2, 1);
+			assert.strictEqual(hover, 'Unique identifier using lowercase letters and hyphens (required)');
+		});
+
+		test('hover on description attribute shows Claude description', async () => {
+			const content = [
+				'---',
+				'name: security-reviewer',
+				'description: Reviews code for security vulnerabilities',
+				'---',
+			].join('\n');
+			const hover = await getClaudeHover(content, 3, 1);
+			assert.strictEqual(hover, 'When to delegate to this subagent (required)');
+		});
+
+		test('hover on tools attribute shows Claude description', async () => {
+			const content = [
+				'---',
+				'name: security-reviewer',
+				'description: Reviews code for security vulnerabilities',
+				'tools: Edit, Grep, AskUserQuestion, WebFetch',
+				'---',
+			].join('\n');
+			const hover = await getClaudeHover(content, 4, 1);
+			assert.strictEqual(hover, 'Array of tools the subagent can use. Inherits all tools if omitted');
+		});
+
+		test('hover on individual Claude tool shows tool description', async () => {
+			const content = [
+				'---',
+				'name: security-reviewer',
+				'description: Reviews code',
+				`tools: ['Edit', 'Grep', 'WebFetch']`,
+				'---',
+			].join('\n');
+			// Hover on 'Edit' tool
+			const hoverEdit = await getClaudeHover(content, 4, 10);
+			assert.strictEqual(hoverEdit, 'Make targeted file edits');
+
+			// Hover on 'Grep' tool
+			const hoverGrep = await getClaudeHover(content, 4, 17);
+			assert.strictEqual(hoverGrep, 'Search file contents with regex');
+
+			// Hover on 'WebFetch' tool
+			const hoverFetch = await getClaudeHover(content, 4, 27);
+			assert.strictEqual(hoverFetch, 'Fetch URL content');
+		});
+
+		test('hover on model attribute shows Claude description', async () => {
+			const content = [
+				'---',
+				'name: security-reviewer',
+				'description: Reviews code',
+				'model: opus',
+				'---',
+			].join('\n');
+			const hover = await getClaudeHover(content, 4, 1);
+			const expected = [
+				'Model to use: sonnet, opus, haiku, or inherit. Defaults to inherit.',
+				'',
+				'Claude model `opus` maps to the following model:',
+				'',
+				'- Name: Claude Opus 4.6',
+				'- Family: claude',
+				'- Vendor: copilot'
+			].join('\n');
+			assert.strictEqual(hover, expected);
+		});
+
+		test('hover on model attribute with sonnet value', async () => {
+			const content = [
+				'---',
+				'name: test-agent',
+				'description: Test',
+				'model: sonnet',
+				'---',
+			].join('\n');
+			const hover = await getClaudeHover(content, 4, 1);
+			const expected = [
+				'Model to use: sonnet, opus, haiku, or inherit. Defaults to inherit.',
+				'',
+				'Claude model `sonnet` maps to the following model:',
+				'',
+				'- Name: Claude Sonnet 4.5',
+				'- Family: claude',
+				'- Vendor: copilot'
+			].join('\n');
+			assert.strictEqual(hover, expected);
+		});
+
+		test('hover on model attribute with haiku value', async () => {
+			const content = [
+				'---',
+				'name: test-agent',
+				'description: Test',
+				'model: haiku',
+				'---',
+			].join('\n');
+			const hover = await getClaudeHover(content, 4, 1);
+			const expected = [
+				'Model to use: sonnet, opus, haiku, or inherit. Defaults to inherit.',
+				'',
+				'Claude model `haiku` maps to the following model:',
+				'',
+				'- Name: Claude Haiku 4.5',
+				'- Family: claude',
+				'- Vendor: copilot'
+			].join('\n');
+			assert.strictEqual(hover, expected);
+		});
+
+		test('hover on model attribute with inherit value', async () => {
+			const content = [
+				'---',
+				'name: test-agent',
+				'description: Test',
+				'model: inherit',
+				'---',
+			].join('\n');
+			const hover = await getClaudeHover(content, 4, 1);
+			const expected = [
+				'Model to use: sonnet, opus, haiku, or inherit. Defaults to inherit.',
+				'',
+				'Inherit model from parent agent or prompt'
+			].join('\n');
+			assert.strictEqual(hover, expected);
+		});
+
+		test('hover on disallowedTools attribute shows Claude description', async () => {
+			const content = [
+				'---',
+				'name: read-only-agent',
+				'description: Read-only analysis agent',
+				`disallowedTools: ['Write', 'Edit', 'Bash']`,
+				'---',
+			].join('\n');
+			const hover = await getClaudeHover(content, 4, 1);
+			assert.strictEqual(hover, 'Tools to deny, removed from inherited or specified list');
+		});
+
+		test('hover on individual disallowedTools value shows tool description', async () => {
+			const content = [
+				'---',
+				'name: read-only-agent',
+				'description: Read-only',
+				`disallowedTools: ['Bash', 'Write']`,
+				'---',
+			].join('\n');
+			// Hover on 'Bash' tool value
+			const hoverBash = await getClaudeHover(content, 4, 20);
+			assert.strictEqual(hoverBash, 'Execute shell commands');
+
+			// Hover on 'Write' tool value
+			const hoverWrite = await getClaudeHover(content, 4, 28);
+			assert.strictEqual(hoverWrite, 'Create/overwrite files');
+		});
+
+		test('hover on permissionMode attribute shows Claude description', async () => {
+			const content = [
+				'---',
+				'name: test-agent',
+				'description: Test',
+				'permissionMode: acceptEdits',
+				'---',
+			].join('\n');
+			const hover = await getClaudeHover(content, 4, 1);
+			assert.strictEqual(hover, 'Permission mode: default, acceptEdits, dontAsk, bypassPermissions, or plan.');
+		});
+
+		test('hover on memory attribute shows Claude description', async () => {
+			const content = [
+				'---',
+				'name: test-agent',
+				'description: Test',
+				'memory: project',
+				'---',
+			].join('\n');
+			const hover = await getClaudeHover(content, 4, 1);
+			assert.strictEqual(hover, 'Persistent memory scope: user, project, or local. Enables cross-session learning.');
+		});
+
+		test('hover on skills attribute shows Claude description', async () => {
+			const content = [
+				'---',
+				'name: test-agent',
+				'description: Test',
+				'skills: ["code-review"]',
+				'---',
+			].join('\n');
+			const hover = await getClaudeHover(content, 4, 1);
+			assert.strictEqual(hover, 'Skills to load into the subagent\'s context at startup.');
+		});
+
+		test('hover on hooks attribute shows Claude description', async () => {
+			const content = [
+				'---',
+				'name: test-agent',
+				'description: Test',
+				'hooks: {}',
+				'---',
+			].join('\n');
+			const hover = await getClaudeHover(content, 4, 1);
+			assert.strictEqual(hover, 'Lifecycle hooks scoped to this subagent.');
+		});
+
+		test('hover on handoffs attribute in Claude agent shows not-used note', async () => {
+			const content = [
+				'---',
+				'name: test-agent',
+				'description: Test',
+				'handoffs:',
+				'  - label: Test',
+				'    agent: Default',
+				'    prompt: Test',
+				'---',
+			].join('\n');
+			const hover = await getClaudeHover(content, 4, 1);
+			// handoffs is not a Claude attribute, so no hover should appear
+			assert.strictEqual(hover, undefined);
+		});
+
+		test('full example: hover on each attribute of a Claude agent', async () => {
+			// Realistic Claude agent file as user provided
+			const content = [
+				'---',
+				'name: security-reviewer',
+				'description: Reviews code for security vulnerabilities',
+				`tools: ['Edit', 'Grep', 'AskUserQuestion', 'WebFetch']`,
+				'model: opus',
+				'---',
+				'You are a senior security engineer.',
+			].join('\n');
+
+			// Hover on name (line 2)
+			const nameHover = await getClaudeHover(content, 2, 1);
+			assert.strictEqual(nameHover, 'Unique identifier using lowercase letters and hyphens (required)');
+
+			// Hover on description (line 3)
+			const descHover = await getClaudeHover(content, 3, 1);
+			assert.strictEqual(descHover, 'When to delegate to this subagent (required)');
+
+			// Hover on tools attribute key (line 4, column 1)
+			const toolsHover = await getClaudeHover(content, 4, 1);
+			assert.strictEqual(toolsHover, 'Array of tools the subagent can use. Inherits all tools if omitted');
+
+			// Hover on 'AskUserQuestion' tool value (line 4)
+			const askHover = await getClaudeHover(content, 4, 28);
+			assert.strictEqual(askHover, 'Ask multiple-choice questions');
+
+			// Hover on model value 'opus' (line 5)
+			const modelHover = await getClaudeHover(content, 5, 1);
+			const expectedModelHover = [
+				'Model to use: sonnet, opus, haiku, or inherit. Defaults to inherit.',
+				'',
+				'Claude model `opus` maps to the following model:',
+				'',
+				'- Name: Claude Opus 4.6',
+				'- Family: claude',
+				'- Vendor: copilot'
+			].join('\n');
+			assert.strictEqual(modelHover, expectedModelHover);
+		});
+	});
 });
