@@ -12,6 +12,7 @@ import { CancellationToken, CancellationTokenSource } from '../../../../base/com
 import { canceled } from '../../../../base/common/errors.js';
 import { Emitter, Event } from '../../../../base/common/event.js';
 import { normalizeDriveLetter } from '../../../../base/common/labels.js';
+import { Lazy } from '../../../../base/common/lazy.js';
 import { Disposable, DisposableMap, DisposableStore, MutableDisposable, dispose } from '../../../../base/common/lifecycle.js';
 import { mixin } from '../../../../base/common/objects.js';
 import * as platform from '../../../../base/common/platform.js';
@@ -65,7 +66,13 @@ export class DebugSession implements IDebugSession {
 	private cancellationMap = new Map<number, CancellationTokenSource[]>();
 	private readonly rawListeners = new DisposableStore();
 	private readonly globalDisposables = new DisposableStore();
-	private fetchThreadsScheduler: RunOnceScheduler | undefined;
+	private fetchThreadsScheduler = new Lazy(() => {
+		const inst = new RunOnceScheduler(() => {
+			this.fetchThreads();
+		}, 100);
+		this.rawListeners.add(inst);
+		return inst;
+	});
 	private passFocusScheduler: RunOnceScheduler;
 	private lastContinuedThreadId: number | undefined;
 	private repl: ReplModel;
@@ -566,8 +573,8 @@ export class DebugSession implements IDebugSession {
 		return this._dataBreakpointInfo({ name: address, bytes, asAddress: true });
 	}
 
-	dataBreakpointInfo(name: string, variablesReference?: number): Promise<{ dataId: string | null; description: string; canPersist?: boolean } | undefined> {
-		return this._dataBreakpointInfo({ name, variablesReference });
+	dataBreakpointInfo(name: string, variablesReference?: number, frameId?: number): Promise<{ dataId: string | null; description: string; canPersist?: boolean } | undefined> {
+		return this._dataBreakpointInfo({ name, variablesReference, frameId });
 	}
 
 	private async _dataBreakpointInfo(args: DebugProtocol.DataBreakpointInfoArguments): Promise<{ dataId: string | null; description: string; canPersist?: boolean } | undefined> {
@@ -1098,15 +1105,8 @@ export class DebugSession implements IDebugSession {
 		this.rawListeners.add(this.raw.onDidThread(event => {
 			statusQueue.cancel([event.body.threadId]);
 			if (event.body.reason === 'started') {
-				// debounce to reduce threadsRequest frequency and improve performance
-				if (!this.fetchThreadsScheduler) {
-					this.fetchThreadsScheduler = new RunOnceScheduler(() => {
-						this.fetchThreads();
-					}, 100);
-					this.rawListeners.add(this.fetchThreadsScheduler);
-				}
-				if (!this.fetchThreadsScheduler.isScheduled()) {
-					this.fetchThreadsScheduler.schedule();
+				if (!this.fetchThreadsScheduler.value.isScheduled()) {
+					this.fetchThreadsScheduler.value.schedule();
 				}
 			} else if (event.body.reason === 'exited') {
 				this.model.clearThreads(this.getId(), true, event.body.threadId);
@@ -1129,25 +1129,38 @@ export class DebugSession implements IDebugSession {
 			}
 		}));
 
-		this.rawListeners.add(this.raw.onDidContinued(event => {
+		this.rawListeners.add(this.raw.onDidContinued(async event => {
 			const allThreads = event.body.allThreadsContinued !== false;
 
-			statusQueue.cancel(allThreads ? undefined : [event.body.threadId]);
+			let affectedThreads: number[] | Promise<number[]>;
+			if (!allThreads) {
+				affectedThreads = [event.body.threadId];
+				if (this.threadIds.includes(event.body.threadId)) {
+					affectedThreads = [event.body.threadId];
+				} else {
+					this.fetchThreadsScheduler.rawValue?.cancel();
+					affectedThreads = this.fetchThreads().then(() => [event.body.threadId]);
+				}
+			} else if (this.fetchThreadsScheduler.value.isScheduled()) {
+				this.fetchThreadsScheduler.value.cancel();
+				affectedThreads = this.fetchThreads().then(() => this.threadIds);
+			} else {
+				affectedThreads = this.threadIds;
+			}
 
-			const threadId = allThreads ? undefined : event.body.threadId;
-			if (typeof threadId === 'number') {
+			statusQueue.cancel(allThreads ? undefined : [event.body.threadId]);
+			await statusQueue.run(affectedThreads, threadId => {
 				this.stoppedDetails = this.stoppedDetails.filter(sd => sd.threadId !== threadId);
 				const tokens = this.cancellationMap.get(threadId);
 				this.cancellationMap.delete(threadId);
 				tokens?.forEach(t => t.dispose(true));
-			} else {
-				this.stoppedDetails = [];
-				this.cancelAllRequests();
-			}
-			this.lastContinuedThreadId = threadId;
+				this.model.clearThreads(this.getId(), false, threadId);
+				return Promise.resolve();
+			});
+
 			// We need to pass focus to other sessions / threads with a timeout in case a quick stop event occurs #130321
+			this.lastContinuedThreadId = allThreads ? undefined : event.body.threadId;
 			this.passFocusScheduler.schedule();
-			this.model.clearThreads(this.getId(), false, threadId);
 			this._onDidChangeState.fire();
 		}));
 
@@ -1177,6 +1190,7 @@ export class DebugSession implements IDebugSession {
 
 					resolved.forEach((child) => {
 						// Since we can not display multiple trees in a row, we are displaying these variables one after the other (ignoring their names)
+						// eslint-disable-next-line local/code-no-any-casts
 						(<any>child).name = null;
 						this.appendToRepl({ output: '', expression: child, sev: outputSeverity, source }, event.body.category === 'important');
 					});
@@ -1316,9 +1330,15 @@ export class DebugSession implements IDebugSession {
 				this.cancelAllRequests();
 				this.model.clearThreads(this.getId(), true);
 
-				const details = this.stoppedDetails;
-				this.stoppedDetails.length = 1;
-				await Promise.all(details.map(d => this.handleStop(d)));
+				const details = this.stoppedDetails.slice();
+				this.stoppedDetails.length = 0;
+				if (details.length) {
+					await Promise.all(details.map(d => this.handleStop(d)));
+				} else if (!this.fetchThreadsScheduler.value.isScheduled()) {
+					// threads are fetched as a side-effect of processing the stopped
+					// event(s), but if there are none, schedule a thread update manually (#282777)
+					this.fetchThreadsScheduler.value.schedule();
+				}
 			}
 
 			const viewModel = this.debugService.getViewModel();
@@ -1475,11 +1495,13 @@ export class DebugSession implements IDebugSession {
 			this.raw.dispose();
 			this.raw = undefined;
 		}
-		this.fetchThreadsScheduler?.dispose();
-		this.fetchThreadsScheduler = undefined;
 		this.passFocusScheduler.cancel();
 		this.passFocusScheduler.dispose();
 		this.model.clearThreads(this.getId(), true);
+		this.sources.clear();
+		this.threads.clear();
+		this.threadIds = [];
+		this.stoppedDetails = [];
 		this._onDidChangeState.fire();
 	}
 
@@ -1487,6 +1509,7 @@ export class DebugSession implements IDebugSession {
 		this.cancelAllRequests();
 		this.rawListeners.dispose();
 		this.globalDisposables.dispose();
+		this._waitToResume = undefined;
 	}
 
 	//---- sources
@@ -1598,7 +1621,7 @@ export class ThreadStatusScheduler extends Disposable {
 	 * Runs the operation.
 	 * If thread is undefined it affects all threads.
 	 */
-	public async run(threadIdsP: Promise<number[]>, operation: (threadId: number, ct: CancellationToken) => Promise<unknown>) {
+	public async run(threadIdsP: Promise<number[]> | number[], operation: (threadId: number, ct: CancellationToken) => Promise<unknown>) {
 		const cancelledWhileLookingUpThreads = new Set<number | undefined>();
 		this.pendingCancellations.push(cancelledWhileLookingUpThreads);
 		const threadIds = await threadIdsP;
