@@ -3,8 +3,8 @@
  *  Licensed under the MIT License. See License.txt in the project root for license information.
  *--------------------------------------------------------------------------------------------*/
 
-import { Disposable, DisposableStore, IDisposable, MutableDisposable, toDisposable } from '../../../../base/common/lifecycle.js';
-import { $, append, clearNode } from '../../../../base/browser/dom.js';
+import { Disposable, DisposableStore, IDisposable, MutableDisposable } from '../../../../base/common/lifecycle.js';
+import { $, append, clearNode, addDisposableListener, EventType } from '../../../../base/browser/dom.js';
 import { Emitter, Event } from '../../../../base/common/event.js';
 import { ExtensionIdentifier, IExtensionManifest } from '../../../../platform/extensions/common/extensions.js';
 import { Orientation, Sizing, SplitView } from '../../../../base/browser/ui/splitview/splitview.js';
@@ -17,9 +17,7 @@ import { getExtensionId } from '../../../../platform/extensionManagement/common/
 import { IListRenderer, IListVirtualDelegate } from '../../../../base/browser/ui/list/list.js';
 import { Button } from '../../../../base/browser/ui/button/button.js';
 import { defaultButtonStyles, defaultKeybindingLabelStyles } from '../../../../platform/theme/browser/defaultStyles.js';
-import { renderMarkdown } from '../../../../base/browser/markdownRenderer.js';
-import { getErrorMessage, onUnexpectedError } from '../../../../base/common/errors.js';
-import { IOpenerService } from '../../../../platform/opener/common/opener.js';
+import { getErrorMessage } from '../../../../base/common/errors.js';
 import { PANEL_SECTION_BORDER } from '../../../common/theme.js';
 import { IThemeService, Themable } from '../../../../platform/theme/common/themeService.js';
 import { DomScrollableElement } from '../../../../base/browser/ui/scrollbar/scrollableElement.js';
@@ -39,6 +37,7 @@ import { ResolvedKeybinding } from '../../../../base/common/keybindings.js';
 import { asCssVariable } from '../../../../platform/theme/common/colorUtils.js';
 import { foreground, chartAxis, chartGuide, chartLine } from '../../../../platform/theme/common/colorRegistry.js';
 import { IHoverService } from '../../../../platform/hover/browser/hover.js';
+import { IMarkdownRendererService } from '../../../../platform/markdown/browser/markdownRenderer.js';
 
 interface IExtensionFeatureElementRenderer extends IExtensionFeatureRenderer {
 	type: 'element';
@@ -52,9 +51,9 @@ class RuntimeStatusMarkdownRenderer extends Disposable implements IExtensionFeat
 
 	constructor(
 		@IExtensionService private readonly extensionService: IExtensionService,
-		@IOpenerService private readonly openerService: IOpenerService,
 		@IHoverService private readonly hoverService: IHoverService,
 		@IExtensionFeaturesManagementService private readonly extensionFeaturesManagementService: IExtensionFeaturesManagementService,
+		@IMarkdownRendererService private readonly markdownRendererService: IMarkdownRendererService,
 	) {
 		super();
 	}
@@ -152,19 +151,11 @@ class RuntimeStatusMarkdownRenderer extends Disposable implements IExtensionFeat
 	}
 
 	private renderMarkdown(markdown: IMarkdownString, container: HTMLElement, disposables: DisposableStore): void {
-		const { element, dispose } = renderMarkdown(
-			{
-				value: markdown.value,
-				isTrusted: markdown.isTrusted,
-				supportThemeIcons: true
-			},
-			{
-				actionHandler: {
-					callback: (content) => this.openerService.open(content, { allowCommands: !!markdown.isTrusted }).catch(onUnexpectedError),
-					disposables
-				},
-			});
-		disposables.add(toDisposable(dispose));
+		const { element } = disposables.add(this.markdownRendererService.render({
+			value: markdown.value,
+			isTrusted: markdown.isTrusted,
+			supportThemeIcons: true
+		}));
 		append(container, element);
 	}
 
@@ -304,15 +295,13 @@ class RuntimeStatusMarkdownRenderer extends Disposable implements IExtensionFeat
 				hoverDisposable.value = undefined;
 			}
 		};
-		svg.addEventListener('mousemove', mouseMoveListener);
-		disposables.add(toDisposable(() => svg.removeEventListener('mousemove', mouseMoveListener)));
+		disposables.add(addDisposableListener(svg, EventType.MOUSE_MOVE, mouseMoveListener));
 
 		const mouseLeaveListener = () => {
 			highlightCircle.style.display = 'none';
 			hoverDisposable.value = undefined;
 		};
-		svg.addEventListener('mouseleave', mouseLeaveListener);
-		disposables.add(toDisposable(() => svg.removeEventListener('mouseleave', mouseLeaveListener)));
+		disposables.add(addDisposableListener(svg, EventType.MOUSE_LEAVE, mouseLeaveListener));
 	}
 }
 
@@ -553,10 +542,10 @@ class ExtensionFeatureView extends Disposable {
 		private readonly extensionId: ExtensionIdentifier,
 		private readonly manifest: IExtensionManifest,
 		readonly feature: IExtensionFeatureDescriptor,
-		@IOpenerService private readonly openerService: IOpenerService,
 		@IInstantiationService private readonly instantiationService: IInstantiationService,
 		@IExtensionFeaturesManagementService private readonly extensionFeaturesManagementService: IExtensionFeaturesManagementService,
 		@IDialogService private readonly dialogService: IDialogService,
+		@IMarkdownRendererService private readonly markdownRendererService: IMarkdownRendererService,
 	) {
 		super();
 
@@ -708,19 +697,11 @@ class ExtensionFeatureView extends Disposable {
 	}
 
 	private renderMarkdown(markdown: IMarkdownString, container: HTMLElement): void {
-		const { element, dispose } = renderMarkdown(
-			{
-				value: markdown.value,
-				isTrusted: markdown.isTrusted,
-				supportThemeIcons: true
-			},
-			{
-				actionHandler: {
-					callback: (content) => this.openerService.open(content, { allowCommands: !!markdown.isTrusted }).catch(onUnexpectedError),
-					disposables: this._store
-				},
-			});
-		this._register(toDisposable(dispose));
+		const { element } = this._register(this.markdownRendererService.render({
+			value: markdown.value,
+			isTrusted: markdown.isTrusted,
+			supportThemeIcons: true
+		}));
 		append(container, element);
 	}
 
@@ -738,7 +719,7 @@ class ExtensionFeatureView extends Disposable {
 	}
 
 	private renderElementData(container: HTMLElement, renderer: IExtensionFeatureElementRenderer): void {
-		const elementData = renderer.render(this.manifest);
+		const elementData = this._register(renderer.render(this.manifest));
 		if (elementData.onDidChange) {
 			this._register(elementData.onDidChange(data => {
 				clearNode(container);
