@@ -7,11 +7,13 @@ import { equals as arraysEqual } from '../../../../base/common/arrays.js';
 import { assertNever } from '../../../../base/common/assert.js';
 import { decodeHex, encodeHex, VSBuffer } from '../../../../base/common/buffer.js';
 import { CancellationToken } from '../../../../base/common/cancellation.js';
-import { Codicon } from '../../../../base/common/codicons.js';
 import { Event } from '../../../../base/common/event.js';
+import { IMarkdownString } from '../../../../base/common/htmlContent.js';
 import { Disposable, IDisposable } from '../../../../base/common/lifecycle.js';
 import { equals as objectsEqual } from '../../../../base/common/objects.js';
-import { IObservable } from '../../../../base/common/observable.js';
+import { IObservable, ObservableMap } from '../../../../base/common/observable.js';
+import { IIterativePager } from '../../../../base/common/paging.js';
+import Severity from '../../../../base/common/severity.js';
 import { URI, UriComponents } from '../../../../base/common/uri.js';
 import { Location } from '../../../../editor/common/languages.js';
 import { localize } from '../../../../nls.js';
@@ -20,12 +22,13 @@ import { RawContextKey } from '../../../../platform/contextkey/common/contextkey
 import { IEditorOptions } from '../../../../platform/editor/common/editor.js';
 import { ExtensionIdentifier } from '../../../../platform/extensions/common/extensions.js';
 import { createDecorator } from '../../../../platform/instantiation/common/instantiation.js';
-import { IGalleryMcpServer, ILocalMcpServer, IQueryOptions } from '../../../../platform/mcp/common/mcpManagement.js';
-import { IMcpDevModeConfig } from '../../../../platform/mcp/common/mcpPlatformTypes.js';
+import { McpGalleryManifestStatus } from '../../../../platform/mcp/common/mcpGalleryManifest.js';
+import { IGalleryMcpServer, IInstallableMcpServer, IGalleryMcpServerConfiguration, IQueryOptions } from '../../../../platform/mcp/common/mcpManagement.js';
+import { IMcpDevModeConfig, IMcpServerConfiguration } from '../../../../platform/mcp/common/mcpPlatformTypes.js';
 import { StorageScope } from '../../../../platform/storage/common/storage.js';
-import { registerIcon } from '../../../../platform/theme/common/iconRegistry.js';
-import { IWorkspaceFolderData } from '../../../../platform/workspace/common/workspace.js';
-import { ToolProgress } from '../../chat/common/languageModelToolsService.js';
+import { IWorkspaceFolder, IWorkspaceFolderData } from '../../../../platform/workspace/common/workspace.js';
+import { IWorkbenchLocalMcpServer, IWorkbencMcpServerInstallOptions } from '../../../services/mcp/common/mcpWorkbenchManagementService.js';
+import { ToolProgress } from '../../chat/common/tools/languageModelToolsService.js';
 import { IMcpServerSamplingConfiguration } from './mcpConfiguration.js';
 import { McpServerRequestHandler } from './mcpServerRequestHandler.js';
 import { MCP } from './modelContextProtocol.js';
@@ -50,8 +53,11 @@ export interface McpCollectionDefinition {
 	readonly label: string;
 	/** Definitions this collection contains. */
 	readonly serverDefinitions: IObservable<readonly McpServerDefinition[]>;
-	/** If 'false', consent is required before any MCP servers in this collection are automatically launched. */
-	readonly isTrustedByDefault: boolean;
+	/**
+	 * Trust behavior of the servers. `Trusted` means it will run without a prompt, always.
+	 * `TrustedOnNonce` means it will run without a prompt as long as the nonce matches.
+	 */
+	readonly trustBehavior: McpServerTrust.Kind.Trusted | McpServerTrust.Kind.TrustedOnNonce;
 	/** Scope where associated collection info should be stored. */
 	readonly scope: StorageScope;
 	/** Configuration target where configuration related to this server should be stored. */
@@ -70,6 +76,8 @@ export interface McpCollectionDefinition {
 		removed?(): void;
 	};
 
+	readonly source?: IWorkbenchMcpServer | ExtensionIdentifier;
+
 	readonly presentation?: {
 		/** Sort order of the collection. */
 		readonly order?: number;
@@ -103,7 +111,7 @@ export namespace McpCollectionDefinition {
 		return a.id === b.id
 			&& a.remoteAuthority === b.remoteAuthority
 			&& a.label === b.label
-			&& a.isTrustedByDefault === b.isTrustedByDefault;
+			&& a.trustBehavior === b.trustBehavior;
 	}
 }
 
@@ -119,9 +127,12 @@ export interface McpServerDefinition {
 	/** If set, allows configuration variables to be resolved in the {@link launch} with the given context */
 	readonly variableReplacement?: McpServerDefinitionVariableReplacement;
 	/** Nonce used for caching the server. Changing the nonce will indicate that tools need to be refreshed. */
-	readonly cacheNonce?: string;
+	readonly cacheNonce: string;
 	/** Dev mode configuration for the server */
 	readonly devMode?: IMcpDevModeConfig;
+	/** Static description of server tools/data, used to hydrate the cache. */
+	readonly staticMetadata?: McpServerStaticMetadata;
+
 
 	readonly presentation?: {
 		/** Sort order of the definition. */
@@ -131,13 +142,28 @@ export interface McpServerDefinition {
 	};
 }
 
+export const enum McpServerStaticToolAvailability {
+	/** Tool is expected to be present as soon as the server is started. */
+	Initial,
+	/** Tool may be present later. */
+	Dynamic,
+}
+
+export interface McpServerStaticMetadata {
+	tools?: { availability: McpServerStaticToolAvailability; definition: MCP.Tool }[];
+	instructions?: string;
+	capabilities?: MCP.ServerCapabilities;
+	serverInfo?: MCP.Implementation;
+}
+
 export namespace McpServerDefinition {
 	export interface Serialized {
 		readonly id: string;
 		readonly label: string;
-		readonly cacheNonce?: string;
+		readonly cacheNonce: string;
 		readonly launch: McpServerLaunch.Serialized;
 		readonly variableReplacement?: McpServerDefinitionVariableReplacement.Serialized;
+		readonly staticMetadata?: McpServerStaticMetadata;
 	}
 
 	export function toSerialized(def: McpServerDefinition): McpServerDefinition.Serialized {
@@ -149,6 +175,7 @@ export namespace McpServerDefinition {
 			id: def.id,
 			label: def.label,
 			cacheNonce: def.cacheNonce,
+			staticMetadata: def.staticMetadata,
 			launch: McpServerLaunch.fromSerialized(def.launch),
 			variableReplacement: def.variableReplacement ? McpServerDefinitionVariableReplacement.fromSerialized(def.variableReplacement) : undefined,
 		};
@@ -157,6 +184,7 @@ export namespace McpServerDefinition {
 	export function equals(a: McpServerDefinition, b: McpServerDefinition): boolean {
 		return a.id === b.id
 			&& a.label === b.label
+			&& a.cacheNonce === b.cacheNonce
 			&& arraysEqual(a.roots, b.roots, (a, b) => a.toString() === b.toString())
 			&& objectsEqual(a.launch, b.launch)
 			&& objectsEqual(a.presentation, b.presentation)
@@ -192,6 +220,17 @@ export namespace McpServerDefinitionVariableReplacement {
 	}
 }
 
+/** An observable of the auto-starting servers. When 'starting' is empty, the operation is complete. */
+export interface IAutostartResult {
+	working: boolean;
+	starting: McpDefinitionReference[];
+	serversRequiringInteraction: Array<McpDefinitionReference & { errorMessage?: string }>;
+}
+
+export namespace IAutostartResult {
+	export const Empty: IAutostartResult = { working: false, starting: [], serversRequiringInteraction: [] };
+}
+
 export interface IMcpService {
 	_serviceBrand: undefined;
 	readonly servers: IObservable<readonly IMcpServer[]>;
@@ -199,9 +238,19 @@ export interface IMcpService {
 	/** Resets the cached tools. */
 	resetCaches(): void;
 
+	/** Resets trusted MCP servers. */
+	resetTrust(): void;
+
 	/** Set if there are extensions that register MCP servers that have never been activated. */
-	readonly lazyCollectionState: IObservable<LazyCollectionState>;
-	/** Activatese extensions and runs their MCP servers. */
+	readonly lazyCollectionState: IObservable<{ state: LazyCollectionState; collections: McpCollectionDefinition[] }>;
+
+	/** Auto-starts pending servers based on user settings. */
+	autostart(token?: CancellationToken): IObservable<IAutostartResult>;
+
+	/** Cancels any current autostart @internal */
+	cancelAutostart(): void;
+
+	/** Activates extension-providing MCP servers that have not yet been discovered. */
 	activateCollections(): Promise<void>;
 }
 
@@ -224,9 +273,47 @@ export interface McpDefinitionReference {
 	label: string;
 }
 
+export class McpStartServerInteraction {
+	/** @internal */
+	public readonly participants = new ObservableMap</* server definition ID */ string, { s: 'unknown' | 'resolved' } | { s: 'waiting'; definition: McpServerDefinition; collection: McpCollectionDefinition }>();
+	choice?: Promise<string[] | undefined>;
+}
+
 export interface IMcpServerStartOpts {
-	isFromInteraction?: boolean;
+	/**
+	 * Automatically trust if changed. This should ONLY be set for afforances that
+	 * ensure the user sees the config before it gets started (e.g. code lenses)
+	 */
+	autoTrustChanges?: boolean;
+	/**
+	 * When to trigger the trust prompt.
+	 * - only-new: only prompt for servers that are not previously explicitly untrusted (default)
+	 * - all-untrusted: prompt for all servers that are not trusted
+	 * - never: don't prompt, fail silently when trying to start an untrusted server
+	 */
+	promptType?: 'only-new' | 'all-untrusted' | 'never';
+	/** True if th servre should be launched with debugging. */
 	debug?: boolean;
+	/** Correlate multiple interactions such that any trust prompts are presented in combination. */
+	interaction?: McpStartServerInteraction;
+	/**
+	 * If true, throw an error if any user interaction would be required during startup.
+	 * This includes variable resolution, trust prompts, and authentication prompts.
+	 */
+	errorOnUserInteraction?: boolean;
+}
+
+export namespace McpServerTrust {
+	export const enum Kind {
+		/** The server is trusted */
+		Trusted,
+		/** The server is trusted as long as its nonce matches */
+		TrustedOnNonce,
+		/** The server trust was denied. */
+		Untrusted,
+		/** The server is not yet trusted or untrusted. */
+		Unknown,
+	}
 }
 
 export interface IMcpServer extends IDisposable {
@@ -234,6 +321,11 @@ export interface IMcpServer extends IDisposable {
 	readonly definition: McpDefinitionReference;
 	readonly connection: IObservable<IMcpServerConnection | undefined>;
 	readonly connectionState: IObservable<McpConnectionState>;
+	readonly serverMetadata: IObservable<{
+		serverName?: string;
+		serverInstructions?: string;
+		icons: IMcpIcons;
+	} | undefined>;
 
 	/**
 	 * Full definition as it exists in the MCP registry. Unlike the references
@@ -241,13 +333,7 @@ export interface IMcpServer extends IDisposable {
 	 */
 	readDefinitions(): IObservable<{ server: McpServerDefinition | undefined; collection: McpCollectionDefinition | undefined }>;
 
-	/**
-	 * Reflects the MCP server trust state. True if trusted, false if untrusted,
-	 * undefined if consent is required but not indicated.
-	 */
-	readonly trusted: IObservable<boolean | undefined>;
-
-	showOutput(): void;
+	showOutput(preserveFocus?: boolean): Promise<void>;
 	/**
 	 * Starts the server and returns its resulting state. One of:
 	 * - Running, if all good
@@ -283,16 +369,20 @@ export interface IMcpResource {
 	/** Identifier of the file as given from the MCP server. */
 	readonly mcpUri: string;
 	readonly name: string;
+	readonly title?: string;
 	readonly description?: string;
 	readonly mimeType?: string;
 	readonly sizeInBytes?: number;
+	readonly icons: IMcpIcons;
 }
 
 export interface IMcpResourceTemplate {
 	readonly name: string;
+	readonly title?: string;
 	readonly description?: string;
 	readonly mimeType?: string;
 	readonly template: UriTemplate;
+	readonly icons: IMcpIcons;
 
 	/** Gets string completions for the given template part. */
 	complete(templatePart: string, prefix: string, alreadyResolved: Record<string, string | string[]>, token: CancellationToken): Promise<string[]>;
@@ -326,11 +416,12 @@ export const enum McpServerCacheState {
 export interface IMcpPrompt {
 	readonly id: string;
 	readonly name: string;
+	readonly title?: string;
 	readonly description?: string;
 	readonly arguments: readonly MCP.PromptArgument[];
 
 	/** Gets string completions for the given prompt part. */
-	complete(argument: string, prefix: string, token: CancellationToken): Promise<string[]>;
+	complete(argument: string, prefix: string, alreadyResolved: Record<string, string>, token: CancellationToken): Promise<string[]>;
 
 	resolve(args: Record<string, string | undefined>, token?: CancellationToken): Promise<IMcpPromptMessage[]>;
 }
@@ -342,25 +433,58 @@ export const mcpPromptPrefix = (definition: McpDefinitionReference) =>
 
 export interface IMcpPromptMessage extends MCP.PromptMessage { }
 
+export interface IMcpToolCallContext {
+	chatSessionId?: string;
+	chatRequestId?: string;
+}
+
+/**
+ * Visibility of an MCP tool, based on the MCP Apps `_meta.ui.visibility` field.
+ * @see https://github.com/anthropics/mcp/blob/main/apps.md
+ */
+export const enum McpToolVisibility {
+	/** Tool is visible to and callable by the language model */
+	Model = 1 << 0,
+	/** Tool is callable by the MCP App UI */
+	App = 1 << 1,
+}
+
+/**
+ * Serializable data for MCP App UI rendering.
+ * This contains all the information needed to render an MCP App webview.
+ */
+export interface IMcpToolCallUIData {
+	/** URI of the UI resource for rendering (e.g., "ui://weather-server/dashboard") */
+	readonly resourceUri: string;
+	/** Reference to the server definition for reconnection */
+	readonly serverDefinitionId: string;
+	/** Reference to the collection containing the server */
+	readonly collectionId: string;
+}
+
 export interface IMcpTool {
 
 	readonly id: string;
 	/** Name for #referencing in chat */
 	readonly referenceName: string;
-
+	readonly icons: IMcpIcons;
 	readonly definition: MCP.Tool;
+	/** Visibility of the tool (Model, App, or both). Defaults to Model | App. */
+	readonly visibility: McpToolVisibility;
+	/** Optional UI resource URI for MCP App rendering */
+	readonly uiResourceUri?: string;
 
 	/**
 	 * Calls a tool
 	 * @throws {@link MpcResponseError} if the tool fails to execute
 	 * @throws {@link McpConnectionFailedError} if the connection to the server fails
 	 */
-	call(params: Record<string, unknown>, token?: CancellationToken): Promise<MCP.CallToolResult>;
+	call(params: Record<string, unknown>, context?: IMcpToolCallContext, token?: CancellationToken): Promise<MCP.CallToolResult>;
 
 	/**
 	 * Identical to {@link call}, but reports progress.
 	 */
-	callWithProgress(params: Record<string, unknown>, progress: ToolProgress, token?: CancellationToken): Promise<MCP.CallToolResult>;
+	callWithProgress(params: Record<string, unknown>, progress: ToolProgress, context?: IMcpToolCallContext, token?: CancellationToken): Promise<MCP.CallToolResult>;
 }
 
 export const enum McpServerTransportType {
@@ -383,6 +507,17 @@ export interface McpServerTransportStdio {
 	readonly envFile: string | undefined;
 }
 
+export interface McpServerTransportHTTPAuthentication {
+	/**
+	 * Authentication provider ID to use to get a session for the initial MCP server connection.
+	 */
+	readonly providerId: string;
+	/**
+	 * Scopes to use to get a session for the initial MCP server connection.
+	 */
+	readonly scopes: string[];
+}
+
 /**
  * MCP server launched on the command line which communicated over SSE or Streamable HTTP.
  * https://spec.modelcontextprotocol.io/specification/2024-11-05/basic/transports/#http-with-sse
@@ -392,6 +527,7 @@ export interface McpServerTransportHTTP {
 	readonly type: McpServerTransportType.HTTP;
 	readonly uri: URI;
 	readonly headers: [string, string][];
+	readonly authentication?: McpServerTransportHTTPAuthentication;
 }
 
 export type McpServerLaunch =
@@ -400,7 +536,7 @@ export type McpServerLaunch =
 
 export namespace McpServerLaunch {
 	export type Serialized =
-		| { type: McpServerTransportType.HTTP; uri: UriComponents; headers: [string, string][] }
+		| { type: McpServerTransportType.HTTP; uri: UriComponents; headers: [string, string][]; authentication?: McpServerTransportHTTPAuthentication }
 		| { type: McpServerTransportType.Stdio; cwd: string | undefined; command: string; args: readonly string[]; env: Record<string, string | number | null>; envFile: string | undefined };
 
 	export function toSerialized(launch: McpServerLaunch): McpServerLaunch.Serialized {
@@ -410,7 +546,7 @@ export namespace McpServerLaunch {
 	export function fromSerialized(launch: McpServerLaunch.Serialized): McpServerLaunch {
 		switch (launch.type) {
 			case McpServerTransportType.HTTP:
-				return { type: launch.type, uri: URI.revive(launch.uri), headers: launch.headers };
+				return { type: launch.type, uri: URI.revive(launch.uri), headers: launch.headers, authentication: launch.authentication };
 			case McpServerTransportType.Stdio:
 				return {
 					type: launch.type,
@@ -422,6 +558,11 @@ export namespace McpServerLaunch {
 				};
 		}
 	}
+
+	export async function hash(launch: McpServerLaunch): Promise<string> {
+		const nonce = await crypto.subtle.digest('SHA-256', new TextEncoder().encode(JSON.stringify(launch)));
+		return encodeHex(VSBuffer.wrap(new Uint8Array(nonce)));
+	}
 }
 
 /**
@@ -455,7 +596,9 @@ export interface IMcpServerConnection extends IDisposable {
 /** Client methods whose implementations are passed through the server connection. */
 export interface IMcpClientMethods {
 	/** Handler for `sampling/createMessage` */
-	createMessageRequestHandler?(req: MCP.CreateMessageRequest['params']): Promise<MCP.CreateMessageResult>;
+	createMessageRequestHandler?(req: MCP.CreateMessageRequest['params'], token?: CancellationToken): Promise<MCP.CreateMessageResult>;
+	/** Handler for `elicitation/create` */
+	elicitationRequestHandler?(req: MCP.ElicitRequest['params'], token?: CancellationToken): Promise<MCP.ElicitResult>;
 }
 
 /**
@@ -508,6 +651,7 @@ export namespace McpConnectionState {
 
 	export interface Stopped {
 		readonly state: Kind.Stopped;
+		readonly reason?: 'needs-user-interaction';
 	}
 
 	export interface Starting {
@@ -540,39 +684,110 @@ export class MpcResponseError extends Error {
 
 export class McpConnectionFailedError extends Error { }
 
+export class UserInteractionRequiredError extends Error {
+	private static readonly prefix = 'User interaction required: ';
+
+	public static is(error: Error): boolean {
+		return error.message.startsWith(this.prefix);
+	}
+
+	constructor(public readonly reason: string) {
+		super(`${UserInteractionRequiredError.prefix}${reason}`);
+	}
+}
+
+export interface IMcpConfigPath {
+	id: string;
+	key: 'userLocalValue' | 'userRemoteValue' | 'workspaceValue' | 'workspaceFolderValue';
+	label: string;
+	scope: StorageScope;
+	target: ConfigurationTarget;
+	order: number;
+	remoteAuthority?: string;
+	uri: URI | undefined;
+	section?: string[];
+	workspaceFolder?: IWorkspaceFolder;
+}
+
 export interface IMcpServerContainer extends IDisposable {
 	mcpServer: IWorkbenchMcpServer | null;
 	update(): void;
 }
 
+export interface IMcpServerEditorOptions extends IEditorOptions {
+	tab?: McpServerEditorTab;
+	sideByside?: boolean;
+}
+
+export const enum McpServerEnablementState {
+	Disabled,
+	DisabledByAccess,
+	Enabled,
+}
+
+export const enum McpServerInstallState {
+	Installing,
+	Installed,
+	Uninstalling,
+	Uninstalled
+}
+
+export const enum McpServerEditorTab {
+	Readme = 'readme',
+	Manifest = 'manifest',
+	Configuration = 'configuration',
+}
+
+export type McpServerEnablementStatus = {
+	readonly state: McpServerEnablementState;
+	readonly message?: {
+		readonly severity: Severity;
+		readonly text: IMarkdownString;
+	};
+};
+
 export interface IWorkbenchMcpServer {
 	readonly gallery: IGalleryMcpServer | undefined;
-	readonly local: ILocalMcpServer | undefined;
+	readonly local: IWorkbenchLocalMcpServer | undefined;
+	readonly installable: IInstallableMcpServer | undefined;
+	readonly installState: McpServerInstallState;
+	readonly runtimeStatus: McpServerEnablementStatus | undefined;
 	readonly id: string;
 	readonly name: string;
 	readonly label: string;
 	readonly description: string;
-	readonly iconUrl?: string;
+	readonly icon?: {
+		readonly dark: string;
+		readonly light: string;
+	};
+	readonly codicon?: string;
 	readonly publisherUrl?: string;
 	readonly publisherDisplayName?: string;
-	readonly installCount?: number;
-	readonly ratingCount?: number;
-	readonly rating?: number;
-	readonly url?: string;
+	readonly starsCount?: number;
+	readonly license?: string;
 	readonly repository?: string;
+	readonly config?: IMcpServerConfiguration | undefined;
+	readonly readmeUrl?: URI;
 	getReadme(token: CancellationToken): Promise<string>;
+	getManifest(token: CancellationToken): Promise<IGalleryMcpServerConfiguration>;
 }
 
 export const IMcpWorkbenchService = createDecorator<IMcpWorkbenchService>('IMcpWorkbenchService');
 export interface IMcpWorkbenchService {
 	readonly _serviceBrand: undefined;
 	readonly onChange: Event<IWorkbenchMcpServer | undefined>;
+	readonly onReset: Event<void>;
 	readonly local: readonly IWorkbenchMcpServer[];
+	getEnabledLocalMcpServers(): IWorkbenchLocalMcpServer[];
 	queryLocal(): Promise<IWorkbenchMcpServer[]>;
-	queryGallery(options?: IQueryOptions, token?: CancellationToken): Promise<IWorkbenchMcpServer[]>;
-	install(mcpServer: IWorkbenchMcpServer): Promise<void>;
+	queryGallery(options?: IQueryOptions, token?: CancellationToken): Promise<IIterativePager<IWorkbenchMcpServer>>;
+	canInstall(mcpServer: IWorkbenchMcpServer): true | IMarkdownString;
+	install(server: IWorkbenchMcpServer, installOptions?: IWorkbencMcpServerInstallOptions): Promise<IWorkbenchMcpServer>;
 	uninstall(mcpServer: IWorkbenchMcpServer): Promise<void>;
-	open(extension: IWorkbenchMcpServer | string, options?: IEditorOptions): Promise<void>;
+	getMcpConfigPath(arg: IWorkbenchLocalMcpServer): IMcpConfigPath | undefined;
+	getMcpConfigPath(arg: URI): Promise<IMcpConfigPath | undefined>;
+	openSearch(searchValue: string, preserveFoucs?: boolean): Promise<void>;
+	open(extension: IWorkbenchMcpServer | string, options?: IMcpServerEditorOptions): Promise<void>;
 }
 
 export class McpServerContainers extends Disposable {
@@ -591,7 +806,7 @@ export class McpServerContainers extends Disposable {
 	update(server: IWorkbenchMcpServer | undefined): void {
 		for (const container of this.containers) {
 			if (server && container.mcpServer) {
-				if (server.name === container.mcpServer.name) {
+				if (server.id === container.mcpServer.id) {
 					container.mcpServer = server;
 				}
 			} else {
@@ -601,14 +816,17 @@ export class McpServerContainers extends Disposable {
 	}
 }
 
-export const McpServersGalleryEnabledContext = new RawContextKey<boolean>('mcpServersGalleryEnabled', false);
-export const HasInstalledMcpServersContext = new RawContextKey<boolean>('hasInstalledMcpServers', false);
+export const McpServersGalleryStatusContext = new RawContextKey<string>('mcpServersGalleryStatus', McpGalleryManifestStatus.Unavailable);
+export const HasInstalledMcpServersContext = new RawContextKey<boolean>('hasInstalledMcpServers', true);
 export const InstalledMcpServersViewId = 'workbench.views.mcp.installed';
-export const mcpServerIcon = registerIcon('mcp-server', Codicon.mcp, localize('mcpServer', 'Icon used for the MCP server.'));
 
 export namespace McpResourceURI {
 	export const scheme = 'mcp-resource';
 
+	// Random placeholder for empty authorities, otherwise they're represente as
+	// `scheme//path/here` in the URI which would get normalized to `scheme/path/here`.
+	const emptyAuthorityPlaceholder = 'dylo78gyp'; // chosen by a fair dice roll. Guaranteed to be random.
+
 	export function fromServer(def: McpDefinitionReference, resourceURI: URI | string): URI {
 		if (typeof resourceURI === 'string') {
 			resourceURI = URI.parse(resourceURI);
@@ -616,11 +834,11 @@ export namespace McpResourceURI {
 		return resourceURI.with({
 			scheme,
 			authority: encodeHex(VSBuffer.fromString(def.id)),
-			path: ['', resourceURI.scheme, resourceURI.authority].join('/') + resourceURI.path,
+			path: ['', resourceURI.scheme, resourceURI.authority || emptyAuthorityPlaceholder].join('/') + resourceURI.path,
 		});
 	}
 
-	export function toServer(uri: URI | string): { definitionId: string; resourceURI: URI } {
+	export function toServer(uri: URI | string): { definitionId: string; resourceURL: URL } {
 		if (typeof uri === 'string') {
 			uri = URI.parse(uri);
 		}
@@ -632,13 +850,16 @@ export namespace McpResourceURI {
 			throw new Error(`Invalid MCP resource URI: ${uri.toString()}`);
 		}
 		const [, serverScheme, authority, ...path] = parts;
+
+		// URI cannot correctly stringify empty authorities (#250905) so we use URL instead to construct
+		const url = new URL(`${serverScheme}://${authority.toLowerCase() === emptyAuthorityPlaceholder ? '' : authority}`);
+		url.pathname = path.length ? ('/' + path.join('/')) : '';
+		url.search = uri.query;
+		url.hash = uri.fragment;
+
 		return {
 			definitionId: decodeHex(uri.authority).toString(),
-			resourceURI: uri.with({
-				scheme: serverScheme,
-				authority,
-				path: '/' + path.join('/'),
-			}),
+			resourceURL: url,
 		};
 	}
 
@@ -670,7 +891,7 @@ export interface ISamplingResult {
 export interface IMcpSamplingService {
 	_serviceBrand: undefined;
 
-	sample(opts: ISamplingOptions): Promise<ISamplingResult>;
+	sample(opts: ISamplingOptions, token?: CancellationToken): Promise<ISamplingResult>;
 
 	/** Whether MCP sampling logs are available for this server */
 	hasLogs(server: IMcpServer): boolean;
@@ -712,3 +933,55 @@ export const enum McpToolName {
 	MaxPrefixLen = 18,
 	MaxLength = 64,
 }
+
+
+export interface IMcpElicitationService {
+	_serviceBrand: undefined;
+
+	/**
+	 * Elicits a response from the user. The `context` is optional and can be used
+	 * to provide additional information about the request.
+	 *
+	 * @param context Context for the elicitation, e.g. chat session ID.
+	 * @param elicitation Request to elicit a response.
+	 * @returns A promise that resolves to an {@link ElicitationResult}.
+	 */
+	elicit(server: IMcpServer, context: IMcpToolCallContext | undefined, elicitation: MCP.ElicitRequest['params'], token: CancellationToken): Promise<ElicitResult>;
+}
+
+export const enum ElicitationKind {
+	Form,
+	URL,
+}
+
+export interface IUrlModeElicitResult extends IDisposable {
+	kind: ElicitationKind.URL;
+	value: MCP.ElicitResult;
+	/**
+	 * Waits until the server tells us the elicitation is completed before resolving.
+	 * Rejects with a CancellationError if the server stops before elicitation is
+	 * complete, or if the token is cancelled.
+	 */
+	wait: Promise<void>;
+}
+
+export interface IFormModeElicitResult extends IDisposable {
+	kind: ElicitationKind.Form;
+	value: MCP.ElicitResult;
+}
+
+export type ElicitResult = IUrlModeElicitResult | IFormModeElicitResult;
+
+export const IMcpElicitationService = createDecorator<IMcpElicitationService>('IMcpElicitationService');
+
+export const McpToolResourceLinkMimeType = 'application/vnd.code.resource-link';
+
+export interface IMcpToolResourceLinkContents {
+	uri: UriComponents;
+	underlyingMimeType?: string;
+}
+
+export interface IMcpIcons {
+	/** Gets the image URI appropriate to the approximate display size */
+	getUrl(size: number): { dark: URI; light?: URI } | undefined;
+}
