@@ -21,7 +21,6 @@ import { IViewsService } from '../../../services/views/common/viewsService.js';
 import { OutputViewPane } from './outputView.js';
 import { ILanguageService } from '../../../../editor/common/languages/language.js';
 import { IContextKey, IContextKeyService } from '../../../../platform/contextkey/common/contextkey.js';
-import { IDefaultLogLevelsService } from '../../logs/common/defaultLogLevels.js';
 import { IFileDialogService } from '../../../../platform/dialogs/common/dialogs.js';
 import { IFileService } from '../../../../platform/files/common/files.js';
 import { localize } from '../../../../nls.js';
@@ -30,6 +29,7 @@ import { VSBuffer } from '../../../../base/common/buffer.js';
 import { telemetryLogId } from '../../../../platform/telemetry/common/telemetryUtils.js';
 import { toLocalISOString } from '../../../../base/common/date.js';
 import { IWorkbenchEnvironmentService } from '../../../services/environment/common/environmentService.js';
+import { IDefaultLogLevelsService } from '../../../services/log/common/defaultLogLevels.js';
 
 const OUTPUT_ACTIVE_CHANNEL_KEY = 'output.activechannel';
 
@@ -132,15 +132,91 @@ class OutputViewFilters extends Disposable implements IOutputViewFilters {
 	filterHistory: string[];
 
 	private _filterText = '';
+	private _includePatterns: string[] = [];
+	private _excludePatterns: string[] = [];
 	get text(): string {
 		return this._filterText;
 	}
 	set text(filterText: string) {
 		if (this._filterText !== filterText) {
 			this._filterText = filterText;
+			const { includePatterns, excludePatterns } = this.parseText(filterText);
+			this._includePatterns = includePatterns;
+			this._excludePatterns = excludePatterns;
 			this._onDidChange.fire();
 		}
 	}
+	private parseText(filterText: string): { includePatterns: string[]; excludePatterns: string[] } {
+		const includePatterns: string[] = [];
+		const excludePatterns: string[] = [];
+
+		// Parse patterns respecting quoted strings
+		const patterns = this.splitByCommaRespectingQuotes(filterText);
+
+		for (const pattern of patterns) {
+			const trimmed = pattern.trim();
+			if (trimmed.length === 0) {
+				continue;
+			}
+
+			if (trimmed.startsWith('!')) {
+				// Negative filter - remove the ! prefix
+				const negativePattern = trimmed.substring(1).trim();
+				if (negativePattern.length > 0) {
+					excludePatterns.push(negativePattern);
+				}
+			} else {
+				includePatterns.push(trimmed);
+			}
+		}
+
+		return { includePatterns, excludePatterns };
+	}
+
+	get includePatterns(): string[] {
+		return this._includePatterns;
+	}
+
+	get excludePatterns(): string[] {
+		return this._excludePatterns;
+	}
+
+	private splitByCommaRespectingQuotes(text: string): string[] {
+		const patterns: string[] = [];
+		let current = '';
+		let inQuotes = false;
+		let quoteChar = '';
+
+		for (let i = 0; i < text.length; i++) {
+			const char = text[i];
+
+			if (!inQuotes && (char === '"')) {
+				// Start of quoted string
+				inQuotes = true;
+				quoteChar = char;
+				current += char;
+			} else if (inQuotes && char === quoteChar) {
+				// End of quoted string
+				inQuotes = false;
+				current += char;
+			} else if (!inQuotes && char === ',') {
+				// Comma outside quotes - split here
+				if (current.length > 0) {
+					patterns.push(current);
+				}
+				current = '';
+			} else {
+				current += char;
+			}
+		}
+
+		// Add the last pattern
+		if (current.length > 0) {
+			patterns.push(current);
+		}
+
+		return patterns;
+	}
 
 	private readonly _trace: IContextKey<boolean>;
 	get trace(): boolean {
@@ -299,7 +375,6 @@ export class OutputService extends Disposable implements IOutputService, ITextMo
 		}));
 
 		this._register(this.loggerService.onDidChangeLogLevel(() => {
-			this.resetLogLevelFilters();
 			this.setLevelContext();
 			this.setLevelIsDefaultContext();
 		}));
@@ -414,7 +489,7 @@ export class OutputService extends Disposable implements IOutputService, ITextMo
 		return id;
 	}
 
-	async saveOutputAs(...channels: IOutputChannelDescriptor[]): Promise<void> {
+	async saveOutputAs(outputPath?: URI, ...channels: IOutputChannelDescriptor[]): Promise<void> {
 		let channel: IOutputChannel | undefined;
 		if (channels.length > 1) {
 			const compoundChannelId = this.registerCompoundLogChannel(channels);
@@ -428,16 +503,19 @@ export class OutputService extends Disposable implements IOutputService, ITextMo
 		}
 
 		try {
-			const name = channels.length > 1 ? 'output' : channels[0].label;
-			const uri = await this.fileDialogService.showSaveDialog({
-				title: localize('saveLog.dialogTitle', "Save Output As"),
-				availableFileSystems: [Schemas.file],
-				defaultUri: joinPath(await this.fileDialogService.defaultFilePath(), `${name}.log`),
-				filters: [{
-					name,
-					extensions: ['log']
-				}]
-			});
+			let uri: URI | undefined = outputPath;
+			if (!uri) {
+				const name = channels.length > 1 ? 'output' : channels[0].label;
+				uri = await this.fileDialogService.showSaveDialog({
+					title: localize('saveLog.dialogTitle', "Save Output As"),
+					availableFileSystems: [Schemas.file],
+					defaultUri: joinPath(await this.fileDialogService.defaultFilePath(), `${name}.log`),
+					filters: [{
+						name,
+						extensions: ['log']
+					}]
+				});
+			}
 
 			if (!uri) {
 				return;
@@ -517,18 +595,6 @@ export class OutputService extends Disposable implements IOutputService, ITextMo
 		return this.instantiationService.createInstance(OutputChannel, channelData, this.outputLocation, this.outputFolderCreationPromise);
 	}
 
-	private resetLogLevelFilters(): void {
-		const descriptor = this.activeChannel?.outputChannelDescriptor;
-		const channelLogLevel = descriptor ? this.getLogLevel(descriptor) : undefined;
-		if (channelLogLevel !== undefined) {
-			this.filters.error = channelLogLevel <= LogLevel.Error;
-			this.filters.warning = channelLogLevel <= LogLevel.Warning;
-			this.filters.info = channelLogLevel <= LogLevel.Info;
-			this.filters.debug = channelLogLevel <= LogLevel.Debug;
-			this.filters.trace = channelLogLevel <= LogLevel.Trace;
-		}
-	}
-
 	private setLevelContext(): void {
 		const descriptor = this.activeChannel?.outputChannelDescriptor;
 		const channelLogLevel = descriptor ? this.getLogLevel(descriptor) : undefined;
@@ -539,7 +605,7 @@ export class OutputService extends Disposable implements IOutputService, ITextMo
 		const descriptor = this.activeChannel?.outputChannelDescriptor;
 		const channelLogLevel = descriptor ? this.getLogLevel(descriptor) : undefined;
 		if (channelLogLevel !== undefined) {
-			const channelDefaultLogLevel = await this.defaultLogLevelsService.getDefaultLogLevel(descriptor?.extensionId);
+			const channelDefaultLogLevel = this.defaultLogLevelsService.getDefaultLogLevel(descriptor?.extensionId);
 			this.activeOutputChannelLevelIsDefaultContext.set(channelDefaultLogLevel === channelLogLevel);
 		} else {
 			this.activeOutputChannelLevelIsDefaultContext.set(false);
