@@ -47,7 +47,7 @@ import { ThemeIcon } from '../../../../base/common/themables.js';
 import { gotoNextLocation, gotoPreviousLocation } from '../../../../platform/theme/common/iconRegistry.js';
 import { Codicon } from '../../../../base/common/codicons.js';
 import { Color } from '../../../../base/common/color.js';
-import { KeyCode, KeyMod } from '../../../../base/common/keyCodes.js';
+import { KeyChord, KeyCode, KeyMod } from '../../../../base/common/keyCodes.js';
 import { getOuterEditor } from '../../../../editor/browser/widget/codeEditor/embeddedCodeEditorWidget.js';
 import { quickDiffDecorationCount } from './quickDiffDecorator.js';
 import { hasNativeContextMenu } from '../../../../platform/window/common/window.js';
@@ -108,7 +108,7 @@ export class QuickDiffPickerBaseAction extends Action {
 
 class QuickDiffWidgetActionRunner extends ActionRunner {
 
-	protected override runAction(action: IAction, context: any): Promise<any> {
+	protected override runAction(action: IAction, context: unknown[]): Promise<void> {
 		if (action instanceof MenuItemAction) {
 			return action.run(...context);
 		}
@@ -130,8 +130,7 @@ class QuickDiffWidgetEditorAction extends Action {
 		@IKeybindingService keybindingService: IKeybindingService,
 		@IInstantiationService instantiationService: IInstantiationService
 	) {
-		const keybinding = keybindingService.lookupKeybinding(action.id);
-		const label = action.label + (keybinding ? ` (${keybinding.getLabel()})` : '');
+		const label = keybindingService.appendKeybinding(action.label, action.id);
 
 		super(action.id, label, cssClass);
 
@@ -140,7 +139,7 @@ class QuickDiffWidgetEditorAction extends Action {
 		this.editor = editor;
 	}
 
-	override run(): Promise<any> {
+	override run(): Promise<void> {
 		return Promise.resolve(this.instantiationService.invokeFunction(accessor => this.action.run(accessor, this.editor, null)));
 	}
 }
@@ -358,9 +357,11 @@ class QuickDiffWidget extends PeekViewWidget {
 		super._fillHead(container, true);
 
 		// Render an empty picker which will be populated later
+		const action = new QuickDiffPickerBaseAction((event?: IQuickDiffSelectItem) => this.switchQuickDiff(event));
+		this._disposables.add(action);
+
 		this.dropdownContainer = dom.prepend(this._titleElement!, dom.$('.dropdown'));
-		this.dropdown = this.instantiationService.createInstance(QuickDiffPickerViewItem,
-			new QuickDiffPickerBaseAction((event?: IQuickDiffSelectItem) => this.switchQuickDiff(event)));
+		this.dropdown = this.instantiationService.createInstance(QuickDiffPickerViewItem, action);
 		this.dropdown.render(this.dropdownContainer);
 	}
 
@@ -462,9 +463,18 @@ class QuickDiffWidget extends PeekViewWidget {
 		return this.diffEditor.hasTextFocus();
 	}
 
+	toggleFocus(): void {
+		if (this.diffEditor.hasTextFocus()) {
+			this.editor.focus();
+		} else {
+			this.diffEditor.focus();
+		}
+	}
+
 	override dispose() {
-		super.dispose();
+		this.dropdown?.dispose();
 		this.menu?.dispose();
+		super.dispose();
 	}
 }
 
@@ -544,6 +554,12 @@ export class QuickDiffEditorController extends Disposable implements IEditorCont
 		this.widget?.showChange(this.widget.index, false);
 	}
 
+	toggleFocus(): void {
+		if (this.widget) {
+			this.widget.toggleFocus();
+		}
+	}
+
 	next(lineNumber?: number): void {
 		if (!this.assertWidget()) {
 			return;
@@ -937,6 +953,26 @@ KeybindingsRegistry.registerCommandAndKeybindingRule({
 	}
 });
 
+KeybindingsRegistry.registerCommandAndKeybindingRule({
+	id: 'togglePeekWidgetFocus',
+	weight: KeybindingWeight.EditorContrib,
+	primary: KeyChord(KeyMod.CtrlCmd | KeyCode.KeyK, KeyCode.F2),
+	when: isQuickDiffVisible,
+	handler: (accessor: ServicesAccessor) => {
+		const outerEditor = getOuterEditorFromDiffEditor(accessor);
+		if (!outerEditor) {
+			return;
+		}
+
+		const controller = QuickDiffEditorController.get(outerEditor);
+		if (!controller) {
+			return;
+		}
+
+		controller.toggleFocus();
+	}
+});
+
 function setPositionAndSelection(change: IChange, editor: ICodeEditor, accessibilityService: IAccessibilityService, codeEditorService: ICodeEditorService) {
 	const position = new Position(change.modifiedStartLineNumber, 1);
 	editor.setPosition(position);
