@@ -8,25 +8,25 @@ import * as dom from '../../../../../base/browser/dom.js';
 import { AutoOpenBarrier } from '../../../../../base/common/async.js';
 import { Event } from '../../../../../base/common/event.js';
 import { KeyCode, KeyMod } from '../../../../../base/common/keyCodes.js';
-import { DisposableStore, MutableDisposable, toDisposable } from '../../../../../base/common/lifecycle.js';
-import { isLinux, isWindows } from '../../../../../base/common/platform.js';
+import { DisposableStore, MutableDisposable, toDisposable, Disposable, DisposableMap } from '../../../../../base/common/lifecycle.js';
+import { isWindows } from '../../../../../base/common/platform.js';
 import { localize2 } from '../../../../../nls.js';
 import { IConfigurationService } from '../../../../../platform/configuration/common/configuration.js';
 import { ContextKeyExpr, IContextKey, IContextKeyService } from '../../../../../platform/contextkey/common/contextkey.js';
 import { IInstantiationService } from '../../../../../platform/instantiation/common/instantiation.js';
 import { KeybindingWeight } from '../../../../../platform/keybinding/common/keybindingsRegistry.js';
-import { GeneralShellType, TerminalLocation } from '../../../../../platform/terminal/common/terminal.js';
+import { TerminalLocation } from '../../../../../platform/terminal/common/terminal.js';
 import { ITerminalContribution, ITerminalInstance, IXtermTerminal } from '../../../terminal/browser/terminal.js';
 import { registerActiveInstanceAction, registerTerminalAction } from '../../../terminal/browser/terminalActions.js';
 import { registerTerminalContribution, type ITerminalContributionContext } from '../../../terminal/browser/terminalExtensions.js';
 import { TerminalContextKeys } from '../../../terminal/common/terminalContextKey.js';
 import { TerminalSuggestCommandId } from '../common/terminal.suggest.js';
-import { terminalSuggestConfigSection, TerminalSuggestSettingId, type ITerminalSuggestConfiguration } from '../common/terminalSuggestConfiguration.js';
+import { terminalSuggestConfigSection, TerminalSuggestSettingId, type ITerminalSuggestConfiguration, registerTerminalSuggestProvidersConfiguration, type ITerminalSuggestProviderInfo } from '../common/terminalSuggestConfiguration.js';
 import { ITerminalCompletionService, TerminalCompletionService } from './terminalCompletionService.js';
+import { ITerminalContributionService } from '../../../terminal/common/terminalExtensionPoints.js';
 import { InstantiationType, registerSingleton } from '../../../../../platform/instantiation/common/extensions.js';
 import { SuggestAddon } from './terminalSuggestAddon.js';
 import { TerminalClipboardContribution } from '../../clipboard/browser/terminal.clipboard.contribution.js';
-import { PwshCompletionProviderAddon } from './pwshCompletionProviderAddon.js';
 import { SimpleSuggestContext } from '../../../../services/suggest/browser/simpleSuggestWidget.js';
 import { SuggestDetailsClassName } from '../../../../services/suggest/browser/simpleSuggestWidgetDetails.js';
 import { EditorContextKeys } from '../../../../../editor/common/editorContextKeys.js';
@@ -37,9 +37,9 @@ import { LspCompletionProviderAddon } from './lspCompletionProviderAddon.js';
 import { createTerminalLanguageVirtualUri, LspTerminalModelContentProvider } from './lspTerminalModelContentProvider.js';
 import { ITextModelService } from '../../../../../editor/common/services/resolverService.js';
 import { ILanguageFeaturesService } from '../../../../../editor/common/services/languageFeatures.js';
-import { env } from '../../../../../base/common/process.js';
-import { PYLANCE_DEBUG_DISPLAY_NAME } from './lspTerminalUtil.js';
-
+import { getTerminalLspSupportedLanguageObj } from './lspTerminalUtil.js';
+import { IOpenerService } from '../../../../../platform/opener/common/opener.js';
+import { Codicon } from '../../../../../base/common/codicons.js';
 
 registerSingleton(ITerminalCompletionService, TerminalCompletionService, InstantiationType.Delayed);
 
@@ -53,14 +53,12 @@ class TerminalSuggestContribution extends DisposableStore implements ITerminalCo
 	}
 
 	private readonly _addon: MutableDisposable<SuggestAddon> = new MutableDisposable();
-	private readonly _pwshAddon: MutableDisposable<PwshCompletionProviderAddon> = new MutableDisposable();
-	private readonly _lspAddon: MutableDisposable<LspCompletionProviderAddon> = new MutableDisposable(); // TODO: Support multiple lspAddons in the future.
+	private readonly _lspAddons: DisposableMap<string, LspCompletionProviderAddon> = this.add(new DisposableMap());
 	private readonly _lspModelProvider: MutableDisposable<LspTerminalModelContentProvider> = new MutableDisposable();
 	private readonly _terminalSuggestWidgetVisibleContextKey: IContextKey<boolean>;
 
 	get addon(): SuggestAddon | undefined { return this._addon.value; }
-	get pwshAddon(): PwshCompletionProviderAddon | undefined { return this._pwshAddon.value; }
-	get lspAddon(): LspCompletionProviderAddon | undefined { return this._lspAddon.value; }
+	get lspAddons(): LspCompletionProviderAddon[] { return Array.from(this._lspAddons.values()); }
 
 	constructor(
 		private readonly _ctx: ITerminalContributionContext,
@@ -74,8 +72,6 @@ class TerminalSuggestContribution extends DisposableStore implements ITerminalCo
 		super();
 		this.add(toDisposable(() => {
 			this._addon?.dispose();
-			this._pwshAddon?.dispose();
-			this._lspAddon?.dispose();
 			this._lspModelProvider?.value?.dispose();
 			this._lspModelProvider?.dispose();
 		}));
@@ -85,8 +81,7 @@ class TerminalSuggestContribution extends DisposableStore implements ITerminalCo
 				const completionsEnabled = this._configurationService.getValue<ITerminalSuggestConfiguration>(terminalSuggestConfigSection).enabled;
 				if (!completionsEnabled) {
 					this._addon.clear();
-					this._pwshAddon.clear();
-					this._lspAddon.clear();
+					this._lspAddons.clearAndDisposeAll();
 				}
 				const xtermRaw = this._ctx.instance.xterm?.raw;
 				if (!!xtermRaw && completionsEnabled) {
@@ -94,6 +89,23 @@ class TerminalSuggestContribution extends DisposableStore implements ITerminalCo
 				}
 			}
 		}));
+
+		// Initialize the dynamic providers configuration manager
+		TerminalSuggestProvidersConfigurationManager.initialize(this._instantiationService);
+
+		// Listen for terminal location changes to update the suggest widget container
+		this.add(this._ctx.instance.onDidChangeTarget((target) => {
+			this._updateContainerForTarget(target);
+		}));
+
+		// The terminal view can be reparented (for example when moved into a new view). Ensure the
+		// suggest widget follows the terminal's DOM when focus returns to the instance.
+		this.add(this._ctx.instance.onDidFocus(() => {
+			const xtermRaw = this._ctx.instance.xterm?.raw;
+			if (xtermRaw) {
+				this._prepareAddonLayout(xtermRaw);
+			}
+		}));
 	}
 
 	xtermOpen(xterm: IXtermTerminal & { raw: RawXtermTerminal }): void {
@@ -109,75 +121,15 @@ class TerminalSuggestContribution extends DisposableStore implements ITerminalCo
 		}));
 	}
 
-	private async _loadPwshCompletionAddon(xterm: RawXtermTerminal): Promise<void> {
-		// Disable when shell type is not powershell. A naive check is done for Windows PowerShell
-		// as we don't differentiate it in shellType
-		if (
-			this._ctx.instance.shellType !== GeneralShellType.PowerShell ||
-			this._ctx.instance.shellLaunchConfig.executable?.endsWith('WindowsPowerShell\\v1.0\\powershell.exe')
-		) {
-			this._pwshAddon.clear();
-			return;
-		}
-
-		// Disable the addon on old backends (not conpty or Windows 11)
-		await this._ctx.instance.processReady;
-		const processTraits = this._ctx.processManager.processTraits;
-		if (processTraits?.windowsPty && (processTraits.windowsPty.backend !== 'conpty' || processTraits?.windowsPty.buildNumber <= 19045)) {
-			return;
-		}
-
-		const pwshCompletionProviderAddon = this._pwshAddon.value = this._instantiationService.createInstance(PwshCompletionProviderAddon, this._ctx.instance.capabilities);
-		xterm.loadAddon(pwshCompletionProviderAddon);
-		this.add(pwshCompletionProviderAddon);
-		this.add(pwshCompletionProviderAddon.onDidRequestSendText(text => {
-			this._ctx.instance.sendText(text, false);
-		}));
-		this.add(this._terminalCompletionService.registerTerminalCompletionProvider('builtinPwsh', pwshCompletionProviderAddon.id, pwshCompletionProviderAddon));
-		// If completions are requested, pause and queue input events until completions are
-		// received. This fixing some problems in PowerShell, particularly enter not executing
-		// when typing quickly and some characters being printed twice. On Windows this isn't
-		// needed because inputs are _not_ echoed when not handled immediately.
-		// TODO: This should be based on the OS of the pty host, not the client
-		if (!isWindows) {
-			let barrier: AutoOpenBarrier | undefined;
-			if (pwshCompletionProviderAddon) {
-				this.add(pwshCompletionProviderAddon.onDidRequestSendText(() => {
-					barrier = new AutoOpenBarrier(2000);
-					this._ctx.instance.pauseInputEvents(barrier);
-				}));
-			}
-			if (this._pwshAddon.value) {
-				this.add(this._pwshAddon.value.onDidReceiveCompletions(() => {
-					barrier?.open();
-					barrier = undefined;
-				}));
-			} else {
-				throw Error('no addon');
-			}
-		}
-	}
-
-	// TODO: Eventually support multiple LSP providers for [non-Python REPLs](https://github.com/microsoft/vscode/issues/249479)
 	private async _loadLspCompletionAddon(xterm: RawXtermTerminal): Promise<void> {
-		const isWsl =
-			isLinux &&
-			(
-				!!env['WSL_DISTRO_NAME'] ||
-				!!env['WSL_INTEROP']
-			);
-
-		// Windows, WSL currently does not support shell integration for Python REPL.
-		if (isWindows || isWsl) {
+		let lspTerminalObj = undefined;
+		// TODO: Change to always load after settings update for terminal suggest provider
+		if (!this._ctx.instance.shellType || !(lspTerminalObj = getTerminalLspSupportedLanguageObj(this._ctx.instance.shellType))) {
+			this._lspAddons.clearAndDisposeAll();
 			return;
 		}
 
-		if (this._ctx.instance.shellType !== GeneralShellType.Python) {
-			this._lspAddon.clear();
-			return;
-		}
-
-		const virtualTerminalDocumentUri = createTerminalLanguageVirtualUri(this._ctx.instance.instanceId, 'py');
+		const virtualTerminalDocumentUri = createTerminalLanguageVirtualUri(this._ctx.instance.instanceId, lspTerminalObj.extension);
 
 		// Load and register the LSP completion providers (one per language server)
 		this._lspModelProvider.value = this._instantiationService.createInstance(LspTerminalModelContentProvider, this._ctx.instance.capabilities, this._ctx.instance.instanceId, virtualTerminalDocumentUri, this._ctx.instance.shellType);
@@ -187,12 +139,13 @@ class TerminalSuggestContribution extends DisposableStore implements ITerminalCo
 		this.add(textVirtualModel);
 
 		const virtualProviders = this._languageFeaturesService.completionProvider.all(textVirtualModel.object.textEditorModel);
-		const provider = virtualProviders.find(p => p._debugDisplayName === PYLANCE_DEBUG_DISPLAY_NAME);
+		const filteredProviders = virtualProviders.filter(p => p._debugDisplayName !== 'wordbasedCompletions');
 
-		if (provider) {
-			const lspCompletionProviderAddon = this._lspAddon.value = this._instantiationService.createInstance(LspCompletionProviderAddon, provider, textVirtualModel, this._lspModelProvider.value);
+		// Iterate through all available providers
+		for (const provider of filteredProviders) {
+			const lspCompletionProviderAddon = this._instantiationService.createInstance(LspCompletionProviderAddon, provider, textVirtualModel, this._lspModelProvider.value);
+			this._lspAddons.set(provider._debugDisplayName, lspCompletionProviderAddon);
 			xterm.loadAddon(lspCompletionProviderAddon);
-			this.add(lspCompletionProviderAddon);
 			this.add(this._terminalCompletionService.registerTerminalCompletionProvider(
 				'lsp',
 				lspCompletionProviderAddon.id,
@@ -208,17 +161,11 @@ class TerminalSuggestContribution extends DisposableStore implements ITerminalCo
 			return;
 		}
 
-		const addon = this._addon.value = this._instantiationService.createInstance(SuggestAddon, this._ctx.instance.shellType, this._ctx.instance.capabilities, this._terminalSuggestWidgetVisibleContextKey);
+		const addon = this._addon.value = this._instantiationService.createInstance(SuggestAddon, this._ctx.instance.sessionId, this._ctx.instance.shellType, this._ctx.instance.capabilities, this._terminalSuggestWidgetVisibleContextKey);
 		xterm.loadAddon(addon);
-		this._loadPwshCompletionAddon(xterm);
 		this._loadLspCompletionAddon(xterm);
 
-		if (this._ctx.instance.target === TerminalLocation.Editor) {
-			addon.setContainerWithOverflow(xterm.element!);
-		} else {
-			addon.setContainerWithOverflow(dom.findParentWithClass(xterm.element!, 'panel')!);
-		}
-		addon.setScreen(xterm.element!.querySelector('.xterm-screen')!);
+		this._prepareAddonLayout(xterm);
 
 		this.add(dom.addDisposableListener(this._ctx.instance.domElement, dom.EventType.FOCUS_OUT, (e) => {
 			const focusedElement = e.relatedTarget as HTMLElement;
@@ -259,7 +206,61 @@ class TerminalSuggestContribution extends DisposableStore implements ITerminalCo
 		}
 		// Relies on shell type being set
 		this._loadLspCompletionAddon(this._ctx.instance.xterm.raw);
-		this._loadPwshCompletionAddon(this._ctx.instance.xterm.raw);
+	}
+
+	private _updateContainerForTarget(target: TerminalLocation | undefined): void {
+		const addon = this._addon.value;
+		if (!addon || !this._ctx.instance.xterm?.raw) {
+			return;
+		}
+
+		this._prepareAddonLayout(this._ctx.instance.xterm.raw);
+	}
+
+
+	private async _prepareAddonLayout(xterm: RawXtermTerminal): Promise<void> {
+		const addon = this._addon.value;
+		if (!addon || this.isDisposed) {
+			return;
+		}
+
+		const xtermElement = xterm.element ?? await this._waitForXtermElement(xterm);
+		if (!xtermElement || this.isDisposed || addon !== this._addon.value) {
+			return;
+		}
+
+		const container = this._resolveAddonContainer(xtermElement);
+		addon.setContainerWithOverflow(container);
+		// eslint-disable-next-line no-restricted-syntax
+		const screenElement = xtermElement?.querySelector('.xterm-screen');
+		if (dom.isHTMLElement(screenElement)) {
+			addon.setScreen(screenElement);
+		}
+	}
+
+	private async _waitForXtermElement(xterm: RawXtermTerminal): Promise<HTMLElement | undefined> {
+		if (xterm.element) {
+			return xterm.element;
+		}
+
+		await Promise.race([
+			Event.toPromise(Event.filter(this._ctx.instance.onDidChangeVisibility, visible => visible)),
+			Event.toPromise(this._ctx.instance.onDisposed)
+		]);
+
+		if (this.isDisposed || this._ctx.instance.isDisposed) {
+			return undefined;
+		}
+
+		return xterm.element ?? undefined;
+	}
+
+	private _resolveAddonContainer(xtermElement: HTMLElement): HTMLElement {
+		if (this._ctx.instance.target === TerminalLocation.Editor) {
+			return xtermElement;
+		}
+
+		return dom.findParentWithClass(xtermElement, 'panel') ?? xtermElement;
 	}
 }
 
@@ -269,26 +270,167 @@ registerTerminalContribution(TerminalSuggestContribution.ID, TerminalSuggestCont
 
 // #region Actions
 
+registerTerminalAction({
+	id: TerminalSuggestCommandId.ChangeSelectionModeNever,
+	title: localize2('workbench.action.terminal.changeSelectionMode.never', 'Selection Mode: None'),
+	tooltip: localize2('workbench.action.terminal.changeSelectionMode.never.tooltip', 'Do not select the top suggestion until down is pressed, at which point Tab or Enter will accept the suggestion. Activate to change.'),
+	f1: false,
+	precondition: ContextKeyExpr.and(
+		ContextKeyExpr.or(TerminalContextKeys.processSupported, TerminalContextKeys.terminalHasBeenCreated),
+		TerminalContextKeys.focus,
+		TerminalContextKeys.isOpen,
+		TerminalContextKeys.suggestWidgetVisible,
+		ContextKeyExpr.equals(`config.${TerminalSuggestSettingId.SelectionMode}`, 'never')
+	),
+	menu: {
+		id: MenuId.MenubarTerminalSuggestStatusMenu,
+		group: 'left',
+		order: 1,
+		when: ContextKeyExpr.and(
+			ContextKeyExpr.equals(`config.${TerminalSuggestSettingId.SelectionMode}`, 'never'),
+			ContextKeyExpr.or(
+				ContextKeyExpr.equals(`config.${TerminalSuggestSettingId.QuickSuggestions}`, true),
+				ContextKeyExpr.equals(`config.${TerminalSuggestSettingId.SuggestOnTriggerCharacters}`, true),
+			)
+		)
+	},
+	run: (c, accessor) => {
+		accessor.get(IConfigurationService).updateValue(TerminalSuggestSettingId.SelectionMode, 'partial');
+	}
+});
+registerTerminalAction({
+	id: TerminalSuggestCommandId.ChangeSelectionModePartial,
+	title: localize2('workbench.action.terminal.changeSelectionMode.partial', 'Selection Mode: Partial (Tab)'),
+	tooltip: localize2('workbench.action.terminal.changeSelectionMode.partial.tooltip', 'Partially select the top suggestion, Tab will accept a suggestion when visible. Activate to change.'),
+	f1: false,
+	precondition: ContextKeyExpr.and(
+		ContextKeyExpr.or(TerminalContextKeys.processSupported, TerminalContextKeys.terminalHasBeenCreated),
+		TerminalContextKeys.focus,
+		TerminalContextKeys.isOpen,
+		TerminalContextKeys.suggestWidgetVisible,
+		ContextKeyExpr.equals(`config.${TerminalSuggestSettingId.SelectionMode}`, 'partial')
+	),
+	menu: {
+		id: MenuId.MenubarTerminalSuggestStatusMenu,
+		group: 'left',
+		order: 1,
+		when: ContextKeyExpr.and(
+			ContextKeyExpr.equals(`config.${TerminalSuggestSettingId.SelectionMode}`, 'partial'),
+			ContextKeyExpr.or(
+				ContextKeyExpr.equals(`config.${TerminalSuggestSettingId.QuickSuggestions}`, true),
+				ContextKeyExpr.equals(`config.${TerminalSuggestSettingId.SuggestOnTriggerCharacters}`, true),
+			)
+		)
+	},
+	run: (c, accessor) => {
+		accessor.get(IConfigurationService).updateValue(TerminalSuggestSettingId.SelectionMode, 'always');
+	}
+});
+registerTerminalAction({
+	id: TerminalSuggestCommandId.ChangeSelectionModeAlways,
+	title: localize2('workbench.action.terminal.changeSelectionMode.always', 'Selection Mode: Always (Tab or Enter)'),
+	tooltip: localize2('workbench.action.terminal.changeSelectionMode.always.tooltip', 'Always select the top suggestion, Tab or Enter will accept a suggestion when visible. Activate to change.'),
+	f1: false,
+	precondition: ContextKeyExpr.and(ContextKeyExpr.or(TerminalContextKeys.processSupported, TerminalContextKeys.terminalHasBeenCreated), TerminalContextKeys.focus, TerminalContextKeys.isOpen, TerminalContextKeys.suggestWidgetVisible),
+	menu: {
+		id: MenuId.MenubarTerminalSuggestStatusMenu,
+		group: 'left',
+		order: 1,
+		when: ContextKeyExpr.and(
+			ContextKeyExpr.equals(`config.${TerminalSuggestSettingId.SelectionMode}`, 'always'),
+			ContextKeyExpr.or(
+				ContextKeyExpr.equals(`config.${TerminalSuggestSettingId.QuickSuggestions}`, true),
+				ContextKeyExpr.equals(`config.${TerminalSuggestSettingId.SuggestOnTriggerCharacters}`, true),
+			)
+		)
+	},
+	run: (c, accessor) => {
+		accessor.get(IConfigurationService).updateValue(TerminalSuggestSettingId.SelectionMode, 'never');
+	}
+});
+
+registerTerminalAction({
+	id: TerminalSuggestCommandId.DoNotShowOnType,
+	title: localize2('workbench.action.terminal.doNotShowSuggestOnType', 'Don\'t show IntelliSense unless triggered explicitly. This disables the quick suggestions and suggest on trigger characters settings.'),
+	f1: false,
+	precondition: ContextKeyExpr.and(ContextKeyExpr.or(TerminalContextKeys.processSupported, TerminalContextKeys.terminalHasBeenCreated), TerminalContextKeys.focus, TerminalContextKeys.isOpen, TerminalContextKeys.suggestWidgetVisible),
+	icon: Codicon.eye,
+	menu: {
+		id: MenuId.MenubarTerminalSuggestStatusMenu,
+		group: 'right',
+		order: 1,
+		when: ContextKeyExpr.and(
+			ContextKeyExpr.or(
+				ContextKeyExpr.equals(`config.${TerminalSuggestSettingId.QuickSuggestions}.commands`, 'on'),
+				ContextKeyExpr.equals(`config.${TerminalSuggestSettingId.QuickSuggestions}.arguments`, 'on'),
+			),
+			ContextKeyExpr.equals(`config.${TerminalSuggestSettingId.SuggestOnTriggerCharacters}`, true),
+		),
+	},
+	run: (c, accessor) => {
+		accessor.get(IConfigurationService).updateValue(TerminalSuggestSettingId.QuickSuggestions, { commands: 'off', arguments: 'off', unknown: 'off' });
+		accessor.get(IConfigurationService).updateValue(TerminalSuggestSettingId.SuggestOnTriggerCharacters, false);
+	}
+});
+
+registerTerminalAction({
+	id: TerminalSuggestCommandId.ShowOnType,
+	title: localize2('workbench.action.terminal.showSuggestOnType', 'Show IntelliSense while typing. This enables the quick suggestions for commands and arguments, and suggest on trigger characters settings.'),
+	f1: false,
+	precondition: ContextKeyExpr.and(ContextKeyExpr.or(TerminalContextKeys.processSupported, TerminalContextKeys.terminalHasBeenCreated), TerminalContextKeys.focus, TerminalContextKeys.isOpen, TerminalContextKeys.suggestWidgetVisible),
+	icon: Codicon.eyeClosed,
+	menu: {
+		id: MenuId.MenubarTerminalSuggestStatusMenu,
+		group: 'right',
+		order: 1,
+		when: ContextKeyExpr.or(
+			ContextKeyExpr.and(
+				ContextKeyExpr.equals(`config.${TerminalSuggestSettingId.QuickSuggestions}.commands`, 'off'),
+				ContextKeyExpr.equals(`config.${TerminalSuggestSettingId.QuickSuggestions}.arguments`, 'off'),
+			),
+			ContextKeyExpr.equals(`config.${TerminalSuggestSettingId.SuggestOnTriggerCharacters}`, false),
+		),
+	},
+	run: (c, accessor) => {
+		accessor.get(IConfigurationService).updateValue(TerminalSuggestSettingId.QuickSuggestions, { commands: 'on', arguments: 'on', unknown: 'off' });
+		accessor.get(IConfigurationService).updateValue(TerminalSuggestSettingId.SuggestOnTriggerCharacters, true);
+	}
+});
+
+registerTerminalAction({
+	id: TerminalSuggestCommandId.LearnMore,
+	title: localize2('workbench.action.terminal.learnMore', 'Learn More'),
+	f1: false,
+	precondition: ContextKeyExpr.and(ContextKeyExpr.or(TerminalContextKeys.processSupported, TerminalContextKeys.terminalHasBeenCreated), TerminalContextKeys.focus, TerminalContextKeys.isOpen, TerminalContextKeys.suggestWidgetVisible),
+	icon: Codicon.question,
+	menu: {
+		id: MenuId.MenubarTerminalSuggestStatusMenu,
+		group: 'right',
+		order: 2
+	},
+	run: (c, accessor) => {
+		(accessor.get(IOpenerService)).open('https://aka.ms/vscode-terminal-intellisense');
+	}
+});
+
 registerTerminalAction({
 	id: TerminalSuggestCommandId.ConfigureSettings,
 	title: localize2('workbench.action.terminal.configureSuggestSettings', 'Configure'),
 	f1: false,
 	precondition: ContextKeyExpr.and(ContextKeyExpr.or(TerminalContextKeys.processSupported, TerminalContextKeys.terminalHasBeenCreated), TerminalContextKeys.focus, TerminalContextKeys.isOpen, TerminalContextKeys.suggestWidgetVisible),
-	keybinding: {
-		primary: KeyMod.CtrlCmd | KeyMod.Shift | KeyCode.Comma,
-		weight: KeybindingWeight.WorkbenchContrib
-	},
+	icon: Codicon.gear,
 	menu: {
 		id: MenuId.MenubarTerminalSuggestStatusMenu,
 		group: 'right',
-		order: 1
+		order: 3
 	},
 	run: (c, accessor) => accessor.get(IPreferencesService).openSettings({ query: terminalSuggestConfigSection })
 });
 
 registerActiveInstanceAction({
-	id: TerminalSuggestCommandId.RequestCompletions,
-	title: localize2('workbench.action.terminal.requestCompletions', 'Request Completions'),
+	id: TerminalSuggestCommandId.TriggerSuggest,
+	title: localize2('workbench.action.terminal.triggerSuggest', 'Trigger Suggest'),
+	f1: false,
 	keybinding: {
 		primary: KeyMod.CtrlCmd | KeyCode.Space,
 		mac: { primary: KeyMod.WinCtrl | KeyCode.Space },
@@ -404,16 +546,18 @@ registerActiveInstanceAction({
 	title: localize2('workbench.action.terminal.acceptSelectedSuggestion', 'Insert'),
 	f1: false,
 	precondition: ContextKeyExpr.and(ContextKeyExpr.or(TerminalContextKeys.processSupported, TerminalContextKeys.terminalHasBeenCreated), TerminalContextKeys.focus, TerminalContextKeys.isOpen, TerminalContextKeys.suggestWidgetVisible),
-	keybinding: {
+	keybinding: [{
 		primary: KeyCode.Tab,
 		// Tab is bound to other workbench keybindings that this needs to beat
+		weight: KeybindingWeight.WorkbenchContrib + 2,
+		when: ContextKeyExpr.and(SimpleSuggestContext.HasFocusedSuggestion)
+	},
+	{
+		primary: KeyCode.Enter,
+		// Enter accepts when: explicitly invoked (ctrl+space), OR not in partial mode, OR not first suggestion, OR user has navigated
+		when: ContextKeyExpr.and(SimpleSuggestContext.HasFocusedSuggestion, ContextKeyExpr.or(SimpleSuggestContext.ExplicitlyInvoked, ContextKeyExpr.notEquals(`config.${TerminalSuggestSettingId.SelectionMode}`, 'partial'), SimpleSuggestContext.FirstSuggestionFocused.toNegated(), SimpleSuggestContext.HasNavigated)),
 		weight: KeybindingWeight.WorkbenchContrib + 1
-	},
-	menu: {
-		id: MenuId.MenubarTerminalSuggestStatusMenu,
-		order: 1,
-		group: 'left'
-	},
+	}],
 	run: (activeInstance) => TerminalSuggestContribution.get(activeInstance)?.addon?.acceptSelectedSuggestion()
 });
 
@@ -426,7 +570,7 @@ registerActiveInstanceAction({
 		primary: KeyCode.Enter,
 		// Enter is bound to other workbench keybindings that this needs to beat
 		weight: KeybindingWeight.WorkbenchContrib + 1,
-		when: ContextKeyExpr.notEquals(`config.${TerminalSuggestSettingId.RunOnEnter}`, 'ignore'),
+		when: ContextKeyExpr.notEquals(`config.${TerminalSuggestSettingId.RunOnEnter}`, 'never'),
 	},
 	run: async (activeInstance) => TerminalSuggestContribution.get(activeInstance)?.addon?.acceptSelectedSuggestion(undefined, true)
 });
@@ -462,3 +606,47 @@ registerActiveInstanceAction({
 });
 
 // #endregion
+
+// #region Dynamic Providers Configuration
+
+class TerminalSuggestProvidersConfigurationManager extends Disposable {
+	private static _instance: TerminalSuggestProvidersConfigurationManager | undefined;
+
+	static initialize(instantiationService: IInstantiationService): void {
+		if (!this._instance) {
+			this._instance = instantiationService.createInstance(TerminalSuggestProvidersConfigurationManager);
+		}
+	}
+
+	constructor(
+		@ITerminalCompletionService private readonly _terminalCompletionService: ITerminalCompletionService,
+		@ITerminalContributionService private readonly _terminalContributionService: ITerminalContributionService
+	) {
+		super();
+		this._register(this._terminalCompletionService.onDidChangeProviders(() => {
+			this._updateConfiguration();
+		}));
+		this._register(this._terminalContributionService.onDidChangeTerminalCompletionProviders(() => {
+			this._updateConfiguration();
+		}));
+		// Initial configuration
+		this._updateConfiguration();
+	}
+
+	private _updateConfiguration(): void {
+		// Add statically declared providers from package.json contributions
+		const providers = new Map<string, ITerminalSuggestProviderInfo>();
+		this._terminalContributionService.terminalCompletionProviders.forEach(o => providers.set(o.extensionIdentifier, { ...o, id: o.extensionIdentifier }));
+
+		// Add dynamically registered providers (that aren't already declared statically)
+		for (const { id } of this._terminalCompletionService.providers) {
+			if (id && !providers.has(id)) {
+				providers.set(id, { id });
+			}
+		}
+
+		registerTerminalSuggestProvidersConfiguration(providers);
+	}
+}
+
+// #endregion
