@@ -5,7 +5,7 @@
 
 import { CancellationToken } from '../../../../base/common/cancellation.js';
 import { createDecorator } from '../../../../platform/instantiation/common/instantiation.js';
-import { BrowserType, IElementData } from '../../../../platform/browserElements/common/browserElements.js';
+import { IElementData, IBrowserTargetLocator } from '../../../../platform/browserElements/common/browserElements.js';
 import { IRectangle } from '../../../../platform/window/common/window.js';
 
 export const IBrowserElementsService = createDecorator<IBrowserElementsService>('browserElementsService');
@@ -14,7 +14,7 @@ export interface IBrowserElementsService {
 	_serviceBrand: undefined;
 
 	// no browser implementation yet
-	getElementData(rect: IRectangle, token: CancellationToken, browserType: BrowserType | undefined): Promise<IElementData | undefined>;
+	getElementData(rect: IRectangle, token: CancellationToken, locator: IBrowserTargetLocator | undefined): Promise<IElementData | undefined>;
 
-	startDebugSession(token: CancellationToken, browserType: BrowserType): Promise<void>;
+	startDebugSession(token: CancellationToken, locator: IBrowserTargetLocator): Promise<void>;
 }
