@@ -6,14 +6,14 @@
 import './media/suggest.css';
 import * as dom from '../../../../base/browser/dom.js';
 import { IListEvent, IListGestureEvent, IListMouseEvent } from '../../../../base/browser/ui/list/list.js';
-import { List } from '../../../../base/browser/ui/list/listWidget.js';
+import { IListStyles, List } from '../../../../base/browser/ui/list/listWidget.js';
 import { ResizableHTMLElement } from '../../../../base/browser/ui/resizable/resizable.js';
 import { SimpleCompletionItem } from './simpleCompletionItem.js';
 import { LineContext, SimpleCompletionModel } from './simpleCompletionModel.js';
 import { getAriaId, SimpleSuggestWidgetItemRenderer, type ISimpleSuggestWidgetFontInfo } from './simpleSuggestWidgetRenderer.js';
 import { CancelablePromise, createCancelablePromise, disposableTimeout, TimeoutTimer } from '../../../../base/common/async.js';
 import { Emitter, Event, PauseableEmitter } from '../../../../base/common/event.js';
-import { MutableDisposable, Disposable } from '../../../../base/common/lifecycle.js';
+import { MutableDisposable, Disposable, IDisposable } from '../../../../base/common/lifecycle.js';
 import { clamp } from '../../../../base/common/numbers.js';
 import { localize } from '../../../../nls.js';
 import { IInstantiationService } from '../../../../platform/instantiation/common/instantiation.js';
@@ -21,11 +21,14 @@ import { SuggestWidgetStatus } from '../../../../editor/contrib/suggest/browser/
 import { MenuId } from '../../../../platform/actions/common/actions.js';
 import { IConfigurationService } from '../../../../platform/configuration/common/configuration.js';
 import { IStorageService, StorageScope, StorageTarget } from '../../../../platform/storage/common/storage.js';
-import { canExpandCompletionItem, SimpleSuggestDetailsOverlay, SimpleSuggestDetailsWidget } from './simpleSuggestWidgetDetails.js';
+import { canExpandCompletionItem, SimpleSuggestDetailsOverlay, SimpleSuggestDetailsWidget, type SimpleSuggestDetailsPlacement } from './simpleSuggestWidgetDetails.js';
 import { IContextKey, IContextKeyService, RawContextKey } from '../../../../platform/contextkey/common/contextkey.js';
 import * as strings from '../../../../base/common/strings.js';
 import { status } from '../../../../base/browser/ui/aria/aria.js';
 import { isWindows } from '../../../../base/common/platform.js';
+import { editorSuggestWidgetForeground, editorSuggestWidgetSelectedBackground } from '../../../../editor/contrib/suggest/browser/suggestWidget.js';
+import { getListStyles } from '../../../../platform/theme/browser/defaultStyles.js';
+import { activeContrastBorder, focusBorder } from '../../../../platform/theme/common/colorRegistry.js';
 
 const $ = dom.$;
 
@@ -58,6 +61,8 @@ const enum WidgetPositionPreference {
 export const SimpleSuggestContext = {
 	HasFocusedSuggestion: new RawContextKey<boolean>('simpleSuggestWidgetHasFocusedSuggestion', false, localize('simpleSuggestWidgetHasFocusedSuggestion', "Whether any simple suggestion is focused")),
 	HasNavigated: new RawContextKey<boolean>('simpleSuggestWidgetHasNavigated', false, localize('simpleSuggestWidgetHasNavigated', "Whether the simple suggestion widget has been navigated downwards")),
+	FirstSuggestionFocused: new RawContextKey<boolean>('simpleSuggestWidgetFirstSuggestionFocused', false, localize('simpleSuggestWidgetFirstSuggestionFocused', "Whether the first simple suggestion is focused")),
+	ExplicitlyInvoked: new RawContextKey<boolean>('simpleSuggestWidgetExplicitlyInvoked', false, localize('simpleSuggestWidgetExplicitlyInvoked', "Whether the simple suggestion widget was explicitly invoked")),
 };
 
 export interface IWorkbenchSuggestWidgetOptions {
@@ -71,6 +76,38 @@ export interface IWorkbenchSuggestWidgetOptions {
 	 * The setting for showing the status bar.
 	 */
 	showStatusBarSettingId?: string;
+
+	/**
+	 * The setting for selection mode.
+	 */
+	selectionModeSettingId?: string;
+
+	/**
+	 * Disables specific detail placements when positioning the details overlay.
+	 */
+	preventDetailsPlacements?: readonly SimpleSuggestDetailsPlacement[];
+}
+
+/**
+ * Controls how suggest selection works
+*/
+export const enum SuggestSelectionMode {
+	/**
+	 * Default. Will show a border and only accept via Tab until navigation has occurred. After that, it will show selection and accept via Enter or Tab.
+	 */
+	Partial = 'partial',
+	/**
+	 * Always select, what enter does depends on runOnEnter.
+	 */
+	Always = 'always',
+	/**
+	 * User needs to press down to select.
+	 */
+	Never = 'never'
+}
+
+const enum Classes {
+	PartialSelection = 'partial-selection',
 }
 
 export class SimpleSuggestWidget<TModel extends SimpleCompletionModel<TItem>, TItem extends SimpleCompletionItem> extends Disposable {
@@ -79,6 +116,7 @@ export class SimpleSuggestWidget<TModel extends SimpleCompletionModel<TItem>, TI
 	private static NO_SUGGESTIONS_MESSAGE: string = localize('suggestWidget.noSuggestions', "No suggestions.");
 
 	private _state: State = State.Hidden;
+	private _loadingTimeout?: IDisposable;
 	private _completionModel?: TModel;
 	private _cappedHeight?: { wanted: number; capped: number };
 	private _forceRenderingAbove: boolean = false;
@@ -114,6 +152,8 @@ export class SimpleSuggestWidget<TModel extends SimpleCompletionModel<TItem>, TI
 
 	private readonly _ctxSuggestWidgetHasFocusedSuggestion: IContextKey<boolean>;
 	private readonly _ctxSuggestWidgetHasBeenNavigated: IContextKey<boolean>;
+	private readonly _ctxFirstSuggestionFocused: IContextKey<boolean>;
+	private readonly _ctxSuggestWidgetExplicitlyInvoked: IContextKey<boolean>;
 
 	constructor(
 		private readonly _container: HTMLElement,
@@ -134,6 +174,8 @@ export class SimpleSuggestWidget<TModel extends SimpleCompletionModel<TItem>, TI
 		this._container.appendChild(this.element.domNode);
 		this._ctxSuggestWidgetHasFocusedSuggestion = SimpleSuggestContext.HasFocusedSuggestion.bindTo(_contextKeyService);
 		this._ctxSuggestWidgetHasBeenNavigated = SimpleSuggestContext.HasNavigated.bindTo(_contextKeyService);
+		this._ctxFirstSuggestionFocused = SimpleSuggestContext.FirstSuggestionFocused.bindTo(_contextKeyService);
+		this._ctxSuggestWidgetExplicitlyInvoked = SimpleSuggestContext.ExplicitlyInvoked.bindTo(_contextKeyService);
 
 		class ResizeState {
 			constructor(
@@ -234,11 +276,11 @@ export class SimpleSuggestWidget<TModel extends SimpleCompletionModel<TItem>, TI
 
 		const details: SimpleSuggestDetailsWidget = this._register(_instantiationService.createInstance(SimpleSuggestDetailsWidget, this._getFontInfo.bind(this), this._onDidFontConfigurationChange.bind(this), this._getAdvancedExplainModeDetails.bind(this)));
 		this._register(details.onDidClose(() => this.toggleDetails()));
-		this._details = this._register(new SimpleSuggestDetailsOverlay(details, this._listElement));
+		this._details = this._register(new SimpleSuggestDetailsOverlay(details, this._listElement, this._options.preventDetailsPlacements));
 		this._register(dom.addDisposableListener(this._details.widget.domNode, 'blur', (e) => this._onDidBlurDetails.fire(e)));
 
 		if (_options.statusBarMenuId && _options.showStatusBarSettingId && _configurationService.getValue(_options.showStatusBarSettingId)) {
-			this._status = this._register(_instantiationService.createInstance(SuggestWidgetStatus, this.element.domNode, _options.statusBarMenuId));
+			this._status = this._register(_instantiationService.createInstance(SuggestWidgetStatus, this.element.domNode, _options.statusBarMenuId, { showIconsNoKeybindings: true }));
 			this.element.domNode.classList.toggle('with-status-bar', true);
 		}
 
@@ -258,7 +300,7 @@ export class SimpleSuggestWidget<TModel extends SimpleCompletionModel<TItem>, TI
 			if (_options.statusBarMenuId && _options.showStatusBarSettingId && e.affectsConfiguration(_options.showStatusBarSettingId)) {
 				const showStatusBar: boolean = _configurationService.getValue(_options.showStatusBarSettingId);
 				if (showStatusBar && !this._status) {
-					this._status = this._register(_instantiationService.createInstance(SuggestWidgetStatus, this.element.domNode, _options.statusBarMenuId));
+					this._status = this._register(_instantiationService.createInstance(SuggestWidgetStatus, this.element.domNode, _options.statusBarMenuId, { showIconsNoKeybindings: true }));
 					this._status.show();
 				} else if (showStatusBar && this._status) {
 					this._status.show();
@@ -356,6 +398,8 @@ export class SimpleSuggestWidget<TModel extends SimpleCompletionModel<TItem>, TI
 
 			}).catch();
 		}
+
+		this._ctxFirstSuggestionFocused.set(index === 0);
 		// emit an event
 		this._onDidFocus.fire({ item, index, model: this._completionModel });
 	}
@@ -384,11 +428,35 @@ export class SimpleSuggestWidget<TModel extends SimpleCompletionModel<TItem>, TI
 		this._persistedSize.reset();
 	}
 
+	relayout(cursorPosition: { top: number; left: number; height: number }): void {
+		if (this._state === State.Hidden) {
+			return;
+		}
+		this._cursorPosition = cursorPosition;
+		this._layout(this.element.size);
+		this._afterRender();
+	}
+
+	showTriggered(explicitlyInvoked: boolean, cursorPosition: { top: number; left: number; height: number }) {
+		if (this._state !== State.Hidden) {
+			return;
+		}
+		this._cursorPosition = cursorPosition;
+		this._ctxSuggestWidgetExplicitlyInvoked.set(!!explicitlyInvoked);
+
+		if (this._ctxSuggestWidgetExplicitlyInvoked.get()) {
+			this._loadingTimeout = disposableTimeout(() => this._setState(State.Loading), 250);
+		}
+	}
+
 	showSuggestions(selectionIndex: number, isFrozen: boolean, isAuto: boolean, cursorPosition: { top: number; left: number; height: number }): void {
 		this._cursorPosition = cursorPosition;
 
-		// this._contentWidget.setPosition(this.editor.getPosition());
-		// this._loadingTimeout?.dispose();
+		this._loadingTimeout?.dispose();
+
+		const selectionMode = this._options?.selectionModeSettingId ? this._configurationService.getValue<SuggestSelectionMode>(this._options.selectionModeSettingId) : undefined;
+		// When explicitly invoked (not auto), always select the first item regardless of selectionMode
+		const noFocus = !this._ctxSuggestWidgetExplicitlyInvoked.get() && selectionMode === SuggestSelectionMode.Never;
 
 		// this._currentSuggestionDetails?.cancel();
 		// this._currentSuggestionDetails = undefined;
@@ -420,8 +488,7 @@ export class SimpleSuggestWidget<TModel extends SimpleCompletionModel<TItem>, TI
 			this._list.splice(0, this._list.length, this._completionModel?.items ?? []);
 			this._setState(isFrozen ? State.Frozen : State.Open);
 			this._list.reveal(selectionIndex, 0);
-			this._list.setFocus([selectionIndex]);
-			// this._list.setFocus(noFocus ? [] : [selectionIndex]);
+			this._list.setFocus(noFocus ? [] : [selectionIndex]);
 		} finally {
 			// this._onDidFocus.resume();
 			// this._onDidSelect.resume();
@@ -433,9 +500,20 @@ export class SimpleSuggestWidget<TModel extends SimpleCompletionModel<TItem>, TI
 			// Reset focus border
 			// this._details.widget.domNode.classList.remove('focused');
 		});
+		this._updateListStyles();
 		this._afterRender();
 	}
 
+	private _updateListStyles(): void {
+		if (this._options.selectionModeSettingId) {
+			const selectionMode = this._configurationService.getValue<SuggestSelectionMode>(this._options.selectionModeSettingId);
+			// When explicitly invoked, always show full selection (background) instead of partial (border)
+			const usePartialStyle = !this._ctxSuggestWidgetExplicitlyInvoked.get() && selectionMode === SuggestSelectionMode.Partial;
+			this._list.style(getListStylesWithMode(usePartialStyle));
+			this.element.domNode.classList.toggle(Classes.PartialSelection, usePartialStyle);
+		}
+	}
+
 	setLineContext(lineContext: LineContext): void {
 		if (this._completionModel) {
 			this._completionModel.lineContext = lineContext;
@@ -483,6 +561,7 @@ export class SimpleSuggestWidget<TModel extends SimpleCompletionModel<TItem>, TI
 				this._details.hide();
 				this._show();
 				this._focusedItem = undefined;
+				status(SimpleSuggestWidget.LOADING_MESSAGE);
 				break;
 			case State.Empty:
 				this.element.domNode.classList.add('message');
@@ -617,8 +696,10 @@ export class SimpleSuggestWidget<TModel extends SimpleCompletionModel<TItem>, TI
 	hide(): void {
 		this._pendingLayout.clear();
 		this._pendingShowDetails.clear();
-		// this._loadingTimeout?.dispose();
+		this._loadingTimeout?.dispose();
 		this._ctxSuggestWidgetHasBeenNavigated.reset();
+		this._ctxFirstSuggestionFocused.reset();
+		this._ctxSuggestWidgetExplicitlyInvoked.reset();
 		this._setState(State.Hidden);
 		this._onDidHide.fire(this);
 		dom.hide(this.element.domNode);
@@ -665,7 +746,7 @@ export class SimpleSuggestWidget<TModel extends SimpleCompletionModel<TItem>, TI
 		// 	width = info.defaultSize.width / 2;
 		// 	this.element.enableSashes(false, false, false, false);
 		// 	this.element.minSize = this.element.maxSize = new dom.Dimension(width, height);
-		// 	this._contentWidget.setPreference(ContentWidgetPositionPreference.BELOW);
+		// 	this._preference = WidgetPositionPreference.Below;
 
 		// } else {
 		// showing items
@@ -678,12 +759,19 @@ export class SimpleSuggestWidget<TModel extends SimpleCompletionModel<TItem>, TI
 		const preferredWidth = this._completionModel ? this._completionModel.stats.pLabelLen * info.typicalHalfwidthCharacterWidth : width;
 
 		// height math
-		const fullHeight = info.statusBarHeight + this._list.contentHeight + this._messageElement.clientHeight + info.borderHeight;
+		// Cap list content height to a reasonable maximum (12 items worth), matching suggestWidget behavior
+		const cappedListContentHeight = Math.min(this._list.contentHeight, info.itemHeight * 12);
+		const fullHeight = info.statusBarHeight + cappedListContentHeight + this._messageElement.clientHeight + info.borderHeight;
 		const minHeight = info.itemHeight + info.statusBarHeight;
 		// const editorBox = dom.getDomNodePagePosition(this.editor.getDomNode());
 		// const cursorBox = this.editor.getScrolledVisiblePosition(this.editor.getPosition());
 		const editorBox = dom.getDomNodePagePosition(this._container);
-		const cursorBox = this._cursorPosition; //this.editor.getScrolledVisiblePosition(this.editor.getPosition());
+		// Convert absolute cursor position to relative position (relative to container)
+		const cursorBox = {
+			top: this._cursorPosition.top - editorBox.top,
+			left: this._cursorPosition.left,
+			height: this._cursorPosition.height
+		};
 		const cursorBottom = editorBox.top + cursorBox.top + cursorBox.height;
 		const maxHeightBelow = Math.min(bodyBox.height - cursorBottom - info.verticalPadding, fullHeight);
 		const availableSpaceAbove = editorBox.top + cursorBox.top - info.verticalPadding;
@@ -704,7 +792,7 @@ export class SimpleSuggestWidget<TModel extends SimpleCompletionModel<TItem>, TI
 		}
 
 		const forceRenderingAboveRequiredSpace = 150;
-		if (height > maxHeightBelow || (this._forceRenderingAbove && availableSpaceAbove > forceRenderingAboveRequiredSpace)) {
+		if ((height > maxHeightBelow && maxHeightAbove > maxHeightBelow) || (this._forceRenderingAbove && availableSpaceAbove > forceRenderingAboveRequiredSpace)) {
 			this._preference = WidgetPositionPreference.Above;
 			this.element.enableSashes(true, true, false, false);
 			maxHeight = maxHeightAbove;
@@ -724,12 +812,22 @@ export class SimpleSuggestWidget<TModel extends SimpleCompletionModel<TItem>, TI
 			? { wanted: this._cappedHeight?.wanted ?? size.height, capped: height }
 			: undefined;
 		// }
-		this.element.domNode.style.left = `${this._cursorPosition.left}px`;
+		// Horizontal positioning: Position widget at cursor, flip to left if would overflow right
+		let anchorLeft = this._cursorPosition.left;
+		const wouldOverflowRight = anchorLeft + width > bodyBox.width;
+
+		if (wouldOverflowRight) {
+			// Position right edge at cursor (extends left)
+			anchorLeft = this._cursorPosition.left - width;
+		}
+
+		this.element.domNode.style.left = `${anchorLeft}px`;
 		if (this._preference === WidgetPositionPreference.Above) {
 			this.element.domNode.style.top = `${this._cursorPosition.top - height - info.borderHeight}px`;
 		} else {
 			this.element.domNode.style.top = `${this._cursorPosition.top + this._cursorPosition.height}px`;
 		}
+		// }
 		this._resize(width, height);
 	}
 
@@ -820,6 +918,7 @@ export class SimpleSuggestWidget<TModel extends SimpleCompletionModel<TItem>, TI
 	}
 
 	selectNext(): boolean {
+		this._clearPartialSelectionState();
 		this._list.focusNext(1, true);
 		const focus = this._list.getFocus();
 		if (focus.length > 0) {
@@ -829,6 +928,7 @@ export class SimpleSuggestWidget<TModel extends SimpleCompletionModel<TItem>, TI
 	}
 
 	selectNextPage(): boolean {
+		this._clearPartialSelectionState();
 		this._list.focusNextPage();
 		const focus = this._list.getFocus();
 		if (focus.length > 0) {
@@ -838,6 +938,7 @@ export class SimpleSuggestWidget<TModel extends SimpleCompletionModel<TItem>, TI
 	}
 
 	selectPrevious(): boolean {
+		this._clearPartialSelectionState();
 		this._list.focusPrevious(1, true);
 		const focus = this._list.getFocus();
 		if (focus.length > 0) {
@@ -847,6 +948,7 @@ export class SimpleSuggestWidget<TModel extends SimpleCompletionModel<TItem>, TI
 	}
 
 	selectPreviousPage(): boolean {
+		this._clearPartialSelectionState();
 		this._list.focusPreviousPage();
 		const focus = this._list.getFocus();
 		if (focus.length > 0) {
@@ -855,6 +957,11 @@ export class SimpleSuggestWidget<TModel extends SimpleCompletionModel<TItem>, TI
 		return true;
 	}
 
+	private _clearPartialSelectionState(): void {
+		this._list.style(getListStylesWithMode(false));
+		this.element.domNode.classList.remove(Classes.PartialSelection);
+	}
+
 	getFocusedItem(): ISimpleSelectedSuggestion<TItem> | undefined {
 		if (this._completionModel) {
 			return {
@@ -885,3 +992,19 @@ export class SimpleSuggestWidget<TModel extends SimpleCompletionModel<TItem>, TI
 		this._forceRenderingAbove = false;
 	}
 }
+
+function getListStylesWithMode(partial?: boolean): IListStyles {
+	// The suggest widget uses the list's inactive focus to mean selection since it's not actually
+	// focused.
+	if (partial) {
+		return getListStyles({
+			listInactiveFocusOutline: focusBorder,
+			listInactiveFocusForeground: editorSuggestWidgetForeground,
+		});
+	} else {
+		return getListStyles({
+			listInactiveFocusBackground: editorSuggestWidgetSelectedBackground,
+			listInactiveFocusOutline: activeContrastBorder
+		});
+	}
+}
